#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: accCreateACCTables.sql,v 1.45 2004/02/13 14:13:05 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# clevin  17/10/95  created
# clevin  12/04/96  - file renamed from accCreateDatabase.sql to
#                     accCreateACCTables.tbl.
#                   - tables "station","prog_environment" and "lcu_bootenv"
#                     where moved to accCreateVCCTables.tbl in order to
#                     have an independent table file for VCC.
# clevin  17/03/97  - table "subnet" was moved to accCreateVCCTables.tbl.
#
#************************************************************************
#   NAME	   
#       accCreateACCTables.sql
#
#   SYNOPSIS	   
#       msql -h $ACC_HOST acc < accCreateACCTables.sql
# 
#   DESCRIPTION    
#       SQL script to create the ACC database tables.
#
#   ENVIRONMENT
#       ACC_HOST - ACC host name
#
#   CAUTIONS
#       - The mSQL server (msqld) should be running.
#       - The ACC database must be created previously 
#         with the msqladmin command.
#       - Only authorized users can run this command.
#
#   SEE ALSO
#       accServer(1)
#
#------------------------------------------------------------------------
#

####################   ENTITIES  #######################

# VLT environments
#
# book_status   -> accBOOKED | accNOT_BOOKED
# active_status -> accACTIVE | accNOT_ACTIVE
#
create table vlt_environment (
        vltenv_name    char(30) primary key,
        description    char(80),
        book_status    char(15),
        active_status  char(15)
)
\g

# Equipments
create table equipment (
        equip_name   char(30) primary key,
        description  char(80)
)
\g

# Components
#
# book_status   -> accBOOKED | accNOT_BOOKED
# active_status -> accACTIVE | accNOT_ACTIVE
#
create table component (
        comp_name      char(30) primary key,
        description    char(80),
        book_status    char(15),
        active_status  char(15)
)
\g

# Users
#
# category -> accOBSERVER | accOPERATOR | accDEVELOPER | accBOOK_MASTER
#
create table user (
        user_name     char(8) primary key,
        description   char(80),
	booking_code  int,
	password      char(15),
        category      char(15)
)
\g

####################   RELATIONSHIPS  #######################

# VLT resource booking information  (resource(N) <--> user(1))
#
# resource_type -> accENVIRONMENT | accCOMPONENT
#
create table resource_booking (
	resource_type     char(15),
	resource_name     char(30),
	booking_code      int,
	station_name      char(30),
	purpose           char(80),
	start_time        int,
	end_time          int
)
\g

# Network access control information  (subnet(N) <--> user(N) <--> station(N))
#
#
#                   N   +--------+   N
#         subnet -------|        |------- user
#                       +---+----+
#                           |
#                          N|
#                           |
#                        station
#
create table network_access (
	subnet_name     char(30),
	user_name       char(8),
	station_name    char(30),
	purpose         char(80),
	start_time      int,
	end_time        int
)
\g

# Active users  (user(N) <--> station(N))
create table active_user (
	user_name       char(8),
	station_name    char(30)
)
\g

# VLT environment / component  (N <--> N)
create table vltenv_comp (
	vltenv_name  char(30),
	comp_name    char(30)
)
\g

# VLT environment / equipment  (N <--> N)
create table vltenv_equip (
	vltenv_name  char(30),
	equip_name   char(30)
)
\g

# equipment / component  (N <--> N)
create table equip_comp (
	equip_name  char(30),
	comp_name   char(30)
)
\g

# process / RTAP-CCSlite environment / component
#
#
#                   N   +--------+   N
#        process -------|        |------- prog_environment
#                       +---+----+
#                           |
#                          1|
#                           |
#                       component
#
##
# proc_type    -> accBOOKABLE | accSHARABLE
##
# book_status  -> accBOOKED | accNOT_BOOKED
##
# error_status -> it is filled internally by the commands RQRESOU and RLRESOU
##
create table proc_progenv_comp (
	proc_name     char(30),
        proc_type     char(30),
	progenv_name  char(30),
	comp_name     char(30),
	ref_comp      char(30),
	proc_owner    char(8),
        book_status   char(15),
        error_status  char(15)
)
\g

# point category / RTAP-CCSlite environment / component
#
#
#                        N   +--------+   N
#      point_category -------|        |------- prog_environment
#                            +---+----+
#                                |
#                               1|
#                                |
#                            component
#
#
##
# pcat_type -> accCAT_READ | accCAT_WRITE | accCAT_RW
#
# progenv_name == ""  --> default category for all RTAP-CCSlite environments
# comp_name    == ""  --> default category for the specific RTAP-CCSlite 
#                         environment
##
create table pcat_progenv_comp (
        pcat_name     char(30),
        pcat_desc     char(80),
	pcat_type     int,
	pcat_number   int,
	progenv_name  char(30),
	comp_name     char(30)
)
\g

#
# ___oOo___
