#ifndef _icmd_h
#define _icmd_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: icmd.h,v 1.104 2005/09/12 09:20:48 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  19/09/96  created
* jstegmei  05/02/98  added icmdGetHost function
* jstegmei  18/02/98  added sdmaFUNCTION_KEY to test multiple function keys
* jstegmei  23/03/98  added icmdVerboseIn(), icmdVerboseOut()
* jstegmei  20/05/98  added icmdPing()
* jstegmei  12/06/98  removed obsolete icmdVerboseIn(), icmdVerboseOut()
* jstegmei  30/06/98  added icmdGetIpAddr()
* jstegmei  22/10/98  added icmdExpression()
* jstegmei  12/01/99  added icmdSetSockTimeout()
* jstegmei  25/05/99  added icmdGetTime(), icmdGetUTC()
* jstegmei  18/09/00  implemented byte-swap for LINUX
*/

/************************************************************************
 *
 *  icmd.h - icmd common constants
 *
 *----------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _XOPEN_SOURCE_EXTENDED
#define icmdSOCKLEN_T socklen_t
#else
#ifdef LINUX
#define icmdSOCKLEN_T socklen_t
#else
#define icmdSOCKLEN_T int
#endif
#endif

#ifndef USE
#define USE(x) x=x
#endif

/*
 *  link type
 */
#define icmdLINK int

/*
 *  swap mode
 */
#ifdef LINUX
#define icmdNO_SWAP 1
#define icmdSWAP    0
#else
#define icmdNO_SWAP 0
#define icmdSWAP    1
#endif

/*
 *  command type
 */
typedef struct
{
    short  id;
    int    pLen;
    char  *pBuf;
} icmdCOM;

/*
 *  reply type
 */
typedef struct
{
    short  errorId;
    short  errorPt;
    int    pLen;
    char  *pBuf;
} icmdREP;

/*
 *  command buffer sizes
 */
#define icmdCOM_H_SIZE       8 
#define icmdREP_H_SIZE      10 
  
/*
 *  maximum command line length (in characters)
 */
#define icmdEditMAX_CMDLEN 256


/*
 *  command stack size
 */
#define icmdEditSTACK_SIZE 20

/*
 *  function keys
 */
#define icmdFUNCTION_KEY(x) ((x == 91) || (x == 79))
#define icmdEditESC              27
#define icmdEditF_CRSR_UP        65
#define icmdEditF_CRSR_DOWN      66
#define icmdEditF_CRSR_LEFT      68
#define icmdEditF_CRSR_RIGHT     67
#define icmdEditF_INSERT         50

/*
 *  comment character
 */
#define icmdEditCOMMENT   '#'

/*
 *  maximum number of open files
 */
#define icmdEditMAX_FILES      32

/*
 *  control keys
 */
#define icmdEditCR             13
#define icmdEditBS              8
#define icmdEditINSERT          9
#define icmdEditEND             5
#define icmdEditBEGIN           2
#define icmdEditDEL             4
#define icmdEditDELLINE        11

/*
 *  keyword parser
 */
#define icmdKEY_COMMENT    '#'
#define icmdKEY_NEWLINE    '\n'
#define icmdKEY_BREAK      ';'
#define icmdKEY_NULL         0
#define icmdKEY_SEP        '.'
#define icmdKEY_VECSEP     ','
#define icmdMAX_KEY_LEN    256
#define icmdMAX_KEYELE_LEN 512

/*
 *  storage type
 */
typedef struct
{
  int test;
} icmdEditTYPE;

/*
 *  expression type
 */
typedef struct
{
  float value;
  char  name[64];
  int   id;
  char  changed;
} icmdVAR;

/*
 *  timer
 */
typedef struct
{
  long tv_sec;    /* seconds since Jan. 1, 1970 */
  long tv_usec;   /* and microseconds           */
} icmdTIME_V;
int  icmdGetTime(struct timeval *, void *);
int  icmdGetUTC(char *, int);
void icmdTime2UTC(struct timeval, char *, int) ;

/*
 *  protoypes
 */
void     icmdSetSockTimeout(int);
int      icmdPing(const char *, int, char *);
int      icmdGetHost(const char *, char *, char *, char *);
int      icmdGetHostName(char *, int);
int      icmdGetIpAddr(const char *, char *, char *);
int      icmdRemote(void);
void     icmdRemoteProcReply(int, char, char *);
int      icmdRemoteInit(const char *, char *);
int      icmdRemoteExit(const char *, char *);
int      icmdRemoteKill(const char *, int, char *);
int      icmdRemoteExec(const char *, int *, const char *, char *, char *);
void     icmdSwap2(void *, int);
void     icmdSwap4(void *, int);
void     icmdSwap8(void *, int);
void     icmdPutValChar(char *, char, int *);
void     icmdGenPutValShort(char, char *, short, int *);
void     icmdGenPutValInt(char, char *, int, int *);
void     icmdGenPutValFloat(char, char *, float, int *);
void     icmdGenPutValDouble(char, char *, double, int *);
char     icmdGetValChar(char *, int *);
short    icmdGenGetValShort(char, char *, int *);
int      icmdGenGetValInt(char, char *, int *);
float    icmdGenGetValFloat(char, char *, int *);
double   icmdGenGetValDouble(char, char *, int *);
void     icmdGenPutVec(char, char *, char*, int, int *, int);
void     icmdGenGetVec(char, char *, char*, int, int *, int);

#ifdef _icmdswap
#define icmdPutValShort(x,y,z)    icmdGenPutValShort(icmdSWAP,x,y,z)
#define icmdPutValInt(x,y,z)      icmdGenPutValInt(icmdSWAP,x,y,z)
#define icmdPutValFloat(x,y,z)    icmdGenPutValFloat(icmdSWAP,x,y,z)
#define icmdPutValDouble(x,y,z)   icmdGenPutValDouble(icmdSWAP,x,y,z)
#define icmdGetValShort(y,z)      icmdGenGetValShort(icmdSWAP,y,z)
#define icmdGetValInt(y,z)        icmdGenGetValInt(icmdSWAP,y,z)
#define icmdGetValFloat(y,z)      icmdGenGetValFloat(icmdSWAP,y,z)
#define icmdGetValDouble(y,z)     icmdGenGetValDouble(icmdSWAP,y,z)
#define icmdPutVecChar(w,x,y,z)   icmdGenPutVec(icmdSWAP,w,(char *)x,y,z,1)
#define icmdPutVecShort(w,x,y,z)  icmdGenPutVec(icmdSWAP,w,(char *)x,y,z,2)
#define icmdPutVecInt(w,x,y,z)    icmdGenPutVec(icmdSWAP,w,(char *)x,y,z,4)
#define icmdPutVecFloat(w,x,y,z)  icmdGenPutVec(icmdSWAP,w,(char *)x,y,z,4)
#define icmdPutVecDouble(w,x,y,z) icmdGenPutVec(icmdSWAP,w,(char *)x,y,z,8)
#define icmdGetVecChar(w,x,y,z)   icmdGenGetVec(icmdSWAP,w,(char *)x,y,z,1)
#define icmdGetVecShort(w,x,y,z)  icmdGenGetVec(icmdSWAP,w,(char *)x,y,z,2)
#define icmdGetVecInt(w,x,y,z)    icmdGenGetVec(icmdSWAP,w,(char *)x,y,z,4)
#define icmdGetVecFloat(w,x,y,z)  icmdGenGetVec(icmdSWAP,w,(char *)x,y,z,4)
#define icmdGetVecDouble(w,x,y,z) icmdGenGetVec(icmdSWAP,w,(char *)x,y,z,8)
#else
#define icmdPutValShort(x,y,z)    icmdGenPutValShort(icmdNO_SWAP,x,y,z)
#define icmdPutValInt(x,y,z)      icmdGenPutValInt(icmdNO_SWAP,x,y,z)
#define icmdPutValFloat(x,y,z)    icmdGenPutValFloat(icmdNO_SWAP,x,y,z)
#define icmdPutValDouble(x,y,z)   icmdGenPutValDouble(icmdNO_SWAP,x,y,z)
#define icmdGetValShort(y,z)      icmdGenGetValShort(icmdNO_SWAP,y,z)
#define icmdGetValInt(y,z)        icmdGenGetValInt(icmdNO_SWAP,y,z)
#define icmdGetValFloat(y,z)      icmdGenGetValFloat(icmdNO_SWAP,y,z)
#define icmdGetValDouble(y,z)     icmdGenGetValDouble(icmdNO_SWAP,y,z)
#define icmdPutVecChar(w,x,y,z)   icmdGenPutVec(icmdNO_SWAP,w,(char *)x,y,z,1)
#define icmdPutVecShort(w,x,y,z)  icmdGenPutVec(icmdNO_SWAP,w,(char *)x,y,z,2)
#define icmdPutVecInt(w,x,y,z)    icmdGenPutVec(icmdNO_SWAP,w,(char *)x,y,z,4)
#define icmdPutVecFloat(w,x,y,z)  icmdGenPutVec(icmdNO_SWAP,w,(char *)x,y,z,4)
#define icmdPutVecDouble(w,x,y,z) icmdGenPutVec(icmdNO_SWAP,w,(char *)x,y,z,8)
#define icmdGetVecChar(w,x,y,z)   icmdGenGetVec(icmdNO_SWAP,w,(char *)x,y,z,1)
#define icmdGetVecShort(w,x,y,z)  icmdGenGetVec(icmdNO_SWAP,w,(char *)x,y,z,2)
#define icmdGetVecInt(w,x,y,z)    icmdGenGetVec(icmdNO_SWAP,w,(char *)x,y,z,4)
#define icmdGetVecFloat(w,x,y,z)  icmdGenGetVec(icmdNO_SWAP,w,(char *)x,y,z,4)
#define icmdGetVecDouble(w,x,y,z) icmdGenGetVec(icmdNO_SWAP,w,(char *)x,y,z,8)
#endif /* _icmdswap */


/*
 *  line editor protoypes
 */
FILE *icmdEditOpenRepStream(const char *, char *);
int   icmdEditTerminate(const char *, char *);
int   icmdEditGetLine(const char *, char *, char *);
int   icmdEditReadStream(FILE *, char *, char *);
int   icmdEditReadKeyboard(char *, char *);
int   icmdEditGetKey(char *, char *);
int   icmdEditPollKey(char *, char *);
int   icmdEditTermTty(char *);

/*
 *  keyword parser prototypes
 */
int icmdKeyGetNext(char *, int,  int *, int *, 
		   char *, char *, char *, char *);
int icmdKeySplit(char *, int *, char **, int *, char *); 
int icmdKeyGetVecInt(char *, int *, char *);
int icmdKeyGetVecFloat(char *, float *, char *);

/*
 *  expression handler prototypes
 */
int icmdExpression(float *, icmdVAR *, char *, int, char *);

/*
 *  external variables
 */
extern int icmdSockTimeout;  /* global socket connection timeout */

#ifdef __cplusplus
}
#endif

#endif /*!_icmd_h*/
