#ifndef _iracqSRV_h
#define _iracqSRV_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: iracqSRV.h,v 1.113 2005/09/13 07:27:10 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  27/05/99  created
*/

/************************************************************************
 *
 *  iracqSRV.h - acquisition control server base class definition (NOCCS)
 *
 *----------------------------------------------------------------------
 */


#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "iracq.h"

#ifndef USE
#define USE(x) x=x
#endif


/*
 *  ascii server reply tags
 */
#define iracqTAG_SUCCESS         "\n!SUCCESS!\n"
#define iracqTAG_ERROR           "\n!ERROR!\n"
#define iracqTAG_ERROR_SEVERE    "\n!ERROR_SEVERE!\n"
#define iracqTAG_VERBOSE         "\n!VB!\n"
#define iracqTAG_STATE_SRV       "\n!STATE_SRV!\n"
#define iracqTAG_SUBSTATE_SRV    "\n!SUBSTATE_SRV!\n"
#define iracqTAG_UPDATE          "\n!UPDATE!\n"
#define iracqTAG_DATA            "\n!DATA!\n"

/*
 *  maximum command length
 */
#define iracqMAX_CMD_SIZE      64
#define iracqMAX_PARAM_SIZE  (16 * 1024)
#define iracqMAX_BUF_SIZE (iracqMAX_PARAM_SIZE + iracqMAX_CMD_SIZE + 2)

/*
 *  maximum number of connections
 */
#define iracqSRV_MAX_CMD  4   /* maximum number of command connections */
#define iracqSRV_MAX_EVT  4   /* maximum number of event connections   */

/*
 *  maximum number of command handler addresses
 */
#define iracqSRV_MAX_FN  512 

/*
 *  update
 */
#define iracqNO_UPDATE 0
#define iracqDO_UPDATE 1

/*
 *  maximum size of log-file
 */
#define iracqMAX_LOG_SIZE (1024 * 1024)

/*
 *  function handle
 */
typedef struct
{
  char     param[iracqMAX_PARAM_SIZE];
  int      cmdSock;
} iracqSRV_HANDLE;


class iracqSRV {
  public:

  typedef int (iracqSRV::*iracqFUNCTION)(iracqSRV_HANDLE *, char *);
  typedef struct {
    char            cmd[64];
    iracqFUNCTION   function;
    int             update;
    char            syntax[64];
    char            help[256];;
  } iracqFN;

  iracqSRV();             // constructor  
  virtual ~iracqSRV();    // destructor 
  
  int   verboseStat;
  int   verboseLevel;
  int   logLevel;
  int   active;

  // initialization routines
  void           PrintUsage(const char *);
  int            ParseInputPars(int, char **);
  int            Init(char *);
  int            InitTest();
  void           AbortServer();
  void           DisableLog();
  const char    *Version();
  const char    *Instance();
  const char    *ProcName(const char *);
  const char    *UserName();
  const char    *GetFileExt(const char *);

  // install signal handler for SIGUSR1/2
  void           InstallSig();

  // asynchronous handlers
  void           VbHandlerGen(const char *);
  void           StateHandlerGen(int, int);
  void           DataHandlerGen(iracqDATA_EVT *);
  void           ExitHandlerGen(int);
  void           BreakHandler(int);
  int            ParamHandler(const char *, char *, int *, char *);
  int            ParamStatHandler(const char *, char *, int *, char *);

  // verbose callback
  virtual void   Verbose(const char *, ...);

  // server state callback
  virtual void   StateHandler(int, int);

  // data transfer task event callback
  virtual void   DataHandler(iracqDATA_EVT *);

  // exit callback
  virtual void   ExitHandler(int);

  // parameter setup callback
  virtual int    ParamCallBack(const char *, const char *, int *, char *);

  // parameter status callback
  virtual int    ParamStatCallBack(const char *, char *, int *, int *, char *);

  // command extension callbacks
  virtual int    ExtOnline(char *);
  virtual int    ExtStandby(char *);
  virtual int    ExtAbort(char *);

  // server main loop
  int            MainLoop();

  // socket server
  int            OpenServer(char *);
  int            CloseServer();
  int            CloseClient(int);
  int            ConnectCmd(char *);
  int            ConnectEvt(char *);
  int            HandleEvtIn(int, char *);
  int            CheckStdIn();
  int            HandleStdIn(int *, char *);
  int            HandleSockIn(int, int *, char *);
  int            GetCmdSock();
  int            GetEvtSock();
  int            FdStat(int *, int *, int *, int *, int *);

  // command handler
  void           EnterSrvCmd();
  void           ExitSrvCmd(int);
  void           CleanupCmd(char *, int *);
  int            CmdHandler(iracqSRV_HANDLE *, const char *, char *, int *);

  // status mapping functions
  void           SeqStat2Str(char *, int);
  void           SeqStat2StrRaw(char *, int);
  void           CldcStat2Str(char *, int);
  void           AcqStat2Str(char *, int);
  void           SysStat2Str(char *, int);
  void           ExpStat2Str(char *, int);
  void           SubStat2Str(char *, int);
  void           OpMode2Str(char *, int);

  // FITS handling
  int            AddFits(const char *, int *, const char *, char *);
  int            AddFits(const char *, float *, const char *, char *);
  int            AddFits(const char *, double *, const char *, char *);
  int            AddFits(const char *, char *, const char *, char *);
  int            AddFits(const char *, char *);
  int            AddFitsDynPar(const char *, const char *, char *);
  int            AddFitsDyn(const char *, const char *, const char *, char *);
  int            AddFitsDyn(const char *, char *);
  void           ClearFitsDyn();
  int            DumpFits(const char *, char *);

  // frame control
  int            FrameDefault(const char *, int, int, int, char *);
  int            AddFrame(int, const char *, int, int, int, char *, char *);
  int            BreakCnt(const char *, int, char *);
  int            FrameGen(const char *, int, char *);
  int            FrameStore(const char *, int, char *);

  // download datasets
  int            DownloadData(int, int, const char *, char *);
  int            DownloadDataRaw(int, int, char *, int, char *);

  // image post processing
  int            ImgOp(int, char *);

  // exposure status
  int            ExpActive();
  int            ExpStatus();
  
  protected:

  private:
  // verbose handling
  int            xterm_;
  int            doStdIn_;
  int            doInitTest_;
  FILE          *logFd_;
  int            logSize_;
  int            disableLog_;
  int            OpenLog_();
  void           CloseLog_();
  void           RemoveLog_();
  int            ResetLog_();
  int            LogMsg_(const char *);

  // data transfer task process name
  char           dttName_[256];

  // system configuration file
  char           sysConfigFile_[256];

  // detector configuration file
  char           detConfigFile_[256];

  // socket handling
  int            srvCmdPort_;                
  int            remoteFd_;
  int            srvCmdSock_;                
  int            cmdSock_[iracqSRV_MAX_CMD];
  int            cmdSockActive_;
  int            srvEvtPort_;                
  int            srvEvtSock_;                
  int            evtSock_[iracqSRV_MAX_EVT]; 
  int            evtMask_[iracqSRV_MAX_EVT]; 
  fd_set         readMask_;                  
  fd_set         readFds_;                   
  int            srvVbChan_;
  iracqFN      **srvFnList_;
  int            srvFnListSize_;
  int            break_;
  int            waitSock_[iracqSRV_MAX_CMD];
  int            numWait_;

  // version string
  char           versionString_[256];

  // process name
  char           procName_[64];
  char           instance_[64];
  char           userName_[64];

  // system (sub-) status
  int            subState_;

  // exposure status
  int            expStatus_;

  // global flags
  int            sim_;
  int            srvOpen_;

  // socket handling functions
  int            SockReadCmd_(int, char *, int, char *);
  int            SockWriteReply_(int, const char *, int, char, char *);
  int            SockVbOut_(int, const char *, ...);
  int            SockPuts_(int, const char *, char *);
  int            SockGets_(int, char *, int, char *);
  int            SockSendEvent_(int, const char *, const char *);
  int            SetReadMask_(fd_set *);
  void           BroadcastEvent_(int *, int, const char*, const char *);

  // command handler
  int            SetupHandle_(char *erms);
  int            AddHandle_(const char *cmd, iracqFUNCTION, int, char *);
  int            AddHandle_(const char *cmd, iracqFUNCTION, int, 
			    char *, char *, char *);
  int            CheckFunction_(iracqFN *, char *);

  // command handling functions
  int            HandleOnline_(iracqSRV_HANDLE *, char *);
  int            HandleStandby_(iracqSRV_HANDLE *, char *);
  int            HandleOff_(iracqSRV_HANDLE *, char *);
  int            HandleExit_(iracqSRV_HANDLE *, char *);
  int            HandleGetConf_(iracqSRV_HANDLE *, char *);
  int            HandleDetConf_(iracqSRV_HANDLE *, char *);
  int            HandleSysConf_(iracqSRV_HANDLE *, char *);
  int            HandleSetOpMode_(iracqSRV_HANDLE *, char *);
  int            HandleSetReadoutMode_(iracqSRV_HANDLE *, char *);
  int            HandleGetParam_(iracqSRV_HANDLE *, char *);
  int            HandleLdv_(iracqSRV_HANDLE *, char *);
  int            HandleLdclk_(iracqSRV_HANDLE *, char *);
  int            HandleLdseq_(iracqSRV_HANDLE *, char *);
  int            HandleLdacq_(iracqSRV_HANDLE *, char *);
  int            HandleVsetup_(iracqSRV_HANDLE *, char *);
  int            HandleTel_(iracqSRV_HANDLE *, char *);
  int            HandleVoltChan_(iracqSRV_HANDLE *, char *);
  int            HandleTelChan_(iracqSRV_HANDLE *, char *);
  int            HandleVoltDc_(iracqSRV_HANDLE *, char *);
  int            HandleVoltClk_(iracqSRV_HANDLE *, char *);
  int            HandleTelDc_(iracqSRV_HANDLE *, char *);
  int            HandleTelClk_(iracqSRV_HANDLE *, char *);
  int            HandleSetCldc_(iracqSRV_HANDLE *, char *);
  int            HandleDac_(iracqSRV_HANDLE *, char *);
  int            HandleStart_(iracqSRV_HANDLE *, char *);
  int            HandleStop_(iracqSRV_HANDLE *, char *);
  int            HandleStartSeq_(iracqSRV_HANDLE *, char *);
  int            HandleStopSeq_(iracqSRV_HANDLE *, char *);
  int            HandleStartAcq_(iracqSRV_HANDLE *, char *);
  int            HandleStopAcq_(iracqSRV_HANDLE *, char *);
  int            HandleResetCnt_(iracqSRV_HANDLE *, char *);
  int            HandleStatus_(iracqSRV_HANDLE *, char *);
  int            HandleSaveVolt_(iracqSRV_HANDLE *, char *);
  int            HandleParamApply_(iracqSRV_HANDLE *, char *);
  int            HandleParamDefault_(iracqSRV_HANDLE *, char *);
  int            HandleParamGet_(iracqSRV_HANDLE *, char *);
  int            HandleParamSet_(iracqSRV_HANDLE *, char *);
  int            HandleStartExp_(iracqSRV_HANDLE *, char *);
  int            HandleStopExp_(iracqSRV_HANDLE *, char *);
  int            HandleEndExp_(iracqSRV_HANDLE *, char *);
  int            HandlePause_(iracqSRV_HANDLE *, char *);
  int            HandleCont_(iracqSRV_HANDLE *, char *);
  int            HandleSetDit_(iracqSRV_HANDLE *, char *);
  int            HandleSeqBase_(iracqSRV_HANDLE *, char *);
  int            HandleSeqCont_(iracqSRV_HANDLE *, char *);
  int            HandleNoNcReset_(iracqSRV_HANDLE *, char *);
  int            HandleNop_(iracqSRV_HANDLE *, char *);
  int            HandleRspeed_(iracqSRV_HANDLE *, char *);
  int            HandleHelp_(iracqSRV_HANDLE *, char *);
  int            HandleAdcSet_(iracqSRV_HANDLE *, char *);
  int            HandleAdcGroupSet_(iracqSRV_HANDLE *, char *);
  int            HandleAdcGet_(iracqSRV_HANDLE *, char *);
  int            HandleAdcRev_(iracqSRV_HANDLE *, char *);
  int            HandleRstat_(iracqSRV_HANDLE *, char *);
  int            HandleWstat_(iracqSRV_HANDLE *, char *);
  int            HandleSetWin_(iracqSRV_HANDLE *, char *);
  int            HandleFrameGen_(iracqSRV_HANDLE *, char *);
  int            HandleFrameStore_(iracqSRV_HANDLE *, char *);
  int            HandleGetFrames_(iracqSRV_HANDLE *, char *);
  int            HandleExtFits_(iracqSRV_HANDLE *, char *);
  int            HandleUsrCmd_(iracqSRV_HANDLE *, char *);
  int            HandleLdData_(iracqSRV_HANDLE *, char *);
  int            HandleDataCube_(iracqSRV_HANDLE *, char *);
  int            HandleNamingType_(iracqSRV_HANDLE *, char *);
  int            HandleReqName_(iracqSRV_HANDLE *, char *);
  int            HandleSeqMode_(iracqSRV_HANDLE *, char *);
  int            HandleBrkCnt_(iracqSRV_HANDLE *, char *);
  int            HandleReset_(iracqSRV_HANDLE *, char *);
  int            HandlePerf_(iracqSRV_HANDLE *, char *);
  int            HandleMset_(iracqSRV_HANDLE *, char *);
  int            HandleCamType_(iracqSRV_HANDLE *, char *);
  int            HandleGetSubPat_(iracqSRV_HANDLE *, char *);
  int            HandleGetSeq_(iracqSRV_HANDLE *, char *);
  int            HandleGetSSD_(iracqSRV_HANDLE *, char *);
  int            HandleWait_(iracqSRV_HANDLE *, char *);
  int            HandleMinCv_(iracqSRV_HANDLE *, char *);
  int            HandleTimeCalc_(iracqSRV_HANDLE *, char *);
  int            HandleAddFrame_(iracqSRV_HANDLE *, char *);
  int            HandleAscii_(iracqSRV_HANDLE *, char *);
};

typedef void (iracqSRV::*iracqSTAT2STR)(char *, int);

void iracqSrvVbHandler(const char *, ...);
void iracqSrvStateHandler(int);
void iracqSrvDataHandler(iracqDATA_EVT *);
void iracqSrvExitHandler(int);
void iracqSrvBreakHandler(int);

#endif /*!_iracqSrv_h*/
