#ifndef _IRPCI_H
#define _IRPCI_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: irpci.h,v 1.104 2005/09/12 09:20:49 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  07/03/02  created
*/

/************************************************************************
 *
 *  irpci.h - irpci driver interface
 *
 *----------------------------------------------------------------------
 */


#ifdef	__cplusplus
extern "C" {
#endif

/* Register mapping */
#define IRPCI_DMA_REG_FIFO    0x00         /* DMA FIFO                   */
#define IRPCI_DMA_REG_STAT    0x04         /* DMA status register        */
#define IRPCI_DMA_REG_CTRL    0x08         /* DMA control register       */
#define IRPCI_COM_REG_FIFO    0x00         /* COM Rx/Tx FIFO             */
#define IRPCI_COM_REG_WR_CNT  0x04         /* COM Tx FIFO status         */
#define IRPCI_COM_REG_RD_CNT  0x08         /* COM Rx FIFO status         */
#define IRPCI_COM_REG_CTRL    0x10         /* COM control register       */

/* Register values and masks */
#define IRPCI_DMA_CLEAR      0x00000001   /* clear DMA FIFO             */
#define IRPCI_DMA_SET_ABORT  0x00000004   /* abort DMA                  */
#define IRPCI_DMA_CLR_ABORT  0x00000008   /* clear-abort DMA            */
#define IRPCI_DMA_HDR        0x00000007   /* DMA header mask            */
#define IRPCI_DMA_SWAP       0x00000008   /* 16-Bit swap                */
#define IRPCI_DMA_TEST       0x00000010   /* DMA test-mode              */
#define IRPCI_DMA_EMPTY      0x00000020   /* DMA FIFO empty             */
#define IRPCI_DMA_FULL       0x00000040   /* DMA FIFO full              */
#define IRPCI_DMA_FULL_W     0x00000080   /* DMA FIFO full - write      */
#define IRPCI_COM_CNT        0x000000FF   /* mask for counter           */
#define IRPCI_COM_SET_RESET  0x00000001   /* reset COM-port             */
#define IRPCI_COM_CLR_RESET  0x00000002   /* reset COM-port             */
#define IRPCI_COM_CLEAR      0x00000004   /* just clear COM-port FIFO   */
#define IRPCI_COM_FIFO_SIZE  128          /* COM-port FIFO size         */

typedef struct ipc_ring {
  int nbuf;
  unsigned long size;
} ipc_ring_t;

typedef struct ipc_param {
  unsigned int reg;
  unsigned int val;
  unsigned int off;
} ipc_param_t;

typedef struct ipc_param64 {
  unsigned int reg;
  unsigned long long val;
  unsigned int off;
} ipc_param64_t;

/*
 * IOCTL's
 */
enum ipc_ioctl { 
  IRPCI_D_COM_GET,
  IRPCI_D_COM_SET,
  IRPCI_D_COM_RESET,
  IRPCI_D_COM_TMO,
  IRPCI_D_DMA_GET,
  IRPCI_D_DMA_SET,
  IRPCI_D_DMA_RESET,
  IRPCI_D_DMA_START,
  IRPCI_D_DMA_WAIT,
  IRPCI_D_DMA_ABORT,
  IRPCI_D_DMA_DONECOUNT,
  IRPCI_D_DMA_BYTECOUNT,
  IRPCI_D_DMA_OVERFLOW,
  IRPCI_D_DMA_CFG_BUF,
  IRPCI_D_DMA_INIT_BUF,
  IRPCI_D_DMA_SYNC_BUF,
  IRPCI_D_DMA_HDR_SET,
  IRPCI_D_CFG_GET,
  IRPCI_D_CFG_SET,
  IRPCI_D_PCFG_GET,
  IRPCI_D_PCFG_SET,
  IRPCI_D_LOC64_GET,
  IRPCI_D_LOC64_SET
};

/* Control bits */




/* ioctl's for communication port */
#define	IRPCI_COM_GET	       _IOR('I', (IRPCI_D_COM_GET), ipc_param_t)
#define	IRPCI_COM_SET	       _IOW('I', (IRPCI_D_COM_SET), ipc_param_t)
#define	IRPCI_COM_RESET	        _IO('I', (IRPCI_D_COM_RESET))
#define	IRPCI_COM_TMO	       _IOW('I', (IRPCI_D_COM_TMO), unsigned int)

/* ioctl's for DMA */
#define	IRPCI_DMA_GET	       _IOR('I', (IRPCI_D_DMA_GET), ipc_param_t)
#define	IRPCI_DMA_SET	       _IOW('I', (IRPCI_D_DMA_SET), ipc_param_t)
#define	IRPCI_DMA_RESET	        _IO('I', (IRPCI_D_DMA_RESET))
#define	IRPCI_DMA_START	        _IO('I', (IRPCI_D_DMA_START))
#define	IRPCI_DMA_WAIT	        _IO('I', (IRPCI_D_DMA_WAIT))
#define	IRPCI_DMA_ABORT	        _IO('I', (IRPCI_D_DMA_ABORT))
#define	IRPCI_DMA_DONECOUNT    _IOR('I', (IRPCI_D_DMA_DONECOUNT), unsigned int)
#define	IRPCI_DMA_BYTECOUNT    _IOR('I', (IRPCI_D_DMA_BYTECOUNT), unsigned int)
#define	IRPCI_DMA_OVERFLOW     _IOR('I', (IRPCI_D_DMA_OVERFLOW), unsigned int)
#define	IRPCI_DMA_CFG_BUF      _IOR('I', (IRPCI_D_DMA_CFG_BUF), ipc_ring_t) 
#define	IRPCI_DMA_INIT_BUF     _IO('I', (IRPCI_D_DMA_INIT_BUF)) 
#define	IRPCI_DMA_SYNC_BUF     _IO('I', (IRPCI_D_DMA_SYNC_BUF)) 
#define	IRPCI_DMA_HDR_SET      _IOW('I', (IRPCI_D_DMA_HDR_SET), unsigned int)
#define	IRPCI_LOC64_GET	       _IOR('I', (IRPCI_D_LOC64_GET), ipc_param64_t)
#define	IRPCI_LOC64_SET	       _IOW('I', (IRPCI_D_LOC64_SET), ipc_param64_t)

/* general GET/SET ioctl's for CFG */
#define	IRPCI_CFG_GET	       _IOR('I', (IRPCI_D_CFG_GET), ipc_param_t)
#define	IRPCI_CFG_SET	       _IOW('I', (IRPCI_D_CFG_SET), ipc_param_t)
#define	IRPCI_PCFG_GET	       _IOR('I', (IRPCI_D_PCFG_SET), ipc_param_t)
#define	IRPCI_PCFG_SET	       _IOW('I', (IRPCI_D_PCFG_SET), ipc_param_t)

#ifdef	__cplusplus
}
#endif

#endif /* !_IRPCI_H */
