 #ifndef oslxFITS_EXT_H
#define oslxFITS_EXT_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxFITS_EXT.h,v 1.253 2005/08/17 13:09:22 vltsccm Exp $" 
*
* oslxFITS_EXT.h
*
* who       when       what
* --------  --------   ----------------------------------------------
* epozna    01/09/00   Created
* epozna    15/10/00   VLTSW00990131  sorting added
* epozna    19/10/00   NOCCS compillation
* epozna    25/04/00   new functions :Cancel() to reset object without saving tmp files 
*                                     RemoveFile(), RemoveTmpFiles()                       
* epozna   18/07/01   added functions: AddIndexToCatgory/ChangeCategory
* - - - --  -- - - MAR2002- - - - - - -- - - - - - - -- - - - - - - -- -- - - -
* epozna   23/07/02 VLTSW20020431  added function 'ReadKeywordStringValue'
*                   compillation on te5 
* - - - --  -- - - APR2003- - - - - - -- - - - - - - -- - - - - - - -- -- - - -
* epozna   08/10/03 define oslxcfitsMOD_ID instead of oslxcfitsMOD_ID  
*                   to make ErrAdd Check easier
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund  19/01/04   VLTSW20040005 Ordering of FITS keywords & EXTEND
*                     keyword is default set to TRUE
* mnylund  22/01/04   VLTSW20040005 Implementation is updated
* - - - - - - - - - - APR2005 - - - - - - - - - - - - - - - - - - - 
* mnylund  01/06/04   VLTSW20030340 and VLTSW20040096 oslxCatFits
*                     Added the methods NrOfHDUs, GetNumberOfKeywordLines,
*                     GetKeywordLine and chmod
* mnylund   28/06/04  oslxFITS: Bug + ... VLTSW20040124
* mnylund   27/10/04  VLTSW20040319 Allow user to specify primary header size 
*                     in oslxFITS_EXT
* - - - - - - - - - - JAN2006 - - - - - - - - - - - - - - - - - - - 
* mnylund  16/05/05   VLTSW20050136 LoadDictionary method is added 
*                     to allow to use this class from the oslx SERVER.
* mnylund  21/07/05   VLTSW20050219 oslxCatFits slow on large files 
*                     Method loadImageFileReadOnly moved from protected to 
*                     public.
*
*/

/************************************************************************
*  Header file for the OSLX FITS Class
*----------------------------------------------------------------------
*/
// includes already included by oslxFITS_DICT_SUPPORT
//#include "oslxPRIVATE.h"      
//#include "oslxFITS_BASE.h"
//#include "oslxDICTIONARY.h"
//#include "oslxKEYWORD.h"
//#include "oslxKEYW_FILTER.h"
//#include "oslxDYN_BUF.h"
// #include "fitsio.h" 

#include  "oslxFITS_DICT_SUPPORT.h"
#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#define oslxCURRENT_HDU        -1
#define oslxcfitsMOD_ID        "oslxcfits"

#define oslxCOMMENT_AUTOADD    "Auto added keyword"
#define oslxLAST_HDU           9999
#define oslxALTER_TAG          "_altered" 
#define oslxUPDATE_TAG         "_update"
#define oslxTMP                "_tmp."
#define oslxTMP_FITS           oslxTMP"fits"

//for backcompatibility
#define oslxFITSEXT_ID         oslxcfitsMOD_ID 

// Class for handling the generation of FITS header information.
class oslxFITS_EXT: public oslxFITS_DICT_SUPPORT
{

public:

    // Constructor method that creates initializes an instance of the
    // class and makes an external reference to a Dictionary
    oslxFITS_EXT(const oslxDICTIONARY  *dictionary = NULL);
    ~oslxFITS_EXT();

    /*
     * FUNCTIONS OPERATING ON FILES
     */

    //create
    ccsCOMPL_STAT CreateImageFile(char * filename,vltLOGICAL isFitsExt=ccsTRUE);
    
    //VLTSW20050136
    //load dictionary (used by oslx server)
    ccsCOMPL_STAT LoadDictionary(const slxDICTIONARY dictionary,
				 const vltLOGICAL    merge = FALSE);
    //load
    ccsCOMPL_STAT LoadImageFile(char * filename,vltLOGICAL update=ccsFALSE);
    ccsCOMPL_STAT LoadHDU(char * filename,vltINT32 hdu);
    
    //append
    ccsCOMPL_STAT Append(char * filename);
    ccsCOMPL_STAT AppendHDU(char * filename,vltINT32 hdu);
    ccsCOMPL_STAT AppendExtentions(char * filename);
    ccsCOMPL_STAT AppendBinaryTables(char * filename);
    
    // merge 
    ccsCOMPL_STAT MergeHDUKeywords(char * filename,vltINT32 hdu);
    ccsCOMPL_STAT MergeHDUKeywords(oslxFITS_EXT &file,vltINT32 hdu);
    ccsCOMPL_STAT MergeHDUKeywords2(char * headerFile);
    ccsCOMPL_STAT MergeBufferOfKwLines(char *kwLineBuf ,vltINT32 hdu=oslxCURRENT_HDU);
    
    //save file
    ccsCOMPL_STAT SaveImageFile(char * filename="",vltLOGICAL isFitsExt=ccsTRUE);
    
        
    // functions to remove files
    ccsCOMPL_STAT  RemoveTmpFiles(char *filename);
    ccsCOMPL_STAT  RemoveFile(char *filename);
    
    /*
     *  FUNCTIONS OPERTAING ON HEADER 
     */

    //functions to add keywords
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const vltDOUBLE       value,
				  int hdunum=oslxCURRENT_HDU );
      
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const vltINT32        value,
				  int hdunum=oslxCURRENT_HDU );
    
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const vltLOGICAL      value,
				  int hdunum=oslxCURRENT_HDU );
    
    
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const char            *value,
				  int hdunum=oslxCURRENT_HDU );

    ccsCOMPL_STAT AddComment (char * comment,
			      int hdunum = oslxCURRENT_HDU );
    
    ccsCOMPL_STAT AddHistory (char * history,
			      int hdunum = oslxCURRENT_HDU );

    // function to delete keywords
    ccsCOMPL_STAT DeleteFitsKeyword( const char  *keyword, 
				     int          hdunum   = oslxCURRENT_HDU,
				     vltLOGICAL   resetHDU = ccsTRUE       );
    
    //functions to retrieve value of keywords
    ccsCOMPL_STAT ReadKeywordValue( const char     *keyword,
				    vltINT32       *value);
    ccsCOMPL_STAT ReadKeywordValue( const char     *keyword,
				    vltDOUBLE      *value);
    ccsCOMPL_STAT ReadKeywordValue( const char     *keyword,
				    vltLOGICAL     *value);
    ccsCOMPL_STAT ReadKeywordValue( const char     *keyword,
				    char           *value);

    ccsCOMPL_STAT ReadKeywordStringValue( const char  *keyword, 
					  char*        value   );

        
    //functions to modify order of the kewyords in the header
    ccsCOMPL_STAT Sort();
    ccsCOMPL_STAT SortKeywordsInHDU(vltINT32 hdu=1);

    //function to modify keywords category in the header
    ccsCOMPL_STAT AddIndexToCategory(char * category ,
				     vltINT32 index,vltINT32 hdu=1);
    ccsCOMPL_STAT ChangeCategory(char * category ,
				 char * newCategory,vltINT32 hdu=1);
    ccsCOMPL_STAT ChangeCategoryInHeaderFile(char* hdrFileName,
					     char * category ,
					     char * newCategory);
  
    //VLTSW20040005
    ccsCOMPL_STAT OverrideDefaultExtend(vltLOGICAL override);

    /* 
     * BINARY TABLE HANDLING FUNCTIONS 
     */
    ccsCOMPL_STAT AddBinaryTable(vltINT32 naxis2,  vltINT32  tfields, 
				 char    *ttype[], char     *tform[], 
				 char    *tunit[], char     *extname,
				 vltINT32 hdu=oslxLAST_HDU);
  
    //VLTSW20040319 nbFitsKewords is added
    ccsCOMPL_STAT CreateBinaryTable(vltINT32 naxis2,vltINT32 tfields, 
				    char *ttype[], char *tform[], 
				    char *tunit[], char *extname,
				    char *filename, vltLOGICAL isFitsExt=ccsTRUE,
				    vltINT32 nbFitsKeywords=0);
    
    ccsCOMPL_STAT AddSimpleColumnToBinaryTable(vltINT32    datatype  ,
					       vltINT32   column    ,
					       vltINT32   nelements ,
					       void      *array     ,   
					       vltINT32   hdu       = oslxCURRENT_HDU);
    
    ccsCOMPL_STAT AddElementsToBinaryTable(vltINT32   datatype,
					   vltINT32   column,
					   vltINT32   firstrow,
					   vltINT32   firstelem , 
					   vltINT32   nelements ,
					   void      *array,
					   vltINT32   hdu=oslxCURRENT_HDU);
    
    
    ccsCOMPL_STAT ReadElementsFromBinaryTable(vltINT32   datatype,
					      vltINT32   column,
					      vltINT32   firstrow,
					      vltINT32   firstelem , 
					      vltINT32   nelements ,
					      void      *array,
					      vltINT32   hdu=oslxCURRENT_HDU);
    
    ccsCOMPL_STAT InsertBlankColumnToBinaryTable(vltINT32  colnum,
						 char  *ttype,
						 char* tform,
						 vltINT32   hdu=oslxCURRENT_HDU);
    

    ccsCOMPL_STAT DeleteColumnFromBinaryTable(vltINT32     colnum,
					      vltINT32     hdu=oslxCURRENT_HDU);
    
    ccsCOMPL_STAT InsertBlankRowsToBinaryTable(vltINT32   firstrow,
					       vltINT32   nrow=1,
					       vltINT32   hdu=oslxCURRENT_HDU);
    
    ccsCOMPL_STAT DeleteRowsFromBinaryTable(vltINT32   firstrow,
					    vltINT32   nrow=1,
					    vltINT32   hdu=oslxCURRENT_HDU);
    
    ccsCOMPL_STAT  GetSizeOfBinaryTable(long &nrows, int &ncolumns);
    vltLOGICAL     IsBinaryTable(vltINT32 hdu=oslxCURRENT_HDU);
    
    
    
    /*
     * functions positioning 
     */
    ccsCOMPL_STAT MoveToHDU(vltINT32 hdunum);
    ccsCOMPL_STAT MoveToFirstHDU();
    ccsCOMPL_STAT MoveToLastHDU();
    ccsCOMPL_STAT MoveToBinaryTable(vltINT32   hdu);
    ccsCOMPL_STAT MoveToFirstBinaryTable();
    ccsCOMPL_STAT MoveToLastBinaryTable();
    ccsCOMPL_STAT MoveToExtention(char*   extname);
    char *        GetSimpleFileName(); 
    char *        GetSimpleFileName(const char* completeName);
    void          Reset();
    void          Cancel();

    //MNY VLTSW20030340 VLTSW20040096 
    ccsCOMPL_STAT NrOfHDUs(vltINT32* hdus);
    ccsCOMPL_STAT GetNumberOfKeywordLines(vltINT32 hdu, vltINT32* numOfKw);
    ccsCOMPL_STAT GetKeywordLine(vltINT32 lineNumber, slxKEYW_LINE keyWordLine);

    //MNY VLTSW20040124
    // Returns the number of 2880 blocks used by the header blocks
    ccsCOMPL_STAT NoOfHdrBlocks(vltINT32* noOfBlocks);
    // Returns the image size of data blocks
    ccsCOMPL_STAT ImageSize(vltINT32* imageSize);
    // Returns the total image size of data blocks
    ccsCOMPL_STAT TotalImageSize(vltINT32* imageSize);
    
    //VLTSW20040319
    ccsCOMPL_STAT NoOfAllocHdrBlocks(vltINT32* noOfBlocks);
    ccsCOMPL_STAT AllocatePrimaryHeaderSize(vltINT32 nbFitsKewords);
    ccsCOMPL_STAT GetHeaderSize(char * headerFile, vltUINT16*  numberOfKeywords);
    ccsCOMPL_STAT RemoveKeyword(const slxKEYW_LINE keyword, 
                                      int           hdunum = oslxCURRENT_HDU, 
                                      vltLOGICAL    resetHDU = ccsTRUE);
    // VLTSW20050219 Moved from protected
    ccsCOMPL_STAT  LoadImageFileReadOnly(char * filename);
protected:
    // file handling support functions
    ccsCOMPL_STAT  GetCompleteFileName(char        *filename,
				       slxFILENAME *completeFileName);
    void           CheckExtension(slxFILENAME *fileName, const char* extension);
    void           YankExtension(slxFILENAME fileName,const char* extension); 
    ccsCOMPL_STAT  filecopy(char *origFileName,  char *newFileName);
    ccsCOMPL_STAT  AppendHDU(oslxFITS_EXT &file,vltINT32 hdu);
    // VLTSW20050219 Moved to public
    //ccsCOMPL_STAT  LoadImageFileReadOnly(char * filename);
    
    // header handling support functions
    ccsCOMPL_STAT  AddKeywordLine(slxKEYW_LINE keywordLine);
    
    ccsCOMPL_STAT  StoreFitsKeyword(int    datatype, 
				    const char  *keyword, 
				    const void  *valuePtr,
				    int          hdunum   = oslxCURRENT_HDU,
				    vltLOGICAL   resetHDU = ccsTRUE       );
    
    ccsCOMPL_STAT  CreateHierachESOKeyword(const slxKEYW_LINE  origKeyword,
					   slxKEYW_LINE       &esoKeyword );
    
    //MNY VLTSW20040096 
     ccsCOMPL_STAT  chmod(char *fileName,  char *newPermMode);

   //keywordLine checking support functions
    char *         DatatypeAsString(vltINT32 type );
    vltLOGICAL     CheckType(vltINT32 type1,vltINT32 type2 );
    
    ccsCOMPL_STAT  CheckHDUKeywords(oslxFITS_EXT &file,vltINT32 hdu);
    ccsCOMPL_STAT  CheckKeywords();
    

    //error handling support
    void           ReportCfitsioError(int status , char* fileline, char * info="" );
    
    // strrstr is not available on te16
    char*          StrrStr(char*s1, const char*s2);
    void           StripSingleQuotes (char * string);
    


    // protected variables
    fitsfile   *imagefile;
    vltLOGICAL created_;

    // VLTSW20040005 Introduce a possibility to override default 
    //               setting on keyword EXTEND
    vltLOGICAL overrideDefaultExt;

    //support function for retrieving values of keywords
    template<typename valuetype>
    ccsCOMPL_STAT ReadKeywordValue_( const char     *keyword,
				     int            kwtypecfio,
				     slxPAR_TYPE    kwtypeslx,
				     valuetype      *value);
    

  
};   // end-class oslxFITS_EXT

#endif /*!oslxFITS_EXT_H*/
