/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: rmac.h,v 1.26 2005/02/25 11:19:40 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* tphan     25/10/00  Split into rmac.h and rmacDefines.h
* alongino  07/02/00  Adapted to Solaris
* fcarbogn  23/09/99  Created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef RMAC_H
#define RMAC_H

/************************************************************************
 * Contains the public global data and data type definitions 
 * to be used in accessing the Reflective Memory
 *----------------------------------------------------------------------
 */


/*
 *******************************************************************************
 * Header Files
 *******************************************************************************
 */
 
/* System Includes */


/* LCC Includes */

#ifndef NOCCS
  #include "CCS.h"
#else
  #include "miscNoCcs.h"
#endif

/* Local Includes */

#include "rmacDefines.h"


/*
 ******************************************************************************
 * Data Types
 ******************************************************************************
 */

#ifndef NOCCS
typedef struct {
	vltUINT32	msgHeader;	/* Message Header*/
	ccsTIMEVAL	timeStamp;	/* Data time stamp */
	vltUINT32	dataCheck;	/* Data Check Field */
	} rmacRTMessage;


/*
 ******************************************************************************
 * Data from OPD 
 ******************************************************************************
 */


typedef struct {
	rmacRTMessage	rtMessage;	/* Message Common Additional Fields */
	vltDOUBLE     	rtOffPos;	/* Real Time Position Offset */
	vltUINT32      	fringesDetected;	/* Flag on Fringe Detection */
        vltUINT32       offsetValid;	/* Flag on RT Position offset */
	} rmacOPDMessage;


/*
 ******************************************************************************
 * Data from DL
 ******************************************************************************
 */

typedef struct {
	rmacRTMessage	rtMessage;	/* Message Common Additional Fields */
        vltDOUBLE       measPos;       	/* DL Position from Metrology System */
	vltUINT32       trkOKFlag;	/* Flag on DL tracking  */	
        } rmacDLMessage;

	
/*
 ******************************************************************************
 * Data from FSU (NOTE: This is FINITO, not the PRIMA FSU!)
 ******************************************************************************
 */

typedef struct {
	rmacRTMessage	rtMessage;	/* Message Common Additional Fields */
	vltDOUBLE  	opdPhase;	/* Optical Path Phase */
	vltUINT32	opdPhaseValid;	/* Validation Flag for Optical Path Phase */	
	vltDOUBLE  	opdCoherence;	/* Optical Path Coherence */
	vltUINT32	opdCoherValid;	/* Validation Flag for Optical Path Coherence */	
	vltDOUBLE      	visibility;	/* Fringes Visibility */
	} rmacFSUMessage;

/*
 ******************************************************************************
 * Data from VINCI
 ******************************************************************************
 */

typedef struct {
	rmacRTMessage	rtMessage;	/* Message Common Additional Fields */
	vltDOUBLE  	opdCoherence;	/* Optical Path Coherence */
	vltUINT32	opdCoherValid;	/* Validation Flag for Optical Path Coherence */	
        vltUINT32       insFringesDetected;
	} rmacVINCIMessage;
/*
 ******************************************************************************
 * Data from MIDI
 ******************************************************************************
 */

typedef struct {
	rmacRTMessage	rtMessage;	/* Message Common Additional Fields */
	vltDOUBLE  	opdCoherence;	/* Optical Path Coherence */
	vltUINT32	opdCoherValid;	/* Validation Flag for Optical Path Coherence */	
        vltUINT32       insFringesDetected;
	} rmacMIDIMessage;
/*
 ******************************************************************************
 * Data from MACAO
 ******************************************************************************
 */

typedef struct {
        rmacRTMessage rtMessage;          /* Message Common Additional Fields */
        vltDOUBLE wavefrontError;         /* Measured Residual Wavefront Error */
        vltUINT32 wavefrontErrorValid;    /* Validation Flag for Wavefront Error */ 
        } rmacMACAOMessage;


/*
 ******************************************************************************
 * Data from PRIMA: Metrology System
 ******************************************************************************
 */

typedef struct {
    rmacRTMessage rtMessage;    /* Message Common Additional Fields */
    vltDOUBLE deltaL;           /* Internal optical path difference */
    vltUINT32 deltaLValid;      /* Validation Flag for Delta L */ 
} rmacPRIMETMessage;

/*
 ******************************************************************************
 * Data from PRIMA: FSU A (Instrument) and B (Fringe Sensor)
 ******************************************************************************
 */

typedef struct {
    rmacRTMessage rtMessage;    /* Message Common Additional Fields */
    vltDOUBLE opd;              /* OPD, as measured by the FSU */
    vltUINT32 opdValid;         /* Validation Flag for OPD */
    vltDOUBLE gd;               /* Group Delay, as measured by the FSU */
    vltUINT32 gdValid;          /* Validation Flag for GD */
    vltDOUBLE ampSquared;       /* Squared fringe amplitude, as measured by the FSU */
    vltUINT32 ampSquaredValid;  /* Validation Flag for the squared fringe amplitude */
} rmacPRIFSUMessage;

/*
 ******************************************************************************
 * Data for MONITOR
 ******************************************************************************
 */

typedef struct {
        rmacRTMessage   rtMessage;      /* Message Common Additional Fields */
        vltDOUBLE       rmacProbe01;
	vltDOUBLE      	rmacProbe02;
	vltDOUBLE       rmacProbe03;
	vltDOUBLE       rmacProbe04;
	vltDOUBLE       rmacProbe05;
	vltDOUBLE       rmacProbe06;
	vltDOUBLE       rmacProbe07;
	vltDOUBLE       rmacProbe08;
	vltDOUBLE       rmacProbe09;
	vltDOUBLE       rmacProbe10;
        } rmacMONITORMessage;
#endif

/*
 **************************************************************************
 * Data Array on Reflective Memory
 **************************************************************************
 */
/* Data type for the buffer on reflective memory */
typedef union 
{
    vltINT32   i;
    vltUINT32  ui;
    vltFLOAT   f;
    vltDOUBLE  d;
} rmacBUFFER;	



#endif /*!RMAC_H*/
