#ifndef _sdma_h
#define _sdma_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: sdma.h,v 1.104 2005/09/12 09:21:03 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  19/09/96  created
* jstegmei  27/01/98  added comments for external variables/semaphores
* jstegmei  28/01/98  added device name to sdmaCAP structure for reopen
* jstegmei  28/01/98  added general parameter sync routine with timeout
* jstegmei  03/02/98  removed scd.h from local include directory
* jstegmei  04/02/98  added sdmaAddOutputFrame(), sdmaRemoveOutputFrame()
* jstegmei  18/02/98  added sdmaCiSimTime()
* jstegmei  26/02/98  added sdmaMicroSleep()
* jstegmei  09/03/98  sdmaShData routines added
* jstegmei  09/03/98  sdmaCiShData routines added
* jstegmei  10/03/98  removed sdmaMemInit(), sdmaMemPtr(), sdmaMemSet()
* jstegmei  26/03/98  added sdmaCiFlushOutput()
* jstegmei  03/04/98  added global sdmaDataInitFlag
* jstegmei  03/04/98  added global sdmaProcName
* jstegmei  06/04/98  added sdmaReAlloc()
* jstegmei  06/04/98  added sdmaReInitFrame()
* jstegmei  08/04/98  added sdmaProcNum macro
* jstegmei  29/04/98  added sdmaCiResCnt(), sdmaCiEnd()
* jstegmei  04/05/98  added sdmaInitClient()
* jstegmei  14/05/98  added parameter check
* jstegmei  20/07/98  added dit-handler (for chopping)
* jstegmei  21/07/98  added sdmaCmdLineHelp()
* jstegmei  21/07/98  added sdmaCiGetErrPort()
* jstegmei  24/07/98  implemented 2^32 wrapper
* kfb       27/07/98  pcd-support
* jstegmei  08/09/98  added posX, posY parameter to shift vector routines
* jstegmei  11/09/98  added external user command handler
* jstegmei  21/11/98  added sdmaCiChipOffset()
* jstegmei  29/06/99  added sdmaNumTypes
*/

/************************************************************************
 *
 *  sdma.h  solaris dma interface constants
 *
 *----------------------------------------------------------------------
 */

#ifdef THR_POSIX
#undef __STRICT_ANSI__
#endif

#ifndef _REENTRANT
#define _REENTRANT    
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/shm.h>
#include <sys/ioctl.h>

#include "sdmaThr.h"
#include "sdmaDrv.h"
#include "rtdImageEvent.h"
#include "sdmaInt.h"

#ifdef __cplusplus
extern "C" {
#endif


/*
 *  minimum ring-buffer size
 */
#define sdmaMIN_RINGBUFSIZE (1024*1024)

/*
 *  maximum number of ring buffer elements
 */
#define sdmaMAX_RINGBUFELE     64

/*
 *  size of error stack
 */
#define sdmaERR_STACK_SIZE 32

/*
 *  default display camera name
 */
#define sdmaCAMERA_DEFAULT "cam"


/*
 *  realtime scheduling
 */
#define sdmaPRI_TS        1   /* real-time class id           */
#define sdmaPRI_IA        2   /* inter-active class id        */
#define sdmaPRI_RT        3   /* time-sharing class id        */
#define sdmaTQ_DEF        0   /* default time quantum         */
#define sdmaTQ_NC        -1   /* don't change time quantum    */
#define sdmaTQ_INF       -2   /* infinite time quantum        */
#define sdmaPRI_CI       20   /* command interpreter priority */
#define sdmaPRI_CAPTURE  19   /* capture process priority     */
#define sdmaPRI_FS       15   /* field-stab. process priority */
#define sdmaPRI_PROC     10   /* application process priority */


/*
 *  external parameter check
 */
typedef int (*sdmaPCHECK)(int *, int, int *);
extern sdmaPCHECK sdmaParamCheck;

/*
 *  external user command handler
 */
typedef void (*sdmaEXT_CMD)(char *, char *);
extern sdmaEXT_CMD sdmaUsrCmdHandler;

/*
 *  built-in simulator
 */   
typedef void (*sdmaSIM)(short **, int, int, int *);
extern sdmaSIM sdmaSimBuf;

/*
 *  chopping mode
 */
#define sdmaCHOP_ZERO  1          /* start position of chopper */

/*
 *  field stabilisation port
 */
#define sdmaFS_PORT                   8067
#define sdmaFS_DEFAULT_BOOK_ID           1

/*
 *  minimum correctio time (in milliseconds)
 */
#define sdmaMIN_COR_TIME   20
#define sdmaPI             3.14159265358979323846

/*
 *  infinite counter
 */
#define sdmaProcNum(x) (x & 0x0000ffff) 
#define sdmaWRAP_CNT 0xffffffff
#define sdmaWRAP_INC(x) if(*x==sdmaWRAP_CNT) {*x=0;} else {(*x)++;}
#define sdmaWRAP_DEC(x) if(*x==0) {*x=sdmaWRAP_CNT;} else {(*x)--;}
#define sdmaWRAP_DIFF(x,a,b) if(b<a){*x=(sdmaWRAP_CNT-a)+b;}else{*x=b-a;}

/*
 *  correction data
 */
typedef struct 
{
  double alpha;
  double delta;
} sdmaFS_ANGLE;
typedef struct  
{
  int          bookId; 
  int          relative;
  sdmaFS_ANGLE angle;
} sdmaCOR;

/*
 *  DMA capture type
 */
typedef struct
{
  unsigned char **buf;
  int             num;
  int             size;
  sdmaHANDLE      fd;
  unsigned int    last;
  unsigned int    next;
  int             dmasize;
  int             allocated;
  int             mode;
  int             shMemId;
  int             shMemLinkId;
  int             shMemPort;
  char            devName[256];
  thread_t       *thid;
  int             semLckId;
  int             simSock[2];
  char            erms[256];
} sdmaCAP;

/*
 *  dynamic param type
 */
typedef struct
{
  char **id;
  int    num;
} sdmaDPM;

/*
 *  dynamic command type
 */
typedef struct
{
  char *name;
  int   code;
} sdmaDYN_CMD;

/*
 *  camera type
 */
typedef struct
{
  int   shMemId;
  char *shMemPtr;
  int   shMemSize;
  rtdIMAGE_EVT_HNDL eventHndl;
  char *fix;
  char *name;
} sdmaCAM;

/*
 *  frame type (internal)
 */
typedef struct
{
  sdmaFRAME_H    h;           /* header                             */
  sdmaFRAME_H   *hr;          /* ring buffer element header         */
  char         **fbuf;        /* output ringbuffer                  */
  int            dptr;        /* pointer to current data            */
  int            nbuf;        /* number of ringbuffer elements      */
  int           *oreq;        /* bitmask to mark output as done     */
  int           *oreq1;       /* bitmask to mark output as done     */
  unsigned int  *cnt;         /* timeStamp                          */
  rwlock_t      *lck;         /* rw-lock                            */
  int            init;        /* init flag                          */
  int            noSort;      /* no output sorting                  */
  int            noWin;       /* no output window                   */
} sdmaFRAME;

/*
 *  parameters for dit-handler
 */
typedef struct
{
  int    ndit;
  int    ncycles;
  int    hcyc;
  int    sdv;
  int    iint;
  int    cycSkip;
  int    startSkip;
  int    setupId;
} sdmaDIT_P;
int sdmaDitHandler(sdmaFRAME *, sdmaFRAME *, sdmaFRAME *, sdmaFRAME *,
		   sdmaFRAME *, sdmaFRAME *, double *, sdmaDIT_P *, char *);
int sdmaCntHandler(sdmaFRAME *, int, int, int, int, int,
		   int *, int *, char *, char *, char *);

/* 
 *  Prototype functions 
 */
int    sdmaGetPrio(short *, int *, int *);
int    sdmaSetPrio(short, int, int);
int    sdmaGetLocHostAddr(char *, char *);
int    sdmaOpen(sdmaCAP *, char *);
int    sdmaClose(sdmaCAP *, char, char *);
int    sdmaReAlloc(int, int, char *);
int    sdmaTerm(sdmaCAP *, char *);
int    sdmaResetReg(sdmaCAP *, char *);
int    sdmaReopen(sdmaCAP *, char *);
int    sdmaInit(sdmaCAP *, unsigned int *, char *);
char   sdmaOverrun(sdmaCAP *);
void   sdmaCapture(void *);
int    sdmaStartCapture(void);
void   sdmaInitClient(void);
int    sdmaStartup(int, char**, int, char*);
void   sdmaCmdLineHelp(char *, char *, char *);
void   sdmaPrintConf(int, int);
void   sdmaPrintHwStat(sdmaCAP *);
char   sdmaCheckStop(void);
void   sdmaDivide(int);
int    sdmaWaitData(short **);
int    sdmaWaitStart(void);
void   sdmaAckData(void);
int    sdmaFlushInput(void);
void   sdmaTermProcessing(void);
int   *sdmaSetupDynParam(int, char **, int, char *);
int    sdmaPSync(int, char *);
int    sdmaDynComHandle(sdmaDYN_CMD *);
int   *sdmaParamPtr(void);
void   sdmaParamDefault(void);
void   sdmaExit(int);
int    sdmaOpenCam(int, char *, char *);
int    sdmaDisplay(int, char *, int, int, int);
int    sdmaProcCreate(void (*calcProcess)(void *), int, char *);
int    sdmaProcWaitStart(int);
int    sdmaProcWaitTrigger(int);
int    sdmaProcAck(int);
int    sdmaProcStart(int);
int    sdmaProcTrigger(int) ;
int    sdmaProcWaitAck(int);
int    sdmaProcSync(int);
int    sdmaProcDelete(int);
int    sdmaMicroSleep(int, int);

/*
 *  error stack
 */
void sdmaSigFailure(char *);
int    sdmaErrStackAdd(int, char *); 
void   sdmaErrStack(void *);

/*
 *  command controller prototypes
 */
void sdmaCommandProcess(void *);
void sdmaCiInitErr(void);
int  sdmaCiSockGetCommand(int, icmdCOM *, char *);
int  sdmaCiSockSendReply(int, icmdREP, char *);
int  sdmaCiSockOpen(char *, char *);
int  sdmaCiSockConnect(int, char *);
int  sdmaCiSockClose(int);
int  sdmaCiParamInit(void);
int  sdmaCiParamSync(icmdCOM, icmdREP *, char *);
int  sdmaCiParamSet(icmdCOM, icmdREP *, char *);
int  sdmaCiParamStat(icmdCOM, icmdREP *, char *);
int  sdmaCiParamConf(icmdCOM, icmdREP *, char *);
int  sdmaCiSetHdr(icmdCOM, icmdREP *, char *);
int  sdmaCiAbort(icmdCOM, icmdREP *, char *);
int  sdmaCiStart(icmdCOM, icmdREP *, char *);
int  sdmaCiStop(icmdCOM, icmdREP *, char *);
int  sdmaCiPause(icmdCOM, icmdREP *, char *);
int  sdmaCiGetStatus(icmdCOM, icmdREP *, char *);
int  sdmaCiGetConfig(icmdCOM, icmdREP *, char *);
int  sdmaCiGetErrPort(icmdCOM, icmdREP *, char *);
int  sdmaCiGetDataPort(icmdCOM, icmdREP *, char *);
int  sdmaCiTime(icmdCOM, icmdREP *, char *);
int  sdmaCiSimTime(icmdCOM, icmdREP *, char *);
int  sdmaCiVerbose(icmdCOM, icmdREP *, char *);
int  sdmaCiDynCmd(icmdCOM, icmdREP *, char *);
int  sdmaCiMemSet(icmdCOM, icmdREP *, char *);
int  sdmaCiMemClose(icmdCOM, icmdREP *, char *);
int  sdmaCiMemSync(icmdCOM, icmdREP *, char *);
int  sdmaCiFlushOutput(icmdCOM, icmdREP *, char *);
int  sdmaCiMemRecv(char *);
int  sdmaCiResetCnt(icmdCOM, icmdREP *, char *);
int  sdmaCiEnd(icmdCOM, icmdREP *, char *);
int  sdmaCiSetFileServerPath(icmdCOM, icmdREP *, char *);
int  sdmaCiSetFileServerBase(icmdCOM, icmdREP *, char *);
int  sdmaCiUsrCmd(icmdCOM, icmdREP *, char *);
int  sdmaCiChipOffset(icmdCOM, icmdREP *, char *);


/*
 *  output prototypes
 */
int  sdmaInitOutputLink(int, char *);
int  sdmaFlushOutput(void);
void sdmaSyncOutput(void);
int  sdmaCloseOutputLink(void);
void sdmaInitFrame(sdmaFRAME *, char **, int);
void sdmaReInitFrameN(sdmaFRAME *, char **, int);
void sdmaReInitFrame(sdmaFRAME *, char **);
int  sdmaReqOut(int, sdmaFRAME *, int, int, int);
void sdmaSockOutputLink(void *);
void sdmaSockOutputSrv(void *);
void sdmaOutLinkDone(int);
int  sdmaOutLinkReq(int, sdmaREQ, int, sdmaFRAME_T *, int *, int *);
void sdmaOutLinkResetReq(int);
void sdmaOutLinkAckData(int);
void sdmaOutLinkAckDataPending(int);
void sdmaOutLinkFlush(int);
int  sdmaInitFileServer(char *, char *);
void sdmaAddOutputFrame(int);
void sdmaRemoveOutputFrame(int);
int  sdmaAddFrameType(int, char *, char *, char *);
void sdmaExportData(void *, int, int, int, int);

/*
 *  file output prototypes
 */
int  sdmaFileOut(int, sdmaFRAME *, int, char, char *);
void sdmaFileSync(void);
int  sdmaSetFileServerPath(int, char *, char *);
int  sdmaSetFileServerBase(char *, char *);


/*
 *  shared memory prototypes
 */
int   sdmaShDataAttach(char, char *);
int   sdmaShDataClose(void);
char *sdmaShDataSetPtr(int, int *, char *);
int   sdmaShDataGetPtr(char **,int , int, char *);
void  sdmaShDataSwap(int);
int   sdmaShDataValid(int);
int   sdmaShDataInvalid(int);
int   sdmaShDataIsValid (int);

/*
 *  calculation tools
 */
int  sdmaGetShift(char *, int  *, char *, int, int, int, int, int, 
		  int *, int *, int *, int *);
int  sdmaGetShiftCen(char *, int  *, char *, int, int, int, int, int, 
		     int *, int *, int *, int *);
int  sdmaGetShiftFloat(char *, float  *, char *, int, int, int, int, int, 
		       int *, int *, int *, int *);
int  sdmaGetShiftCenFloat(char *, float  *, char *, int, int, int, int, int, 
			  int *, int *, int *, int *);
int  sdmaShiftAddInt(int *, int *, int, int, int, int);
int  sdmaShiftAddIntN(int *, int *, int, int, int, int, int, int);
int  sdmaShiftAddFloat(float *, float *, int, int, int, int);
int  sdmaShiftAddFloatN(float *, float *, int, int, int, int, int, int);
int  sdmaCutWindow(char *, int, int, char *, int, int, int, int, int, 
		   int, int);
void sdmaApplyPixMap(char *, char *, int, int);

/*
 *  field stabilisation routines
 */
int sdmaFsConnect(char *, char *);
int sdmaFsDisconnect(int);
int sdmaFsSend(int, sdmaCOR *, char *);

/* 
 *  global variables 
 */
extern int        sdmaVflag;                    /* verbose mode flag       */
extern int        sdmaNumCam;                   /* number of rtd's         */
extern sdmaCAM    sdmaCamera[sdmaMAX_CAM];      /* camera structures       */
extern sdmaCAP   *sdmaDev;                      /* data input devices      */
extern int        sdmaNumDev;                   /* number of acq.-devices  */
extern thread_t   sdmaThCapture[sdmaMAX_DEV];   /* capture process(es)     */
extern char       sdmaCapInitFlag[sdmaMAX_DEV]; /* capture init flag       */
extern char       sdmaStatus;                   /* processing status       */
extern char       sdmaRunning;                  /* running flag            */
extern char       sdmaCiInitFlag;               /* cmd.interp. init flag   */
extern char       sdmaErrInitFlag;              /* err-stack init flag     */
extern char       sdmaDataInitFlag;             /* data server init flag   */
extern char       sdmaDataSrvName[256];         /* data server device name */
extern char     **sdmaOutLinkName;              /* output link names       */
extern int        sdmaNumOutLink;               /* number of output links  */
extern int        sdmaNumDataClients;           /* number of data clients  */
extern int        sdmaNumDataFiles;             /* number of data files    */
extern int        sdmaNumFileSrv;               /* number of file servers  */
extern int       *sdmaDynPtr;                   /* pointer to param. buffer*/
extern int        sdmaParamBuf[3][sdmaMAX_PARAMSIZE]; /* parameter buffers */
extern sdmaDPM    sdmaDynParam;                 /* parameter configuration */
extern double     sdmaTime[2];                  /* calc.-time information  */
extern int        sdmaDynCmd;                   /* dyn. command code       */
extern int        sdmaDynCmdAck;                /* dyn. command code ack.  */
extern int        sdmaNumDynCmd;                /* nr of dyn.-cmd. codes   */
extern int        sdmaSimTime;                  /* simulation interval     */
extern int        sdmaCmdClients;               /* number of cmd clients   */
extern int        sdmaKeyVal;                   /* key value for shMem     */
extern int        sdmaTransferTypes;            /* valid frame types       */
extern char       sdmaPrintPars;                /* just print params flag  */
extern char       sdmaProcName[128];            /* process name            */
extern int        sdmaRemoteFd;                 /* remote file descriptor  */
extern char       sdmaRstCnt;                   /* reset counters flag     */
extern char       sdmaEndFlg;                   /* end of measurement      */
extern int        sdmaCmdPort;                  /* port of error stack     */
extern int        sdmaErrPort;                  /* port of error stack     */
extern int        sdmaDataPort;                 /* port of data server     */
extern char       sdmaOutHost[sdmaMAX_OUTLINK][256];
extern int        sdmaOutFcnt[sdmaMAX_OUTLINK];
extern int        sdmaOutRcnt[sdmaMAX_OUTLINK];
extern char       sdmaCmdHost[sdmaCI_MAX_CON][256];
extern int        sdmaStartX;                   /* chip offset             */
extern int        sdmaStartY;                   /* chip offset             */
extern int        sdmaChopZero;                 /* chop zero position      */
extern char      *sdmaErmsCi;                   /* ci init error           */
extern char      *sdmaErmsData;                 /* data server init error  */
extern int        sdmaNumTypes;                 /* number of defined types */
extern sdmaFTYPE  sdmaFrameType[32];            /* frame types             */
extern int       *sdmaPixMap;                   /* pixel map               */
extern char       sdmaPauseFlag;                /* pause flag              */
extern int        sdmaNumDiv;                   /* DMA sub-divisions       */
extern int        sdmaNumDet;                   /* # of detector instances */
extern short      sdmaIfHdr;                    /* DMA interface header    */
extern int        sdmaFrameSize;                /* output frame size       */

/*
 *  semaphores
 */
extern sema_t semStart[sdmaMAX_DEV];            /* start sync              */
extern sema_t semStartAck[sdmaMAX_DEV];         /* start acknowledge sync  */
extern sema_t semCap[sdmaMAX_DEV];              /* capture sync            */
extern sema_t semCapInit[sdmaMAX_DEV];          /* capture init sync       */
extern sema_t semCapClient[sdmaMAX_DEV];        /* client sync             */
extern sema_t semCapClientStart[sdmaMAX_DEV];   /* client start sync       */
extern sema_t semCapClientInit[sdmaMAX_DEV];    /* capture init ready      */
extern sema_t semRun;                           /* running flag access     */
extern sema_t semStat;                          /* status access           */
extern sema_t semParam;                         /* parameter access        */
extern sema_t semParamLock;                     /* parmeter lock sync      */
extern sema_t semCmd;                           /* dyn. cmd sync           */
extern sema_t semCmdAck;                        /* dyn. cmd ack sync       */
extern sema_t semBuf;                           /* data set access         */
extern sema_t semCiInit;                        /* cmd. interp. init sync  */
extern sema_t semErrInit;                       /* err. stack init sync    */
extern sema_t semDataInit;                      /* data server init sync   */
extern sema_t semClientInit;                    /* client init sync        */
extern sema_t semMem;                           /* data set access         */
extern sema_t semTerm;                          /* termination             */
extern sema_t semCam;                           /* camera config access    */


#ifdef __cplusplus
}
#endif

#endif /*!_sdma_h*/
