#ifndef _sdmaInt_h
#define _sdmaInt_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: sdmaInt.h,v 1.104 2005/09/12 09:21:06 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  19/09/96  created
* jstegmei  27/01/98  removed minphys and maxdma from config-info
* jstegmei  28/01/98  sdmaAllocCom added
* jstegmei  02/02/98  removed connect, exec and kill command
* jstegmei  04/02/98  added frame type output service
* jstegmei  16/02/98  added types sdmaPARAM_NAMES, sdmaPARAM_VALUES
* jstegmei  17/02/98  added sdmaSetSimTime
* jstegmei  17/02/98  added sdmaSendDataRequest, sdmaWaitDataReply
* jstegmei  18/02/98  added sdmaProcDelete 
* jstegmei  19/02/98  added more information to config structure
* jstegmei  20/02/98  changed sdmaFRAME_H (sx -> start_x, 
*                     sy -> start_y)
* jstegmei  23/02/98  removed obsolete sdmaReadFrame()
* jstegmei  24/02/98  added sdmaPingDataLine()
* jstegmei  26/03/98  added sdmaFlushOutputQueue()
* jstegmei  01/04/98  added sdmaScale(), sdmaScale2()
* jstegmei  03/04/98  added sdmaDATA_SERVICE_PROCNAME
* jstegmei  28/04/98  added sdmaFRAME_HCYCLE
* jstegmei  04/05/98  added frames element to header structure
* jstegmei  04/05/98  new config structure
* jstegmei  26/05/98  removed obsolete sdmaServerTimeout 
*                     and sdmaCONNECT_TIMEOUT
* jstegmei  16/06/98  added error codes sdmaERR_HANDSHAKE_FRAME and
*                     sdmaERR_HANDSHAKE_REQ
* jstegmei  21/07/98  added sdmaGetErrPort()
* jstegmei  08/09/98  added new header elements (tx, ty)
* jstegmei  11/09/98  added external user command handler
* jstegmei  18/09/98  added sdmaGetParamType()
* jstegmei  18/09/98  added bufSize element to config-structure
* jstegmei  28/10/98  added sdmaGetDataPort()
* jstegmei  21/11/98  added sdmaSetChipOffset()
* jstegmei  27/11/98  added sdmaGetFrameTypeByName(), 
*                     sdmaGetFrameNameByType()
* jstegmei  09/01/01  added sdmaERR_GENERAL
*/

/************************************************************************
 *
 *  sdmaInt.h - sdma common constants
 *
 *----------------------------------------------------------------------
 */

#include <icmd.h>

#ifdef __cplusplus
extern "C" {
#endif


/*
 *  verbose
 */ 
#define sdmaVB(STR) if (sdmaVflag) printf STR
/*#define sdmaVB(STR)*/  

/*
 *  environment settings
 */
#define sdmaENV_ERR_PORT      "SDMA_ERR"
#define sdmaENV_DATA_PORT     "SDMA_DATA"
#define sdmaENV_COMMAND_PORT  "SDMA_CMD"
#define sdmaENV_HOST          "SDMA_HOST"
#define sdmaENV_DEVNAME       "SDMA_DEV"

/*
 *  socket ports (default values)
 */
#define sdmaCOMMAND_PORT  8001
#define sdmaDATA_PORT     0
#define sdmaERR_PORT      0
#define sdmaSIM_PORT      8004

/*
 *  default timeout
 */
#define sdmaNO_TIMEOUT       0
#define sdmaDEFAULT_TIMEOUT  sdmaNO_TIMEOUT

/*
 *  status definitions
 */
#define sdmaSTAT_IDLE        0
#define sdmaSTAT_WAITING     1
#define sdmaSTAT_PROCESSING  2
#define sdmaSTAT_READY       3
#define sdmaSTAT_FAILURE     4


/*
 *  external user command handler
 */
#define sdmaMAX_USRCMD_LEN  1024 /* maximum size of the user command/reply */

/*
 *  datatype
 */
#define sdmaDTYPE_BYTE          0
#define sdmaDTYPE_REAL32        1
#define sdmaDTYPE_REAL64        2
#define sdmaDTYPE_INT16         3
#define sdmaDTYPE_INT32         4
#define sdmaDTYPE_INT64         5

/*
 *  frame type
 */
#define sdmaMAX_FRAME_TYPES            32
#define sdmaFRAME_ANY                  0xffffffff
#define sdmaFRAME_NO_FRAME             0x00000000
#define sdmaFRAME_SNAPSHOT             0x00000001
#define sdmaFRAME_DIT                  0x00000002
#define sdmaFRAME_INT                  0x00000004
#define sdmaFRAME_INTERMEDIATE_INT     0x00000008
#define sdmaFRAME_INTERMEDIATE_DIT     0x00000010
#define sdmaFRAME_SDV                  0x00000020
#define sdmaFRAME_SAMPLE               0x00000040
#define sdmaFRAME_HCYCLE1              0x00000080
#define sdmaFRAME_HCYCLE2              0x00000100
#define sdmaFRAME_TRACK                0x00000200
#define sdmaFRAME_USER                 0x00000200

/*
 *  frame names
 */
#define sdmaFRAME_NAME_ANY                  "ANY_FRAME"
#define sdmaFRAME_NAME_NO_FRAME             "NO_FRAME"
#define sdmaFRAME_NAME_SNAPSHOT             "SNAPSHOT"
#define sdmaFRAME_NAME_DIT                  "DIT"
#define sdmaFRAME_NAME_INT                  "INT"
#define sdmaFRAME_NAME_INTERMEDIATE_INT     "INTERM-INT"
#define sdmaFRAME_NAME_INTERMEDIATE_DIT     "INTERM-DIT"
#define sdmaFRAME_NAME_SDV                  "STDEV"
#define sdmaFRAME_NAME_SAMPLE               "SAMPLE"
#define sdmaFRAME_NAME_TRACK                "TRACK"
#define sdmaFRAME_NAME_HCYCLE1              "HCYCLE1"
#define sdmaFRAME_NAME_HCYCLE2              "HCYCLE2"


/*
 *  dynamic parameters
 */
#define sdmaMAX_PARAMSIZE    64    /* Max. number of dynamic parameters */
#define sdmaNUM_ELE(x) ((sizeof x) / (sizeof x[0]))
typedef char sdmaPARAM_NAMES[sdmaMAX_PARAMSIZE][256];
typedef int  sdmaPARAM_VALUES[sdmaMAX_PARAMSIZE];
#define sdmaPARAM_TYPE_FLOAT sdmaDTYPE_REAL32
#define sdmaPARAM_TYPE_INT   sdmaDTYPE_INT32

/*
 *  system services command codes
 */
#define sdmaCOM_TIMEOUT      -3

/*
 *  command codes
 */
#define sdmaCOM_NOP         400   /* no operation (ping)          */
#define sdmaCOM_START       401   /* start acquisition            */
#define sdmaCOM_STOP        402   /* stop  acquisition            */
#define sdmaCOM_STAT        403   /* get processing status        */
#define sdmaCOM_TIME        404   /* get on-line calcualtion time */
#define sdmaCOM_MSET        405   /* download data-set            */
#define sdmaCOM_PSET        406   /* set parameter                */
#define sdmaCOM_PSTAT       407   /* get parameter status         */
#define sdmaCOM_PSYNC       408   /* parameter synchronization    */
#define sdmaCOM_MSYNC       409   /* data-set synchronization     */
#define sdmaCOM_EXIT        410   /* exit command interpreter     */
#define sdmaCOM_ICONF       411   /* get configuration            */
#define sdmaCOM_PCONF       412   /* get parameter configuration  */
#define sdmaCOM_DYNCMD      413   /* send dynamic command code    */
#define sdmaCOM_VERBOSE     414   /* switch verbose mode on/off   */
#define sdmaCOM_SIMTIME     415   /* set simulation interval time */
#define sdmaCOM_MCLOSE      416   /* remove shared memory data    */
#define sdmaCOM_FLUSHOUT    417   /* flush all output data queues */
#define sdmaCOM_RSTCNT      418   /* reset counters               */
#define sdmaCOM_END         419   /* end of measurement           */
#define sdmaCOM_FILEBASE    420   /* set file server base name    */
#define sdmaCOM_FILEPATH    421   /* send file server path name   */
#define sdmaCOM_ERRPORT     422   /* send error stack port        */
#define sdmaCOM_USRCMD      423   /* execute ext. user command    */
#define sdmaCOM_DATAPORT    424   /* send data server port        */
#define sdmaCOM_CHIPOFFSET  425   /* set chip offset              */
#define sdmaCOM_PAUSE       426   /* pause                        */
#define sdmaCOM_ABORT       427   /* abort all                    */
#define sdmaCOM_HDR         428   /* set DMA-interface header     */
#define sdmaNUM_COM        (429 - sdmaCOM_NOP)

/*
 *  general failure
 */
#define sdmaFAILURE -1

/*
 *  error definitions
 */
#define sdmaERR_NO_ERR              0  /* good                            */
#define sdmaERR_IO_ERROR           -1  /* i/o error                       */
#define sdmaERR_GENERAL           400  /* general error                   */
#define sdmaERR_NO_VALID_COMMAND  401  /* no valid command                */
#define sdmaERR_OUT_OF_RANGE      402  /* out of range                    */
#define sdmaERR_OPEN              405  /* open error                      */
#define sdmaERR_ALLOC             406  /* allocation error                */
#define sdmaERR_TERM              407  /* error terminating dma transfers */
#define sdmaERR_UNKNOWN           408  /* unknown element                 */
#define sdmaERR_STARTED           409  /* acquisition is already started  */
#define sdmaERR_NOT_STARTED       410  /* acquisition is not started      */
#define sdmaERR_SYSTEM            411  /* system call                     */
#define sdmaERR_TIMEOUT           412  /* timeout                         */
#define sdmaERR_INIT              413  /* init failure                    */
#define sdmaERR_CMD               414  /* command could not be handled    */

/*
 *  data channel handshake errors
 */
#define sdmaERR_HANDSHAKE_REQ     451  /* request handshake error         */
#define sdmaERR_HANDSHAKE_FRAME   452  /* frame handshake error           */

/*
 *  command/reply message-id
 */
#define sdmaMES_COMMAND   0xffff       /* command message     */
#define sdmaMES_DATA      0x0002       /* data message        */
#define sdmaMES_BACKREPLY 0x0003       /* reply message       */
#define sdmaMES_MESSAGE   0x0004       /* unformatted message */

/*
 *  maximum number of output links
 */
#define sdmaMAX_OUTLINK   16          /* maximum number of output links */

/*
 *  output sync modes
 */
#define sdmaQUEUE_SKIP       0  /* skip frame, if queue is full       */
#define sdmaQUEUE_BLOCK      1  /* block until queue is free          */
#define sdmaQUEUE_SETERR     2  /* set error in frame header and skip */

/*
 *  maximum number of connections to command interpreter
 */
#define sdmaCI_MAX_CON 4       /* maximum command connections  */

/*
 *  asynchronous read methods
 */
#define sdmaMODE_SIM    1     /* built in simulator            */
#define sdmaMODE_NORMAL 2     /* read-threads                  */

/*
 *  maximum number of dma devices
 */
#define sdmaMAX_DEV  2         /* maximum number of dma devices */

/*
 *  maximum number of cameras
 */
#define sdmaMAX_CAM  2         /* maximum number of displays    */

/*
 *  command buffer sizes
 */
#define sdmaMAX_COMBUFSIZE   (8*1024)    /* command buffer size in bytes */

/*
 *  dynamic command codes
 */
#define sdmaDYN_CMD_IVLD         0       /* invalid command        */
#define sdmaDYN_CMD_RESET        1       /* reset counters         */   
#define sdmaDYN_CMD_RESERVED   100       /* reserved command codes */

/*
 *  datasets
 */
#define sdmaFLATFIELD   1                /* flatfield               */
#define sdmaBADPIXMASK  2                /* bad-pixel mask          */

/*
 *  data services
 */
#define sdmaDATA_SERVICE_PING      1      /* ping request       */
#define sdmaDATA_SERVICE_CONNECT   2      /* connection request */
#define sdmaDATA_SERVICE_ABORT     3      /* abort request      */
#define sdmaDATA_SERVICE_TYPES     4      /* get frame types    */
#define sdmaDATA_SERVICE_PROCNAME  5      /* get process name   */
#define sdmaDATA_SERVICE_TYPEDEF   6      /* get defines types  */
#define sdmaDATA_SERVICE_EXPORT    7      /* export internals   */

/*
 *  transfer modes
 */
#define sdmaTMODE_NOTRANS    0       /* no transfer      */
#define sdmaTMODE_SCIENCE    1       /* science transfer */
#define sdmaTMODE_VIDEO      2       /* video transfer   */

/*
 *  simple header
 */
typedef struct
{
  int bitPix;
  int nx;
  int ny;
  int ftype;
  int data[16];
} sdmaSIMPLE_HDR;

/*
 *  output service type
 */
typedef struct
{
  int      serviceId;     /* requested service id          */
  int      clientId;      /* process id of calling client  */
  int      tmode;         /* transfer mode                 */
} sdmaDATA_SRV_T;

/*
 *  frame request type
 */
typedef struct
{
  int  type;           /* requested frame type                         */
  int  blocking;       /* block until frame is ready                   */
  int  retransmit;     /* transfer last buffer again (video mode only) */
  int  start_x;        /* window start-x                               */
  int  start_y;        /* window start-y                               */
  int  nx;             /* window nx                                    */
  int  ny;             /* window ny                                    */
} sdmaREQ;

/*
 *  frame request type
 */
typedef struct
{
  int      transfer;        /* transfer flag (1-transfer, 0-skip)  */
  int      start_x;         /* window start-x                      */
  int      start_y;         /* window start-y                      */
  int      nx;              /* window nx                           */
  int      ny;              /* window ny                           */
} sdmaREQ_ACK;

/*
 *  frame header type
 */
typedef struct
{
  int   dtype;               /* data type                          */
  int   ftype;               /* frame type                         */
  int   start_x;             /* window start-x                     */
  int   start_y;             /* window start-y                     */
  int   nx;                  /* window nx                          */
  int   ny;                  /* window ny                          */
  int   scal;                /* scale factor                       */
  int   cnt;                 /* frame counter                      */
  int   setupId;             /* setup-id                           */
  int   err;                 /* error-id                           */
  int   overrun;             /* overrun flag                       */
  int   frames;              /* available frame types              */
  int   tx;                  /* track point x                      */
  int   ty;                  /* track point y                      */
} sdmaFRAME_H;

/*
 *  frame type (external)
 */
typedef struct
{
  sdmaFRAME_H   h;            /* header structure */
  char         *fbuf;         /* frame buffer     */
} sdmaFRAME_T;

/*
 *  frame type definition
 */
typedef struct
{
  int        type;            /* frame type                           */
  char       name[64];        /* frame type name                      */
  char       paramName[64];   /* parameter name to switch type on/off */
} sdmaFTYPE;


/*
 *  configuaration (for get config command)
 */
typedef struct
{ 
  int    numDev;                    /* number of dma devices                */
  int    inputMode[sdmaMAX_DEV];    /* array of input modes for each dev.   */
  int    dmaStat[sdmaMAX_DEV];      /* number of bytes in dma for each dev. */
  int    bufSize[sdmaMAX_DEV];      /* buffer-size for each device          */
  int    maxDataClients;            /* max. number of data clients          */
  int    dataClients;               /* number of data clients               */
  int    rcnt[sdmaMAX_OUTLINK];
  int    fcnt[sdmaMAX_OUTLINK];
  char   dataHost[sdmaMAX_OUTLINK][256];
  int    numFileSrv;                /* number of file servers               */
  int    maxCmdClients;             /* max. number of command clients       */
  int    cmdClients;                /* number of command clients            */
  char   cmdHost[sdmaCI_MAX_CON][256];
  int    frameTypes;                /* available frame types                */
  int    simTime;                   /* simulation interval (built-in sim.)  */
} sdmaCONF;

/*
 *  error stack type
 */
typedef struct
{
  int  id;
  char time[128];
  char erms[256];
} sdmaERRMSG;

#define sdmaFRAME_H_SIZE sizeof(sdmaFRAME_H)

/*
 *  command socket protoypes
 */
int   sdmaSockComHandler(int, icmdCOM *, icmdREP *, 
			 char *, char *, char *, char *);
int   sdmaComSockOpen(const char *, const char *, char *);
int   sdmaComSockClose(int);
int   sdmaSendSockCommand(int, icmdCOM, char *);
int   sdmaGetSockReply(int, icmdREP *, char *);

/*
 *  command CCS protoypes
 */
/*
int   sdmaCCSComHandler(int, icmdCOM *, icmdREP *, 
			 char *, char *, char *, char *);
int   sdmaComCCSOpen(char *, char *, char *);
int   sdmaComCCSClose(int);
int   sdmaSendCCSCommand(int, icmdCOM, char *);
int   sdmaGetCCSReply(int, icmdREP *, char *);
*/

/*
 *  protoypes
 */
int   sdmaExec(const char *, const char *, char, int *, char *);
int   sdmaKill(const char *, int, char *);
int   sdmaComExec(icmdLINK *, const char *, const char *, char, int *, char *);
int   sdmaComKill(const char *, int, char *);
int   sdmaCleanup(const char *, char *);
int   sdmaSelectCom(char *, char *);
int   sdmaAllocCom(char *);
int   sdmaOpenComLine(const char *, int *, char *);
int   sdmaCloseComLine(int);
void  sdmaErr(char *, short);
int   sdmaGetParamSetup(icmdLINK, int *, sdmaPARAM_NAMES, 
			sdmaPARAM_VALUES, char *);
int   sdmaGetParamType(icmdLINK, const char *, int *, char *);
int   sdmaSetParam(icmdLINK, const char *, int, char *);
int   sdmaSetParamBuf(icmdLINK, int *, char *);
int   sdmaGetParam(icmdLINK, const char *, int *, char *);
int   sdmaGetParamBuf(icmdLINK, int *, char *);
int   sdmaParamSync(icmdLINK, int, char *);
int   sdmaSetChipOffset(icmdLINK, int, int, char *);
int   sdmaSetHdr(icmdLINK, int, char *);
int   sdmaTimeout(icmdLINK, int, char *);
int   sdmaStart(icmdLINK, char *);
int   sdmaResetCnt(icmdLINK, char *);
int   sdmaEnd(icmdLINK, char *);
int   sdmaStop(icmdLINK, char *);
int   sdmaPause(icmdLINK, int, char *);
int   sdmaAbort(icmdLINK, char *);
int   sdmaExitComServer(icmdLINK, char *);
int   sdmaConnect(icmdLINK *, const char *, char *);
int   sdmaPing(icmdLINK, char *);
int   sdmaGetStatus(icmdLINK, int *, char *);
int   sdmaGetTime(icmdLINK, double *, double *, char *);
int   sdmaSetSimTime(icmdLINK, int, char *);
int   sdmaVerbose(icmdLINK, char, char *);
int   sdmaGetConfig(icmdLINK, sdmaCONF *, char *);
int   sdmaGetErrPort(icmdLINK, int *, char *);
int   sdmaGetDataPort(icmdLINK, int *, char *);
int   sdmaSendDynCmd(icmdLINK, int, int, char *);
int   sdmaExecUsrCmd(icmdLINK, const char *, char *, char *);
int   sdmaMclose(icmdLINK, char *);
int   sdmaMsync(icmdLINK, int id, char *);
int   sdmaLoadBuf(char *, char *, int *, char *);
int   sdmaDownloadBuf(icmdLINK, char *, int, int, int, char *);
int   sdmaFlushOutputQueue(icmdLINK, char *);
int   sdmaFileBase(icmdLINK, char *, char *);
int   sdmaFilePath(icmdLINK, int, char *, char *);

/*
 *  data transfer prototypes
 */
int   sdmaOpenDataLine(char *, int *, int, char *); 
int   sdmaCloseDataLine(int);
int   sdmaPingDataLine(char *, char *, char *);
int   sdmaAbortReq(char *, int, char *);
int   sdmaCancelReq(int, char *);
int   sdmaSendDataRequest(int, sdmaREQ, char *);
int   sdmaWaitDataReply(int, sdmaFRAME_H *, char *);
int   sdmaWaitNextFrame(int, sdmaREQ, sdmaFRAME_H *, char *);
int   sdmaPollNextFrame(int, sdmaREQ, sdmaFRAME_H *, char *);
int   sdmaSkipFrame(int, char *);
int   sdmaAcceptFrame(int, sdmaREQ, sdmaFRAME_T *, char *);
int   sdmaAcceptFrameN(int, sdmaREQ, sdmaFRAME_T *, int, int, char *);
int   sdmaAckFrame(int  linkId, sdmaREQ, sdmaFRAME_H *, char *);
int   sdmaGetFrameBuf(int, sdmaFRAME_T *, int, int, char *);
int   sdmaGetTypeDef(char *, int *, sdmaFTYPE *, char *);
int   sdmaGetFrameTypes(char *, int *, char *);
void  sdmaScale(sdmaFRAME_T *);
void  sdmaScale2(float *, sdmaFRAME_T *);
int   sdmaGetFrameTypeByName(int *, char *, char *);
int   sdmaGetFrameNameByType(int, char *, char *);
int   sdmaAttachData(char *, int, char *);
int   sdmaDetachData(char *);
sdmaSIMPLE_HDR *sdmaImportData(void **, char *);
int   sdmaReturnData(char *);

/*
 *  system type
 */
typedef int (*sdmaFUNC_OPEN)(const char *, const char *, char *);
typedef int (*sdmaFUNC_C)(int, icmdCOM, char *);
typedef int (*sdmaFUNC_R)(int, icmdREP *, char *);
typedef int (*sdmaFUNC_CLOSE)(int);
typedef struct
{
  sdmaFUNC_OPEN  open;
  sdmaFUNC_CLOSE close;
  sdmaFUNC_C     sendCommand; 
  sdmaFUNC_R     getReply;
} sdmaSYS;

extern int       sdmaSockTimeout;        /* timeout for immediate reply    */
extern int       sdmaMaxComBufSize;      /* maximum command buffer length  */
extern int       sdmaMaxRepBufSize;      /* maximum reply buffer length    */
extern sdmaSYS   sdmaSys;                /* command system module          */

#define sdmaSendCommand(l,c,e) sdmaSys.sendCommand(l,c,e)
#define sdmaGetReply(l,c,e)    sdmaSys.getReply(l,c,e)

/*
 *  verbose mode (from external)
 */
extern int sdmaVflag;

#ifdef __cplusplus
}
#endif

#endif /*!_sdmaInt_h*/
