#ifndef SEQ_H
#define SEQ_H
/*******************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: seq.h,v 2.97 2005/09/14 09:47:23 vltsccm Exp $" 
*
* seq.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* eallaert  30/03/94  created
* eallaert  28/07/95  differentiation of seqLONGNAME according to use of CCS
* eallaert  11/10/95  fixed various problems for compilation without CCS
* eallaert  03/05/96  Seq_Init name compatible with "load" in Tcl 7.5
*/

/************************************************************************
*  This is the public interface file for the Sequencer extensions to Tcl/Tk
*
*  Remark that if you use it, the following compilation flags should be set:
*  - CCS_FULL if the compilation is with full CCS
*  - CCS_LIGHT if the compilation is with CCS-light
*  - neither of the above if the compilation is without CCS
*
*----------------------------------------------------------------------
*/
/*
 * Other header files 
 */
#if defined(CCS_LIGHT)
#include "seqErrors.h"		/* error interface file */
#else
#include "seqErrorsNoCcs.h"	/* error interface file */
#endif

#include <tclExtend.h>		/* interface file for Tcl/TclX */

#if defined(CCS_LIGHT)
#include <ccs.h>		/* declares the vlt* and ccs* datatypes */
#endif	/* CCS_FULL || CCS_LIGHT */
/*
 * Definition of constants
 */
#define seqVERSION "$Revision: 2.97 $"	/* string containing revision code */
                                /* "tclAppVersion" points to digits only */
                                /* "infox appversion" returns digits only */

#define seqNAME "sequencer"	/* default name stored in tclAppName */

/* string returned by the command "infox applongname" for all CCS-cases 
 * (this string is pointed to by tclAppLongName)
 */
#if defined(CCS_LIGHT)
#define seqLONGNAME "The VLT Sequencer with CCS"
#define seqCCSTYPE "light"
#else
#define seqLONGNAME "The VLT Sequencer without CCS"
#define seqCCSTYPE "none"
#endif

#define tclPRECISION "tcl_precision"	/* Tcl-var linked to seqPrecision */
#define seqMODULEID "seq_moduleId"	/* Tcl-var linked to seqModuleId */
#define seqERRLOGGING "seq_errLogging"	/* Tcl-var linked to seqErrLogging */
#define seqERRREPLYMERGING "seq_errReplyMerging"/* linked to seqErrReplyMerging */
#define seqCMDCHECK "seq_ccsCmdCheck"	/* Tcl-var linked to seqCmdCheck */
#define seqOBISCRIPT "seq_obiScript"	/* Tcl-var linked to seqObiScript */

#define seqREPLYTIMEOUT "reply timed out"	/* return string value for a */
					/* timedout reply */

#define seqDEFAULTID "seq"		/* default module Id - cf errAdd() */
					/* package name is derived from this */
					/* (1st character is upshifted) */
#define seqDEFAULTCHECK "NO_CHECK"	/* def check - cf msgSendCommand() */

#define seqCMD_HANDLE_BASE "cmd"	/* basis for command handle name */
#define seqEVT_HANDLE_BASE "event"	/* basis for event handle name */
#define seqSCR_HANDLE_BASE "script"	/* basis for script handle name */
#define seqLIST_HANDLE_BASE "list"	/* basis for db-list handle name */
#define seqALRM_HANDLE_BASE "alarm"	/* basis for alarm handle name */


/*
 * Global variables-  see also seqInit.c, seqMsgDispatch
 */
extern unsigned int	seqPrecision;	/* C-variable linked to tclPRECISION */
					/* type must correspond to int */

extern char	*seqModuleId;		/* C-variable linked to seqMODULEID */
					/* see a.o. errAdd(); max 6 chars!!! */

extern int	seqErrLogMethod;	/* C-var reflecting seqERRLOGGING */
					/* 0=off, 1=auto, 2=manual */

extern int	seqErrReplyMergeMethod;	/* C-var corr. to seqERRREPLYMERGING */
					/* 0=off, 1=internal, 2=local */

extern char	*numericVersion;	/* string containing version */

#if defined(CCS_LIGHT)

extern ccsERROR		lastError;	/* CCS error at last errCloseStack */

extern char		*seqObiScript;	/* C-var linked 2 string seqOBISCRIPT */
					/* cf obituaries in seqMsgDispatch */
extern ccsENVNAME	myEnvName;	/* local environment name */
extern ccsPROCNUM	myProcNum;	/* Sequencer process number */
extern ccsPROCNAME	myProcName;	/* the CCS name of this process */

extern vltLOGICAL	cmdsAllowed;	/* reflects whether Sequencer event */
					/* loop will allow incoming cmds */

#endif	/* CCS_LIGHT */

/*
 * Data types
 */

/*
 * Function prototypes
 */
int Seq_Init (			/* Sequencer interpreter initialization */
	Tcl_Interp *interp	/* pointer to interpreter structure */
	);			/* (called by Tcl_AppInit & seqInitInterp) */

#if defined(CCS_LIGHT)
EXTERN ccsCOMPL_STAT seqInitInterp(  /* creates/initializes a Sequencer interp */
	char 	   *display,	/* string with display-term name */
	char 	   *name,	/* label for main window */
	vltLOGICAL interactive, /* flags if application is shell-like */
	Tcl_Interp **interp,	/* Tcl interpreter structure pointer-pointer */
	ccsERROR   *error);

EXTERN ccsCOMPL_STAT seqEval (	/* Sequencer interface to Tcl_Eval */
	Tcl_Interp *interp,	/* Tcl interpreter pointer */
	char	   *script,	/* string with script */
	ccsERROR   *error);	/* error structure pointer */

EXTERN ccsCOMPL_STAT seqMonitorQ ( /* enables/disables monitoring of msg queue */
	Tcl_Interp *interp,	/* result string goes to interp->result */
	vltLOGICAL stateflag,	/* ccsTRUE => enable; ccsFALSE => disable */
	ccsERROR   *error);	/* returned error structure */

#endif	/* CCS_LIGHT */
EXTERN ccsCOMPL_STAT saveAndCloseStack (/* does a errCloseStack after saving */
	ccsERROR   *error);	/* this error structure in global lastError */

#endif	/*!SEQ_H*/
