#ifndef _tcomChk_h
#define _tcomChk_h
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tcomChk.h,v 1.104 2005/09/12 09:21:29 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  04/05/98  created
* jstegmei  13/05/98  added tcomSclSysCheckResultBuf()
* jstegmei  29/06/98  adds for memory test support
* jstegmei  30/06/98  added tcomSclChkInit()/Free()
* jstegmei  10/05/02  replaced unsigned long type
*/

/************************************************************************
 *
 *  tcomChk.h - tcom subsystem check constants
 *
 *----------------------------------------------------------------------
 */


#ifdef __cplusplus
extern "C" {
#endif


/*
 *  system check type
 */
typedef struct tpstats
{
    struct tpstats *parent;        /* where I was born                     */
    struct tpstats *next;          /* sequential list, handy to have       */
    void	   *extra;         /* used to extend struct                */
    char	    info[256];     /* original line                        */
    int		    tpid;          /* number assigned by CHECKOUT          */
    int	            tptype;        /* Transputer type                      */
    int	            route;         /* incremental route here               */
    int	            routelen;      /* route length from host               */
    int	            bootlink;      /* link CHECKOUT is using for lifeline  */
    int             procspeed;     /* processor speed                      */
    float	    linkspeed;     /* Mbytes/sec down boot link            */
    struct tpstats *links[4];      /* pointers to other Transputers        */
    int		    tpno[4];       /* temporary - other transputer number  */
    int		    linkno[4];     /* link numbers, or non-transputer type */
}tcomCHK_TP;

/*
 *  memory test result type
 */
typedef struct meminfo
{
    unsigned char testno, endmem;
    int           memspeed;
    unsigned int error, memsize, watermark;
}tcomCHK_MEM;
 
/*
 *  linktype
 */
typedef int (*tcomCHK_OPEN)(char *, char *);
typedef int (*tcomCHK_TIMEOUT)(int, int);
typedef int (*tcomCHK_CLOSE)(int);
typedef int (*tcomCHK_RES)(int);
typedef int (*tcomCHK_R)(int, char *, int, char *);
typedef int (*tcomCHK_W)(int, char *, int, char *);
typedef struct
{
  tcomCHK_TIMEOUT timeout;
  tcomCHK_OPEN    open;
  tcomCHK_CLOSE   close;
  tcomCHK_RES     reset;
  tcomCHK_R       read; 
  tcomCHK_W       write;
} tcomCHK_MOD;

extern tcomCHK_MOD tcomChkMod;

#define tcomChkTimeout(l,t) tcomChkMod.timeout(l,t)
#define tcomChkOpen(d,e) tcomChkMod.open(d,e)
#define tcomChkClose(l) tcomChkMod.close(l)
#define tcomChkReset(l) tcomChkMod.reset(l)
#define tcomChkRead(l,b,s,e) tcomChkMod.read(l,b,s,e)
#define tcomChkWrite(l,b,s,e) tcomChkMod.write(l,b,s,e)

struct tcomCHK16_STRUCT 
{
  unsigned char code[914];
  char          target[10];
  char          mode[10];
  long          codeSize;
  long          offset;
  long          workSpace;
  long          vectorSpace;
  int           bytesPerWord;
  int           totalSpace;
};

struct tcomCHK32_STRUCT 
{
  unsigned char code[976];
  char          target[10];
  char          mode[10];
  long          codeSize;
  long          offset;
  long          workSpace;
  long          vectorSpace;
  int           bytesPerWord;
  int           totalSpace;
};

struct tcomTYPE16_STRUCT 
{
  unsigned char code[186];
  char          target[10];
  char          mode[10];
  long          codeSize;
  long          offset;
  long          workSpace;
  long          vectorSpace;
  int           bytesPerWord;
  int           totalSpace;
};

struct tcomTYPE32_STRUCT 
{
  unsigned char code[284];
  char          target[10];
  char          mode[10];
  long          codeSize;
  long          offset;
  long          workSpace;
  long          vectorSpace;
  int           bytesPerWord;
  int           totalSpace;
};

int  tcomSclMemTest (char *, tcomCHK_TP *, char *);
int  tcomSclSysCheck (char *, tcomCHK_TP *, char *);
int  tcomSclMemTestResult (FILE *, tcomCHK_TP, char *);
int  tcomSclMemTestResultBuf (char *, tcomCHK_TP, char *);
int  tcomSclSysCheckResult (FILE *, tcomCHK_TP, char **, char *);
int  tcomSclSysCheckResultBuf (char *, tcomCHK_TP, char **, char *);
void tcomSclCheckInit (tcomCHK_TP *);
void tcomSclCheckFree (tcomCHK_TP *);

#ifdef tcomDEF_CHK_STRUCT
static struct tcomCHK16_STRUCT tcomChk16Code = {
  {			/* code */
    0x20,0x61,0xBE,0x24, 0xF2,0xD4,0x24,0xF2, 0x51,0xD5,0x24,0xF2, 
    0x52,0xD6,0x24,0xF2, 0x53,0xD7,0x24,0xF2, 0x54,0xDE,0x24,0xF2, 
    0x55,0xDF,0x24,0xF2, 0x56,0x21,0xD0,0x24, 0xF2,0x57,0x21,0xD1, 
    0x21,0x73,0x1E,0xFA, 0xD2,0x72,0x30,0xD0, 0x13,0x70,0x42,0xF7, 
    0x13,0x70,0x42,0xF7, 0x1B,0xD8,0x1C,0xD9, 0x1D,0xDA,0x24,0xF2, 
    0xDB,0x24,0xF2,0xDC, 0x24,0xF2,0xDD,0x43, 0xD1,0x21,0x2E,0x44, 
    0x21,0xFB,0xD0,0x2C, 0x4E,0x62,0x1C,0xFD, 0x21,0x29,0x40,0x6A, 
    0x13,0xFD,0x60,0xB7, 0x40,0x11,0x81,0x23, 0xFB,0x11,0x51,0xF1, 
    0x1D,0xFA,0xD8,0x14, 0x7B,0x30,0x42,0xF7, 0x74,0x60,0xCF,0x2A, 
    0xA3,0x11,0x7B,0x30, 0x41,0xF7,0x11,0xF1, 0xC0,0x26,0xAC,0x7B, 
    0x30,0xD3,0x14,0x73, 0x42,0xF7,0x11,0x51, 0x73,0x41,0xF7,0x74, 
    0x60,0x8F,0xD4,0x40, 0x11,0x81,0x23,0xFB, 0x74,0xC0,0x23,0xA2, 
    0x11,0x51,0xF1,0x2F, 0xCF,0x22,0xA4,0x41, 0xD5,0x43,0xD6,0x75, 
    0x21,0x7C,0xF5,0x44, 0x21,0xFF,0xD0,0x70, 0x21,0x17,0xFA,0xD7, 
    0x70,0x1D,0xFA,0xD3, 0x77,0x21,0xF2,0x73, 0x21,0xF2,0x15,0x21, 
    0x4C,0x22,0xF1,0x21, 0x2F,0xFF,0x05,0x41, 0x11,0x83,0x23,0xFB, 
    0x21,0x0F,0x11,0x51, 0xF1,0x1D,0xFA,0xD5, 0x40,0x11,0x83,0x23, 
    0xFB,0x75,0x30,0xD3, 0x73,0x60,0x4F,0xFF, 0x73,0x40,0xFE,0x40, 
    0xD0,0x14,0x75,0x30, 0x7B,0x30,0x22,0x26, 0x94,0x22,0x0C,0x41, 
    0x11,0x83,0xF1,0xF4, 0xAD,0x78,0x30,0xD3, 0x73,0x60,0x4F,0xFF, 
    0x73,0x11,0xF1,0xFE, 0x21,0x09,0x11,0xF1, 0xC2,0xA6,0x41,0x11, 
    0x81,0x23,0xFB,0x0F, 0x11,0xF1,0xC9,0xA9, 0x24,0xF2,0x78,0x30, 
    0x21,0x20,0x41,0xFB, 0x02,0x21,0xF0,0x0C, 0x11,0x81,0xF1,0xD0, 
    0x14,0x78,0x30,0x7B, 0x30,0x22,0x22,0x99, 0x6C,0x0F,0x44,0xD1, 
    0x2B,0x4F,0x21,0xFB, 0xD0,0x60,0x1C,0xD2, 0x41,0xD3,0x43,0xD4, 
    0x10,0x72,0xE1,0x73, 0x72,0xE0,0x4B,0x72, 0x60,0x5C,0xFD,0x72, 
    0x62,0x55,0xD2,0x13, 0x21,0x44,0x22,0xF1, 0x10,0xF3,0x75,0x23, 
    0x37,0x74,0xF5,0x44, 0x21,0xFF,0xD3,0x24, 0xF2,0xD2,0x42,0xD1, 
    0x28,0x4D,0x21,0xFB, 0xD0,0x24,0x47,0x60, 0x11,0xFD,0x60,0xBC, 
    0x77,0x79,0x22,0x58, 0xFA,0xD3,0x40,0xD2, 0x73,0x30,0xD1,0x21, 
    0x2C,0x42,0x21,0xFB, 0x71,0x21,0x47,0xFB, 0x71,0x77,0x21,0x20, 
    0x40,0x21,0xF3,0xFE, 0x12,0x16,0x41,0xF7, 0x72,0xC0,0x21,0xA1, 
    0x73,0x30,0xD1,0x71, 0x79,0x22,0x37,0xFF, 0x71,0x77,0x21,0x20, 
    0x40,0x21,0xF3,0xFE, 0x0E,0x72,0xC1,0xA9, 0x29,0x4A,0x21,0xFB, 
    0x73,0x30,0x42,0xFB, 0x02,0x21,0xF0,0x62, 0x07,0x21,0x73,0x60, 
    0x8F,0x21,0x74,0x22, 0x5C,0xFA,0xD7,0x21, 0x72,0x21,0x74,0x23, 
    0x52,0xFA,0xD1,0x1F, 0x27,0x9F,0x16,0x71, 0x30,0x42,0xF7,0x76, 
    0x64,0x21,0x21,0xC7, 0x21,0xA3,0x40,0xD4, 0x21,0x46,0xD5,0x12, 
    0x71,0x30,0x41,0xF7, 0x14,0x49,0x22,0xF1, 0x21,0x11,0x40,0xFE, 
    0x0A,0x40,0xD0,0x16, 0x77,0x30,0x71,0x30, 0x21,0x26,0x96,0x62, 
    0x05,0x75,0xF3,0x22, 0x14,0xF3,0x24,0xF3, 0x40,0xD1,0x43,0xD2, 
    0x71,0x2B,0x15,0xFA, 0x30,0x41,0x24,0xF8, 0x11,0x4C,0x22,0xF1, 
    0x24,0xF4,0x40,0xD1, 0x43,0xD2,0x71,0x2B, 0x15,0xFA,0x30,0x41, 
    0x40,0x22,0xFF,0xA2, 0x71,0xD3,0x11,0x21, 0x41,0x22,0xF1,0x24, 
    0xF5,0x15,0x73,0x2B, 0x15,0xFA,0x30,0x42, 0xF7,0x40,0xD0,0x73, 
    0x2B,0x15,0xFA,0x30, 0xD4,0x15,0x2C,0x70, 0x2B,0x11,0xFA,0x30, 
    0x74,0x21,0x21,0x99, 0x64,0x08,0x21,0xB2, 0x22,0xF0,0x02,0x00, 
    0x00,0x60,0xB7,0x40, 0xD1,0x41,0xD2,0x22, 0xF2,0xD3,0x72,0x23, 
    0xAC,0x24,0xFE,0x7A, 0x60,0x32,0x30,0x41, 0x24,0xF8,0x73,0x2F, 
    0x2A,0x40,0xF2,0x41, 0x24,0xF7,0x25,0xF1, 0x7A,0x60,0x32,0x30, 
    0x41,0x40,0x22,0xFF, 0x73,0x2F,0x2A,0x40, 0xF2,0x41,0x21,0x41, 
    0x22,0xFE,0x24,0xF5, 0x71,0x84,0x48,0x21, 0xFF,0x15,0xF2,0x7A, 
    0x60,0x32,0x30,0x41, 0xF7,0x71,0x81,0xD1, 0x02,0x40,0xD2,0x63, 
    0x01,0x71,0xC3,0x21, 0xAF,0x40,0x15,0x83, 0x23,0xFB,0x7A,0x33, 
    0x21,0x20,0x40,0x21, 0xF3,0x15,0x87,0x23, 0xFB,0x7A,0x60,0x38, 
    0x30,0xD4,0x74,0x45, 0xFF,0x15,0x83,0x74, 0x45,0xFB,0x28,0x05, 
    0x71,0x21,0xC8,0x21, 0xAF,0x40,0x15,0x51, 0x23,0xFB,0x7A,0x33, 
    0x21,0x20,0x40,0x21, 0xF3,0x15,0x52,0x23, 0xFB,0x7A,0x60,0x38, 
    0x30,0xD4,0x74,0x43, 0xFF,0x15,0x51,0x74, 0x43,0xFB,0x26,0x01, 
    0x71,0xC1,0x22,0xAD, 0x15,0x52,0xF1,0xC0, 0x21,0xAB,0x7A,0x60, 
    0x38,0x30,0xD4,0x74, 0x42,0xFF,0x44,0x10, 0x23,0xFB,0x7A,0x33, 
    0x21,0x20,0x40,0x21, 0xF3,0x10,0x81,0x23, 0xFB,0x10,0x74,0x42, 
    0xFB,0x7A,0x52,0x41, 0xFE,0x15,0x7A,0x60, 0x32,0x30,0x41,0xF7, 
    0x0F,0x15,0x52,0xF1, 0x2A,0xCA,0xA7,0x2A, 0x4A,0x15,0x85,0xF1, 
    0xF4,0xA2,0x40,0xD1, 0x71,0x21,0x4F,0x21, 0xFB,0xF2,0xF1,0x15, 
    0x23,0xFB,0x7A,0x33, 0x21,0x20,0x40,0x21, 0xF3,0x15,0x81,0x23, 
    0xFB,0x7A,0x60,0x38, 0x30,0xD4,0x74,0x42, 0xFF,0x15,0x74,0x42, 
    0xFB,0xB9,0x22,0xF0, 0x00,0x01,0x02,0x00, 0x03,0x00,0x17,0xB1, 
    0xD1,0x24,0xF2,0x21, 0xFC,0x24,0xF2,0x21, 0xF8,0xF0,0x60,0x5C, 
    0x2A,0x2A,0x2A,0x4A, 0xFF,0x21,0x2F,0xFF, 0x02,0x60,0xB1,0x41, 
    0x21,0x73,0xF4,0xA6, 0x21,0x71,0x21,0x72, 0x30,0xFF,0x21,0x72, 
    0x30,0x40,0xF9,0x23, 0xA6,0x21,0x72,0x30, 0x21,0x4C,0xF9,0x21, 
    0xA6,0x11,0x21,0x70, 0x21,0x4C,0xF7,0x11, 0x21,0x71,0x21,0x4C, 
    0xFB,0x21,0x72,0x30, 0x61,0x84,0x21,0x72, 0xE0,0x21,0x06,0x21, 
    0x72,0x30,0xD0,0x11, 0x21,0x70,0x70,0xF7, 0x21,0x72,0x30,0xD0, 
    0x11,0x21,0x71,0x70, 0xFB,0x40,0x21,0x72, 0xE0,0x63,0x03,0xBF, 
    0x22,0xF0 },
  
  "TA",	       /* target         */
  "Undefined", /* mode           */
  914,	       /* code size      */
  1,	       /* offset         */
  213,	       /* workspace      */
  0,	       /* vectorspace    */
  2,	       /* bytes per word */
  1340	       /* total code requirement not including parameter space  */
  
};


static struct tcomCHK32_STRUCT tcomChk32Code = {
  {			/* code */
    0x20,0x20,0x20,0x61, 0xBD,0x22,0x26,0x4E, 0x21,0xFB,0xD4,0x24, 
    0xF2,0xD5,0x24,0xF2, 0x51,0xD6,0x24,0xF2, 0x52,0xD7,0x24,0xF2, 
    0x53,0xD8,0x24,0xF2, 0x54,0xDF,0x24,0xF2, 0x55,0x21,0xD0,0x24, 
    0xF2,0x56,0x21,0xD1, 0x24,0xF2,0x57,0x21, 0xD2,0x21,0x74,0x1F, 
    0xFA,0xD2,0x72,0x30, 0xD0,0x13,0x70,0x42, 0xF7,0x13,0x70,0x42, 
    0xF7,0x1C,0xD9,0x1D, 0xDA,0x1E,0xDB,0x24, 0xF2,0xDC,0x24,0xF2, 
    0xDD,0x24,0xF2,0xDE, 0x43,0xD1,0x22,0x21, 0x4D,0x21,0xFB,0xD0, 
    0x2E,0x48,0x61,0x12, 0xFD,0x21,0x2B,0x4B, 0x69,0x1D,0xFD,0x60, 
    0xB7,0x40,0x11,0x81, 0x23,0xFB,0x11,0x82, 0xF1,0x1E,0xFA,0xD8, 
    0x15,0x7B,0x30,0x42, 0xF7,0x75,0x2F,0x2F, 0x2F,0x4F,0x24,0xF6, 
    0x28,0x20,0x20,0x40, 0x23,0xFA,0xD4,0x74, 0x60,0xCF,0x2A,0xAF, 
    0x11,0x7B,0x30,0x41, 0xF7,0x11,0xF1,0xC0, 0x27,0xA8,0x7B,0x30, 
    0xD2,0x15,0x72,0x42, 0xF7,0x11,0x82,0x72, 0x41,0xF7,0x75,0x2F, 
    0x2F,0x2F,0x4F,0x24, 0xF6,0x28,0x20,0x20, 0x40,0x23,0xFA,0x60, 
    0x8F,0xD4,0x40,0x11, 0x81,0x23,0xFB,0x74, 0xC0,0x23,0xA2,0x11, 
    0x82,0xF1,0x2F,0xCF, 0x22,0xA4,0x41,0xD2, 0x43,0xD3,0x72,0x21, 
    0x7D,0xF5,0x44,0x21, 0xFF,0xD0,0x70,0x21, 0x18,0xFA,0xD7,0x70, 
    0x1E,0xFA,0xD6,0x77, 0x21,0xF2,0x76,0x21, 0xF2,0x12,0x21,0x4C, 
    0x22,0xF1,0x21,0x2F, 0xFF,0x05,0x41,0x11, 0x83,0x23,0xFB,0x21, 
    0x0F,0x11,0x82,0xF1, 0x1E,0xFA,0xD3,0x40, 0x11,0x83,0x23,0xFB, 
    0x73,0x30,0xD2,0x7D, 0x72,0x42,0xFB,0x72, 0x40,0xFE,0x40,0xD0, 
    0x14,0x73,0x30,0x7B, 0x30,0x22,0x28,0x94, 0x22,0x0C,0x41,0x11, 
    0x83,0xF1,0xF4,0xAD, 0x78,0x30,0xD2,0x7D, 0x72,0x42,0xFB,0x72, 
    0x11,0xF1,0xFE,0x21, 0x09,0x11,0xF1,0xC2, 0xA6,0x41,0x11,0x81, 
    0x23,0xFB,0x0F,0x11, 0xF1,0xC9,0xA9,0x24, 0xF2,0x78,0x30,0x21, 
    0x20,0x41,0xFB,0x02, 0x21,0xF0,0x0C,0x11, 0x81,0xF1,0xD0,0x14, 
    0x78,0x30,0x7B,0x30, 0x22,0x24,0x99,0x6D, 0x05,0x44,0xD1,0x2D, 
    0x40,0x21,0xFB,0xD0, 0x60,0x1C,0xD2,0x41, 0xD3,0x43,0xD4,0x10, 
    0x72,0xE1,0x73,0x72, 0xE0,0x4B,0x72,0x60, 0x5C,0xFD,0x72,0x62, 
    0x5C,0xD2,0x13,0x21, 0x44,0x22,0xF1,0x10, 0xF3,0x75,0x23,0x32, 
    0x74,0xF5,0x44,0x21, 0xFF,0xD3,0x24,0xF2, 0xD2,0x42,0xD1,0x29, 
    0x4E,0x21,0xFB,0xD0, 0x24,0x48,0x60,0x12, 0xFD,0x60,0xBC,0x77, 
    0x79,0x22,0x53,0xFA, 0xD3,0x40,0xD2,0x73, 0x30,0xD1,0x21,0x2E, 
    0x41,0x21,0xFB,0x71, 0x21,0x47,0xFB,0x71, 0x77,0x21,0x20,0x40, 
    0x21,0xF3,0xFE,0x12, 0x16,0x41,0xF7,0x72, 0xC0,0x21,0xA2,0x73, 
    0x30,0xD1,0x79,0x22, 0x51,0x71,0x42,0xFB, 0x71,0x77,0x21,0x20, 
    0x40,0x21,0xF3,0xFE, 0x0E,0x72,0xC1,0xA9, 0x2B,0x48,0x21,0xFB, 
    0x73,0x30,0x42,0xFB, 0x02,0x21,0xF0,0x62, 0x06,0x21,0x72,0x60, 
    0x8F,0x21,0x73,0x22, 0x57,0xFA,0xD6,0x21, 0x71,0x21,0x73,0x22, 
    0x5D,0xFA,0xD1,0x1E, 0x2A,0x95,0x40,0xD5, 0x15,0x71,0x30,0x41, 
    0xF7,0x75,0xD4,0x15, 0x71,0x30,0x41,0xF7, 0x75,0x48,0x24,0xF1, 
    0x74,0xF5,0xD4,0x74, 0x2B,0x21,0x21,0xC7, 0x21,0xA3,0x40,0xD2, 
    0x21,0x46,0xD3,0x10, 0x71,0x30,0x41,0xF7, 0x12,0x49,0x22,0xF1, 
    0x21,0x10,0x40,0xFE, 0x0A,0x40,0xD0,0x14, 0x76,0x30,0x71,0x30, 
    0x21,0x27,0x95,0x63, 0x05,0x75,0xF3,0x21, 0x1E,0xF3,0x24,0xF3, 
    0x40,0xD1,0x43,0xD2, 0x71,0x29,0x1C,0xFA, 0x30,0x41,0x24,0xF8, 
    0x11,0x4C,0x22,0xF1, 0x24,0xF4,0x40,0xD1, 0x43,0xD2,0x71,0x29, 
    0x1C,0xFA,0x30,0x41, 0x40,0x22,0xFF,0xA2, 0x71,0xD3,0x11,0x21, 
    0x41,0x22,0xF1,0x24, 0xF5,0x16,0x73,0x29, 0x1C,0xFA,0x30,0x42, 
    0xF7,0x76,0x2F,0x2F, 0x2F,0x4F,0x24,0xF6, 0x28,0x20,0x20,0x40, 
    0x23,0xFA,0xD5,0x40, 0xD0,0x73,0x29,0x1C, 0xFA,0x30,0xD4,0x15, 
    0x2A,0x77,0x29,0x18, 0xFA,0x30,0x74,0x21, 0x21,0x9A,0x65,0x0A, 
    0x21,0xB3,0x22,0xF0, 0x02,0x00,0x00,0x00, 0xFF,0xFF,0xFF,0xFF, 
    0x00,0x00,0x00,0x60, 0xB9,0x2D,0x4F,0x21, 0xFB,0xD6,0x40,0xD1, 
    0x41,0xD4,0x22,0xF2, 0xD5,0x74,0x23,0xAC, 0x24,0xFE,0x78,0x60, 
    0x33,0x30,0x41,0x24, 0xF8,0x75,0x2F,0x2A, 0x40,0xF2,0x41,0x24, 
    0xF7,0x25,0xF1,0x78, 0x60,0x33,0x30,0x41, 0x40,0x22,0xFF,0x75, 
    0x2F,0x2A,0x40,0xF2, 0x41,0x21,0x41,0x22, 0xFE,0x24,0xF5,0x71, 
    0x84,0x48,0x21,0xFF, 0x12,0xF2,0x78,0x60, 0x33,0x30,0x41,0xF7, 
    0x71,0x81,0xD1,0x02, 0x40,0xD4,0x63,0x01, 0x71,0xC3,0x22,0xA1, 
    0x40,0x12,0x83,0x23, 0xFB,0x78,0x33,0x21, 0x20,0x40,0x21,0xF3, 
    0x12,0x87,0x23,0xFB, 0x78,0x60,0x38,0x30, 0xD0,0x76,0x52,0x70, 
    0x42,0xFB,0x12,0x83, 0x70,0x45,0xFB,0x26, 0x08,0x71,0x21,0xC8, 
    0x22,0xA1,0x40,0x12, 0x82,0x23,0xFB,0x78, 0x33,0x21,0x20,0x40, 
    0x21,0xF3,0x12,0x51, 0x23,0xFB,0x78,0x60, 0x38,0x30,0xD0,0x76, 
    0x51,0x70,0x42,0xFB, 0x12,0x82,0x70,0x43, 0xFB,0x24,0x02,0x71, 
    0xC1,0xAE,0x60,0x0E, 0x78,0x52,0x41,0xFE, 0x12,0x78,0x60,0x33, 
    0x30,0x41,0xF7,0x0F, 0x12,0x51,0xF1,0x2A, 0xCA,0xA7,0x2A,0x4A, 
    0x12,0x85,0xF1,0xF4, 0xA2,0x40,0xD1,0x71, 0x22,0x40,0x21,0xFB, 
    0xF2,0xF1,0x12,0x23, 0xFB,0x78,0x33,0x21, 0x20,0x40,0x21,0xF3, 
    0x12,0x81,0x23,0xFB, 0x78,0x60,0x38,0x30, 0xD0,0x76,0x70,0x42, 
    0xFB,0x12,0x70,0x42, 0xFB,0xB7,0x22,0xF0, 0x00,0x01,0x02,0x00, 
    0x03,0x00,0x00,0x00, 0x02,0x00,0x00,0x00, 0x03,0x00,0x00,0x00, 
    0x05,0x00,0x00,0x00, 0x17,0xB1,0xD1,0x24, 0xF2,0x21,0xFC,0x24, 
    0xF2,0x21,0xF8,0xF0, 0x60,0x5C,0x2A,0x2A, 0x2A,0x4A,0xFF,0x21, 
    0x2F,0xFF,0x02,0x00, 0x60,0xB7,0x41,0x7D, 0xF4,0xAD,0x7C,0x30, 
    0x28,0x20,0x20,0x40, 0x25,0xF6,0xD0,0x10, 0x7B,0x42,0xFB,0x7C, 
    0x30,0x40,0xF9,0x22, 0xAC,0x7C,0x30,0x22, 0x40,0xF9,0x21,0xA2, 
    0x11,0x7A,0x22,0x40, 0xF7,0x11,0x7B,0x22, 0x40,0xFB,0x7C,0x30, 
    0x61,0x80,0x7C,0xE0, 0x21,0x01,0x7C,0x30, 0xD0,0x11,0x7A,0x70, 
    0xF7,0x7C,0x30,0xD0, 0x11,0x7B,0x70,0xFB, 0x40,0x7C,0xE0,0x63, 
    0x0E,0xB9,0x22,0xF0 },
  
  "TA",	       /* target         */
  "Undefined", /* mode           */
  976,	       /* code size      */
  3,	       /* offset         */
  182,	       /* workspace      */
  0,	       /* vectorspace    */
  4,	       /* bytes per word */
  1704	       /* total code requirement not including parameter space  */
  
};

static struct tcomTYPE16_STRUCT tcomType16Code = {
  {			/* code */
    0x60,0xB1,0x24,0xF2, 0xD8,0x24,0xF2,0x51, 0xD9,0x24,0xF2,0x52, 
    0xDA,0x24,0xF2,0x53, 0xDB,0x24,0xF2,0x54, 0xD4,0x24,0xF2,0x55, 
    0xD5,0x24,0xF2,0x56, 0xD6,0x24,0xF2,0x57, 0xD7,0x21,0x70,0x18, 
    0xFA,0xD3,0x21,0x70, 0x14,0xFA,0xD2,0x42, 0xD1,0x28,0x46,0x21, 
    0xFB,0xD0,0x4B,0x21, 0xFB,0x68,0xD8,0x68, 0x19,0x23,0xF9,0x68, 
    0xB6,0x28,0x1A,0xF3, 0x29,0x13,0xD5,0x60, 0x49,0x61,0x4D,0x40, 
    0x21,0x27,0xFC,0xD6, 0xD4,0x74,0xC0,0xA4, 0x76,0x75,0xE0,0x03, 
    0x74,0x75,0xE0,0x22, 0xF2,0xD3,0x23,0x2E, 0x48,0x24,0xF1,0x22, 
    0xF2,0xD2,0x23,0x2F, 0x4C,0x72,0x73,0xF4, 0x22,0xFC,0x21,0x20, 
    0x40,0x21,0xF3,0x29, 0x13,0x51,0x23,0xFB, 0x29,0x77,0x21,0x20, 
    0x40,0x21,0xF3,0x29, 0x13,0x83,0x23,0xFB, 0x29,0x13,0x52,0xD1, 
    0x40,0xD0,0x10,0x28, 0x79,0x30,0x41,0xF7, 0x22,0xF2,0xD3,0x17, 
    0x28,0x79,0x30,0x21, 0x20,0x40,0xF7,0x22, 0xF2,0xD2,0x72,0x73, 
    0xF4,0x71,0xE0,0x28, 0x7A,0x30,0xD1,0x71, 0x46,0xFF,0x29,0x13, 
    0x71,0x46,0xFB,0x21, 0x2F,0xFF,0x10,0x81, 0x23,0xF9,0x21,0xF5, 
    0x28,0x17,0xF3,0xBF, 0x22,0xF0 },
  
  "TA",	       /* target         */
  "Undefined", /* mode           */
  186,	       /* code size      */
  0,	       /* offset         */
  155,	       /* workspace      */
  0,	       /* vectorspace    */
  2,	       /* bytes per word */
  496	       /* total code requirement not including parameter space  */
  
};

static struct tcomTYPE32_STRUCT tcomType32Code = {
  {			/* code */
    0x20,0x20,0x60,0xB2, 0x21,0x20,0x47,0x21, 0xFB,0xD4,0x24,0xF2, 
    0xDA,0x24,0xF2,0x51, 0xDB,0x24,0xF2,0x52, 0xDC,0x24,0xF2,0x53, 
    0xDD,0x24,0xF2,0x54, 0xD6,0x24,0xF2,0x55, 0xD7,0x24,0xF2,0x56, 
    0xD8,0x24,0xF2,0x57, 0xD9,0x7F,0x1A,0xFA, 0xD5,0x42,0xD1,0x2D, 
    0x4A,0x21,0xFB,0xD0, 0x4A,0x21,0xFB,0x60, 0xD7,0x60,0x18,0x23, 
    0xF9,0x60,0xB5,0x1B, 0xF3,0x21,0x77,0x1E, 0xFA,0xD6,0x1A,0xD1, 
    0x60,0x49,0x61,0x4D, 0x40,0x21,0x27,0xFC, 0xD5,0xD4,0x74,0xC0, 
    0xAE,0x75,0x28,0x20, 0x20,0x40,0x25,0xF6, 0xD0,0x10,0x71,0x42, 
    0x24,0xFA,0x0D,0x74, 0x28,0x20,0x20,0x40, 0x25,0xF6,0xD0,0x10, 
    0x71,0x42,0x24,0xFA, 0x71,0x10,0x42,0x24, 0xFA,0x70,0x2F,0x2F, 
    0x2F,0x4F,0x24,0xF6, 0x28,0x20,0x20,0x40, 0x23,0xFA,0x60,0xC9, 
    0x22,0xA0,0x7C,0x32, 0xD0,0x10,0x28,0xFE, 0x10,0x28,0xFE,0x28, 
    0xFF,0xC0,0xA4,0x29, 0xF0,0x60,0xAC,0x10, 0x28,0xF8,0x7C,0x32, 
    0x70,0xF4,0xC0,0xA6, 0x7C,0x51,0x71,0x42, 0x24,0xFA,0x22,0xF2, 
    0xD3,0x23,0x2E,0x48, 0x24,0xF1,0x22,0xF2, 0xD2,0x23,0x2F,0x4C, 
    0x72,0x73,0xF4,0x22, 0xFC,0x21,0x20,0x40, 0x21,0xF3,0x1A,0x82, 
    0x23,0xFB,0x21,0x77, 0x21,0x20,0x40,0x21, 0xF3,0x1A,0x83,0x23, 
    0xFB,0x1A,0x51,0xD7, 0x40,0xD0,0x10,0x76, 0x30,0x41,0xF7,0x22, 
    0xF2,0xD3,0x24,0xF2, 0x21,0x2C,0x50,0x76, 0x30,0x21,0x20,0x40, 
    0xF7,0x22,0xF2,0xD2, 0x72,0x73,0xF4,0x28, 0x20,0x20,0x40,0x25, 
    0xF6,0xD0,0x10,0x77, 0x42,0x24,0xFA,0x7D, 0x30,0xD0,0x7C,0x70, 
    0x42,0xFB,0x1A,0x70, 0x46,0xFB,0x21,0x2F, 0xFF,0x10,0x81,0x23, 
    0xF9,0x21,0xF5,0x18, 0xF3,0xBE,0x22,0xF0, 0x06,0x00,0x00,0x00, 
    0xF8,0xFF,0xFF,0xFF, 0x00,0x00,0x80,0x7F },
  
  "TA",	       /* target         */
  "Undefined", /* mode           */
  284,	       /* code size      */
  2,	       /* offset         */
  27,	       /* workspace      */
  0,	       /* vectorspace    */
  4,	       /* bytes per word */
  392	       /* total code requirement not including parameter space  */
  
};


struct mtest16_struct {
	unsigned char Code[1378];
	char Target[10];
	char Mode[10];
        long CodeSize;
        long Offset;
        long WorkSpace;
        long VectorSpace;
        int  BytesPerWord;
        int  TotalSpace;
};

struct mtest16_struct mtest16_code = {

    {			/* code */
	0x20,0x60,0xB3,0x24, 0xF2,0xD4,0x24,0xF2, 0xD2,0x24,0xF2,0x21, 
	0xD0,0x24,0xF2,0xD9, 0x24,0xF2,0x51,0xDA, 0x24,0xF2,0x52,0xDB, 
	0x24,0xF2,0x53,0xDC, 0x24,0xF2,0x54,0xD5, 0x24,0xF2,0x55,0xD6, 
	0x24,0xF2,0x56,0xD7, 0x24,0xF2,0x57,0xD8, 0x7E,0x15,0xFA,0x30, 
	0xD3,0x42,0xD1,0x22, 0x24,0x41,0x21,0xFB, 0xD0,0x2D,0x40,0x21, 
	0xFB,0x60,0xDD,0x60, 0x1E,0x23,0xF9,0x64, 0xB9,0x40,0xD5,0x22, 
	0x23,0x40,0x21,0xFB, 0x16,0x21,0x40,0x24, 0xFA,0x10,0x24,0x1B, 
	0x42,0xF7,0x16,0x24, 0x1B,0x21,0x40,0xF7, 0x16,0x51,0xF1,0xD5, 
	0x16,0x52,0xD1,0x71, 0x30,0xD4,0x16,0x52, 0xD2,0x16,0x54,0xDE, 
	0x16,0x56,0xD3,0x40, 0x72,0xE0,0x40,0x16, 0x83,0x23,0xFB,0x40, 
	0x73,0xE0,0x16,0x83, 0xF1,0xC0,0x28,0xAB, 0x75,0x16,0x51,0x23, 
	0xFB,0x72,0x30,0x25, 0xAF,0x16,0xD0,0x72, 0x30,0x72,0x30,0x24, 
	0xF2,0xFA,0x24,0x20, 0x40,0xF0,0x22,0x25, 0x96,0x24,0x20,0x40, 
	0xD1,0x72,0x30,0x71, 0xF9,0x21,0xA0,0x71, 0x24,0xF2,0xFA,0x30, 
	0x71,0xF4,0xC0,0xA7, 0x71,0x24,0x20,0x80, 0xD1,0x61,0x0A,0x73, 
	0x30,0xA7,0x41,0x16, 0x83,0x23,0xFB,0x22, 0x0B,0x71,0x72,0x30, 
	0xF4,0xA8,0x2F,0x4F, 0x16,0x83,0x23,0xFB, 0x21,0x0E,0x74,0x72, 
	0x30,0x22,0x20,0x50, 0xF9,0xC0,0xA6,0x42, 0x16,0x83,0x23,0xFB, 
	0x0F,0x72,0x30,0x22, 0x20,0x50,0x67,0x20, 0x20,0xC0,0xA5,0x42, 
	0x16,0x83,0x23,0xFB, 0x24,0x19,0x21,0x40, 0xFF,0x7E,0xD0,0x24, 
	0x20,0x40,0x72,0x30, 0x24,0xF2,0xFA,0x24, 0x17,0x21,0x2D,0x90, 
	0x16,0x24,0x19,0x21, 0x40,0xFB,0x72,0x30, 0x22,0x20,0x50,0x72, 
	0xE0,0x69,0x0F,0x6C, 0x0A,0x43,0xD1,0x21, 0x25,0x4D,0x21,0xFB, 
	0xD0,0x21,0x41,0x60, 0x14,0xFD,0x2D,0x48, 0x62,0x1C,0xFD,0x60, 
	0xBF,0x22,0xF2,0xD0, 0x10,0x21,0x13,0x42, 0xFB,0x60,0x06,0x40, 
	0x11,0x81,0x23,0xFB, 0x13,0x21,0x71,0x42, 0xF7,0x73,0x60,0xCF, 
	0x29,0xA9,0x11,0x21, 0x71,0x41,0xF7,0x11, 0xF1,0xC0,0x26,0xA9, 
	0x13,0x21,0x71,0x42, 0xF7,0x11,0x51,0x21, 0x71,0x41,0xF7,0x73, 
	0x60,0x8F,0xD3,0x40, 0x11,0x81,0x23,0xFB, 0x73,0xC0,0x23,0xA3, 
	0x11,0x51,0xF1,0x2F, 0xCF,0x22,0xA5,0x41, 0xD6,0x43,0xD7,0x76, 
	0x21,0x7C,0xF5,0x44, 0x21,0xFF,0xD0,0x70, 0x21,0x13,0xFA,0x30, 
	0xD5,0x70,0x21,0x17, 0xFA,0x30,0xD4,0x15, 0x21,0xF2,0x14,0x21, 
	0xF2,0x16,0x21,0x4F, 0x22,0xF1,0x60,0x0E, 0x41,0x11,0x83,0x23, 
	0xFB,0x21,0x0C,0x11, 0x51,0xF1,0x21,0x17, 0xFA,0x30,0xD4,0x40, 
	0x11,0x83,0x23,0xFB, 0x74,0x60,0x4F,0xFF, 0x74,0x40,0xFE,0x40, 
	0xD0,0x13,0x74,0x21, 0x71,0x2E,0x92,0x22, 0x04,0x41,0x11,0x83, 
	0xF1,0xF4,0x21,0xA1, 0x11,0x51,0xF1,0x21, 0x17,0xFA,0x30,0xD4, 
	0x74,0x60,0x4F,0xFF, 0x11,0x74,0x41,0xFB, 0x0C,0x11,0xF1,0xC2, 
	0xA6,0x41,0x11,0x81, 0x23,0xFB,0x02,0x21, 0xF0,0x22,0x02,0x11, 
	0x51,0xF1,0xC4,0xAD, 0x11,0x81,0xF1,0xD0, 0x13,0x21,0x12,0x21, 
	0x71,0x2A,0x9A,0x21, 0x00,0x11,0x81,0xF1, 0xD0,0x13,0x11,0x51, 
	0xF1,0x21,0x17,0xFA, 0x30,0x21,0x71,0x29, 0x98,0x6C,0x09,0x24, 
	0xF3,0x41,0xD2,0x43, 0xD3,0x23,0x74,0x72, 0xF5,0x44,0x21,0xFF, 
	0x22,0x1B,0xFA,0x30, 0xD1,0x71,0x41,0x24, 0xF8,0x12,0x21,0x45, 
	0x22,0xF1,0x22,0x18, 0x41,0x24,0xF8,0x24, 0xF4,0x41,0xD2,0x43, 
	0xD3,0x23,0x74,0x72, 0xF5,0x44,0x21,0xFF, 0x22,0x1B,0xFA,0x30, 
	0xD1,0x71,0x41,0x40, 0x22,0xFF,0xA2,0x72, 0xD5,0x12,0x21,0x49, 
	0x22,0xF1,0x22,0x18, 0x41,0x21,0x4E,0x22, 0xFF,0x24,0xF5,0x23, 
	0x74,0x75,0xF5,0x44, 0x21,0xFF,0x22,0x1B, 0xFA,0x30,0xD1,0x14, 
	0x71,0x42,0xF7,0x40, 0xD0,0x14,0x23,0x74, 0x22,0x1F,0xFA,0x30, 
	0x71,0x23,0x92,0x21, 0x02,0x14,0x22,0x18, 0x42,0xF7,0x40,0xD0, 
	0x14,0x23,0x74,0x22, 0x1F,0xFA,0x30,0x22, 0x18,0x21,0x9E,0x67, 
	0x06,0x10,0x81,0x23, 0xF9,0x21,0xF5,0x12, 0xF3,0xBD,0x22,0xF0, 
	0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00, 
	0x00,0x00,0x00,0x00, 0x00,0x60,0xB5,0x41, 0x7F,0xF4,0xA4,0x7E, 
	0x7D,0x42,0xFB,0x7E, 0x30,0x40,0xF9,0x22, 0xAC,0x7E,0x30,0x21, 
	0x44,0xF9,0x21,0xA2, 0x11,0x7C,0x21,0x44, 0xF7,0x11,0x7D,0x21, 
	0x44,0xFB,0x7E,0x30, 0x61,0x8C,0x7E,0xE0, 0x21,0x01,0x7E,0x30, 
	0xD0,0x11,0x7C,0x70, 0xF7,0x7E,0x30,0xD0, 0x11,0x7D,0x70,0xFB, 
	0x40,0x7E,0xE0,0x63, 0x0E,0xBB,0x22,0xF0, 0x60,0xBE,0x10,0x73, 
	0x21,0x50,0x42,0xF7, 0x74,0x74,0x75,0x23, 0xF4,0x24,0xFA,0x10, 
	0x73,0x21,0x50,0x42, 0xF7,0x11,0x73,0x21, 0x50,0x42,0xF7,0x71, 
	0x70,0xF4,0x76,0xE0, 0xB2,0x22,0xF0,0x60, 0xBD,0x77,0x56,0xD1, 
	0x71,0x41,0x24,0xF6, 0x41,0x21,0xF3,0x22, 0xF2,0xD2,0x77,0x51, 
	0xF1,0xD0,0x45,0x70, 0xF9,0x24,0xAE,0x71, 0x30,0xC0,0x24,0xA9, 
	0x70,0x42,0xF9,0xAA, 0x43,0x70,0xF4,0x22, 0xAF,0x70,0xC4,0xAE, 
	0x23,0x01,0x70,0xAC, 0x41,0x70,0xF4,0x21, 0xA6,0x70,0xC2,0xA2, 
	0x21,0x08,0x21,0xF0, 0x71,0x60,0x4F,0x10, 0x21,0x2D,0x9B,0x71, 
	0x40,0x10,0x21,0x2D, 0x95,0x21,0x09,0x71, 0x12,0x10,0x25,0x9B, 
	0x21,0x02,0x71,0x12, 0x10,0x2E,0x96,0x0C, 0x71,0x75,0x74,0x21, 
	0x27,0x93,0x05,0x71, 0x76,0x10,0x21,0x92, 0x70,0x81,0xD0,0x65, 
	0x0D,0x70,0x21,0x20, 0x40,0x21,0xF3,0x77, 0x51,0x23,0xFB,0xB3, 
	0x22,0xF0,0x60,0xBE, 0x40,0xD0,0x73,0x35, 0xD1,0x71,0xAC,0x74, 
	0x70,0xF5,0x70,0x73, 0x34,0xFA,0xE0,0x10, 0x4C,0x22,0xF1,0x40, 
	0xD0,0x73,0x35,0xD1, 0x71,0x21,0xA5,0x74, 0x70,0xF5,0x70,0x73, 
	0x34,0xFA,0x30,0x23, 0xF3,0x75,0x30,0x24, 0xFB,0x75,0xE0,0x10, 
	0x21,0x45,0x22,0xF1, 0xB2,0x22,0xF0,0x60, 0xB6,0x7B,0x35,0x23, 
	0xF4,0xD6,0x7B,0x34, 0xD2,0x40,0xD1,0x7C, 0x21,0x29,0x9B,0x40, 
	0xD4,0x42,0xD5,0x74, 0xC0,0xA4,0x7C,0x30, 0xD9,0x03,0x79,0x7C, 
	0xE0,0x4F,0x7C,0x30, 0x24,0xF6,0xD3,0x4A, 0x73,0xF9,0xC0,0xA6, 
	0x26,0x26,0x26,0x41, 0xD3,0x08,0x73,0x2D, 0x4D,0x21,0xFB,0xFA, 
	0x30,0xD3,0x7C,0x21, 0x27,0x90,0x27,0x2F, 0x2F,0x4F,0x7C,0x30, 
	0x24,0xF6,0x7B,0x35, 0x21,0xFF,0xD0,0x40, 0xD7,0x7B,0x35,0x42, 
	0x22,0xFC,0xD8,0x78, 0x24,0xF2,0x21,0xF3, 0x23,0xA3,0x70,0x73, 
	0xF5,0x76,0x21,0xFF, 0xD0,0x7C,0x21,0x24, 0x99,0x2F,0x4F,0x7C, 
	0x30,0x24,0xF6,0x21, 0x20,0x40,0x21,0xF3, 0xD1,0x74,0xC0,0xA7, 
	0x71,0x70,0x72,0xF2, 0x23,0xFB,0x0D,0x70, 0x72,0xF2,0xF1,0x71, 
	0x23,0xF3,0x7D,0x30, 0x24,0xFB,0x7D,0xE0, 0x17,0x23,0x43,0x22, 
	0xF1,0x14,0x27,0x4B, 0x22,0xF1,0xBA,0x22, 0xF0,0x60,0xB8,0x7A, 
	0x21,0x21,0x93,0x40, 0xD3,0x42,0xD4,0x73, 0xC0,0xA4,0x7A,0x30, 
	0xD7,0x03,0x77,0x7A, 0xE0,0x4F,0x7A,0x30, 0x24,0xF6,0xD2,0x4A, 
	0x72,0xF9,0xC0,0xA6, 0x26,0x26,0x26,0x41, 0xD2,0x08,0x72,0x25, 
	0x45,0x21,0xFB,0xFA, 0x30,0xD2,0x7A,0x2E, 0x99,0x27,0x2F,0x2F, 
	0x4F,0x7A,0x30,0x24, 0xF6,0x79,0x35,0x21, 0xFF,0xD0,0x40,0xD5, 
	0x79,0x35,0x42,0x22, 0xFC,0xD6,0x76,0x24, 0xF2,0x21,0xF3,0x22, 
	0xAB,0x70,0x72,0xF5, 0x79,0x35,0x21,0xFF, 0xD0,0x7A,0x2C,0x92, 
	0x7A,0x30,0xD1,0x73, 0xC0,0xA7,0x71,0x70, 0x79,0x34,0xFA,0xE0, 
	0x0E,0x70,0x79,0x34, 0xFA,0x30,0x71,0x23, 0xF3,0x7B,0x30,0x24, 
	0xFB,0x7B,0xE0,0x15, 0x22,0x4B,0x22,0xF1, 0x13,0x27,0x42,0x22, 
	0xF1,0xB8,0x22,0xF0, 0xFB,0x03,0xA3,0x04, 0x31,0x0C,0x53,0x12, 
	0x67,0x1F,0x9F,0x22, 0x45,0x37,0xEF,0x3E, 0x8D,0x46,0x61,0x66, 
	0x00,0x60,0xBC,0x76, 0x23,0xF4,0xD2,0x67, 0x21,0x28,0x41,0x75, 
	0xE0,0x76,0x60,0x8E, 0xD1,0x76,0x60,0x8F, 0xD3,0x75,0x75,0x51, 
	0x71,0x23,0xF4,0x24, 0xFA,0x62,0x2F,0x26, 0x40,0x75,0xE0,0x67, 
	0x21,0x24,0x40,0x75, 0xE1,0x26,0x20,0x28, 0x41,0x76,0x60,0x8D, 
	0x75,0xFA,0xE0,0x69, 0x27,0x4F,0x76,0x60, 0x8E,0x75,0xFA,0xE0, 
	0x75,0xF6,0xD0,0xD0, 0x72,0x70,0xFC,0x60, 0x88,0x77,0x30,0x24, 
	0xFB,0x77,0xE0,0xB4, 0x22,0xF0,0x60,0xBE, 0x74,0x73,0x34,0xE0, 
	0x73,0x35,0x60,0x8F, 0xD0,0x73,0x35,0x60, 0x8F,0xD1,0x73,0x34, 
	0x73,0x34,0x51,0x70, 0x23,0xF4,0x24,0xFA, 0x40,0xD0,0x73,0x35, 
	0xD1,0x71,0x21,0xA3, 0x70,0x73,0x34,0xFA, 0x30,0x74,0x23,0xF3, 
	0x75,0x30,0x24,0xFB, 0x75,0xE0,0x10,0x21, 0x43,0x22,0xF1,0xB2, 
	0x22,0xF0,0x43,0x71, 0x30,0x24,0xF6,0xD3, 0x73,0xA3,0x73,0xC3, 
	0xA8,0x71,0x30,0x41, 0x24,0xF0,0x71,0xE0, 0x0B,0x71,0x30,0x41, 
	0x24,0xF0,0x24,0xF2, 0x24,0xFB,0x71,0xE0, 0x22,0xF0 },

	"TA",	/* target */
        "Undefined",	/* mode */
	1378,	/* code size */
	1,	/* offset */
	111,	/* workspace */
	0,	/* vectorspace */
	2,	/* bytes per word */
	1600	/* total code requirement not including parameter space  */

};	/* end of generated code */


struct mtest32_struct {
	unsigned char Code[1372];
	char Target[10];
	char Mode[10];
        long CodeSize;
        long Offset;
        long WorkSpace;
        long VectorSpace;
        int  BytesPerWord;
        int  TotalSpace;
};

struct mtest32_struct mtest32_code = {

    {			/* code */
	0x20,0x20,0x20,0x60, 0xB2,0x22,0x24,0x4A, 0x21,0xFB,0xD5,0x24, 
	0xF2,0x21,0xD1,0x24, 0xF2,0xDA,0x24,0xF2, 0x51,0xDB,0x24,0xF2, 
	0x52,0xDC,0x24,0xF2, 0x53,0xDD,0x24,0xF2, 0x54,0xD6,0x24,0xF2, 
	0x55,0xD7,0x24,0xF2, 0x56,0xD8,0x24,0xF2, 0x57,0xD9,0x7F,0x16, 
	0xFA,0x30,0xD3,0x24, 0xF2,0xD4,0x24,0xF2, 0xD2,0x42,0xD1,0x22, 
	0x20,0x4D,0x21,0xFB, 0xD0,0x2B,0x44,0x21, 0xFB,0x60,0xDD,0x60, 
	0x1E,0x23,0xF9,0x63, 0xBB,0x40,0xD9,0x10, 0x23,0x19,0x42,0xF7, 
	0x13,0x23,0x19,0x21, 0x40,0xF7,0x13,0x82, 0xF1,0xD9,0x13,0x51, 
	0xD1,0x71,0x30,0xD8, 0x13,0x51,0xD2,0x13, 0x52,0xDA,0x13,0x53, 
	0xD7,0x40,0x72,0xE0, 0x40,0x13,0x83,0x23, 0xFB,0x40,0x77,0xE0, 
	0x13,0x83,0xF1,0xC0, 0x27,0xA9,0x79,0x13, 0x82,0x23,0xFB,0x72, 
	0x30,0x24,0xAE,0x13, 0xD0,0x72,0x30,0x72, 0x30,0x24,0xF2,0xFA, 
	0x22,0x20,0x40,0xF0, 0x22,0x22,0x9D,0x22, 0x20,0x40,0xD1,0x72, 
	0x30,0x71,0xF9,0x21, 0xA0,0x71,0x24,0xF2, 0xFA,0x30,0x71,0xF4, 
	0xC0,0xA7,0x71,0x22, 0x20,0x80,0xD1,0x61, 0x0A,0x77,0x30,0xA7, 
	0x41,0x13,0x83,0x23, 0xFB,0x21,0x0A,0x71, 0x72,0x30,0xF4,0xA7, 
	0x2F,0x4F,0x13,0x83, 0x23,0xFB,0x0E,0x78, 0x72,0x30,0x22,0x20, 
	0x80,0xF9,0xC0,0xA5, 0x42,0x13,0x83,0x23, 0xFB,0x23,0x17,0x21, 
	0x40,0xFF,0x7A,0xD0, 0x22,0x20,0x40,0x72, 0x30,0x24,0xF2,0xFA, 
	0x23,0x15,0x21,0x2B, 0x98,0x13,0x23,0x17, 0x21,0x40,0xFB,0x72, 
	0x30,0x28,0x50,0x72, 0xE0,0x67,0x01,0x6A, 0x06,0x43,0xD1,0x21, 
	0x24,0x45,0x21,0xFB, 0xD0,0x21,0x41,0x60, 0x18,0xFD,0x2B,0x48, 
	0x61,0x15,0xFD,0x60, 0xBF,0x22,0xF2,0xD0, 0x10,0x21,0x14,0x44, 
	0xFB,0x60,0x06,0x40, 0x11,0x81,0x23,0xFB, 0x21,0x22,0x4F,0x21, 
	0xFB,0x12,0x44,0x24, 0xFA,0x12,0x7D,0x42, 0xF7,0x72,0x2F,0x2F, 
	0x2F,0xCF,0x27,0xA3, 0x11,0x7D,0x41,0xF7, 0x11,0xF1,0xC0,0x24, 
	0xA4,0x12,0x7D,0x42, 0xF7,0x11,0x82,0x7D, 0x41,0xF7,0x72,0x60, 
	0x8F,0xD2,0x40,0x11, 0x81,0x23,0xFB,0x72, 0xC0,0x21,0xA1,0x11, 
	0x82,0xF1,0x2F,0xCF, 0xA4,0x21,0x2F,0xFF, 0x05,0x41,0x11,0x83, 
	0x23,0xFB,0x21,0x0B, 0x11,0x82,0xF1,0x21, 0x14,0xFA,0x30,0xD3, 
	0x40,0x11,0x83,0x23, 0xFB,0x7F,0x73,0x42, 0xFB,0x73,0x40,0xFE, 
	0x40,0xD0,0x12,0x73, 0x7D,0x2D,0x9E,0x22, 0x04,0x41,0x11,0x83, 
	0xF1,0xF4,0x21,0xA1, 0x11,0x82,0xF1,0x21, 0x14,0xFA,0x30,0xD0, 
	0x7F,0x70,0x42,0xFB, 0x11,0x70,0x41,0xFB, 0x0C,0x11,0xF1,0xC2, 
	0xA6,0x41,0x11,0x81, 0x23,0xFB,0x02,0x21, 0xF0,0x21,0x0E,0x11, 
	0x82,0xF1,0xC4,0xAA, 0x11,0x81,0xF1,0xD0, 0x12,0x1E,0x7D,0x2A, 
	0x98,0x0F,0x11,0x81, 0xF1,0xD0,0x12,0x11, 0x82,0xF1,0x21,0x14, 
	0xFA,0x30,0x7D,0x29, 0x98,0x69,0x02,0x28, 0x49,0x21,0xFB,0x14, 
	0x44,0x24,0xFA,0x24, 0xF3,0x41,0xD2,0x43, 0xD3,0x22,0x7C,0x72, 
	0xF5,0x44,0x21,0xFF, 0x22,0x13,0xFA,0x30, 0xD1,0x71,0x41,0x24, 
	0xF8,0x12,0x21,0x45, 0x22,0xF1,0x21,0x1F, 0x41,0x24,0xF8,0x24, 
	0xF4,0x41,0xD2,0x43, 0xD3,0x22,0x7C,0x72, 0xF5,0x44,0x21,0xFF, 
	0x22,0x13,0xFA,0x30, 0xD1,0x71,0x41,0x40, 0x22,0xFF,0xA2,0x72, 
	0xD5,0x12,0x21,0x49, 0x22,0xF1,0x21,0x1F, 0x41,0x21,0x4E,0x22, 
	0xFF,0x24,0xF5,0x22, 0x7C,0x75,0xF5,0x44, 0x21,0xFF,0x22,0x13, 
	0xFA,0x30,0xD1,0x14, 0x71,0x42,0xF7,0x40, 0xD0,0x14,0x22,0x7C, 
	0x22,0x17,0xFA,0x30, 0x71,0x22,0x9A,0x21, 0x02,0x14,0x21,0x1F, 
	0x44,0xF7,0x40,0xD0, 0x14,0x22,0x7C,0x22, 0x17,0xFA,0x30,0x21, 
	0x1F,0x21,0x96,0x68, 0x0E,0x10,0x81,0x23, 0xF9,0x21,0xF5,0x12, 
	0xF3,0xBE,0x22,0xF0, 0x00,0x00,0x00,0x00, 0xFF,0xFF,0xFF,0xFF, 
	0x00,0x60,0xBA,0x41, 0x7A,0xF4,0xAD,0x79, 0x30,0x28,0x20,0x20, 
	0x40,0x25,0xF6,0xD0, 0x10,0x78,0x42,0xFB, 0x79,0x30,0x40,0xF9, 
	0x22,0xAC,0x79,0x30, 0x21,0x44,0xF9,0x21, 0xA2,0x11,0x77,0x21, 
	0x44,0xF7,0x11,0x78, 0x21,0x44,0xFB,0x79, 0x30,0x61,0x8C,0x79, 
	0xE0,0x21,0x01,0x79, 0x30,0xD0,0x11,0x77, 0x70,0xF7,0x79,0x30, 
	0xD0,0x11,0x78,0x70, 0xFB,0x40,0x79,0xE0, 0x63,0x0E,0xB6,0x22, 
	0xF0,0x60,0xBE,0x10, 0x73,0x21,0x51,0x44, 0xF7,0x74,0x74,0x75, 
	0x23,0xF4,0x24,0xFA, 0x10,0x73,0x21,0x51, 0x44,0xF7,0x11,0x73, 
	0x21,0x51,0x44,0xF7, 0x71,0x70,0xF4,0x76, 0xE0,0xB2,0x22,0xF0, 
	0x60,0xBD,0x77,0x53, 0xD1,0x71,0x43,0x24, 0xF6,0x41,0x21,0xF3, 
	0x22,0xF2,0xD2,0x77, 0x82,0xF1,0xD0,0x45, 0x70,0xF9,0x24,0xAE, 
	0x71,0x30,0xC0,0x24, 0xA9,0x70,0x42,0xF9, 0xAA,0x43,0x70,0xF4, 
	0x22,0xAF,0x70,0xC4, 0xAE,0x23,0x01,0x70, 0xAC,0x41,0x70,0xF4, 
	0x21,0xA6,0x70,0xC2, 0xA2,0x21,0x08,0x21, 0xF0,0x71,0x60,0x4F, 
	0x10,0x22,0x20,0x90, 0x71,0x40,0x10,0x21, 0x2F,0x9A,0x21,0x09, 
	0x71,0x12,0x10,0x25, 0x9B,0x21,0x02,0x71, 0x12,0x10,0x2E,0x97, 
	0x0C,0x71,0x75,0x74, 0x21,0x29,0x9C,0x05, 0x71,0x76,0x10,0x21, 
	0x92,0x70,0x81,0xD0, 0x65,0x0D,0x70,0x21, 0x20,0x40,0x21,0xF3, 
	0x77,0x82,0x23,0xFB, 0xB3,0x22,0xF0,0x60, 0xBE,0x40,0xD0,0x73, 
	0x35,0xD1,0x71,0xAC, 0x74,0x70,0xF5,0x70, 0x73,0x34,0xFA,0xE0, 
	0x10,0x4C,0x22,0xF1, 0x40,0xD0,0x73,0x35, 0xD1,0x71,0x21,0xA5, 
	0x74,0x70,0xF5,0x70, 0x73,0x34,0xFA,0x30, 0x23,0xF3,0x75,0x30, 
	0x24,0xFB,0x75,0xE0, 0x10,0x21,0x45,0x22, 0xF1,0xB2,0x22,0xF0, 
	0x60,0xB6,0x7B,0x35, 0x23,0xF4,0xD6,0x7B, 0x34,0xD2,0x40,0xD1, 
	0x7C,0x21,0x2C,0x90, 0x40,0xD4,0x42,0xD5, 0x74,0xC0,0xA4,0x7C, 
	0x30,0xD9,0x03,0x79, 0x7C,0xE0,0x4F,0x7C, 0x30,0x24,0xF6,0xD3, 
	0x4E,0x73,0xF9,0xC0, 0xA7,0x21,0x28,0x2A, 0x2F,0x47,0xD3,0x08, 
	0x73,0x2D,0x4F,0x21, 0xFB,0xFA,0x30,0xD3, 0x7C,0x21,0x29,0x94, 
	0x24,0xF2,0x23,0xF2, 0x7C,0x30,0x24,0xF6, 0x7B,0x35,0x21,0xFF, 
	0xD0,0x40,0xD7,0x7B, 0x35,0x42,0x22,0xFC, 0xD8,0x78,0x24,0xF2, 
	0x21,0xF3,0x23,0xA3, 0x70,0x73,0xF5,0x76, 0x21,0xFF,0xD0,0x7C, 
	0x21,0x26,0x9D,0x2F, 0x4F,0x7C,0x30,0x24, 0xF6,0x21,0x20,0x40, 
	0x21,0xF3,0xD1,0x74, 0xC0,0xA7,0x71,0x70, 0x72,0xF2,0x23,0xFB, 
	0x0D,0x70,0x72,0xF2, 0xF1,0x71,0x23,0xF3, 0x7D,0x30,0x24,0xFB, 
	0x7D,0xE0,0x17,0x23, 0x43,0x22,0xF1,0x14, 0x27,0x4C,0x22,0xF1, 
	0xBA,0x22,0xF0,0x60, 0xB8,0x7A,0x21,0x23, 0x97,0x40,0xD3,0x42, 
	0xD4,0x73,0xC0,0xA4, 0x7A,0x30,0xD7,0x03, 0x77,0x7A,0xE0,0x4F, 
	0x7A,0x30,0x24,0xF6, 0xD2,0x4E,0x72,0xF9, 0xC0,0xA7,0x21,0x28, 
	0x2A,0x2F,0x47,0xD2, 0x08,0x72,0x25,0x46, 0x21,0xFB,0xFA,0x30, 
	0xD2,0x7A,0x21,0x20, 0x9B,0x24,0xF2,0x23, 0xF2,0x7A,0x30,0x24, 
	0xF6,0x79,0x35,0x21, 0xFF,0xD0,0x40,0xD5, 0x79,0x35,0x42,0x22, 
	0xFC,0xD6,0x76,0x24, 0xF2,0x21,0xF3,0x22, 0xAB,0x70,0x72,0xF5, 
	0x79,0x35,0x21,0xFF, 0xD0,0x7A,0x2E,0x94, 0x7A,0x30,0xD1,0x73, 
	0xC0,0xA7,0x71,0x70, 0x79,0x34,0xFA,0xE0, 0x0E,0x70,0x79,0x34, 
	0xFA,0x30,0x71,0x23, 0xF3,0x7B,0x30,0x24, 0xFB,0x7B,0xE0,0x15, 
	0x22,0x4B,0x22,0xF1, 0x13,0x27,0x44,0x22, 0xF1,0xB8,0x22,0xF0, 
	0xFB,0x03,0x00,0x00, 0xA3,0x04,0x00,0x00, 0x31,0x0C,0x00,0x00, 
	0x53,0x12,0x00,0x00, 0x67,0x1F,0x00,0x00, 0x9F,0x22,0x00,0x00, 
	0x45,0x37,0x00,0x00, 0xEF,0x3E,0x00,0x00, 0x8D,0x46,0x00,0x00, 
	0x61,0x66,0x00,0x00, 0xA5,0xAD,0x00,0x00, 0x6B,0x19,0x01,0x00, 
	0xF7,0x8A,0x01,0x00, 0xA3,0x86,0x01,0x00, 0x00,0x00,0x00,0x60, 
	0xBC,0x23,0x47,0x21, 0xFB,0xD0,0x70,0x32, 0x75,0xE0,0x76,0x60, 
	0x8E,0xD2,0x76,0x60, 0x8F,0xD3,0x75,0x75, 0x51,0x72,0x23,0xF4, 
	0x24,0xFA,0x70,0x31, 0x75,0xE0,0x70,0x30, 0x76,0x60,0x8D,0x75, 
	0xFA,0xE0,0x75,0xF6, 0xD1,0xD1,0x76,0x23, 0xF4,0xD2,0x72,0x71, 
	0xFC,0x60,0x82,0x77, 0x30,0x24,0xFB,0x77, 0xE0,0xB4,0x22,0xF0, 
	0x81,0x60,0x7F,0xF6, 0x60,0xDF,0x40,0x81, 0x81,0x81,0x81,0x81, 
	0x60,0xBE,0x74,0x73, 0x34,0xE0,0x73,0x35, 0x60,0x8F,0xD0,0x73, 
	0x35,0x60,0x8F,0xD1, 0x73,0x34,0x73,0x34, 0x51,0x70,0x23,0xF4, 
	0x24,0xFA,0x40,0xD0, 0x73,0x35,0xD1,0x71, 0x21,0xA3,0x70,0x73, 
	0x34,0xFA,0x30,0x74, 0x23,0xF3,0x75,0x30, 0x24,0xFB,0x75,0xE0, 
	0x10,0x21,0x43,0x22, 0xF1,0xB2,0x22,0xF0, 0x43,0x71,0x30,0x24, 
	0xF6,0xD3,0x73,0xA3, 0x73,0xC3,0xA8,0x71, 0x30,0x41,0x24,0xF0, 
	0x71,0xE0,0x0B,0x71, 0x30,0x41,0x24,0xF0, 0x24,0xF2,0x24,0xFB, 
	0x71,0xE0,0x22,0xF0 },

	"TA",	/* target */
        "Undefined",	/* mode */
	1372,	/* code size */
	3,	/* offset */
	94,	/* workspace */
	0,	/* vectorspace */
	4,	/* bytes per word */
	1748	/* total code requirement not including parameter space  */

};	/* end of generated code */

#endif

#ifdef __cplusplus
}
#endif

#endif /*!_tcomChk_h*/
