#ifndef _tcomCmd_h
#define _tcomCmd_h
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tcomCmd.h,v 1.104 2005/09/12 09:21:30 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  03/01/98  created
*/

/************************************************************************
 *
 *  tcomCmd.h  tcom command constants
 *
 *----------------------------------------------------------------------
 */

#include "tcomInt.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 *  command arguments
 */
#define tcomK_ENABLE  "enable"
#define tcomK_DISABLE "disable"

#define tcomFAILURE -1

/*
 *  interactive mode status structure
 */
typedef struct
{
  int  linkId;
  char comType[256];
  char srvName[256];
  char active;
  char connected;
  char booted;
  char protectedMode;
  char forceCommand;
} tcomIM_STAT;

#define tcomCMD_UNPROTECTED 0
#define tcomCMD_PROTECTED   1

/*
 *  command handler
 */
typedef int (*tcomCMD_HANDLER)(tcomIM_STAT *, char *, FILE *, char *);

/*
 *  command structure
 */
typedef struct
{
  const char      *cmd;
  const char      *syntax;
  const char      *help;
  const char       protected;
  tcomCMD_HANDLER  cmdHandler;
} tcomIM_CMD;

/*
 *  command list
 */
extern tcomIM_CMD tcomSclImCmdAbort;
extern tcomIM_CMD tcomSclImCmdExit;
extern tcomIM_CMD tcomSclImCmdHelp;
extern tcomIM_CMD tcomSclImCmdCldcNum;
extern tcomIM_CMD tcomSclImCmdDac;
extern tcomIM_CMD tcomSclImCmdTemp;
extern tcomIM_CMD tcomSclImCmdRstat;
extern tcomIM_CMD tcomSclImCmdWstat;
extern tcomIM_CMD tcomSclImCmdEstat;
extern tcomIM_CMD tcomSclImCmdSync;
extern tcomIM_CMD tcomSclImCmdPstat;
extern tcomIM_CMD tcomSclImCmdIconf;
extern tcomIM_CMD tcomSclImCmdPset;
extern tcomIM_CMD tcomSclImCmdPget;
extern tcomIM_CMD tcomSclImCmdTel;
extern tcomIM_CMD tcomSclImCmdVset;
extern tcomIM_CMD tcomSclImCmdFset;
extern tcomIM_CMD tcomSclImCmdCset;
extern tcomIM_CMD tcomSclImCmdSeqCheck;
extern tcomIM_CMD tcomSclImCmdFoTest;
extern tcomIM_CMD tcomSclImCmdLdv;
extern tcomIM_CMD tcomSclImCmdLdSeq;
extern tcomIM_CMD tcomSclImCmdLdClk;
extern tcomIM_CMD tcomSclImCmdTcalc;
extern tcomIM_CMD tcomSclImCmdExec;
extern tcomIM_CMD tcomSclImCmdKill;
extern tcomIM_CMD tcomSclImCmdBoot;
extern tcomIM_CMD tcomSclImCmdConnect;
extern tcomIM_CMD tcomSclImCmdStat;
extern tcomIM_CMD tcomSclImCmdStart;
extern tcomIM_CMD tcomSclImCmdStop;
extern tcomIM_CMD tcomSclImCmdEnd;
extern tcomIM_CMD tcomSclImCmdMode;
extern tcomIM_CMD tcomSclImCmdTrigger;
extern tcomIM_CMD tcomSclImCmdSrvStat;
extern tcomIM_CMD tcomSclImCmdForceCmd;
extern tcomIM_CMD tcomSclImCmdProtectCmd;
extern tcomIM_CMD tcomSclImCmdTclose;
extern tcomIM_CMD tcomSclImCmdSrvAbort;
extern tcomIM_CMD tcomSclImCmdSv;
extern tcomIM_CMD tcomSclImCmdPsync;
extern tcomIM_CMD tcomSclImCmdReset;
extern tcomIM_CMD tcomSclImCmdTrim;
extern tcomIM_CMD tcomSclImCmdEcho;
extern tcomIM_CMD tcomSclImCmdCldcTest;
extern tcomIM_CMD tcomSclImCmdAdcTest;
extern tcomIM_CMD tcomSclImCmdAscii;
extern tcomIM_CMD tcomSclImCmdTmo;

/*
 *  command handler protoypes
 */
int  tcomSclHandleAbort(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleExit(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleHelp(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleCldcNum(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleDac(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleTemp(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleRstat(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleWstat(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleEstat(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleSync(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandlePstat(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandlePset(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandlePget(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleIconf(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleTel(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleVset(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleFset(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleCset(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleSeqCheck(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleFoTest(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleLdv(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleSv(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleLdSeq(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleLdClk(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleTcalc(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleExec(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleKill(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleBoot(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleConnect(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleStatus(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleStart(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleStop(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleEnd(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleMode(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleTrigger(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleSrvStat(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleForceCmd(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleProtectCmd(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleTclose(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleSrvAbort(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandlePsync(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleReset(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleTrim(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleEcho(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleCldcTest(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleAdcTest(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleAscii(tcomIM_STAT *, char *, FILE *, char *);
int  tcomSclHandleTmo(tcomIM_STAT *, char *, FILE *, char *);

extern tcomIM_CMD *tcomSclCmdList[];
extern int tcomSclCmdListSize;
extern int tcomSclTrim(int, int, char *, int, int, int, float, float,
			float *, char *);

/*
 *  protoypes
 */
int      tcomSclHelp(FILE *, char *str);
void     tcomSclInteractive(void *);

/*
 *  line editor prototype
 */
typedef int (*tcomFUNC_LEDIT)(const char *, char *, char *);

extern tcomFUNC_LEDIT tcomLineEditor;


#ifdef __cplusplus
}
#endif

#endif /*!_tcomCmd_h*/
