#ifndef _tcomInt_h
#define _tcomInt_h
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tcomInt.h,v 1.104 2005/09/12 09:21:30 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  19/09/96  created
* jstegmei  02/02/98  added tcomSclNumCldc
* jstegmei  04/03/98  structure tcomPAT needs no longer be initilaized
*                     (states have static size tcomISEQ_MAXPATLEN)
* jstegmei  04/03/98  removed tcomSclInitPat()
* jstegmei  12/03/98  added tcomSclTrigger(), tcomSclSetMode()
* jstegmei  14/04/98  added tcomSclParseSSD
* jstegmei  05/05/98  added tcomSRV_COM_STAT
*                     added tcomCONF structure
*                     added tcomSclSrvStat()
*                     added tcomSclSrvAbort()
* jstegmei  18/05/98  added server process-id to config structure
* jstegmei  29/06/98  added global tcomSclTimeoutValue
* jstegmei  14/10/98  increased tcomISEQ_MAXPAT to 256 for revA
* jstegmei  14/10/98  added tcomSclTempEnable/Disable
* jstegmei  21/10/98  added tcomSclNumStat()
*/

/************************************************************************
 *
 *  tcomInt.h  transputer command interface constants
 *
 *----------------------------------------------------------------------
 */


#include <stdio.h>

#include "icmd.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 *  environments
 */
#define tcomENV_SCL_SIM_PORT "TCOM_SCL_SIM"
#define tcomENV_SRV_PORT     "TCOM_SRV"
#define tcomENV_SCL_DEV      "TCOM_SCL_DEV"

/*
 *  default ports
 */
#define tcomSCL_SIM_PORT  8006
#define tcomSRV_PORT      8008

/*
 *  verbose
 */ 
#define tcomVB( STR ) if ( tcomVflag ) printf STR

/*
 *  buffer sizes for transputer command interpreter
 */
#define tcomCI_MAX_COMBUFSIZE   8192         /* bytes */


/*
 *  default timeout (in seconds)
 */  
#define tcomDEFAULT_TIMEOUT    5
#define tcomCONNECT_TIMEOUT   20
#define tcomMAX_TIMEOUT      100

/*
 *  maximum number of server connections
 */
#define tcomSRV_MAX_CON 4

/*
 *  message id
 */
#define tcomMES_COMMAND   1
#define tcomMES_DATA      2
#define tcomMES_BACKREPLY 3
#define tcomMES_MESSAGE   4
#define tcomMES_BOOT      5

/*
 *  irace sequencer and cldc commands
 */
#define tcomSCL_COM_BOOT              -1  /* scl boot service                */
#define tcomSCL_COM_RESET             -2  /* scl reset service               */
#define tcomSCL_COM_TIMEOUT           -3  /* scl timeout service             */
#define tcomSCL_COM_CLOSE             -4  /* scl close service               */
#define tcomSCL_COM_CONNECT           -5  /* scl reopen service              */

#define tcomSRV_COM_STAT              10  /* tcom server status              */
#define tcomSRV_COM_ABORT             11  /* abort tcom server               */
#define tcomSRV_COM_ASCII             12  /* ascii command abort tcom server */

#define tcomCI_COM_GET_CONFIG         51  /* get configuration               */
#define tcomCI_COM_ECHO               52  /* echo service                    */

/*
 *  irace sequencer commands
 */
#define tcomISEQ_COM_START           101  /* start loaded sequence           */
#define tcomISEQ_COM_STOP            102  /* stop sequence                   */
#define tcomISEQ_COM_STATUS          103  /* get sequencer status            */
#define tcomISEQ_COM_SET_PAT         104  /* load subpatterns to sequencer   */
#define tcomISEQ_COM_PARAMS          105  /* load parameters to sequencer    */
#define tcomISEQ_COM_SEQ_PROG        106  /* load sequencer program          */
#define tcomISEQ_COM_READOUTTIMES    107  /* get detector readout times      */
#define tcomISEQ_COM_PARAM_STATUS    109  /* send parameter status           */
#define tcomISEQ_COM_SYNC            110  /* request frame sync signal       */
#define tcomISEQ_COM_SET_CLKP        111  /* load clkp to sequencer          */
#define tcomISEQ_COM_MODE            112  /* set sequencer mode              */
#define tcomISEQ_COM_GET_PAT         113  /* get subpatterns from sequencer  */
#define tcomISEQ_COM_GET_CLKP        114  /* get clkp from sequencer         */
#define tcomISEQ_COM_CLKP_PROC       115  /* load clkp process               */
#define tcomISEQ_COM_CLR_CLKP        116  /* clear clock pattern             */
#define tcomISEQ_COM_CLR_SUBPAT      117  /* clear subpattern setup          */
#define tcomISEQ_COM_TRIGGER         118  /* trigger synchronous start       */
#define tcomISEQ_COM_CLKP_LOOP       119  /* load clkp loop                  */

/* 
 *  irace cldc commands
 */
#define tcomCLDC_COM_CHANNEL         201  /* set voltage at a single channel */
#define tcomCLDC_COM_VOLTS           202  /* load voltage setup to CLDC      */
#define tcomCLDC_COM_ENABLE          203  /* enable  CLDC output             */
#define tcomCLDC_COM_DISABLE         204  /* disable CLDC output             */
#define tcomCLDC_COM_STATUS          206  /* get CLDC and ADC-FIFO status    */
#define tcomCLDC_COM_TEL_CHAN        207  /* select telemetry channel        */
#define tcomCLDC_COM_CONVERT         208  /* perform a number of converts    */
#define tcomCLDC_COM_READ_CHAN       210  /* read tel_channel (macro)        */
#define tcomCLDC_COM_READ_ALL        211  /* read all tel.channels (macro)   */
#define tcomCLDC_COM_TEMP_ENABLE     212  /* enable temp. measurement        */
#define tcomCLDC_COM_TEMP_DISABLE    213  /* disable temp. measurement       */

/*
 *  irace status handler commands
 */
#define tcomISH_COM_RSTAT            701  /* hw-status read request          */
#define tcomISH_COM_WSTAT            702  /* hw-status write request         */
#define tcomISH_COM_ESTAT            703  /* hw-status error request         */
#define tcomISH_COM_SYNC             705  /* wait for sync                   */
#define tcomISH_COM_NSTAT            706  /* get number of status processes  */

/*
 *  common commands
 */
#define tcomNOP                   0  /* no operation                      */
#define tcomABORT               999  /* abort command interpreter         */

/*
 *  tcom error id
 */
#define tcomERR_NO_ERR                     0  /* good                        */
#define tcomERR_IO_ERROR                  -1  /* i/o error                   */
#define tcomERR_OUT_OF_RANGE              -2  /* out of range                */
#define tcomERR_OPEN_ERR                  -3  /* open error                  */
#define tcomERR_SYS_ERR                   -4  /* system error                */
#define tcomERR_BOOT_ERR                  -5  /* boot error                  */

/*
 *  transputer command interpreter error id
 */
#define tcomCI_ERR_NO_VALID_COMMAND       51  /* no valid command            */
#define tcomCI_ERR_WRONG_MES_SIZE         52  /* wrong message size          */
#define tcomCI_ERR_OUT_OF_RANGE           53  /* ci out of range             */
#define tcomCI_ERR_NOT_IMPLEMENTED        54  /* not implemented             */

/*
 *  irace sequencer error id
 */
#define tcomISEQ_ERR_OUT_OF_RANGE        101 /* iseq out of range            */
#define tcomISEQ_ERR_NOT_STARTED         102 /* iseq is not started          */
#define tcomISEQ_ERR_STARTED             103 /* iseq is already started      */
#define tcomISEQ_ERR_NOT_READY           104 /* iseq not ready               */
#define tcomISEQ_ERR_NOT_LOADED          105 /* iseq not loaded              */
#define tcomISEQ_ERR_NOT_INITIALIZED     106 /* iseq not initialized         */
#define tcomISEQ_ERR_LOOP_NO_END         107 /* iseq loop has no end         */
#define tcomISEQ_ERR_LOOP_NOT_CLOSED     108 /* iseq loop not closed         */
#define tcomISEQ_ERR_LOOP_PARSER         109 /* iseq loop parse error        */
#define tcomISEQ_ERR_LOOP_OVERFLOW       110 /* iseq loop overflow           */
#define tcomISEQ_ERR_LOOP_STACK_OVERFLOW 111 /* iseq loop stack overflow     */
#define tcomISEQ_ERR_ALLOCATION_FAILURE  112 /* iseq allocation failure      */
#define tcomISEQ_ERR_TIMEOUT             113 /* iseq fifo timeout            */
#define tcomISEQ_ERR_FIFO_OVERRUN        114 /* iseq fifo overrun            */
#define tcomISEQ_ERR_NOT_WAITING         115 /* iseq not in waiting state    */
#define tcomISEQ_ERR_NOT_RUNNING         116 /* iseq not in running state    */
#define tcomCLDC_ERR_OUT_OF_RANGE        201 /* cldc out of range            */
#define tcomCLDC_ERR_NO_VALID_DATA       202 /* no valid telemetry data      */
#define tcomISH_ERR_OUT_OF_RANGE         701 /* hw-status value out of range */
#define tcomISH_ERR_INVALID              702 /* no valid module              */
#define tcomISH_ERR_GENERAL              703 /* hw-status bus error          */
#define tcomISH_ERR_TIMEOUT              704 /* hw-status sync timeout       */

/*
 *  transputer board id
 */
#define tcomID_ISEQ_A          0x0A01      /* irace sequencer revision A   */
#define tcomID_ISEQ_3          0x0101      /* irace seqeuncer revision 3   */
#define tcomID_CLDC_C          0x0C02      /* irace cldc revision A        */
#define tcomID_CLDC_3          0x0102      /* irace cldc revision 3        */
#define tcomID_ISH             0x0003      /* irace status bus handler     */
#define tcomID_SCSI_TARGET     0x0011      /* nc scsi target module        */
#define tcomID_QT9_DISPLAY     0x0012      /* nc qt9 display               */
#define tcomID_QT9_REMOTE      0x0013      /* nc qt9 display remote ctrl   */ 

/*
 *  irace sequencer masks
 */
#define tcomISEQ_RSPEED_MSK    0x0000007F  /* AND with lower 32 bits       */
#define tcomISEQ_ADDR_MSK_3    0x0000001f  /* AND with ptr to get address  */
#define tcomISEQ_ADDR_MSK_A    0x000000ff  /* AND with ptr to get address  */
#define tcomISEQ_REP_CLR_MSK   0xffffb07b  /* AND to clear rep.factor      */
#define tcomISEQ_RFAC_MSK      0x00003f00  
 
/*
 *  irace sequencer loop commands
 */
#define tcomISEQ_LOOP_START         -101   /* loop start                   */
#define tcomISEQ_LOOP_END           -102   /* loop end                     */ 
#define tcomISEQ_LOOP_COM_END       -103   /* loop command end             */
#define tcomISEQ_LOOP_SYNC          -104   /* loop sync                    */
#define tcomISEQ_LOOP_INFINITE        -1   /* infinite loop counter        */ 

/*
 *  irace sequencer states
 */
#define tcomISEQ_STAT_IDLE           1  /* idle                           */
#define tcomISEQ_STAT_INITIALIZED    2  /* subpattern pages initialized   */
#define tcomISEQ_STAT_LOADED         3  /* clock pattern(s) loaded        */
#define tcomISEQ_STAT_READY          4  /* setup is ready                 */
#define tcomISEQ_STAT_STOPPED        5  /* stopped                        */
#define tcomISEQ_STAT_RUNNING        6  /* running                        */
#define tcomISEQ_STAT_FAILURE        7  /* failure                        */
#define tcomISEQ_STAT_STARTING       8  /* starting fifo                  */
#define tcomISEQ_STAT_WAITING        9  /* waiting for async. start pulse */
 
/*
 *  irace sequencer modes
 */
#define tcomISEQ_MOD_CLK       0    /* normal running mode        */
#define tcomISEQ_MOD_TRIG_EXT  1    /* external trigger           */
#define tcomISEQ_MOD_TRIG_COM  2    /* trigger via command link   */

/*
 *  irace cldc states
 */
#define tcomCLDC_STAT_ENABLED        1  /* dac output enabled             */ 
#define tcomCLDC_STAT_DISABLED       2  /* dac output disabled            */

/*
 *  irace sequencer clock
 */
#define tcomISEQ_CLK  50.0e-6 

/*
 *  irace sequencer readspeed change permission
 */
#define tcomISEQ_CHANGE_ON   1   /* readspeed of state can be changed   */
#define tcomISEQ_CHANGE_OFF  0   /* readspeed of state is locked        */

/*
 *  irace sequencer maximum number of clock patterns
 */
#define tcomISEQ_MAX_CLKP    128

/*
 *  irace sequencer maximum repetition factor for subpattern
 */
#define tcomISEQ_MAX_REP_FACTOR      63

/*
 *  irace sequencer maximum readspeed
 */
#define tcomISEQ_MAX_RSPEED         127
#define tcomISEQ_MIN_RSPEED           0

/*
 *  subpattern limits for irace sequencer
 */
#define tcomISEQ_MAXPAT                256
#define tcomISEQ_MAXPAT_REV3            32
#define tcomISEQ_MAXPATLEN             128
#define tcomISEQ_MAX_CLK                48

/*
 *  irace sequencer maximum length of loop 
 *  command line (integer elements)
 */
#define tcomISEQ_MAX_LOOP_COM_LEN   1024

/*
 *  irace cldc constants
 */
#define tcomCLDC_NUM_TEL_CHANNELS       64 
#define tcomCLDC_NUM_DAC_CHANNELS       48 
#define tcomCLDC_MAX_CLK_CHAN           16 
#define tcomCLDC_MAX_DC_CHAN            16 
#define tcomCLDC_ADC_ZERO                2.048e+3 
#define tcomCLDC_ADC_RANGE               1.0e+1
#define tcomCLDC_ADC_CORR                1.0 
#define tcomCLDC_DAC_ZERO                2.048e+3 
#define tcomCLDC_DAC_RANGE               1.0e+1
#define tcomCLDC_DAC_CORR                1.0 

/*
 *  irace sequencer subpattern state type
 */
typedef struct
{
  int  w1;
  int  w2;
  int  rspeed;
  char perm;
} tcomPATSTATE;

/*
 *  irace sequencer subpattern type
 */
typedef struct
{
  int           rfac;
  int           len;
  tcomPATSTATE  state[tcomISEQ_MAXPATLEN];
} tcomPAT;
typedef struct
{
  char      name[256];
  tcomPAT   pattern;
} tcomPAT_OBJ;

/*
 *  irace cldc clock voltage type
 */
typedef struct
{
  char      nameHi[256];
  char      nameLo[256];
  float     valueHi;
  float     valueLo;
  float     range1Hi;
  float     range2Hi;
  float     range1Lo;
  float     range2Lo;
  int       dacHi;
  int       dacLo;
  int       telHi;
  int       telLo;
} tcomCLK_OBJ;

/*
 *  irace cldc dc voltage type
 */
typedef struct
{
  char      name[256];
  float     value;
  float     range1;
  float     range2;
  int       dac;
  int       tel;
} tcomDC_OBJ;

/*
 *  irace sequencer frame type
 */
typedef struct
{
  char      name[256];
  int       clkpnr;
  int       value;
  float     time;
  int       pos[tcomISEQ_MAX_LOOP_COM_LEN];
} tcomFRAME_OBJ;

/*
 *  irace sequencer clock pattern type
 */
typedef struct
{
  char      file[256];
  char      lflag;
} tcomCLKP_OBJ;

/*
 *  expressions
 */
typedef struct
{
  char      expr[256];
  int       pos;
  char      name[64];
} tcomEXPR;

#define tcomVAR_SSD  0x00000001
#define tcomVAR_SEQ  0x00000002
#define tcomVAR_FREE 0x00000004

/*
 *  maximum number of variables
 */
#define tcomMAX_VAR 64

/*
 *  system type
 */
typedef int (*tcomFUNC_C)(int, icmdCOM, char *);
typedef int (*tcomFUNC_R)(int, icmdREP *, char *);
typedef struct
{
    tcomFUNC_C   sendCommand; 
    tcomFUNC_R   getReply;
} tcomSYS;

/*
 *  configuaration (for get config command)
 */
typedef struct
{ 
  int    booted;                        /* booted state                   */
  int    liveTime;                      /* server live-time               */
  int    srvPid;                        /* server process-id              */
  int    sim;                           /* simulation mode if set to 1    */
  int    maxCmdClients;                 /* max. number of command clients */
  int    cmdClients;                    /* number of command clients      */
  char   cmdHost[tcomSRV_MAX_CON][256]; /* host info                      */
} tcomCONF;


/*
 *  irace sequencer & cldc system services prototypes
 */
int tcomSclComHandler (int, icmdCOM *, icmdREP *,
		       char *, char *, char *, char *);
int tcomSclSysOpen (char *, char *, char *);
int tcomSclSysClose (int);
int tcomSclBoot (int, char *, int, char *);
int tcomSclConnect (int *, char *, char *, char *);
int tcomSclAbort (int, char *);
int tcomSclReset (int, char *);
int tcomSclClose (int, char *);
int tcomSclTimeout (int, int, char *);
int tcomSclOpenComLine (char * , char *, int *, char *);
int tcomSclCloseComLine (int);
int tcomSclAscii(int, char *, char *, char *);

/*
 *  irace sequencer & cldc command function prototypes
 */
int  tcomSclSubpatSetup (char *, int, tcomPAT_OBJ *, int *, int *, char *);
void tcomSclPrintSubpat (FILE *, tcomPAT_OBJ);
int  tcomSclVoltSetup (char *, int , tcomCLK_OBJ *, tcomDC_OBJ *, char *);
int  tcomSclVoltSave (const char *, char *, int, tcomCLK_OBJ *, tcomDC_OBJ *,
		      char *);
void tcomSclPrintVoltSetup (FILE *, tcomCLK_OBJ *, tcomDC_OBJ *);
void tcomSclPrintTel (FILE *, float *, tcomCLK_OBJ *, tcomDC_OBJ *);
int  tcomSclFrameSetup (char *, int, tcomCLKP_OBJ *, int *, 
			tcomFRAME_OBJ *, int *, int *, int *,
			icmdVAR *, tcomEXPR *, int *, char *);
void tcomSclPrintFrameSetup (FILE *, tcomCLKP_OBJ *, tcomFRAME_OBJ *, 
			     int, int *, int);
void tcomSclPrintFrameSetupRaw (FILE *, int *, int);
int  tcomSclUpdateTimes (icmdLINK, icmdVAR *, tcomFRAME_OBJ *, int, char *);
void tcomSclChooseUpdate (icmdVAR *, char *, char *);
int  tcomSclUpdateSeq (icmdLINK, int *, int, icmdVAR  *, tcomEXPR *, 
		       int, char *);
int  tcomSclParseSSD (char *, int, int *, int *, icmdVAR *, char *);
int  tcomSclInitCom (int, int);
int  tcomSclSendTrCommand (int, icmdCOM, char *);
int  tcomSclGetTrReply (int, icmdREP *, char *);
int  tcomSclBinToInt (char *);
void tcomSclIntToBin (char *, int);
void tcomSclBinToState (tcomPATSTATE *, char *);
void tcomSclErr (char *, short);
int  tcomSclWaitInit (icmdLINK, char*);
int  tcomSclPing (icmdLINK, char *);
int  tcomSclEcho (icmdLINK, float *, char *);
int  tcomSclEcho2 (icmdLINK, char *, int, char *, char *);
int  tcomSclClrPat (icmdLINK, char*);
int  tcomSclClrClkp (icmdLINK, char*);
int  tcomSclSrvStat (icmdLINK, tcomCONF *, char*);
int  tcomSclStatSeq (icmdLINK, int *, char*);
int  tcomSclStatCldc (icmdLINK, int, int *, char*);
int  tcomSclSeqStart (icmdLINK, char *);
int  tcomSclSeqStop (icmdLINK, char *);
int  tcomSclSeqEnd (icmdLINK, char *);
int  tcomSclTrigger (icmdLINK, char *);
int  tcomSclSetMode (icmdLINK, int, char *);
int  tcomSclDacEnable (icmdLINK, int, char *);
int  tcomSclDacDisable (icmdLINK, int, char *);
int  tcomSclTempEnable (icmdLINK, int, char *);
int  tcomSclTempDisable (icmdLINK, int, char *);
int  tcomSclSetPat (icmdLINK, int, tcomPAT, char *);
int  tcomSclGetPat (icmdLINK, int, tcomPAT *, char *);
int  tcomSclSetClkp (icmdLINK, int, int *, int, char *);
int  tcomSclGetClkp (icmdLINK, int, int *, int *, char *);
int  tcomSclClkpProg (icmdLINK, int, char *, char *);
int  tcomSclClkpLoop (icmdLINK, int, int *, int, char *);
int  tcomSclSeqProg (icmdLINK, int *, int, char *);
int  tcomSclTime (icmdLINK, float *, int *, char *);
int  tcomSclSetSpeed (icmdLINK, int, int, char *);
int  tcomSclGetSpeed (icmdLINK, int *, int *, char *);
int  tcomSclSetVolt (icmdLINK, int, int, float, char *);
int  tcomSclSetVoltAll (icmdLINK, int, float *, char *);
int  tcomSclTelChan (icmdLINK, int, int, float *, char *);
int  tcomSclTelemetry (icmdLINK, int, float *, char *);
int  tcomSclPstat (icmdLINK, int *, int *, int *, int *, int *, char *);
int  tcomSclGetConf (icmdLINK, int *, int *, int *, char *);
int  tcomSclNumCldc (icmdLINK, int *, char *);
int  tcomSclReadHwStat (icmdLINK, int, int, int, int *, char *);
int  tcomSclWriteHwStat (icmdLINK, int, int, int, int, char *);
int  tcomSclErrStat (icmdLINK, int *, int *, char *);
int  tcomSclNumStat (icmdLINK, int *, char *);
int  tcomSclSyncStat (icmdLINK, int, int, int, int, int, char *);
int  tcomSclExec (const char *, char *, const char *, const char *, 
		  int, char , char, int *, char *);
int  tcomSclKill (const char *, int, char *);
int  tcomSclSrvAbort (icmdLINK, char *);
int  tcomSclCldcTest(icmdLINK, double, char *);
int  tcomSclAdcTest(icmdLINK, char *);

/*
 *  transputer systems
 */
extern tcomSYS tcomSysScl;
extern tcomSYS tcomSysNc;
extern int     tcomSclBooted;              /* booted - flag              */
extern int     tcomSclTimeoutValue;        /* current timeout in seconds */ 
extern char    tcomCommandServerName[128];
extern char    tcomSimulatorName[128];
extern int     tcomServerExt;

/*
 *  irace sequencer & cldc command model
 */
#define tcomSclSendCommand(l,c,e) tcomSysScl.sendCommand(l,c,e)
#define tcomSclGetReply(l,c,e)    tcomSysScl.getReply(l,c,e)

/* 
 *  global variables 
 */
extern int tcomVflag;

#ifdef __cplusplus
}
#endif

#endif /*!_tcomInt_h*/
