#ifndef _tcomSim_h
#define _tcomSim_h
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tcomSim.h,v 1.104 2005/09/12 09:21:30 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  03/01/98  created
* jstegmei  14/10/98  simulate additional telemtry channels for 
*                     temperature measurement
*/

/************************************************************************
 *
 *  tcomSim.h  tcom simulator constants
 *
 *----------------------------------------------------------------------
 */

#include "tcomInt.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 *  irace sequencer masks
 */
#define tcomISEQ_DISABLE_MSK     0x0000c000
#define tcomISEQ_ENABLE_REGS     0xffff07e0

  
/* 
 *  maximum number of status bus modules
 */
#define tcomISH_MAX_MOD 16
#define tcomISH_MAX_REG 16

/*
 *  cldc type
 */
#define tcomCLDC_NUM_TEL_CHANNELS_SIM (tcomCLDC_NUM_TEL_CHANNELS + 8)
typedef struct
{
  int    revision;
  short  status;
  short  dac[tcomCLDC_NUM_DAC_CHANNELS];
  short  tel[tcomCLDC_NUM_TEL_CHANNELS_SIM];
} tcomCLDC_T;

/*
 *  irace status handler module type
 */
typedef struct
{
  short   moduleId;
  short   val[tcomISH_MAX_REG];
} tcomISHMOD_T;

/*
 *  irace status handler board type
 */
typedef struct
{
  short         boardId;
  tcomISHMOD_T  module[tcomISH_MAX_MOD];
} tcomISHBOARD_T;

/*
 *  irace status handler type
 */
typedef struct
{
  tcomISHBOARD_T board[2];
} tcomISH_T;

/*
 *  subpattern type
 */
typedef struct
{
  int    len;
  int    rfac;
  char   perm[tcomISEQ_MAXPATLEN];
  int    w1[tcomISEQ_MAXPATLEN];
  int    w2[tcomISEQ_MAXPATLEN];
} tcomSUBPAT_T;

/*
 *  clkp type
 */
typedef struct
{
  int    len;
  int    alloc;
  int   *buf;
} tcomCLKP_T;

/*
 *  iseq type
 */
typedef struct
{
  int            revision;
  short          status;
  int            factor;
  int            add;
  int            factorOld;
  int            addOld;
  int            loopCmdLen;
  int            loopCmd[tcomISEQ_MAX_LOOP_COM_LEN];
  int            numSubPat;
  int            subPatIdx[tcomISEQ_MAXPAT];
  tcomSUBPAT_T   subPat[tcomISEQ_MAXPAT];
  int            numClkp;
  struct timeval timeStart;
  tcomCLKP_T     clkp[tcomISEQ_MAX_CLKP];
} tcomISEQ_T;

/*
 *  prototypes
 */
void tcomSclSim (void *);
int  tcomSclSimDacEnable (icmdCOM, icmdREP *, char *);
int  tcomSclSimDacDisable (icmdCOM, icmdREP *, char *);
int  tcomSclSimTempEnable (icmdCOM, icmdREP *, char *);
int  tcomSclSimTempDisable (icmdCOM, icmdREP *, char *);
int  tcomSclSimSetDacChan (icmdCOM, icmdREP *, char *);
int  tcomSclSimSetDacAll (icmdCOM, icmdREP *, char *);
int  tcomSclSimTelChan (icmdCOM, icmdREP *, char *);
int  tcomSclSimTelAll (icmdCOM, icmdREP *, char *);
int  tcomSclSimCldcStat (icmdCOM, icmdREP *, char *);
int  tcomSclSimIseqStat (icmdCOM, icmdREP *, char *);
int  tcomSclSimClrSubPat (icmdCOM, icmdREP *, char *);
int  tcomSclSimClrClkp (icmdCOM, icmdREP *, char *);
int  tcomSclSimSetPat (icmdCOM, icmdREP *, char *);
int  tcomSclSimGetPat (icmdCOM, icmdREP *, char *);
int  tcomSclSimSetClkp (icmdCOM, icmdREP *, char *);
int  tcomSclSimClkpProc (icmdCOM, icmdREP *, char *);
int  tcomSclSimClkpLoop (icmdCOM, icmdREP *, char *);
int  tcomSclSimGetClkp (icmdCOM, icmdREP *, char *);
int  tcomSclSimMode (icmdCOM, icmdREP *, char *);
int  tcomSclSimStart (icmdCOM, icmdREP *, char *);
int  tcomSclSimStop (icmdCOM, icmdREP *, char *);
int  tcomSclSimSeqProg (icmdCOM, icmdREP *, char *);
int  tcomSclSimPstat (icmdCOM, icmdREP *, char *);
int  tcomSclSimParam (icmdCOM, icmdREP *, char *);
int  tcomSclSimTcalc (icmdCOM, icmdREP *, char *);
int  tcomSclSimRstat (icmdCOM, icmdREP *, char *);
int  tcomSclSimWstat (icmdCOM, icmdREP *, char *);
int  tcomSclSimEstat (icmdCOM, icmdREP *, char *);
int  tcomSclSimNstat (icmdCOM, icmdREP *, char *);
int  tcomSclSimSync (icmdCOM, icmdREP *, char *);
int  tcomSclSimIconf (icmdCOM, icmdREP *, char *);


extern tcomCLDC_T *tcomCldc;
extern tcomISEQ_T *tcomIseq;
extern tcomISH_T  *tcomIsh;
extern int         tcomSimNumIseq; 
extern int         tcomSimNumCldc; 
extern int         tcomSimNumIsh; 
extern int         tcomSimNstat; 
extern char        tcomSimBooted;
extern char        tcomSimReset;

#ifdef __cplusplus
}
#endif

#endif /*!_tcomSim_h*/
