#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobFileSel.tcl,v 1.145 2005/10/27 09:07:07 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  30/11/96    created
# eallaert  1998-09-25  protection against UifFileBrowser problems (SPR 980491)
#

#************************************************************************
#   NAME
#   bobFileSel - select a file via file selection box
# 
#   SYNOPSIS
#   bobFileSel <title> <example> ?<fileType>?
# 
#   DESCRIPTION
#   <title> : string with the title for the file-browser's window.
#   <example>: example filename from which directory to list and filter to 
#	apply is derived; directory is the dirname-part of the <example>, the
#       filter is the extension of <example>
#   <fileType>: the type of file one wants (accepted values are as returned by
#       by the "file type" Tcl-command, whereby links get resolved). This
#       procedure will loop until a selection of proper type is made. By
#       default, the type is not checked. 
#
#   Returns the file selected; it is set to "" if the file selection 
#   was canceled.
#
#   PUBLIC VARIABLES
#   
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   UifFileBrowser(3)
#
#
#------------------------------------------------------------------------
#
proc bobFileSel {title example {fileType {}}} {
    global env cv
    set dir [file dirname $example]
    if {[string length $dir] <= 1} {
	set dir $env(INS_ROOT)/$env(INS_USER)/COMMON/TEMPLATES/OBD
	if {![file exists $dir]} {
	    set dir [pwd]
	}
    }

    set fsb .fsb
    ##set dirWidth [expr {[string length $dir] + 7}]
    ##set len 10
    if {[winfo exists $fsb]} {
	wm deiconify $fsb
    } else {
	# An alternative to "UifFileBrowser" would be "panSelectFile"
	UifFileBrowser $fsb -title $title -iconify 1
	##    -fileGeometry 25x${len} -dirGeometry ${dirWidth}x${len}
    }

    $fsb CD $dir
    if {[string first * [file extension $example]] >= 0} {
	# The example contains already wildcard - use it as filter
	$fsb filter [file extension $example]
    } else {
	$fsb filter *[file extension $example]
    }

    while {1} {
	foreach {filePath fileInfo} [$fsb getFile] {}
	set selFileType [lindex $fileInfo 0]
	set link $filePath
	set level 0
        while {[string equal $selFileType link]} {
	    set link [file readlink $link]
	    set selFileType [file type $link]
	    if {[incr level] > 500} {
		bobDialog "You have selected\n\n$filePath\n\nIt seems to be a\
			circular link" application
		continue
	    }
	}
	switch -- $filePath {} {
	    break
	}

	switch -- $fileType {} {
	    break
	} $selFileType {
	    break
	} link {
	    # cannot get here - see above.
	} default {
	    bobDialog "You have selected\n\n$filePath\n\n\
		    It is of type \"$selFileType\",\n\
		    while it should be of type \"$fileType\"." application
	}	
    }
    wm withdraw $fsb;	# remove icon from screen
    focus $cv;		# put the focus back on bob's window
    return $filePath
}
