#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobPlaySound.tcl,v 1.145 2005/10/27 09:07:11 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  1996-11-30  created
# eallaert  1999-07-14  update to use the sound-extension
# eallaert  2000-10-27  made compatible with snack 1.7
# eallaert  2002-02-04  check for play-errors (important for HP-UX)
# eallaert  2005-03-04  updated to use audioPlay (if available)
#

#************************************************************************
#   NAME
#   bobPlaySound - play a soundfile
# 
#   SYNOPSIS
#   bobPlaySound <soundFile>
# 
#   DESCRIPTION
#   bobPlaySound will play a <soundFile>
#
#   FILES
#
#   ENVIRONMENT
#   ::bob::internal::config(bobSoundOn) : flags whether sound is required (1)
#                                         or not (0).
#   env(AUDIO) : if not set, $env(DISPLAY) will be used
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#
# The AUDIO environment variable does not need to be set - "send_sound" will
# try to output to $DISPLAY if AUDIO is not set, and this is just fine.
# 
# Try to find the "audio" package and load the audioPlay proc
catch {package require Audio; auto_load audio::audioPlay}

# Pick up the audio/sound command from the snack package; if it cannot be
# found; signal this as an error 
# (In principle, for HP-UX we could then try "send_sound", and for Solaris
# /usr/bin/audioplay, but "snack" should really be there).
proc ::bob::bobPlaySound { {soundFile {}} } {
    global env
    if {!$::bob::internal::config(bobSoundOn) || [string equal $soundFile {}]} return

    # Default to .au format
    switch -- [file extension $soundFile] {} {
	set soundFile ${soundFile}.au
    }
    if {![file exists $soundFile]} return

    if {[string equal [info command ::audio::audioPlay] ::audio::audioPlay]} {
	if {$::bob::internal::config(bobSoundSetVolAtPlay)} {
	    if {[catch {::audio::audioPlay \
			    -vol [expr {int($::bob::internal::config(bobSoundVolume))}]\
			    $soundFile} msg]} {
		# something is wrong with the sound server.  Disable, so it is 
		# clear there is a problem but do not flag fatal failure - like
		# this we can recover when the sound server is back up
		##set ::bob::internal::info(audioFailed) 1
		set ::bob::internal::config(bobSoundOn) 0
		bobDialog "Turned audio off:\n$msg." none 8000
	    } 
	} else {
	    if {[catch {::audio::audioPlay $soundFile} msg]} {
		##set ::bob::internal::info(audioFailed) 1
		set ::bob::internal::config(bobSoundOn) 0
		bobDialog "Turned audio off:\n$msg." none 8000
	    }
	}	    
	return
    }

    # If we get here, the Audio package was not found. Must be older than JAN2006.
    # Keep this part for as long as this version of bob should still run under APR2004.
    switch -- [info command snack::sound] {} {
	if {!$::bob::internal::info(audioFailed)} {
	    if {[catch {package require snack} msg]} {
		# couldn't find snack sound extension????
		# Disable sounds, so we don't keep on stumbling over this problem
		set ::bob::internal::info(audioFailed) 1
		set ::bob::internal::config(bobSoundOn) 0
		error "snack not found"
	    }
	}
    }	    

    audio play_gain [expr {int($::bob::internal::config(bobSoundVolume))}]
    set outputs [snack::mixer outputs]
    if {[llength $outputs] == 0 && [llength [snack::mixer inputs]] == 0} {
	# something is wrong with the sound server or audio HW.  Disable
	# sounds, so we don't keep on stumbling over this problem
	set ::bob::internal::info(audioFailed) 1
	set ::bob::internal::config(bobSoundOn) 0
	return
    }
    foreach output $outputs {
	snack::mixer output $output
    }

    if {[catch {snack::sound bobSnd -file $soundFile} msg]} {
	# something is wrong with the soundfile. Should give a bobDialog
	return
    }
	
    if {[catch {bobSnd play -blocking false -command "$bobSnd destroy"}]} {
	# something is wrong with the sound server. Disable sounds,
	# so we don't keep on stumbling over this problem.
	set ::bob::internal::info(audioFailed) 1
	set ::bob::internal::config(bobSoundOn) 0
	return
    }
}



