#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobPromptDialog.tcl,v 1.145 2005/10/27 09:07:07 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mbec      13/03/1997  created
# eallaert  25/07/1997  added borderwidth to buttonconfigure
#

#************************************************************************
#   NAME
#   bobPromptDialog - popup dialog + text entry line
#
#   SYNOPSIS
#   bobPromptDialog <message> <modality> ?<timeout> ?<type> ?<userLevel>??? 
#
#   DESCRIPTION
#   bobPromptDialog will display a dialog box with:
#   <message> : text displayed in the dialog box
#   <modality>: determines the degree of blocking of the application.  Can be
#       "global" -> all applications are blocked while this dialog is active.
#       "application" -> only this application is blocked
#       "none" -> no application will be blocked
#   <timeout> : timeout value in ms after which the dialog will deactivate
#               automatically.  A 0 value indicates no timeout (default).
#   <type> : type of dialog box; can be "error" (default), "info" or anything 
#            else
#   <userLevel> : do this only if the configured user-level is <userLevel>
#                 (0 for Beginner, 1 for Advanced, empty means all levels)
#
#   This procedure is implemented based on [incr Widgets]'s messagedialog.
#    
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#   text entry field - up to 100 max char
#
#   CAUTIONS
#   Lines longer than 100 chars will be truncated.
#
#   EXAMPLES
#
#   SEE ALSO
#   [incr widgets] documentation (http://www.tcltk.com)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc bobPromptDialog {text mod {timeout 0} {type error} {specialbutton {}} {userLevel {}} } {
    global gvar
    
    set maxLabelTextLength 100
    
    if {![string equal $userLevel {}] && $userLevel != $gvar(bobUserLevel)} {
	return
    }

    # Check text length
    foreach line [split $text \n] {
	if {[string length $line] > $maxLabelTextLength} {
	    set line "[string range $line 0 [expr {$maxLabelTextLength -4}]] ..."
	}
	append ltext "${line}\n"
    }

    if {[string equal [info command .bobPD] {}]} {
    	promptdialog .bobPD -title "BOB $type" \
		-labeltext $ltext -modality $mod -width 50 -padx 10 -pady 10
	.bobPD buttonconfigure OK -command { .bobPD deactivate 1 } \
		-padx 3 -pady 3 -borderwidth 2
	.bobPD buttonconfigure Cancel -command { .bobPD deactivate 0 } \
		-padx 3 -pady 3 -borderwidth 2
	.bobPD hide Help
	if {$specialbutton == {}} {
	    .bobPD hide Apply
	} else {
	    .bobPD show Apply
	    .bobPD buttonconfigure Apply -command { .bobPD deactivate 2 } \
		    -padx 3 -pady 3 -text $specialbutton -borderwidth 2
	}
    } else {
	while {[winfo viewable .bobPD]} {
	    # means the previous pass one has either an "after" command
	    # issued to deactivate after a timeout, or is waiting
	    # for acknowledge; in any case, let's wait till the window
	    # is gone
	    # This condition can only occur if an external process
	    # would send a command while the dialog box has a grab
	    # on the display, i.e. it cannot happen by pressing
	    # buttons.
	    after 50; update idletasks
	}
	.bobPD configure -labeltext $ltext -modality $mod -title "BOB $type"
	.bobPD delete 0 end
	if {$specialbutton == {}} {
	    .bobPD hide Apply
	} else {
	    .bobPD show Apply
	    .bobPD buttonconfigure Apply -command { .bobPD deactivate 2 } \
		    -padx 3 -pady 3 -text $specialbutton -borderwidth 2
	}
    }
    
    if {[string equal $type passwd]} {
    	.bobPD configure -show *
    } else {
	.bobPD configure -show {}
    }

    # Take care of automatic deactivation if so required.
    if {$timeout > 0} {
	after $timeout {.bobPD deactivate 0}
    }

    set ans [.bobPD activate]
    ###update idletasks
    return [list $ans [.bobPD get]]
}

#
# ___oOo___



