#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobTree.tcl,v 1.145 2005/10/27 09:07:08 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mbec     09/04/97  	created
# mbec      16/04/97  manpage & vltstandard - last touch
# mbec      15/05/97  added rank method
#					  optional position argument to add method
#					  added 'cut' - remove from tree node but 'kept alive'
#
#

#************************************************************************
#   NAME
#   BobTree - define standard tree object
#
#   SYNOPSIS
#   BobTree <obj_name>|#auto <args>
# 
#   DESCRIPTION
#   BobTree defines a standard tree node object
#   
#   METHODS
#   constructor: see [incr Tcl]
#   destructor: destroy this and all his children; see [incr Tcl]
#   configure: see [incr Tcl]
#
#   add <obj> ?pos? ?oobj?: register obj as a new child
#		position end (default)
#			 after oobj
#			 before oobj
#			 <integer>
#
#   up: returns the parent
#   contents: returns list of children
#
#   cut ?obj? : remove the object from child list (but doesn't delete it)
#		understand: if obj is defined : this remove obj from its list
#	                        if not: this cut itself out of his parent.
#
#   CLASS PROCEDURES
#
#   VARIABLES
#   children: list of children objects
#   parent: parent object
#
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#=============================================================================
#=============================================================================
itcl::class BobTree {
    constructor {args} {}
    destructor {}

    public method add {obj {pos "end"} {who "undefined"}} {}
    public method up {} {}
    public method cut { {obj  "myself"}} {};
    
    public method contents {} {}

    public method clearChildren {} {};   # destroy all children objects
    private method parent {pobj} {};      # register pobj as parent
    #  rank ?obj? : return position index
    #    understand: if obj is defined : return obj rank in this children
    #    if not: return obj rank from its parent
    private method rank {{obj  "myself"}} {}

    private variable children {}
    private variable parent {}
}

#=============================================================================
#=============================================================================

# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
itcl::body BobTree::constructor {args} {
    eval configure $args
}

# ------------------------------------------------------------------
#                        DESTRUCTOR
# ------------------------------------------------------------------
itcl::body BobTree::destructor {} {
    clearChildren
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD add
# ------------------------------------------------------------------
itcl::body BobTree::add {obj {pos "end"} {who "undefined"}} {
    $obj parent $this
    
    switch -regexp $pos {
	
	end {
	    #puts "add end"
	    lappend children $obj
	}
	after {
	    #puts "add after $who"
	    set i [rank $who]
	    set children [linsert $children [expr {$i +1}] $obj]
	}
	before {
	    #puts "add before $who"
	    set i [rank $who]
	    set children [linsert $children $i $obj]
	}
	{^([0-9]+)$} {
	    #puts "add position $pos"
	    set children [linsert $children $pos $obj]
	}
	default {
	    "WARNING: call 'add' method with wrong syntax: add $obj $pos $who"
	    puts "I add to end and continue"
	    lappend children $obj
	}
    }
}


# ------------------------------------------------------------------
#                        PUBLIC METHOD contents
# ------------------------------------------------------------------
itcl::body BobTree::contents {} {
    return $children
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD cut
# ------------------------------------------------------------------
itcl::body BobTree::cut {{obj myself}} {
    if {[string equal $obj myself]} {
	$parent cut $this;	#remove this from its parent list
    } else {
	set i [rank $obj];	#remove obj from this list
	set children [lreplace $children $i $i]
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD up
# ------------------------------------------------------------------
itcl::body BobTree::up {} {
    return $parent
}
# ------------------------------------------------------------------
#                        PRIVATE METHODS
# ------------------------------------------------------------------
itcl::body BobTree::clearChildren {} {
    if {$children != ""} {
        eval delete object $children
    }
    set children {}
}

# -----------------------------------------------------------------
itcl::body BobTree::rank {{obj myself}} {
    if {[string equal $obj myself]} {
	return [$parent rank $this]
    } else {
	return [lsearch $children $obj]
    }
}
#--------------------------------------------------------------------------

itcl::body BobTree::parent {pobj} {
    set parent $pobj
}



#
# ___oOo___





