#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobUifInit.tcl,v 1.145 2005/10/27 09:07:08 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  30/11/1996  created
# mbec      16/04/1997  manpage & vltstandard
#                       replaced gif by pixmap
#                       added preferences - debug - repeat buttons
#                       removed 'fun' buttons
#                       patched tk bug for button state
# mbec      07/05/1997  pixmap moved to their respective class
# mbec      07/05/1997  split in bobUifInit bobUifObservationToolbar 
#                       bobUifEngineerToolbar bobUifInitDisplay to support 
#                       both interface type
# mbec      13/06/1997  fixed minor bug concerning the actual name of the 
#                       canvas in a scrolledcanvas
# eallaert  1998-03-25  added seq_errDisplay button (SPR 980056)
# eallaert  2002-09-17  added _bobUifSetCanvasLabel (SPR 20020527)
# eallaert  2002-10-10  added snapshot button (SPR 20020511)
#

#************************************************************************
#   NAME
#   _bobUifInitDisplay - init the BOB user interface
#   _bobUifInit - get common stuff opnce for all (image)
#   _bobUifObservationToolbar - init the BOB user interface : observation mode
#   _bobUifEngineerToolbar - init the BOB user interface : build mode
# 
#   SYNOPSIS
#   _bobUifInitDisplay <hullForDisplay>
#   _bobUifInit
#   _bobUifEngineerToolbar <hullForToolbar>
#   _bobUifObservationToolbar <hullForToolbar>
# 
#   DESCRIPTION
#   _bobUifInit will initialize the BOB user interface.
#   <hullForDisplay>: widget name that is intended as the parent for the 
#       OB-canvas.  This canvas is implemented based on [incr Widgets]'s 
#       scrolledcanvas. 
#   <hullForDisplay>: widget name that is intended as the parent for the 
#       toolbar
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   [incr widgets] documentation (http://www.tcltk.com)
#
#   BUGS
#------------------------------------------------------------------------
#
# Make sure that the namespaces we will be dealing with in here do exist
namespace eval ::bob {}
namespace eval ::bob::internal {}

proc ::bob::internal::_bobUifInitDisplay {} {
    global gvar

    set height $gvar(pwFrameHeight)
    set width  [$gvar(pwFrame) cget -width]
    # Create paned windows
    if {$::tk_version < 8.4} {
	panedwindow $gvar(pw) -sashcursor double_arrow -thickness 2
	$gvar(pw) add top -margin 2 -minimum 0
	$gvar(pw) add bottom -margin 2 -minimum 0
	set hullSc [$gvar(pw) childsite 0]  
	set hullSt [$gvar(pw) childsite 1]  
	pack $gvar(pw) -fill both -expand yes
    } else {
	# option -handlepad: indicates how many pixels from left
	panedwindow $gvar(pw) \
	    -handlesize 8       \
	    -handlepad [expr {$width - 25}] \
	    -orient vertical    \
	    -sashcursor double_arrow \
	    -sashpad 2          \
	    -sashrelief sunken  \
	    -sashwidth 2
	set hullSc $gvar(pw)
	set hullSt $gvar(pw)
    }

    #####################
    #      Canvas       #
    #####################
    iwidgets::Scrolledcanvas $hullSc.sc -autoresize 1 -automargin 0 \
	    -labelpos n -textbackground Grey80 \
	    -sbwidth 10
    if {[catch {$hullSc.sc config -labelfont $::bob::internal::config(bobTitleFont)}]} {
	puts "The title font $::bob::internal::config(bobTitleFont) does not exist -\
                please edit your preferences"
    }

    if {$::tk_version < 8.4} {
	pack $hullSc.sc -fill both -expand yes -padx 1 -pady 5 -anchor nw
    }

    # There are a few things which do not seem to work as well on the
    # containing scrolledcanvas widget as directly on its canvas component,
    # e.g. the repositioning after folding  may lead to displaying the
    # OB at the bottom of the canvas, instead of the top. This depends also
    # on the "autoresize" setting. For this reason, we pass directly the
    # childsite name ($gvar(cvOB))to the OBs, while we keep the scrolledcanvas
    # name ($gvar(scOB)) to set its label- see _bobUifSetCanvasLabel and
    # bobPreferences.
    set gvar(scOB) $hullSc.sc
    set gvar(cvOB) [$hullSc.sc childsite]; # canvas only (no scrollbars etc)

    # make sure ROOT is defined, before it gets accessed all over the place
    set ::ROOT ""
    
    #####################
    #       Text        #
    #####################
    iwidgets::Scrolledtext ${hullSt}.st -labelpos n -labeltext "Template log-messages" \
	    -vscrollmode static -hscrollmode static -wrap none \
	    -sbwidth 10 -textfont $::bob::internal::config(bobLogFont)
    # -textbackground Grey80

    if {[catch {$hullSt.st config -labelfont $::bob::internal::config(bobTitleFont)}]} {
	puts "The title font $::bob::internal::config(bobTitleFont) does not exist -\
                please edit your preferences"
    }

    if {$::tk_version < 8.4} {
	pack ${hullSt}.st -padx 1 -pady 1 -fill both -expand yes
    } else {
	$gvar(pw) add $hullSc.sc ${hullSt}.st
	pack $gvar(pw) -fill both -expand yes
    }
    set gvar(stTpl) ${hullSt}.st

    #####################
    #    Progress Bar   #
    #####################
    if {[catch {SeqProgressBar .progressBar ::bob::internal::progress \
		    -font $::bob::internal::config(bobDialogFont)} ::bob::internal::progressBar]} {
	set ::bob::internal::progressBar {}
	wm withdraw .progressBar
    } else {
	wm title .progressBar {BOB's Progress Bar}
	wm withdraw .progressBar
    }
    set ::bob::internal::progressBar {};	# don't use it (yet)

    # Trap the window-deletion - we have our own clean-up to do.
    wm protocol . WM_DELETE_WINDOW {catch {delete object $::ROOT}; panClosePanel }

}

#------------------------------------------------------------------------
proc ::bob::internal::_bobUifInit {} {
    # create the different images which may be used in the canvas later on
	

    # observation toolbar
    image create pixmap file_trans -file [bobFindVlt bitmaps/bobFile_trans.xpm]
    image create pixmap fileSaveObd -file [bobFindVlt bitmaps/bobSaveFile.xpm]
    image create pixmap file_file -file [bobFindVlt bitmaps/bobFile.xpm]
    image create pixmap telescope -file [bobFindVlt bitmaps/bobTelescope.xpm]
    image create pixmap hand_up -file [bobFindVlt bitmaps/bobHand_up.xpm]
    image create pixmap hand -file [bobFindVlt bitmaps/bobHand.xpm]
    image create pixmap abort -file [bobFindVlt bitmaps/bobToolbarAbort.xpm]
    image create pixmap repeat -file [bobFindVlt bitmaps/bobRepeat.xpm]
    image create pixmap tools -file [bobFindVlt bitmaps/bobTools.xpm]
    image create pixmap preferences -file [bobFindVlt bitmaps/bobPreferences.xpm]
    image create pixmap sound? -file [bobFindVlt bitmaps/bobSound.xpm]
    image create pixmap soundOn -file [bobFindVlt bitmaps/bobSoundOn.xpm]
    image create pixmap print -file [bobFindVlt bitmaps/bobReports.xpm]
    image create pixmap snapshot -file [bobFindVlt bitmaps/bobCamera.xpm]

    # engineering toolbar
    image create pixmap _bobEngCut -file [bobFindVlt bitmaps/bobEngCut.xpm]
    image create pixmap _bobEngCopy -file [bobFindVlt bitmaps/bobEngCopy.xpm]
    image create pixmap _bobEngPaste -file [bobFindVlt bitmaps/bobEngPaste.xpm]
    image create pixmap _bobEngDrag -file [bobFindVlt bitmaps/bobEngDrag.xpm]
    image create pixmap _bobEngEdit -file [bobFindVlt bitmaps/bobEngEdit.xpm]

    image create pixmap _bobEngXcut -file [bobFindVlt bitmaps/bobEngXcut.xpm]
    image create pixmap _bobEngXcopy -file [bobFindVlt bitmaps/bobEngXcopy.xpm]
    image create pixmap _bobEngXpaste -file [bobFindVlt bitmaps/bobEngXpaste.xpm]
    image create pixmap _bobEngXdrag -file [bobFindVlt bitmaps/bobEngXdrag.xpm]
    image create pixmap _bobEngXedit -file [bobFindVlt bitmaps/bobEngXedit.xpm]

    image create pixmap _bobEngErrDisp -file [bobFindVlt bitmaps/bobEngErrDisp.xpm]
    image create pixmap _bobEngToggleupLog -file [bobFindVlt bitmaps/bobEngToggleupLog.xpm]
    image create pixmap _bobEngToggleLog -file [bobFindVlt bitmaps/bobEngToggleLog.xpm]

    image create pixmap refresh_disp -file [bobFindVlt bitmaps/bobEngRefresh.xpm]

}
#------------------------------------------------------------------------
proc ::bob::internal::_bobUifObservationToolbar {hull2} {
    global gvar cv
    
    set vtb $hull2.vtb
    set gvar(vtb) $vtb

    # first destroy existing one
    if {[winfo exist $gvar(vtb)]} {
    	destroy $gvar(vtb)
    }
    
    ################################
    #    Observation Toolbar       #
    ################################

    global env
    toolbar $vtb -orient vertical -helpvariable $gvar(mainHelpVar) \
	    -selectcolor black \
	    -balloondelay1 0 \
	    -balloondelay2 0 \
	    -borderwidth 2 -selectcolor Grey90 -activebackground Grey90
    set fc 0

    ##$vtb add frame filler[incr fc] -width 10 -height 43

    $vtb add button getOB \
	    -helpstr {Get the next OB from OH} \
	    -image file_trans \
	    -balloonstr {Fetch an OB from OH} \
	    -command {
	if {[string equal $::bob::internal::config(bobEnvNameOH) {}] || \
		[string equal [lindex $::bob::internal::config(bobProcNameOH) 0] {}]} {
	    ::bob::bobNextObsBlocks file
	} else {
	    ::bob::bobNextObsBlocks 1
	}
    }

    $vtb add button getOBfile \
	    -helpstr {Get the next OB from a file (via a file selection box)} \
	    -image file_file \
	    -balloonstr {Fetch an OB from file} \
	    -command {::bob::bobNextObsBlocks file}
    
    $vtb add button fileSaveObd \
	    -helpstr {Save description of selected OB into a file} \
	    -image fileSaveObd \
	    -balloonstr {Save OB-description} \
	    -command ::bob::bobSave

    $vtb add frame filler[incr fc] -width 10 -height 30

    $vtb add button telescope \
	    -helpstr {Start the execution of OBs} \
	    -image telescope \
	    -balloonstr {Start observing} \
	    -state disabled \
	    -command ::bob::bobStart
    
    # tk bug ????
    $vtb itemconfigure telescope -state disabled

    $vtb add button hand_up \
	    -helpstr {Pauses a running OB} \
	    -image hand_up \
	    -balloonstr Pause \
	    -state disabled \
	    -command ::bob::bobPause

    # tk bug ????
    $vtb itemconfigure hand_up -state disabled

    $vtb add button hand \
	    -helpstr {Continues a paused OB} \
	    -image hand \
	    -balloonstr Continue \
	    -state disabled \
	    -command ::bob::bobContinue

    # tk bug ????
    $vtb itemconfigure hand -state disabled

    $vtb add button abort \
	    -helpstr {Aborts a running or paused OB} \
	    -image abort \
	    -balloonstr Abort \
	    -state disabled \
	    -command ::bob::bobAbort

    # tk bug ????
    $vtb itemconfigure abort -state disabled

    $vtb add button repeat \
	    -helpstr {Request repetition of the selected OB(s) to the schedule} \
	    -image repeat \
	    -balloonstr {Repeat OB} \
	    -command ::bob::bobRepeat
    $vtb itemconfigure repeat -state disabled

    $vtb add frame filler[incr fc] -width 10 -height 30

    $vtb add button tools \
	    -helpstr {Will pop up the configuration menu for bob} \
	    -image tools \
	    -balloonstr {Configure Bob} \
	    -command {panImportPanel bobConfigEnv}

    $vtb add button preferences \
	    -helpstr {Will pop up the preferences menu for bob (first time may take a while due to font queries)} \
	    -image preferences \
	    -balloonstr Preferences \
	    -command {if {![info exists gvar(fontFamilies)]} {
		    set gvar(fontFamilies) [lsort [font families]]
		}
	        panImportPanel bobPreferences
            }

    $vtb add checkbutton sound? \
	    -helpstr {Enable/disable sounds (requires audio hardware)} \
	    -image sound? -selectimage soundOn\
	    -balloonstr {Enable/disable sounds} \
	    -variable ::bob::internal::config(bobSoundOn)\
	    -indicatoron false\
	    -command {if {$::::bob::internal::info(audioFailed)} {
	        set ::bob::internal::config(bobSoundOn) 0
	        bobDialog "Your terminal either has some problem with the audio-hardware,\n\
                           or doesn't have any such hardware.\n\
                           Hence sounds cannot be enabled." none 8000
                      } else {
                           if {$::bob::internal::config(bobSoundOn)} {::bob::bobPlaySound [bobFindSound $::bob::internal::config(bobWarningSound)]}}}
    
    $vtb add frame filler[incr fc] -width 10 -height 30
    $vtb add button print \
	    -helpstr {Print the currently displayed log of template messages} \
	    -image print \
	    -balloonstr {Print log} \
	    -command ::bob::bobPrint

    $vtb add button snapshot \
	    -helpstr {Take a snapshot of activities  and logs related to BOB} \
	    -image snapshot \
	    -balloonstr {Take a snapshot} \
	    -command ::bob::bobSnapshot
    pack $vtb -side top -anchor n    
}
#------------------------------------------------------------------------
proc ::bob::internal::_bobUifEngineerToolbar {hull2} {
    global gvar cv env
    
    set htb $hull2.htb
    set gvar(htb) $htb

    # default is edit mode
    set gvar(engineer_mode) edit

    ##############################
    #     Engineer Toolbar       #
    ##############################
    
    # first destroy existing one
    if {[winfo exist $gvar(htb)]} {
    	destroy $gvar(htb)
    }
    
    toolbar $htb -orient horizontal -helpvariable $gvar(mainHelpVar) \
	    -selectcolor black \
	    -balloondelay1 0 \
	    -balloondelay2 0 \
	    -borderwidth 2 -selectcolor Grey90 -activebackground Grey90
       
    $htb add label currentMode -borderwidth 1 -relief groove -pady 3 -width 5 -height 1 -textvariable gvar(engineer_mode)
    catch {$htb.tbFrame.currentMode config -background white} ;# catch for dummy toolbar
    
    set fc 0
    $htb add frame filler[incr fc] -width 2 -height 15
	
    $htb add radiobutton cut \
	    -helpstr {Cut the selected block} \
	    -image _bobEngCut -selectimage _bobEngXcut\
	    -indicatoron false\
	    -balloonstr Cut \
	    -command BobVisualTree::cleanupCanvas\
	    -variable gvar(engineer_mode) -value cut 
    $htb itemconfigure cut -state normal
    
    $htb add radiobutton copy \
	    -helpstr {Copy the selected block} \
	    -image _bobEngCopy  -selectimage _bobEngXcopy\
	    -indicatoron false\
	    -balloonstr Copy \
	    -command BobVisualTree::cleanupCanvas\
	    -variable gvar(engineer_mode) -value copy 
    $htb itemconfigure copy -state normal

    $htb add radiobutton paste \
	    -helpstr {Paste the selection} \
	    -image _bobEngPaste  -selectimage _bobEngXpaste\
	    -indicatoron false\
	    -balloonstr Paste \
	    -command BobVisualTree::cleanupCanvas\
	    -variable gvar(engineer_mode) -value paste 
    $htb itemconfigure paste -state normal
    
    $htb add radiobutton drag \
	    -helpstr {Drag selected block} \
	    -image _bobEngDrag -selectimage _bobEngXdrag\
	    -indicatoron false\
	    -balloonstr Drag \
	    -command BobVisualTree::cleanupCanvas\
	    -variable gvar(engineer_mode) -value drag 
    $htb itemconfigure drag -state normal

    $htb add radiobutton edit \
	    -helpstr {Edit the selected block} \
	    -image _bobEngEdit -selectimage _bobEngXedit\
	    -indicatoron false\
	    -balloonstr Edit \
	    -command BobVisualTree::cleanupCanvas\
	    -variable gvar(engineer_mode) -value edit 
    $htb itemconfigure edit -state normal
	    	
    $htb add frame filler[incr fc] -width 330 -height 15

    $htb add button errDisplay \
	    -helpstr {Display the CCS error stack in a separate window} \
	    -image _bobEngErrDisp \
	    -balloonstr {Show error stack} \
	    -command seq_errDisplay
    $htb add frame filler[incr fc] -width 10 -height 15

    # shift the sash bar to show/hide Template message
    set fraction1 85
    set fraction2 100
    if {$::tk_version < 8.4} {
	$htb add button toggleLog \
	    -helpstr {Show/hide the template logs} \
	    -image _bobEngToggleLog \
	    -balloonstr {Hide logs} \
	    -command "global gvar cv; \
	    if {\[string equal \[$gvar(htb) itemcget toggleLog -balloonstr\] \
	    {Show logs}\]} {$gvar(htb) itemconfigure toggleLog \
	    -balloonstr {Hide logs} -image _bobEngToggleLog; \
            $gvar(pw) fraction $fraction1 [expr {100 - $fraction1}]} else {\
	    $gvar(htb) itemconfigure toggleLog \
	    -balloonstr {Show logs} -image _bobEngToggleupLog; \
	    $gvar(pw) fraction $fraction2 [expr {100 - $fraction2}]} "
    } else {
	
	$htb add button toggleLog \
	    -helpstr {Show/hide the template logs} \
	    -image _bobEngToggleLog \
	    -balloonstr {Hide logs} \
	    -command "global gvar cv; \
	    if {\[string equal \[$gvar(htb) itemcget toggleLog -balloonstr\] \
	    {Show logs}\]} {$gvar(htb) itemconfigure toggleLog \
	    -balloonstr {Hide logs} -image _bobEngToggleLog; \
	    $gvar(pw) sash place 0 0 [expr {$gvar(pwFrameHeightEng)*$fraction1/100}]} else {\
	    $gvar(htb) itemconfigure toggleLog \
	    -balloonstr {Show logs} -image _bobEngToggleupLog; \
	    $gvar(pw) sash place 0 0 [expr {$gvar(pwFrameHeightEng)*$fraction2/100}]} "
    }

    $htb add frame filler[incr fc] -width 10 -height 15

    $htb add button refreshDisplay \
	    -helpstr {Refresh the display} \
	    -image refresh_disp \
	    -balloonstr Re-draw \
	    -command { catch {[BobVisualTree::root] BobVisualTree::refresh}}

    pack $htb -side top -anchor w  
}

# for <component> set to "OH", only the help-text of the toolbar fetch-button
# will be updated. However, if there is no OB loaded yet, it has the same effect
# as setting <component> to "both" (see further).
# If set to "OS", the help-text of the toolbar start-OB button will be updated,
# and the OS-part of the canvas label as well. 
# If set to "both", both helpstrings and the full label will be updated.
proc ::bob::internal::_bobUifSetCanvasLabel {component} {

    #!!#
    ##puts "entering _bobUifSetCanvasLabel (component $component)"
    global gvar env seq_ccsStatus

    if {![info exists gvar(cvOB)]} return; # not yet defined at start-up

    switch -- $component "OH" - "both" {
	set OHlabel [lindex $::bob::internal::config(bobProcNameOH) 1]
	if {[string length $OHlabel] == 0} {
	    set OHlabel $::bob::internal::config(bobProcNameOH)
	}
	
	if {[string length $OHlabel] > 0} {
	    $gvar(vtb) itemconfigure getOB -helpstr "Get the next OB from \"$OHlabel\""
	} else {
	    $gvar(vtb) itemconfigure getOB -helpstr \
		"Get the next OB from OH (process name not defined - see config)"
	}
    }

    switch -- $component "OH" {
	if {![string equal $::ROOT ""]} return
    }

    set procNameOS [lindex $::bob::internal::config(bobProcNameOS) 0]
    if {[string length $procNameOS] != 0} {
	set OSlabel [lindex $::bob::internal::config(bobProcNameOS) 1]
	if {[string length $OSlabel] == 0} {
	    # no label given in the config file, so interrogate OS
	    if {[info exists seq_ccsStatus]} {
		# means ccsInit was given before - we can use seq_msgSendCommand
		foreach {cmd par} [list \
			STATUS {-function INS.ID} \
			FORWARD {-subSystem ICS -command STATUS -arguments 1,INS.ID}] {
		    #!!#
		    ##puts "I'm sending the command $cmd to retrieve the OS name"
		    if {[catch {seq_msgSendCommand -nocheck \
				    [subst $::bob::internal::config(bobEnvNameOS)] \
				    [subst $procNameOS] $cmd $par} cmdId]} {
			#!!#puts "got a communication problem with $procNameOS"
			break;	# means a communication problem
		    }
		    #!!#
		    ##puts "command was sent, cmdId = $cmdId - [seq_isoTime]"
		    ##update
		    if {![catch {seq_msgRecvReply -4000 -all -ascii $cmdId error lR} reply ]} {
			if {$error == 0} {
			    set OSlabel $reply
			    #!!#
			    ##puts "got a reply - OSlabel = $OSlabel"
			    break
			} else {
			    #!!#
			    ##puts "got error\# $error back from $procNameOS, reply = $reply"
			}
		    } 
			
		}
	    }
	}
	if {[string length $OSlabel] == 0} {
	    set OSlabel $procNameOS
	}
    } else {
	set OSlabel "OS"
    }
    
    #!!#
    ##puts "OSlabel = $OSlabel"

    $gvar(vtb) itemconfigure telescope -helpstr \
	"Start the execution of OBs, sending commands to \"$OSlabel\""

    switch -- $component "both" - "OH" {
	if {[string equal $bob::internal::info(obdSource) "file"] || \
		[string length [lindex $::bob::internal::config(bobProcNameOH) 0]] == 0} {
	    set OHlabel "(file)"
	 
	}
	set ::bob::internal::OHlabel $OHlabel
	set ::bob::internal::OSlabel $OSlabel

    } "OS" {
	set OHlabel $::bob::internal::OHlabel;	# keep on displaying the 'old' value
    }

    #!!#
    ##puts "OHlabel = $OHlabel"
    $gvar(scOB) configure -labeltext "OBs: $OHlabel -> bob -> $OSlabel"
}


