#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobVisual.tcl,v 1.145 2005/10/27 09:07:08 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mbec      09/04/97    created
# mbec      16/04/97    manpage & vltstandard - last touch
# eallaert  2000-12-11  made sure that label does not include newlines
#                         and is no longer than 200 chars
#

#************************************************************************
#   NAME
#   BobVisual - define standard visual object for a canvas
#   SYNOPSIS
#   BobVisual <obj_name>|#auto <canvas> <args>
# 
#   DESCRIPTION
#   BobVisual defines a standard object to be represented on a specific canvas
#   This element will show up as an icon followed by a text label and a
#   rectangle used for eventual hilight purpose
#   
#   METHODS
#   constructor: canvas specify the canvas the object belongs to;see [incr Tcl]
#   destructor: destroy this and clear it from the canvas;see [incr Tcl]
#   configure: see [incr Tcl]
#
#   draw <ulx> <uly> <tags>: draw this object on the canvas at the
#      coordinates specified by ulx,uly (upper left corner). The 
#      upper-left y-coordinate for a next similar object is returned.
#   canvas <args> : returns the canvas if no args specified, otherwise
#      configure the canvas according to args.
#
#   CLASS PROCEDURES
#
#   VARIABLES
#   icon: icon name
#   label: label going next to it
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#=============================================================================
#=============================================================================
itcl::class BobVisual {
    constructor {cwin args} {}
    destructor {}
    
    public method canvas {args} {}
    public method draw {ulx uly moretags} {}

    public variable icon "folded"
    public variable label ""
    public variable labelTruncated 0;	# flag if label was truncated
    
    protected common icon_w 20
    protected common icon_h 20
    protected common separator_h 5
    
    protected variable canvas "";      # canvas the object belongs to
}
#=============================================================================
#=============================================================================

# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
itcl::body BobVisual::constructor {cwin args} {
    set canvas $cwin
    eval configure $args
}
# ------------------------------------------------------------------
#                        DESTRUCTOR
# ------------------------------------------------------------------
itcl::body BobVisual::destructor {} {
    $canvas delete $this
}

# ------------------------------------------------------------------
#                        CONFIGBODY
# ------------------------------------------------------------------
itcl::configbody BobVisual::label {
    if {[set idx [string first \n $label]] != -1} {
	set label "[string range $label 0 [incr idx -1]] ..."
    }
    if {[string length $label] > 200} {
	set label "[string range $label 0 199] ..."
	set labelTruncated 1
    } else {
	set labelTruncated 0
    }

    $canvas itemconfigure $this-text -text $label
}


# ------------------------------------------------------------------
#                        PUBLIC METHOD draw
# ------------------------------------------------------------------
itcl::body BobVisual::draw {ulx uly moretags} {
    global gvar
    
    # play with anchor !
    set ymid [expr {$uly+$icon_h/2}]
    
    $canvas create line [expr {$ulx - $icon_w/2}] $ymid \
	[expr {$ulx + $icon_w/2}] $ymid \
	-tags [concat $this $this-line $moretags]
    $canvas create image $ulx $ymid -anchor w -image img_$icon \
	-tags [concat $this $this-icon $moretags]

    $canvas create text [expr {$ulx+$icon_w}] $ymid -text $label -anchor w \
	-tags [concat $this $this-text $moretags] \
	-font $::bob::internal::config(bobCanvasFont)

    #set uly [expr {$uly+$icon_h+$separator_h}]
    return [expr {$uly+$icon_h+$separator_h}]
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD canvas
# ------------------------------------------------------------------

itcl::body BobVisual::canvas {args} {
    if {[llength $args] == 0} {
        return $canvas
    }
    regsub -all {%o} $args $this args
    eval $canvas $args
}


#
# ___oOo___



