#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobVisualPCTree.tcl,v 1.145 2005/10/27 09:07:09 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mbec  09/04/97  created
#

#************************************************************************
#   NAME
#   BobVisualPCTree - define visual tree nodes concept with program counter
#
#   SYNOPSIS
#   BobVisualPCTree <obj_name> <canvas> <args>
#
#   DESCRIPTION
#   BobVisualPCTree - this class inherits from VisualTree and add
#   a program counter (PC) functionality to support dynamic deplacements
#   in the tree. The PC is an index to the current activeChild.
#
#   
#   Public:
#      - Methods:
#          getNextChild: increment the PC and return corresponding 
#            active child
#          setPC <child>: set the PC to the index value corresponding to
#            the specified child and return PC value.
#          getPC: return PC value
#          resetPC: reset the PC
#          setActiveChild <child>: set <child> as activeChild
#          activeChild: returns current activeChild
#
#      - Variables:
#            abort: boolean indicating an abort pending
#            pause: boolean indicating a pause pending
#            activeChild: child which bears the PC
#            
#
#
#
#   Private:
#      - Variables:
#            PC: index pointing to the activeChild in the list of children
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#
itcl::class BobVisualPCTree {
    inherit BobVisualTree

    constructor {cwin args} {
	BobVisualTree::constructor $cwin
    } {
	eval configure $args
    }
     
    public variable abort 0
    public variable mustRepeat 0
    public variable pause 0;	# these define the status for the object

    public variable activeChild ""
    private variable PC 0
    
    public method getNextChild {} {}
    # this method used on root when starting from selected OB
    public method setPC { child } {}
    public method getPC { } {}
    public method resetPC { } {}
    # parent select its own active child
    public method setActiveChild { obj } {}
    public method activeChild { } {}
}


#-----------------------------------------------------------------------------
itcl::body BobVisualPCTree::activeChild { } {
    return $activeChild
}

#-----------------------------------------------------------------------------
itcl::body BobVisualPCTree::setActiveChild { obj } {
    set activeChild $obj
}

#----------------------------------------------------------------------------
itcl::body BobVisualPCTree::getNextChild {} {
    incr PC
    return [lindex [contents] $PC]
}
#-----------------------------------------------------------------------------
itcl::body BobVisualPCTree::resetPC { } {
    set PC -1
}

#-----------------------------------------------------------------------------
itcl::body BobVisualPCTree::setPC { child } {
    set pc [lsearch -exact [contents] $child]
    if {$pc == -1} {
	set PC 0
    } else {
	set PC $pc
    }
    return $pc
}
#-----------------------------------------------------------------------------
itcl::body BobVisualPCTree::getPC { } {
    return $PC
}



#
# ___oOo___




