#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: feditCreateIncludeFile.tcl,v 2.11 2003/11/12 09:10:34 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    09/11/95  created
# gfilippi  21/11/95  override write protection (needed when installing)
# mcomin    03/12/95  Add check if offsetRec exists
# bgilli    03/02/00  Added () surrounding casted define, and conditional
#                     inclusion of ccs.h
# bgilli    15/10/03  Stronger casting for C++: SPR 20030405
# bgilli    20/10/03  SPR 20030405: additional () for defines without offset
# bgilli    21/10/03  SPR 20030405: Isolated special C++ casting only for errors.
#

#************************************************************************
#   NAME
#   
#   feditCreateIncludeFile - Creates the include file containing definitions
#                            for error/log mnemonics
#
#   SYNOPSIS
#
#   feditCreateIncludeFile  <what>  <mod>
#
#   DESCRIPTION
#
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


proc feditCreateIncludeFile { what  module } {

   global recList offsetRec

   # Index constants to access record information
   
   global rec_offset rec_id  rec_type rec_memo rec_format   
   global err_help err_offset   
    
   switch $what {
     "ERRORS" { 
              set includeFile  "${module}Errors.h"
              set headTitle    "Error Include File"
              }
     "LOGS"   { 
              set includeFile  "${module}Logs.h" 
              set headTitle    "Logs Include File"
              }
   }     
 
   if { [ set filePath [ findModuleFile include/${includeFile}]] == "" } {
       set filePath  [getRelPath $module "include" ]
       append filePath "/$includeFile"       
   }
    
   catch { exec chmod +w $filePath } ; # to be sure that the file, if any,
                                       # is writable
   set fh [open $filePath w+ ] ;       # Open file 

   feditWriteHeader "include" $fh  $headTitle
  
   puts $fh "#ifndef ${module}${what}_H"
   puts $fh "#define ${module}${what}_H \n"
   puts $fh "#ifndef CCS_H"
   puts $fh "#include \"ccs.h\""
   puts $fh "#endif \n"

   #
   # Save offset definitions
   #

   if { $what == "ERRORS" && [info exists offsetRec] } { 
      foreach name [array names offsetRec] { 
         if {$name != "none" } { puts $fh "#define $name $offsetRec($name)" } 
      }  
      puts $fh " " 
   }
 
   #
   #  Calculate the maximum length of the mnemonics
   #

   set maxLen 0
 
   foreach errNum  [array names recList] {       
      set len [string length [lindex $recList($errNum) $rec_memo]]        
      if {$len > $maxLen} { set maxLen $len }
   }
    
   incr maxLen 2


   # 
   # Normal C Style
   #
   if { $what == "ERRORS" } { 
       puts $fh "#ifndef __cplusplus\n"
   }

   set lineFormat "#define %-${maxLen}s   ((vltINT16) %-s)"

      
   foreach myId [lsort -integer [array names recList]] {
   
       set myMemo   [lindex $recList($myId) $rec_memo]
       set myValue  $myId
       
       if { $what == "ERRORS" } {
       
          set myOffset [lindex $recList($myId) $err_offset] 
               
          if { $myOffset != "" } {
                set myValue "( $myOffset + $myValue )"
          }
       }

       puts $fh [format $lineFormat $myMemo $myValue]
   } 
   # 
   # C++ declarations with strong casting
   #
   if { $what == "ERRORS" } { 
       puts  $fh "\n#else\n"   

       set lineFormat "#define %-${maxLen}s   (static_cast<vltINT16> %-s)"
      
       foreach myId [lsort -integer [array names recList]] {
	   
	   set myMemo   [lindex $recList($myId) $rec_memo]
	   set myValue  $myId
	   
	   if { $what == "ERRORS" } {
	       
	       set myOffset [lindex $recList($myId) $err_offset] 
               
	       if { $myOffset != "" } {
		   set myValue "( $myOffset + $myValue )"
	       } else {
		   set myValue "( $myValue )"
	       }
	   }
	   
	   puts $fh [format $lineFormat $myMemo $myValue]
       } 
       
       puts  $fh "\n#endif"
   }     
 # End of C++ special casting for errors.  

   puts  $fh "\n#endif"  
   puts  $fh "\n";           # Write the EOF of the file 
   close $fh;                # Close the file
}
 

#
# ___oOo___
