#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: feditMisc.tcl,v 2.11 2003/11/12 09:10:35 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    12/11/95  created
# mcomin    05/12/95  Fixed bug in feditGetFile
# ------------------
# mcomin    14/03/96  Ported to TK4.0 and itcl2.0
#           21/09/96  Use seq_displayTerm to show file contents
#                     (This is included in the seq tcl library) 
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc getFileList { what } {
   if {[catch {set flist [glob $what]} ]} {
      return {}
   } else {
      return $flist
   }
}


#------------------------------------------------------------------------------
# PROCEDURE : feditQuitEditor - 
#------------------------------------------------------------------------------

proc feditQuitEditor { } {
   global par
   
   if { $par(SAVE_FLAG) } {
     if {[tk_dialog .dialog "Dialog Box" $par(NOT_SAVED) question 1 "Exit Anyway" Cancel]} { return }
   }
   
   exit 0
}


#-------------------------------------------------------------------------
# PROCEDURE : feditGenFiles - 
#-------------------------------------------------------------------------

proc feditGenFiles { } {
   global par 

   switch $par(WHAT) {
      "LOGS"    { set do nothing}
      "ERRORS"  { feditGenErrDefFile $par(MODULE) }
      default   { return }
   }

   feditCreateIndexFile   $par(WHAT)  $par(MODULE)
   feditCreateIncludeFile $par(WHAT)  $par(MODULE)
   
   if {[info exists par(SH_AREA)] && [winfo exists $par(SH_AREA)]} {
      $par(SH_AREA) displayText  "Configuration Files GENERATED"
   }
   set par(SAVE_FLAG) 0
}


#------------------------------------------------------------------------
# PROCEDURE : feditShowFile - Display the generated files
#------------------------------------------------------------------------

proc feditShowFile { type } {
   global par 
   
   set what $par(WHAT)   
   
   switch $what {
      "LOGS"    { set incFile Logs }
      "ERRORS"  { set incFile Errors }
      default   { return }
   }
   
   switch $type {
     "include"     {set file "include/$par(MODULE)${incFile}.h" }
     "definition"  {set file "${what}/$par(MODULE)_${what}"     }
     "index"       {set file "${what}/$par(MODULE)${what}.IDX"  }
     default       { return }   
   }   

   if { [set filePath [findVLTFile $file]] == "" } {
      $par(SH_AREA) displayText "NO $type - file FOUND"
      return
   }   

   set fh       [open $filePath "r"]
   set fileSize [file size $filePath] 
   set buf      [read $fh $fileSize]
   close $fh

   seq_displayTerm .txtDisplay  "Show $type File" $filePath $buf
}

#------------------------------------------------------------------------
# PROCEDURE : feditCleanData -
#------------------------------------------------------------------------

proc feditCleanData {} {
   global par deleteHelpList recList totalElem  errMemo offsetRec

   feditConstants
   catch {unset recList errMemo offsetRec }

   set deleteHelpList {}
   set recList(0)     {}
   set errMemo(1)     {}
   set totalElem      0
   set par(EDIT_MODE) "r+"
   set par(SAVE_FLAG) 0
   set par(LAST_ID)   0
   set par(OLD_MEMO) ""
   $par(wdgErrList) deleteItem 0 end ; # Delete content of listBox
}


#------------------------------------------------------------------------
# PROCEDURE : feditNewFile -
#------------------------------------------------------------------------

proc feditNewFile {} {
   global par 

   if {[set par(MODULE) [promptDialog ".root" "Module Name " ""]] == "" } {
      return 
   }
   feditCleanData
}

#-----------------------------------------------------------------------------
# PROCEDURE : feditConfigPanel  - Open the configuration panel
#-----------------------------------------------------------------------------

proc feditConfigPanel {} {
   global par myIndex
   
   feditClearConfig
   set myIndex [$par(wdgErrList) getIndex]
   displayError $myIndex
}
  
#-----------------------------------------------------------------------------
# PROCEDURE : feditDisplay - Display a format definition file
#-----------------------------------------------------------------------------

proc feditDisplay { what } {
   global recList myIndex par 

   set myIndex 0
   feditClearConfig

   if {$what == "ERRORS"} {
     
      $par(wdgErrList) updateSortList "-increasing"  "-integer" \
                               recList        feditErrorFormat
      displayError $myIndex

   } elseif {$what == "LOGS"} {
       set do nothing
   }

   $par(wdgErrList) setSelection $myIndex
}


#-----------------------------------------------------------------------------
# PROCEDURE : feditOpenMod - Open and read a format definition file
#-----------------------------------------------------------------------------

proc feditOpenMod { mode } {
  global par 

  set what $par(WHAT)

  if { $par(SAVE_FLAG) } {
      set ret [tk_dialog .dialog "Dialog Box" \
               $par(NOT_SAVED) question 1 Continue Cancel]
                           
      if { $ret }  { return }
  }
  
  feditCleanData

  set par(EDIT_MODE) $mode
  feditConfigMode

  # Read the new file
         
  if { [ set par(FILE) [feditGetFile $what $mode]] == "" } { return }

  feditReadDefFile  $what $par(FILE) $mode
  feditDisplay      $what
  set par(SAVE_FLAG) 0

}

#-----------------------------------------------------------------------------
# PROCEDURE : feditConfigMode -
#-----------------------------------------------------------------------------

proc feditConfigMode {} {
  global par mbOpt insMenu mbFile

  if {[winfo exists ".errConfig"]} { destroy .errConfig }

  if {[cequal $par(EDIT_MODE) "r+"] } { 
     set myState normal
  } else {
     set myState disabled
  }

  $mbFile  entryconfigure 6 -state $myState
  $mbOpt   entryconfigure 6 -state $myState
  $mbOpt   entryconfigure 8 -state $myState
  $insMenu entryconfigure 1 -state $myState
  $par(actFrame).b1 configure -state $myState
  $par(actFrame).b2 configure -state $myState
  $par(actFrame).b3 configure -state $myState
  $par(actFrame).b4 configure -state $myState
}

#-----------------------------------------------------------------------------
# PROCEDURE : feditGetFile -
#-----------------------------------------------------------------------------

proc feditGetFile { what mode } {
   global par
   
   if {[set par(MODULE) [promptDialog ".root" "Module Name " "" ]] == "" } {
       return ""
   }

   set file "$par(MODULE)_${what}"      ;# File name   
   
   if { $mode == "r" } {
      set filePath  [findVLTFile  ${what}/$file]
   } else {      
      set filePath  [findModuleFile ${what}/$file]
   }      
   
   if { $filePath == "" } {    
       set tmp  "File $file Not Found in Module Directory "
       tk_dialog .dialog "Dialog Box" $tmp info 0 OK     
       return ""
   }

   return $filePath
}


#-----------------------------------------------------------------------------
# PROCEDURE :  -
#-----------------------------------------------------------------------------

proc feditWhichRec { idx } {
   global recList
   set list [lsort -integer [array names recList]]
   return   [lindex $list $idx]
}

proc feditGetRec { item idx } {
   global recList
   return [lindex $recList($item) $idx]
}

proc feditSaveRec { item idx value } {
   global recList
   set list           $recList($item)  
   set recList($item) [lreplace $list $idx $idx $value ] 
}





#------------------------------------------------------------------------
# PROCEDURE : feditClearConfig - Clears the information on the screen
#------------------------------------------------------------------------

proc feditClearConfig {} {
   global par myNumber myMnemonic myFormat mySeverity myHelpFile myOffset

    if { ! [cequal  $par(WHAT) "ERRORS"]} { return }

    set myNumber     ""
    set mySeverity   W
    set myFormat     {}
    set myHelpFile   {}
    set myOffset     ""
    set myMnemonic  [feditErrSetMnemonic $par(MODULE)]

    if {[info exists par(wdgErrHelp)] && [winfo exists $par(wdgErrHelp)]} {
         $par(wdgErrHelp) deleteText 0.0 end
     }
}


# ___oOo___
