#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifClass.tcl,v 1.113 2005/09/13 07:27:24 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  20/12/99  created
# jstegmei  10/01/00  added iracqPerf_uifClass
# jstegmei  12/01/00  dynamic add/remove of available frame types
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# send CCS command
proc iracqClassSendCmd {server cmd param {errDisp 1}} {
    global gvar seq_ccsEnvName
    
    # send command
    if {[catch \
	    {seq_msgSendCommand $seq_ccsEnvName $server $cmd $param} \
	    cmdHandle]} {
	if {$errDisp} {
	    seq_errDisplay
	}
	return 0
    }

    # wait for result
    set lastReply 0
    while {$lastReply == 0} {
	if {[catch {seq_msgRecvReply $cmdHandle errorNumber \
		lastReply} reply]} {
	    if {$errDisp} {
		seq_errDisplay
	    }
	    return 0
	}
    }

    if {$errorNumber != 0} {
	if {$errDisp} {
	    seq_errDisplay
	}
	return 0
    }

    return 1
}

# text widget
proc iracqClassTextWidget {w width height} {
    global gvar 

    frame $w.box 
    pack $w.box -side top -fill both -expand 1

    text $w.text -relief sunken -fg black -bd 2 \
	    -cursor left_ptr \
	    -width $width -height $height \
	    -wrap none -state disabled \
	    -xscrollcommand "$w.scrollx set" \
	    -yscrollcommand "$w.scrolly set" 


    $w.text tag configure err -foreground red3
 
    scrollbar $w.scrolly -command "$w.text yview" \
	    -orient vertical -cursor left_ptr
    
    scrollbar $w.scrollx -command "$w.text xview" \
	    -orient horizontal -cursor left_ptr
   
    pack $w.scrollx -side bottom -fill x -in $w.box
    pack $w.scrolly -side right -fill y -in $w.box
    pack $w.text -side left -fill both -expand 1 -in $w.box

    return $w.text
}

# clear text widget
proc iracqClassClearText {w} {

    # clear text
    $w configure -state normal
    $w delete 1.0 end
    $w configure -state disabled
}

# add string to text widget
proc iracqClassAddText {w instance data} {
    global gvar 

    set value "[join [lindex $data 2]]"

    if {$gvar($instance\_enable)} {
	# enable verbose box for insertion
	$w configure -state normal
	
	# append result to diagText
	$w insert end "$value\n"
	$w see end 
        
	# disable insertion 
	$w configure -state disabled
    }   
}

# add filename to text widget
proc iracqClassAddFile {w instance data} {
    global gvar 

    set value [lindex $data 2]

    if {$gvar($instance\_enable)} {
	# enable verbose box for insertion
	$w configure -state normal
	
	# append result to diagText
	$w insert end "[file tail $value]\n"
	$w see end 
        
	# disable insertion 
	$w configure -state disabled
    }   
}

proc iracqClassSetDit {server detIdx instance} {
    global gvar

    iracqClassSendCmd $server "SETUP" \
	    "-function DET$detIdx\.DIT $gvar($instance\_ditSet)"
    set gvar($instance\_ditSet) ""
}

proc iracqClassInitDit {server detIdx dbPoint instance {w ""}} {
    global gvar cv
    
    if {$w == ""} {
	set w $cv.$instance.cv
    }

    bind [$w.iracqDit-ditSet component uifEntry] <Return> \
	"iracqClassSetDit $server \"$detIdx\" $instance "
    
    $w.iracqDit-apply configure -command \
	    "iracqClassSendCmd $server SETUP \"-function DET$detIdx\.DIT 0\""
}

proc iracqClassInitStatus {server detIdx dbPoint instance {w ""}} {
    global gvar cv
    
}

proc iracqClassFrameGen {server detIdx instance name} {
    global gvar
    
    set tmp [expr {1 - $gvar($instance\_gen_$name)}]
 
    if {$gvar($instance\_gen_$name)} {
	set value "T"
    } else {
	set value "F"
    }

    set param "-name $name -gen $value"

    if {$detIdx != ""} {
	set param "$param -det $detIdx"
    }

    if {![iracqClassSendCmd $server FRAME $param]} {
	set gvar($instance\_gen_$name) $tmp
    }
}

proc iracqClassFrameStore {server detIdx instance name} {
    global gvar
    
    set tmp [expr {1 - $gvar($instance\_store_$name)}]

    if {$gvar($instance\_store_$name)} {
	set value "T"
    } else {
	set value "F"
    }

    set param "-name $name -store $value"

    if {$detIdx != ""} {
	set param "$param -det $detIdx"
    }

    if {![iracqClassSendCmd $server FRAME "$param"]} {
	set gvar($instance\_store_$name) $tmp
    }
}

proc iracqClassFrameDisp {server detIdx instance name} {
    global gvar
    
    set tmp [expr {1 - $gvar($instance\_disp_$name)}]

    if {!$gvar($instance\_disp_$name)} {
	set fname "NO_FRAME"
    } else {
	set fname $name
    }

    set param "-name $fname -display"

    if {$detIdx != ""} {
	set param "$param -det $detIdx"
    }

    if {![iracqClassSendCmd $server FRAME "$param"]} {
	set gvar($instance\_disp_$name) $tmp
    }
}

proc iracqClassFrameAdd {server detIdx instance name w} {

    global gvar cv

    set fw [$w.iracqFrames-main component uifFrame]

    set e frm-$name

    if {[winfo exists $fw.$e]} { 
	return
    }

    frame $fw.$e -bd 0 
    pack $fw.$e -side top -fill both
    label $fw.$e.name -anchor nw -width 21 -fg black \
	    -text $name 
    pack $fw.$e.name -side left -anchor nw
    checkbutton  $fw.$e.gen -anchor w \
	    -offvalue 0 -onvalue 1 \
	    -variable gvar($instance\_gen_$name) \
	    -command "iracqClassFrameGen $server $detIdx $instance $name"
    checkbutton  $fw.$e.store -anchor w \
	    -offvalue 0 -onvalue 1 \
	    -variable gvar($instance\_store_$name) \
	    -command "iracqClassFrameStore $server $detIdx $instance $name"
    checkbutton  $fw.$e.disp -anchor w \
	    -offvalue 0 -onvalue 1 \
	    -variable gvar($instance\_disp_$name) \
	    -command "iracqClassFrameDisp $server $detIdx $instance $name"
    pack $fw.$e.gen -side left 
    pack $fw.$e.store -side left -padx 45
    pack $fw.$e.disp -side left 
}


proc iracqClassFrameUpdate {server detIdx attr instance w} {
    global gvar cv

    set fw [$w.iracqFrames-main component uifFrame]

    set fchild [winfo children $fw]
    
    set frameTable [seq_dbReadSymbolic $attr]

    for {set i 0} {$i<32} {incr i 1} {
	set record [lindex $frameTable $i]
	set name [lindex $record 0]
	if {$name != ""} {
	    set idx [lsearch -exact $fchild $fw.frm-$name]
	    if {$idx != -1} {
		set fchild [lreplace $fchild $idx $idx]
	    }
	    set generate [lindex $record 1]
	    set store [lindex $record 2]
	    set disp [lindex $record 3]
	    set gvar($instance\_gen_$name) $generate
	    set gvar($instance\_store_$name) $store
	    set gvar($instance\_disp_$name) $disp
	    iracqClassFrameAdd $server $detIdx $instance $name $w
	}
    }

    if {"$fchild" != ""} {
	catch {eval destroy $fchild}
    }
    
}
proc iracqClassInitFrames {server detIdx dbPoint instance {w ""}} {
    global gvar cv

    if {$w == ""} {
	set w $cv.$instance.cv
    }

    set attr "$dbPoint:irace.frames"

    iracqClassFrameUpdate $server \"$detIdx\" $attr $instance $w
    seq_evtAttach $attr "w" \
	    "iracqClassFrameUpdate $server \"$detIdx\" $attr $instance $w"    
    
}

proc iracqClassParamUpdate {attr instance w} {
    global gvar cv

    set pw $w.iracqParam-paramList

    $pw deleteItem 0 end

    set attrInfo [seq_dbGetAttrInfo $attr]
    set size [lindex $attrInfo 2]  
    set paramTable [seq_dbReadSymbolic $attr]
    
    for {set i 0} {$i<$size} {incr i 1} {
	set record [lindex $paramTable $i]
	set name [lindex $record 0]
	if {$name != ""} {
	    set value [lindex $record 1]
	    set line [format "%-30s %s" $name $value]
	    set parArray($line) $line
	} else {
	    break
	}
    }

    $pw updateSortList -increasing -ascii parArray
}

proc iracqClassParamSelect {par1 par2 par3} {
    global gvar iracqParam iracqParamSet
    
    set i1 [string first "_paramSel" $par2]
    set instance [string range $par2 0 [expr {$i1 - 1}]]
    set gvar($instance\_paramSelName)  [lindex $gvar($par2) 0]
    set gvar($instance\_paramSelValue) ""
    
}

proc iracqClassParamSet {server instance detIdx} {
    global gvar

    if {$gvar($instance\_paramSelValue) != ""} {
	set name $gvar($instance\_paramSelName)
	set i1 [string first "." $name]
	set str [string range $name $i1 end]
	set name "DET$detIdx$str"
	iracqClassSendCmd $server SETUP \
		"-function $name $gvar($instance\_paramSelValue)"
	set gvar($instance\_paramSelValue) ""
    }
}

proc iracqClassAttachParam {dbPoint instance w} {

    set attr "$dbPoint:irace.dynPars"
    iracqClassParamUpdate $attr $instance $w
    seq_evtAttach $attr "w" \
	    "iracqClassParamUpdate $attr $instance $w"    
}

proc iracqClassInitParam {server detIdx dbPoint instance {w ""}} {
    global gvar cv
 
    if {$w == ""} {
	set w $cv.$instance.cv
    }

    if {$server == ""} {
	catch {destroy $w.iracqParam-paramDefault}
    } else {
	$w.iracqParam-paramDefault configure -command \
		"iracqClassSendCmd $server SETUP \"-default\""
    }

    iracqClassAttachParam $dbPoint $instance $w
    
    if {$server == ""} {
	catch {destroy $w.iracqParam-paramEntry}
	catch {destroy $w.iracqParam-paramSel}
    } else {
	trace variable gvar($instance\_paramSel) w iracqClassParamSelect
	bind [$w.iracqParam-paramEntry component uifEntry] <Return> \
		"iracqClassParamSet $server $instance \"$detIdx\""
    }
}

proc iracqClassReadMode {instance attr} {
    global gvar
    
    set gvar($instance\_readMode) [seq_dbReadSymbolic $attr]
}

proc iracqClassUpdateReadMode {server detIdx dbPoint instance w} {
    global gvar cv

    set rw $w.iracqReadMode-option
    
    set numRm [seq_dbReadSymbolic "$dbPoint:irace.numRm"]
    set rmTable [seq_dbReadSymbolic "$dbPoint:irace.rmDef"]

    $rw deleteEntry 0 last  

    for {set i 0} {$i<$numRm} {incr i 1} {
	set record [lindex $rmTable $i]
	set ele [lindex $record 0]
	$rw addItem $ele \
		"iracqClassSendCmd $server SETUP \
		\"-function DET$detIdx\.NCORRS.NAME $ele\""
    }    
}

proc iracqClassInitReadMode {server detIdx dbPoint instance {w ""}} {
    global gvar cv
    
    if {$w == ""} {
	set w $cv.$instance.cv
    }

    set attr "$dbPoint:irace.readoutMode"
    iracqClassReadMode $instance $attr
    seq_evtAttach $attr "!=" "iracqClassReadMode $instance $attr"

    iracqClassUpdateReadMode $server $detIdx $dbPoint $instance $w
    seq_evtAttach "$dbPoint:irace.rmDef" "w" \
	    "iracqClassUpdateReadMode $server \"$detIdx\" $dbPoint $instance $w"
}

proc iracqClassHwWinEvent {instance attr} {
    global gvar

    set gvar($instance\_hwWindow) [seq_dbReadSymbolic $attr]
}

proc iracqClassWinCmd {server detIdx instance} {
    global gvar

    set plist ""
    if {$gvar($instance\_startXSet) != ""} {
	set plist "$plist DET$detIdx\.WIN.STARTX $gvar($instance\_startXSet)"
    }
    if {$gvar($instance\_startYSet) != ""} {
	set plist "$plist DET$detIdx\.WIN.STARTY $gvar($instance\_startYSet)"
    }
    if {$gvar($instance\_nxSet) != ""} {
	set plist "$plist DET$detIdx\.WIN.NX $gvar($instance\_nxSet)"
    }
    if {$gvar($instance\_nySet) != ""} {
	set plist "$plist DET$detIdx\.WIN.NY $gvar($instance\_nySet)"
    }

    iracqClassSendCmd $server SETUP "-function $plist"
    set gvar($instance\_startXSet) ""
    set gvar($instance\_startYSet) ""
    set gvar($instance\_nxSet) ""
    set gvar($instance\_nySet) ""
}

proc iracqClassApplyHwWin {server detIdx instance} {
    global gvar

    iracqClassSendCmd $server SETUP \
	    "-function DET$detIdx\.WIN.TYPE $gvar($instance\_hwWindow)"
}

proc iracqClassInitWindow {server detIdx dbPoint instance {w ""}} {
    global gvar cv
    
    if {$w == ""} {
	set w $cv.$instance.cv
    }

    bind [$w.iracqWindow-startXSet component uifEntry] <Return> \
	"iracqClassWinCmd $server \"$detIdx\" $instance"
    bind [$w.iracqWindow-startYSet component uifEntry] <Return> \
	"iracqClassWinCmd $server \"$detIdx\" $instance"
    bind [$w.iracqWindow-nxSet component uifEntry] <Return> \
	"iracqClassWinCmd $server \"$detIdx\" $instance"
    bind [$w.iracqWindow-nySet component uifEntry] <Return> \
	"iracqClassWinCmd $server \"$detIdx\" $instance"

    set attr "$dbPoint:exposure.hwWindow"

    set gvar($instance\_hwWindow) [seq_dbReadSymbolic $attr]

    seq_evtAttach $attr "!=" \
	    "iracqClassHwWinEvent $instance $attr"

    $w.iracqWindow-hwWindow configure -command \
	    "iracqClassApplyHwWin $server \"$detIdx\" $instance"

    $w.iracqWindow-fullFrame configure -command \
	    "iracqClassSendCmd $server SETUP \"-function \
	    DET$detIdx\.WIN.STARTX 1 \
	    DET$detIdx\.WIN.STARTY 1 \
	    DET$detIdx\.WIN.NX 0 \
	    DET$detIdx\.WIN.NY 0\""

}

proc iracqClassAttachAction {w instance attr} {
    global gvar
    
    if {$gvar($instance\_enable)} {
	set gvar($instance\_evt) [seq_evtAttach $attr "w" \
		"iracqClassAddText $w $instance %D"]
    } else {
	seq_evtDetach $gvar($instance\_evt)
    }
}

proc iracqClassAttachFile {w instance attr} {
    global gvar
    
    if {$gvar($instance\_enable)} {
	set gvar($instance\_evt) [seq_evtAttach $attr "w" \
		"iracqClassAddFile $w $instance %D"]
    } else {
	seq_evtDetach $gvar($instance\_evt)
    }
}

proc iracqClassInitAction {server detIdx dbPoint instance {w ""}} {
    global gvar cv

    if {$w == ""} {
	set w $cv.$instance.cv
    }

    set tw [iracqClassTextWidget [$w.iracqAction-frame component uifFrame] \
	    34 5 ]
    
    set attr "$dbPoint:irace.currentAction"

    $w.iracqAction-clear configure -command \
	    "iracqClassClearText $tw"

    $w.iracqAction-enable configure -command \
	    "iracqClassAttachAction $tw $instance $attr"
}

proc iracqClassInitFiles {server detIdx dbPoint instance {w ""}} {
    global gvar cv

    if {$w == ""} {
	set w $cv.$instance.cv
    }

    set tw [iracqClassTextWidget [$w.iracqFiles-frame component uifFrame] \
	    34 5 ]
    
    set attr "$dbPoint:exposure.newDataFileName"

    $w.iracqFiles-clear configure -command \
	    "iracqClassClearText $tw"

    $w.iracqFiles-enable configure -command \
	    "iracqClassAttachFile $tw $instance $attr"
}

proc iracqClassPlot {x0 y0 x1 attr w updateTime minY maxY} {
    global gvar

    set y1 [seq_dbReadSymbolic $attr]

    set deltaX [expr {$x1 - $x0}]
    set sy [expr {$maxY - $minY}]

    set xw [$w cget -width]
    set yw [$w cget -height]

    set yy0 [expr {($y0-$minY)*($yw/$sy)}]
    set yy1 [expr {($y1-$minY)*($yw/$sy)}]
    set ly0 [expr {$yw-$yy0}]
    set ly1 [expr {$yw-$yy1}]

    if {$x1 > $xw} {
	$w move lineTag -${deltaX} 0
	set x1 $x0
	set x0 [expr {$x0 - $deltaX}]
    }

    set id [$w create line ${x0} ${ly0} ${x1} ${ly1} \
	    -fill yellow]
    $w addtag lineTag withtag $id

    set x0 $x1
    set y0 $y1

    set x1 [expr {$x0 + $deltaX}]

    set gvar($w\_monId) [after $updateTime [list iracqClassPlot $x0 $y0 $x1 \
	    $attr $w $updateTime $minY $maxY]]
}

proc iracqClassPlotClear {w} {
    global gvar

    $w delete lineTag
}

proc iracqClassPlotStart {w attr updateTime minY maxY} {
    global gvar

    if {[info exists gvar($w\_monId)]} {
	# already running
	return
    }

    iracqClassPlotClear $w
    iracqClassPlot 0 0 2 $attr $w $updateTime $minY $maxY
}

proc iracqClassPlotStop {w} {
    global gvar

    if {[info exists gvar($w\_monId)]} {
	after cancel $gvar($w\_monId) 
	unset gvar($w\_monId) 
    }
}

# plotter widget
proc iracqClassPlotWidget {w} {
    global gvar
    global px0 py0

    $w configure -relief sunken -bd 2

    # performance canvas
    canvas $w.canvas -width 120 -height 50 \
	    -relief flat -bd 0 -bg black 
    pack $w.canvas -side left -padx 0m -pady 0m -fill both

    set id [$w.canvas create line 0 5 120 5 -fill red]
    $w.canvas addtag alarmTag withtag $id

    for {set x 0} {$x < 120} {incr x 10} {
	set id [$w.canvas create line $x 0 $x 50 -fill grey]
	$w.canvas addtag gridTag withtag $id
    }

    for {set y 10} {$y < 50} {incr y 5} {
	set id [$w.canvas create line 0 $y 120 $y -fill grey]
	$w.canvas addtag gridTag withtag $id
    }

    bind $w <Destroy> "iracqClassPlotStop $w"

    return $w.canvas
}

# start performance monitor
proc iracqClassPerfStart {server detIdx attr w} {
    global gvar cv

    set param "-perfStart"

    if {$detIdx != ""} {
	set param "$param -det $detIdx"
    }

    if {[iracqClassSendCmd $server "MISC" $param]} {
	iracqClassPlotStart $w $attr 2000 0.0 100.0
    } else {
	return 0
    }

    return 1
}

proc iracqClassPerfStop {server detIdx w} {
    global gvar cv

    iracqClassPlotStop $w

    set param "-perfStop"

    if {$detIdx != ""} {
	set param "$param -det $detIdx"
    }

    iracqClassSendCmd $server "MISC" $param 0
    
    return 1
}

proc iracqClassPerfEnable {server dbPoint detIdx instance w} {
    global gvar

    set attr $dbPoint:irace.perf
    
    if {$gvar($instance\_enabled)} {
	if {![iracqClassPerfStart $server $detIdx $attr $w]} {
	    set gvar($instance\_enabled) 0
	}
    } else {
	if {![iracqClassPerfStop $server $detIdx $w]} {
	    set gvar($instance\_enabled) 1
	}
    }
}

proc iracqClassDtStatus {instance attr} {
    global gvar
    
    set status [seq_dbReadSymbolic $attr]

    if {$status == 1} {
	set gvar($instance\_recv) 1
	set gvar($instance\_store) 0
    } else {
	if {$status == 2} {
	    set gvar($instance\_recv) 0
	    set gvar($instance\_store) 1
	} else {
	    set gvar($instance\_recv) 0
	    set gvar($instance\_store) 0
	}
    }

}

proc iracqClassInitPerf {server detIdx dbPoint instance {w ""}} {
    global gvar cv

    if {$w == ""} {
	set w $cv.$instance.cv
    }

    set attr "$dbPoint:exposure.dtStatus"
    iracqClassDtStatus $instance $attr
    seq_evtAttach $attr "!=" "iracqClassDtStatus $instance $attr"

    set plotWidget [$w.iracqPerf-main component uifFrame]
    
    set perfWidget [iracqClassPlotWidget $plotWidget]

    $w.iracqPerf-enable configure -command \
	    "iracqClassPerfEnable $server $dbPoint \"$detIdx\" $instance $perfWidget"
}


# configure panel editor class
proc iracqClassUpdate { w dbPoint { relief flat }} {
    global shArea
    
    $w configure -cwp $dbPoint
    $w init

    catch { $w configure -helparea $shArea } 
    $w display

    $w configure -relief $relief

    return $w
}  

# create notebook
proc iracqClassNoteBookCreate {w} {
    global gvar

    set tabPath [tabnotebook $w.tabB \
	    -tabpos n -equaltabs false -raiseselect 1]

    pack $tabPath -fill both -side left -expand 1

    return $tabPath
}

# add notebook page
proc iracqClassNoteBookAdd {w page name instance dbPoint} {
    global gvar

    set childPath [$w add -label $name]

    if {[catch {set childClass [$page $childPath.$instance \
	    -library libiracqClass -cwp $dbPoint]}]} {
	$w delete end
	return 0
    }

    pack [iracqClassUpdate $childClass $dbPoint flat] \
	    -padx 2 -pady 2 -fill both -expand 1

    return 1
}

# intitialize notebook
proc iracqClassNoteBookInit {w} {
    global gvar

    if {[catch {set n [$w childsite]}]} {
	$w add -label "No Plug-In"
    }  

    $w select 0
}

# get notebock path
proc iracqClassNoteBookPath {w child} {

    set childpath [$w childsite $child]

    return $childpath
}

proc iracqClassInitOs {server detIdx dbPoint instance {w ""}} {
    global gvar cv

    if {$w == ""} {
	set w $cv.$instance.cv
    }

    $w.iracqOs-main.uifFrame configure -relief sunken -borderwidth 2

    set nb [iracqClassNoteBookCreate [$w.iracqOs-main component uifFrame]]
    
    iracqClassNoteBookAdd $nb iracqReadMode_uifClass ReadMode \
	    $instance\_rm $dbPoint

    iracqClassNoteBookAdd $nb iracqFrames_uifClass Frames \
	    $instance\_frm $dbPoint

    iracqClassNoteBookAdd $nb iracqParam_uifClass Param \
	    $instance\_param $dbPoint

    iracqClassNoteBookAdd $nb iracqDit_uifClass DIT \
	    $instance\_dit $dbPoint

    iracqClassNoteBookAdd $nb iracqWindow_uifClass Window \
	    $instance\_win $dbPoint

    iracqClassNoteBookAdd $nb iracqStatus_uifClass Status \
	    $instance\_stat $dbPoint
    
    iracqClassNoteBookAdd $nb iracqPerf_uifClass PerfMon \
	    $instance\_perf $dbPoint
    
    iracqClassNoteBookInit $nb

    set wnb [iracqClassNoteBookPath $nb ReadMode]
    iracqClassInitReadMode $server $detIdx $dbPoint \
	    $instance\_rm $wnb.$instance\_rm.cv

    set wnb [iracqClassNoteBookPath $nb Frames]
    iracqClassInitFrames $server $detIdx $dbPoint \
	    $instance\_frm $wnb.$instance\_frm.cv

    set wnb [iracqClassNoteBookPath $nb Param]
    iracqClassInitParam $server $detIdx $dbPoint \
	    $instance\_param $wnb.$instance\_param.cv

    set wnb [iracqClassNoteBookPath $nb DIT]
    iracqClassInitDit $server $detIdx $dbPoint \
	    $instance\_dit $wnb.$instance\_dit.cv
    
    set wnb [iracqClassNoteBookPath $nb Window]
    iracqClassInitWindow $server $detIdx $dbPoint \
	    $instance\_win $wnb.$instance\_win.cv
    
    set wnb [iracqClassNoteBookPath $nb PerfMon]
    iracqClassInitPerf $server $detIdx $dbPoint \
	    $instance\_perf $wnb.$instance\_perf.cv
    
    
}
#
# ___oOo___
