#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqSTARTUP.tcl,v 1.113 2005/09/13 07:27:21 vltsccm Exp $"
#
# iracqSTARTUP.tcl
#
# who        when      what
# ---------- --------  ---------------------------------------------
# jstegmei   02/06/99  created
# jstegmei   13/12/99  support multiple server instances
# jstgemei   15/12/99  added iracqWaitExit method
# jstegmei   04/01/00  increased server start-up timeout
# jstegmei   04/07/00  added CheckVersion method (for NOCCS)
# jstegmei   24/04/02  added command line option for panel
# jstegmei   29/07/03  fixed bug: hardcoded instance number when stopping
#                      GUI
# jstegmei   11/11/03  updated check for running CCS environment
#
#************************************************************************
#   NAME
#   iracqSTARTUP - class for starting/stopping the IRACE acquisition 
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#			
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#/*


#************************************************************************
#*# class: iracqSTARTUP
#------------------------------------------------------------------------

class iracqSTARTUP {

    inherit itk::Widget
    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    common seq_ccsCmdCheck

    constructor {args} {
	eval itk_initialize $args
	wm withdraw .

	ParseArguments
	
	CheckEnvironments 

	Initialize
    }

    #****************************************************************
    #*# destructor
    #----------------------------------------------------------------

    destructor {
    }

    #****************************************************************
    #*# method: Exec (will exit on error)
    #----------------------------------------------------------------
    private method Exec {cmd {bg ""}} {	

	Verbose "Executing: $cmd..."
	
	if {[catch {
	    set pid [eval "exec $cmd $bg "]
	    if {$bg == ""} {
		Verbose "$pid"
	    }
	} result]} {		
	    Error "$result"
	    return 0
	}

	return 1
    }

    #****************************************************************
    #*# method: Exec1 (will not exit on error but issue a warning)
    #----------------------------------------------------------------
    private method Exec1 {cmd {bg ""}} {	

	Verbose "Executing: $cmd..."
	
	if {[catch {
	    set pid [eval "exec $cmd $bg "]
	    if {$bg == ""} {
		Verbose "$pid"
	    }
	} result]} {		
	    Warning "$result"
	    return 0
	}

	return 1
    }

    #****************************************************************
    #*# method: LocalSend
    #----------------------------------------------------------------
    private method LocalSend {procName command text} {
	if {$instance == ""} {
	    set regName $procName
	} else {
	    set regName "$procName\_$instance"
	}
	set cmdId [seq_msgSendCommand "" $regName $command "$text"]
	set reply [seq_msgRecvReply $cmdId errorNumber -last]
	return $reply
    }

    #****************************************************************
    #*# method: GetPid
    #----------------------------------------------------------------
    private method GetPid {proc} {
	global ::env
	
	# check process name
	if {[catch {set result \
		[exec ps -u $env(USER) -ef \
		| grep "iracqServerNoCcs" \
		| grep -v grep \
		| grep -v "iracqCtrl" \
		| grep -v "iracqStart" \
		| grep -v "iracqStop" \
		| grep -v "display" \
		     | grep "$env(USER) "]}]} {
	    return 0
	}

	if { $result != "" } {
	    set pid  [lindex $result 1]
	    set parentPid  [lindex $result 2]
	    	    
	    return $pid
	}
	
	return 0    
    }
 
    #****************************************************************
    #*# method: GetPidWish
    #----------------------------------------------------------------
    private method GetPidWish {proc} {
	global ::env
	
	# check process name
	if {$instance == ""} {
	    if {[catch {set result \
		    [exec ps -u $env(USER) -ef \
		    | grep "$proc" \
		    | grep -v grep \
		    | grep -v "\\-inst" \
		    | grep -v "iracqStart" \
		    | grep -v "iracqStop" \
		    | grep "$env(USER)" \
			 | grep -i wish]}]} {
		return 0
	    }
	} else {
	    if {[catch {set result \
		    [exec ps -u $env(USER) -ef \
		    | grep "$proc" \
		    | grep "\\-inst $instance" \
		    | grep -v grep \
		    | grep -v "iracqStart" \
		    | grep -v "iracqStop" \
		    | grep "$env(USER)" \
			 | grep -i wish]}]} {
		return 0
	    }
	}

	if { $result != "" } {
	    set pid  [lindex $result 1]
	    set parentPid  [lindex $result 2]
	    	    
	    return $pid
	}
	
	return 0    
    }

    #****************************************************************
    #*# method: CheckEnvironements
    #----------------------------------------------------------------
    private method CheckEnvironments {} {
	global ::env

	Verbose "Checking environments..."

	# check, whether RTAP is running
	if {!$noccs} {
	    if {![catch {set RTAPENV $env(RTAPENV)}]} {
		if {![catch \
			"exec ps -ef | grep Scheduler | grep $env(USER) | grep -v grep" \
			msg]} {
		    if {$msg != ""} {
		    } else {
			set noccs 1
		    }
		} else {
		    set noccs 1
		}
	    } else {
		set noccs 1
	    }
	}

	if {$noccs && $serverName == "iracqServer"} {
	    set serverName "iracqServerNoCcs"
	}
    }
    
    #****************************************************************
    #*# method: ParseArguments
    #----------------------------------------------------------------
    private method ParseArguments {} {
	global ::gvar

	set noccs 0
	set sim 0
	set gui 0
	set verbose 0

	set cnt 0
	while {$itk_option(-argc) != $cnt} {
	    set a [lindex $itk_option(-argv) $cnt]
	    incr cnt
	    switch -- $a {
		"-gui" {
		    set gui 1
		}
		"-sim" {
		    set sim 1
		}
		"-man" {
		    iracqExtHelp
		    if {$gvar(iracqHelpWdg) != "" && \
			    [winfo exists $gvar(iracqHelpWdg)]} {
			tkwait variable gvar(iracqHelpDestroy)
		    }
		    exit 0
		}
		"-noccs" {
		    set noccs 1
		}
		"-dtt" {
		    set dtt [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-cfg" {
		    set cfg [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-det" {
		    set det [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-verbose" { 
		    set verbose [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-log" { 
		    set log [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-inst" { 
		    set instance [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-xterm" {
		    set xterm 1
		}
		"-server" { 
		    set serverName [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-panel" { 
		    set panelName [lindex $itk_option(-argv) $cnt]; incr cnt
		    set gui 1
		}
		"-port" { 
		    set cmdPort [lindex $itk_option(-argv) $cnt]; incr cnt  
		}
		"-load" { 
		    incr cnt  
		}
		"-scratch" { 
		}
		default {
		    Usage
		}
	    }
	}	
    }

    #****************************************************************
    #*# method: Verbose
    #----------------------------------------------------------------
    private method Verbose {text} {
	if {$verbose} {Log $text}
    }

    #****************************************************************
    #*# method: Warning
    #----------------------------------------------------------------
    private method Warning {text} {
	Log "Warning: $text"
    }

    #****************************************************************
    #*# method: Error
    #----------------------------------------------------------------
    private method Error {text} {
	Log "Error: $text"
	exit 1
    }

    #****************************************************************
    #*# method: Log
    #----------------------------------------------------------------
    private method Log {text} {
	global ::env
	
	puts "$text"
    }

    #****************************************************************
    #*# method: WaitExit
    #----------------------------------------------------------------
    private method WaitExit {process {timeout 5}} {
	global ::env ::gvar
	Verbose "Waiting for exit: $process..."

	if {$instance == ""} {
	    set regName $process
	} else {
	    set regName "$process\_$instance"
	}

	if {![iracqWaitExit $regName $timeout]} {
	    return 0
	}

	return 1
    }

    #****************************************************************
    #*# method: CheckVersion
    #----------------------------------------------------------------
    private method CheckVersion {process} {
	global ::env ::gvar

	if {!$noccs} {
	    # send version command
	    if {[catch {LocalSend $process VERSION "" }]} {
		return 0
	    }

	    return 1
	} else {
	    if {$cmdPort == 0} {
		return 1
	    }

	    if {[catch {set srvHost $env(HOST)}]} {
		return 1
	    }

	    # open server socket
	    if {[catch {set sockFd \
		    [socket $srvHost $cmdPort]}]} {
		return 0
	    }
	    
	    # get version string
	    if {[catch {set version [gets $sockFd]}]} {
		close $sockFd
		return 0
	    }
	    
	    # get event port number
	    if {[catch {set evtPort [gets $sockFd]}]} {
		close $sockFd
		return 0
	    }
	    
	    # get command file descriptor
	    if {[catch {set cmdFd [gets $sockFd]}]} {
		close $sockFd
		return 0
	    }
	    close $sockFd
	    return 1
	}
    }

    #****************************************************************
    #*# method: CheckProcess
    #----------------------------------------------------------------
    private method CheckProcess {process {timeout 5}} {
	global ::env ::gvar
	global seq_errLogging

	set seq_errLogging "off"

	Verbose "Checking process: $process..."

	set downCounter [expr {$timeout * 5}]

	while {$downCounter > 0} {
	    if {![CheckVersion $process]} {
		set downCounter [expr {$downCounter - 1}]
		after 200
	    } else {
		set seq_errLogging "auto"
		return 1
	    }
	}
	
	set seq_errLogging "auto"
	return 0
    }

    #****************************************************************
    #*# method: Initialize
    #----------------------------------------------------------------
    public method Initialize {} {
	global ::env ::gvar

	if {!$noccs} {
	    # ccs-init
	    if {[catch {seq_ccsInit $itk_option(-name)} result]} {
		Error "Could not initialize with RTAP environment \
			$env(RTAPENV)"
	    }
	    
	    set seq_ccsCmdCheck CHECK_ALL
	}
    }
    
    #****************************************************************
    #*# method: Usage
    #----------------------------------------------------------------
    public method Usage {} {
	Log "\n$itk_option(-name) options:\n"
	Log "    -man                 - just show online manual"
	Log "    -verbose <level>     - set verbose level"
	Log "                           (default: 0)"
	Log "    -log <level>         - set log level"
	Log "                           (default: 0)"
	Log "    -inst <instance>     - server instance"
	Log "                           (default: <>)"
	Log "    -xterm               - start processes in xterms"
	Log "    -cfg <name>          - system configuration file name"
	Log "                           (default: sys.cfg)"
	Log "    -det <name>          - detector configuration file name"
	Log "    -sim                 - start server in simulation mode"
	Log "    -gui                 - starts also the control panel"
	Log "    -noccs               - do not use CCS"
	Log "    -port <server port>  - specify the command server port"
	Log "                           (only for maintenance and NOCCS)"  
	Log "    -server <serverName> - uses <serverName> as control server"
	Log "                           (default: iracqServer/iracqServerNoCcs)"
	Log "    -panel <panelName>   - uses <panelName> as control panel"
	Log "                           (default: iracqCtrl)"
	Log "    -dtt <name>          - process name of data transfer task"
	Log "                           (default: iracqDtt)"
	Log "    -load <mymod>        - load uif-classes of <mymod> into panel"
	Log "                           (this might be repeated)"
	Log ""  
	exit 0
    }
    
    #****************************************************************
    #*# method: ServerName
    #----------------------------------------------------------------
    public method ServerName {name} {
	set serverName $name
	Verbose "Server is $name..."
    }
    
    #****************************************************************
    #*# method: LoadCdts
    #----------------------------------------------------------------
    public method LoadCdts {} {
	if {!$noccs} {
	    Log "Loading CDT's..."

	    Exec "cmdSetup -parse $serverName"

	    if {$itk_option(-cdts) != ""} {
		foreach cdt $itk_option(-cdts) {
		    Exec "cmdSetup -parse $cdt"
		}
	    } 
	}
    }
    
    #****************************************************************
    #*# method: StartServers
    #----------------------------------------------------------------
    public method StartServer {} {
	global ::env

	StopServer

	Log "Starting Server..."

	set cmd "$serverName" 
	if {$sim} {
	    set cmd "$cmd -sim"
	}
	if {$xterm} {
	    set cmd "$cmd -xterm"
	}
	if {$xterm && $verbose} {
	    set cmd "$cmd -v $verbose"
	}
	if {$log} {
	    set cmd "$cmd -log $log"
	}
	if {$dtt != ""} {
	    set cmd "$cmd -dtt $dtt"
	}
	if {$cfg != ""} {
	    set cmd "$cmd -cfg $cfg"
	}
	if {$det != ""} {
	    set cmd "$cmd -det $det"
	}

	if {$noccs} {
	    if {$cmdPort == 0} {
		if {[catch {set cmdPort $env(IRACQ_PORT)}]} {
		    set cmdPort 0
		}
	    }
	    
	    set cmd "$cmd -port $cmdPort"
	} else {
	    if {$instance != ""} {
		set cmd "$cmd -inst $instance -port 0"
	    }
	}
	
	# now execute the command
	if {$xterm} {
	    Exec "xterm -geometry 75x15+0+300 \
		    -display $env(DISPLAY) -e $cmd" &
	} else {
	    Exec "$cmd >&/dev/null" &
	}

	if {!$noccs} {
	    if {$itk_option(-cdts) != ""} {
		foreach cdt $itk_option(-cdts) {
		    Exec "$cdt >&/dev/null" &
		}
	    } 
	}
    }

    #****************************************************************
    #*# method: StopServers
    #----------------------------------------------------------------
    public method StopServer {} {
	global seq_errLogging


	Log "Stopping Server..."
	if {!$noccs} {
	    set seq_errLogging "off"
	    if {[catch {LocalSend $serverName EXIT "" }]} {
		Warning "Could not communicate to $serverName \
			- assuming not running"
	    } else {
		if {![WaitExit $serverName]} {
		    Error "Error stopping server <$serverName>"
		}
	    }

	    if {$itk_option(-cdts) != ""} {
		foreach cdt $itk_option(-cdts) {
		    if {[catch {LocalSend $cdt EXIT "" }]} {
			Warning "Could not communicate to $cdt \
				- assuming not running"
		    } 
		}
	    set seq_errLogging "auto"
	    }

	} else {	    
	    set pid [GetPid "$serverName"]
	    if {$pid != 0} {
		Exec1 "kill $pid"
	    }
	}

    }

    #****************************************************************
    #*# method: CheckServer
    #----------------------------------------------------------------
    public method CheckServer {} {

	if {![CheckProcess $serverName 60]} {
	    Warning "Could not communicate to $serverName"
	} else {
	    Verbose "$serverName running properly..."
	}
	
	if {$itk_option(-cdts) != ""} {
	    foreach cdt $itk_option(-cdts) {
		if {![CheckProcess $cdt 60]} {
		    Warning "Could not communicate to $cdt"
		} else {
		    Verbose "$cdt running properly..."
		}
	    }
	}
    }
    
    #****************************************************************
    #*# method: StartGui
    #----------------------------------------------------------------
    public method StartGui {} {
	global ::env

	if {$gui} {    

	    StopGui

	    Log "Starting gui..."

	    if {!$noccs} {
		set cmdArg "$itk_option(-argv)"
		if {$instance != ""} {
		    # this mus be done to ensure that -inst is 
		    # the first option
		    set cmdArg "-inst $instance $cmdArg"
		}
	    } else {
		set cmdArg "$itk_option(-argv)"    
	    }
	    
	    # now start the panel
	    iracqStartPanel $panelName $cmdArg
	}
    }

    #****************************************************************
    #*# method: StopGui
    #----------------------------------------------------------------
    public method StopGui {} {
	global ::env ::gvar

	if {$gui} {    
	    set pid [GetPidWish $panelName]
	    if {$pid != 0} {
		Exec1 "kill $pid"
	    }
	}
    }

    #****************************************************************
    #*# Public variables
    #----------------------------------------------------------------
    itk_option define -name         name        Name        "iracqSTARTUP"
    itk_option define -argc         argc        Argc        0
    itk_option define -argv         argv        Argv        ""
    itk_option define -cdts         cdts        Cdts        "" 
    
    #****************************************************************
    #*# Private variables
    #----------------------------------------------------------------    
    private variable RTAPENV     ""
    private variable gui         0
    private variable verbose     0
    private variable log         0
    private variable xterm       0
    private variable sim         0
    private variable noccs       0
    private variable serverName  "iracqServer"
    private variable panelName   "iracqCtrl"
    private variable dtt         ""
    private variable cfg         ""
    private variable det         ""
    private variable cmdPort     0
    private variable instance    ""
    private variable error       0
}

#************************************************************************

# ___oOo___
