#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifHelp.tcl,v 1.113 2005/09/13 07:27:24 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  01/09/00  created
# jstegmei  20/12/00  fixed bug in ON-LINE help search
# jstegmei  20/12/00  improved search function
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc iracqForAllMatches {w pattern script} {

    # search keyword in text and execute script for all matches
    scan [$w index end] %d numLines
    for {set i 1} {$i < $numLines} {incr i} {
	$w mark set last $i.0
	while {[regexp -nocase -indices $pattern \
		[$w get last "last lineend"] indices]} {
	    $w mark set first \
		    "last + [lindex $indices 0] chars"
	    $w mark set last "last + 1 chars \
		    + [lindex $indices 1] chars"
	    uplevel $script
	}
    }
}

proc iracqSearchFile {fileName} {
    global env

    if {[string index $fileName 0] == "/"} {
	# absolute path
	if {[file exists $fileName]} {
	    return "$fileName"
	} else {
	    return ""
	}
	
    }

    # search in $INS_ROOT
    if {[catch {set INS_USER $env(INS_USER)}]} {
	set INS_USER "SYSTEM"
    }
    if {![catch {set INS_ROOT $env(INS_ROOT)}]} {
	if {[file exists $INS_ROOT/$INS_USER/MISC/$fileName]} {
	    return "$INS_ROOT/$INS_USER/MISC/$fileName"
	}
    }

    # check extension
    set subDir "config"
    set ext [file extension $fileName]
    if {$ext == ""} {
	set subDir "bin"
    } else {
	switch -glob -- $ext {
	    ".class" -
	    ".db" {
		set subDir "dbl"
	    }
	    ".xbm" -
	    ".gif" {
		set subDir "bitmaps"
	    }
	}
    }

    # search in module
    if {[file exists ../$subDir/$fileName]} {
	return "../$subDir/$fileName"
    }

    if {$subDir == "config"} {
	set subDir "config/irace"
    }

    # search in $INTROOT
    if {![catch {set INTROOT $env(INTROOT)}]} {
	if {[file exists $INTROOT/$subDir/$fileName]} {
	    return "$INTROOT/$subDir/$fileName"
	}
    }

    # search in $VLTROOT
    if {![catch {set VLTROOT $env(VLTROOT)}]} {
	if {[file exists $VLTROOT/$subDir/$fileName]} {
	    return "$VLTROOT/$subDir/$fileName"
	}
    }

    return ""
}

proc iracqReplaceEnv {repString} {
    global env gvar
    upvar $repString result

    if {![info exists result]} {
	set result ""
	return
    }

    if {$result == ""} {
	return
    }
    set tmpStr $result

    regsub -all -- {\$gvar} $tmpStr "gvar" x1
    regsub -all -- {\$[^\$/]*} $x1 "\$env(&)" x2
    regsub -all -- {(env\(\$)} $x2 "env\(" x3
    regsub -all -- {gvar} $x3 "\$gvar" x4
    set tmpStr $x4

    while {[regexp -indices -- {(gvar\([^\)]*\))} $tmpStr idx]} {
	set i1 [lindex $idx 0]
	set i2 [lindex $idx 1]
	set subStr [string range $tmpStr $i1 $i2]
	set tmpStr [string range $tmpStr $i2 end]
	set tlist [trace vinfo $subStr]
	set ele [lsearch $tlist {w iracqHelpUpdate}]
	if {$ele != -1} {
	    set tlist [lindex $tlist $ele]
	    set op [lindex $tlist 0]
	    set command [lindex $tlist 1]
	    trace vdelete $subStr $op $command
	} else {
	    lappend gvar(iracqHelpGvarList) $subStr
	}
	trace variable $subStr w {iracqHelpUpdate}
    }

    eval set result $x4
}

proc iracqHelpUpdate {name1 name2 op} {
    global gvar
    
    # store current topic
    set lastTopic $gvar(iracqHelpTopic)

    # reset stack
    $gvar(iracqHelpCtrl).fwd configure -state disabled
    $gvar(iracqHelpCtrl).bck configure -state disabled
    iracqStackReset helpTopic

    # reinitialize
    iracqHelpInit $gvar(iracqHelpFile)
    
    # set index
    iracqHelpIndex

    # restore last topic
    iracqHelpTopic $lastTopic 0

    # push topic to browse-stack
    iracqStackClear helpTopic
    iracqStackPush helpTopic $gvar(iracqHelpTopic)
    if {[iracqStackSize helpTopic] > 1} {
	$gvar(iracqHelpCtrl).bck configure -state normal
    }
    iracqStackDown helpTopic
    $gvar(iracqHelpCtrl).fwd configure -state disabled
}

proc iracqHelpInit {helpFile} {
    global gvar env
    global iracqHelpDb

    set gvar(iracqHelpTopic) -1
    set gvar(iracqHelpTopicSet) ""
    set gvar(iracqHelpHeading) ""
    set iracqHelpDb(index) ""
    set iracqHelpDb(keys) ""
    set iracqHelpDb(text) ""
    set iracqHelpDb(type) ""
    set iracqHelpDb(pos) ""
    set iracqHelpDb(size) 0
    set gvar(iracqHelpDestroy) 0
    set gvar(iracqHelpFile) $helpFile
    
    set type 0

    if {![file readable $helpFile]} {
	iracqVB "no help available -\
		$helpFile is not readable..."
	return 0
    }

    # read file
    set first 1
    set next 1
    if {[catch {set fd [open $helpFile r]} result]} {
	iracqVB "no help available - $result..."
	return 0
    }
    while {[gets $fd line] >= 0} {
	set char0 [string index [string trim $line] 0]
	if {$char0 == "#" && $first} {
	    # ignore comment
	    continue
	}
	if {$char0 == "" && $first} {
	    # ignore first empty lines
	    continue
	}

	if {$char0 == "!"} {
	    set str [string range $line 1 end]
	    set str [string trim $str]
	    if {[string range $str 0 5] == "IMPORT"} {
		set importFile [string trim \
			[string range $str 6 end] " \""]
		if {![catch {iracqReplaceEnv importFile} result]} {
		    if {$importFile  == ""} {
			continue
		    }
		    set res [iracqSearchFile $importFile]
		    if {$res == ""} {
			iracqVB "$importFile not found..."
			continue
		    }
		    set importFile $res
		    
		    if {![file readable $importFile]} {
			iracqVB "$importFile is not readable..."
			continue
		    }
		    if {[catch {set fdi [open $importFile r]} result]} {
			iracqVB "$result"
			continue
		    }
		    
		    set helpBuf "$helpBuf\n\<!<"
		    while {[gets $fdi importLine] >= 0} {
			set helpBuf "$helpBuf\n$importLine"
		    }
		    
		    close $fdi
		    set helpBuf "$helpBuf\n\>!>"
		}
		continue
	    }

	    if {$next} {
		if {!$first} {
		    # next topic -> store previous
		    iracqHelpAdd $heading $keys $helpBuf $type
		    set type 0
		}
		set first 0
		set helpBuf ""
		set keys ""
		# first line is heading
		set heading [string range $line 1 end]
		set heading [string trim $heading]
		set next 0
	    } else {
		# add search keywords
		set line [string range $line 1 end]
		set line [string trim $line]
		set tlist [split $line " "]
		set keys [concat $keys $tlist]
	    }
	} else {
	    if {![info exists helpBuf]} {
		iracqVB "no help available -\
			help-file has invalid format..."
		close $fd
		return 0
	    }

	    # add free help-text
	    set last1 [string index $helpBuf \
			   [expr {([string length $helpBuf] - 1)}]]
	    if {[string trim $line] == ""} {
		if {$last1 == "\n"} {
		    set helpBuf "$helpBuf\n"
		} else {
		    set helpBuf "$helpBuf\n\n"
		}
	    } else {
		set firstChar [string index $line 0]
		if {$firstChar == " " || $firstChar == "\t"} {
		    if {$last1 != "\n"} {
			set helpBuf "$helpBuf\n"
			set last1 "\n"
		    }
		}
		if {!$next || $last1 == "\n" || $last1 == ""} {
		    set helpBuf "$helpBuf$line"
		} else {
		    set helpBuf "$helpBuf $line"
		}
	    }
	    set next 1
	}
    }
    close $fd

    if {$first} {
	iracqVB "no help available -\
		help-file has invalid format (no topic found)..."
	return 0
    }

    # store last topic
    iracqHelpAdd $heading $keys $helpBuf $type
    
    # setup index topic
    iracqHelpAdd "Available Topics:" "" $iracqHelpDb(index) 1

    return 1
}

proc iracqHelpAdd {heading keys text {type 0}} {
    global gvar
    global iracqHelpDb

    if {[lsearch $iracqHelpDb(index) $heading] != -1} {
	# topic does already exist (do not overwrite)
	return
    }

    lappend iracqHelpDb(index) $heading
    lappend iracqHelpDb(keys) $keys
    lappend iracqHelpDb(text) $text
    lappend iracqHelpDb(type) $type
    lappend iracqHelpDb(pos) 0.0
    set iracqHelpDb(size) [llength $iracqHelpDb(index)]
}

proc iracqHelpSelect {text} {
    global gvar
    global iracqHelpSelTag

    if {![info exists iracqHelpSelTag]} {
	set iracqHelpSelTag 0
    }
    if {$iracqHelpSelTag == 0} {
	set iracqHelpSelTag 1
	$gvar(iracqHelpWdg) insert end \
		"$text" {choose bold1 act1} "\n" {}
    } else {
	set iracqHelpSelTag 0
	$gvar(iracqHelpWdg) insert end \
		"$text" {choose bold1 act2} "\n" {}
    }
}

proc iracqHelpIndex {} {
    global gvar
    global iracqHelpDb

    # choose index topic
    iracqHelpChooseTopic "Available Topics:"
}

proc iracqHelpSearch {name} {
    global gvar
    global iracqHelpDb

    if {$name == ""} {
	return
    }

    regsub -all -- "-" [string toupper $name] " " searchName

    # search topics and create list containing all matches
    set topicList ""
    for {set i 0} {$i < $iracqHelpDb(size)} {incr i 1} {
	set heading [lindex $iracqHelpDb(index) $i]
	set keys [lindex $iracqHelpDb(keys) $i]
	set found 0
	foreach ele $keys {
	    foreach searchEle $searchName {
		if {[string match "$searchEle\*" "$ele\S"] || \
			[string match "$searchEle" "$ele\ES"]} {
		    lappend topicList $heading
		    set found 1
		    break
		}
	    }
	    if {$found} {
		break
	    }
	}
    }

    if {$topicList == ""} {
	$gvar(iracqHelpWdg) insert end "No matching item found!" {bold1}
    }

    # add selection topic
    iracqHelpAdd "Search Result for $name:" "" $topicList 1
    iracqHelpChooseTopic "Search Result for $name:"
}

proc iracqHelpChooseTopic {heading {type 0}} {
    global gvar
    global iracqHelpDb

    # choose topic by heading
    set index -1
    foreach ele $iracqHelpDb(index) {
	if {[regexp -nocase $heading $ele]} {
	    set index [lsearch -exact $iracqHelpDb(index) $ele]
	    break
	}
    }
    if {$index < 0} {
	return
    }

    # set new topic
    iracqHelpTopic $index

    # push topic to browse-stack
    iracqStackClear helpTopic
    iracqStackPush helpTopic $gvar(iracqHelpTopic)
    if {[iracqStackSize helpTopic] > 1} {
	$gvar(iracqHelpCtrl).bck configure -state normal
    }
    iracqStackDown helpTopic
    $gvar(iracqHelpCtrl).fwd configure -state disabled
}

proc iracqHelpTopic {topic {clrSel 1}} {
    global gvar
    global iracqHelpDb

    if {$topic == "" || $topic < 0 || $topic >= $iracqHelpDb(size)} {
	return
    }

    # store position
    if {$gvar(iracqHelpTopic) >= 0} {
	set oldPos  [lindex [$gvar(iracqHelpWdg) yview] 0]
	set iracqHelpDb(pos) [lreplace $iracqHelpDb(pos) \
		$gvar(iracqHelpTopic) $gvar(iracqHelpTopic) $oldPos]
    }

    # set new topic
    set heading [lindex $iracqHelpDb(index) $topic]
    set keys [lindex $iracqHelpDb(keys) $topic]
    set text [lindex $iracqHelpDb(text) $topic]
    set type [lindex $iracqHelpDb(type) $topic]
    set pos [lindex $iracqHelpDb(pos) $topic]

    $gvar(iracqHelpWdg) configure -state normal
    $gvar(iracqHelpWdg) delete 1.0 end

    # insert heading
    if {[regsub "Search Result for" \
	    [string trim $heading " :!"] "" newHeading]} {
	set newHeading "$newHeading\?"
    }
    if {[regsub -nocase -- "APPENDIX:" \
	    $heading "" newHeading]} {
	set newHeading "A:$newHeading"
    }
    set gvar(iracqHelpHeading) $newHeading

    $gvar(iracqHelpWdg) insert end "$heading" {heading2} "\n" {}
    
    if {$type == 0} {
	# insert help text
	$gvar(iracqHelpWdg) insert end "$text" {} "\n" {}

	set i1 ""
	set i2 ""
	iracqForAllMatches $gvar(iracqHelpWdg) {<!<} {
	    lappend i1 [$gvar(iracqHelpWdg) index first]
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {>!>} {
	    lappend i2 [$gvar(iracqHelpWdg) index first]
	}

	if {[llength $i1] > 0} {
	    for {set i 0} {$i < [llength $i1]} {incr i} {
		set a [lindex $i1 $i]
		set b [lindex $i2 $i]
		$gvar(iracqHelpWdg) tag add teleType $a $b
	    }
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {<!<} {
	    $gvar(iracqHelpWdg) delete first last
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {>!>} {
	    $gvar(iracqHelpWdg) delete first last
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {\"[^\"]*\"|'[^\']*'} {
	    $gvar(iracqHelpWdg) tag add italic1 first last
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {^[0-9]*\..*} {
	    $gvar(iracqHelpWdg) tag add bold1 first last
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {~.*:\ *$} {
	    $gvar(iracqHelpWdg) tag add bold1 first last
	    $gvar(iracqHelpWdg) delete first 
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {<I\ .*>} {
	    set idx [$gvar(iracqHelpWdg) index first]
	    set imgName [$gvar(iracqHelpWdg) get \
		    "first + 3 chars" "last -1 chars"]
	    $gvar(iracqHelpWdg) delete first last
	    iracqInsertImage $idx $imgName
	}

	iracqForAllMatches $gvar(iracqHelpWdg) {@[^@]*@} {
	    $gvar(iracqHelpWdg) tag add ref first last
	    $gvar(iracqHelpWdg) delete first 
	    $gvar(iracqHelpWdg) delete "last -1 chars"
	}
    } else {
	if {$type == 1} {
	    $gvar(iracqHelpWdg) insert end "\n" {}
	    # insert index for selection
	    foreach ele $text {
		iracqHelpSelect $ele
	    }
	} 
    }

    $gvar(iracqHelpWdg) yview moveto $pos

    $gvar(iracqHelpWdg) configure -state disabled

    set gvar(iracqHelpTopic) $topic

    if {$clrSel} {
	event generate $gvar(iracqHelpWdg) <Button-1>
    }
}

proc iracqInsertImage {idx imgName} {
    global gvar
    set res [iracqSearchFile $imgName]
    if {$res == ""} {
	iracqVB "$imgName not found..."
	$gvar(iracqHelpWdg) insert $idx ":$imgName:"
	return 0
    }

    set imgList [image names]
    set refName [file rootname [file tail $res]]

    if {[lsearch -exact $imgList $refName] == -1} {
	iracqVB "loading $res..."
	set w1 [winfo width $gvar(iracqHelpWdg)]
	set refName [image create photo \
		[file rootname [file tail $res]]]
	if {[catch {set ldName [image create photo \
		-file $res]}]} {
	    iracqVB "could not load $res..."
	    $gvar(iracqHelpWdg) insert $idx ":$imgName:"
	    return 0
	}
	set w2 [image width $ldName]
	if {$w2 > $w1} {
	    set s1 [expr {(int($w2 / $w1) + 1)}]
	} else {
	    set s1 1
	}
	$refName copy $ldName -subsample $s1 $s1
    }
    set img [$gvar(iracqHelpWdg) image create $idx \
	    -padx 10 -align center -image $refName]
    return 1
}

proc iracqHelpFwd {} {
    global gvar
    global iracqHelpDb

    set ele [iracqStackUp helpTopic]
    if {$ele == ""} {
	iracqStackDown helpTopic
    }
    iracqHelpTopic $ele
    if {[iracqStackIdx helpTopic] > 0} {
	$gvar(iracqHelpCtrl).bck configure -state normal
    }
    if {[iracqStackIdx helpTopic] >= \
	    [expr {[iracqStackSize helpTopic]-1}]} {
	$gvar(iracqHelpCtrl).fwd configure -state disabled
    }
}

proc iracqHelpBck {} {
    global gvar
    global iracqHelpDb

    set ele [iracqStackDown helpTopic]
    iracqHelpTopic $ele
    $gvar(iracqHelpCtrl).fwd configure -state normal
    if {[iracqStackIdx helpTopic] <= 0} {
	$gvar(iracqHelpCtrl).bck configure -state disabled
    }
}

proc iracqHelpDestroy {w} {
    global gvar

    bind $w <Destroy> {}

    set gvar(iracqHelpDestroy) 1
    
    foreach var $gvar(iracqHelpGvarList) {
	set tlist [trace vinfo $var]
	set ele [lsearch $tlist {w iracqHelpUpdate}]
	if {$ele != -1} {
	    set tlist [lindex $tlist $ele]
	    set op [lindex $tlist 0]
	    set command [lindex $tlist 1]
	    trace vdelete $var $op $command
	}
    }
}

proc iracqHelpPrint {} {
    global gvar
    global iracqHelpDb
}

proc iracqExtHelp {} {
    global gvar env
    global iracqHelpDb

    set w .iracqHelp

    if {[iracqPopUp $w]} {
	# widget does already exist
	return
    }

    set gvar(iracqHelpGvarList) ""
    set gvar(iracqHelpWdg) ""
    set gvar(iracqHelpCv) ""
    set gvar(iracqHelpCtrl) ""

    set fileName [iracqSearchFile "IRACE/SYS/iracqHelp.hlp"]
    if {$fileName == ""} {
	set fileName [iracqSearchFile "iracqHelp.hlp"]
	if {$fileName == ""} {
	    iracqVB "no help available -\
		    iracqHelp.hlp not found..."
	    return 0
	}
    }

    iracqVB "initializing online-help ($fileName)..."

    if {![iracqHelpInit $fileName]} {
	# initialization of help data failed (no help available)
	return
    }

    # create new toplevel widget
    toplevel $w
    bind $w <Destroy> "iracqHelpDestroy $w"
    wm iconname $w "Help"
    wm title $w "IRACE HELP"
    wm resizable $w 0 1 

    # top/bottom area
    frame $w.top -relief raised -bd 2 -height 60
    set gvar(iracqHelpCtrl) $w.top
    frame $w.bot -relief raised -bd 2 
    pack $w.top -side top -fill both -expand 0 
    pack $w.bot -side bottom -fill both -expand 1 

    # control widgets
    button $w.top.dismiss -text "Dismiss" \
	    -command "set gvar(iracqHelpDestroy) 1; destroy $w"

    button $w.top.index -text "Index" -command {
	iracqHelpIndex
    }
    button $w.top.fwd -text ">>" -state disabled -command {
	iracqHelpFwd
    }
    button $w.top.bck -text "<<" -state disabled -command {
	iracqHelpBck
    }
    button $w.top.search -text "Search" -command {
	if {$gvar(iracqHelpTopicSet) != ""} {
	    iracqHelpSearch $gvar(iracqHelpTopicSet)
	    set gvar(iracqHelpTopicSet) ""
	}
    }
    entry $w.top.searchEntry -relief sunken -bd 2 -width 16 \
	    -textvariable gvar(iracqHelpTopicSet)
    bind $w.top.searchEntry <Return> {
	if {$gvar(iracqHelpTopicSet) != ""} {
	    iracqHelpSearch $gvar(iracqHelpTopicSet)
	    set gvar(iracqHelpTopicSet) ""
	}
    }
    entry $w.top.topic -relief ridge -bd 2 -width 34 \
	    -justify center -fg blue -state disabled \
	    -textvariable gvar(iracqHelpHeading) 

    # pack control widgets
    pack $w.top.dismiss $w.top.index $w.top.bck $w.top.fwd $w.top.search \
	    -side left -ipadx 1m
    pack $w.top.searchEntry  \
	    -side left -padx 2
    pack $w.top.topic  \
	    -side right -padx 2
    
    # help area
    set helpArea $w.bot.help
    #canvas $helpArea
    frame $helpArea
    pack $helpArea -side top -fill both -expand 1

    # help text widget
    text $helpArea.text -relief sunken -borderwidth 2\
	    -foreground black \
	    -background white \
	    -cursor left_ptr \
	    -selectforeground white \
	    -selectbackground black \
	    -selectborderwidth 0 \
	    -wrap none -state disabled \
	    -width 80 -height 40 \
	    -wrap word \
	    -yscrollcommand "$helpArea.scrolly set"  \
	    -font -Adobe-helvetica-medium-r-normal--14*
    scrollbar $helpArea.scrolly -command "$helpArea.text yview" \
	    -orient vertical -cursor left_ptr

    set gvar(iracqHelpWdg) $helpArea.text
    
    # create/bind tags
    $helpArea.text tag configure italic1 \
	    -font -Adobe-helvetica-medium-o-normal--14*

    $helpArea.text tag configure italic2 \
	    -font -Adobe-helvetica-medium-o-normal--18*

    $helpArea.text tag configure heading1 \
	    -font -Adobe-helvetica-bold-r-normal--14* \
	    -underline 1

    $helpArea.text tag configure heading2 \
	    -font -Adobe-helvetica-bold-r-normal--18* \
	    -underline 1

    $helpArea.text tag configure bold1 \
	    -font -Adobe-helvetica-bold-r-normal--14*

    $helpArea.text tag configure bold2 \
	    -font -Adobe-helvetica-bold-r-normal--18*

    $helpArea.text tag configure teleType \
	    -font -Adobe-courier-medium-o-normal--12* \
	    -tabs {56 left}

    $helpArea.text tag configure choose -foreground green4 \
	    -spacing1 4 -spacing3 4 -lmargin1 10 -lmargin2 10
    $helpArea.text tag configure ref -foreground green3
    $helpArea.text tag configure high -background green4 -foreground white

    $helpArea.text tag bind act1 <Enter> \
	    "$helpArea.text configure -cursor hand2; \
	    iracqSelectText $helpArea.text %x %y 1"
    $helpArea.text tag bind act1 <Leave> \
	    "$helpArea.text configure -cursor left_ptr; \
	    iracqSelectText $helpArea.text %x %y 0"
    $helpArea.text tag bind act2 <Enter> \
	    "$helpArea.text configure -cursor hand2; \
	    iracqSelectText $helpArea.text %x %y 1"
    $helpArea.text tag bind act2 <Leave> \
	    "$helpArea.text configure -cursor left_ptr; \
	    iracqSelectText $helpArea.text %x %y 0"

    $helpArea.text tag bind ref <Enter> \
	    "$helpArea.text configure -cursor hand2"
    $helpArea.text tag bind ref <Leave> \
	    "$helpArea.text configure -cursor left_ptr"
    $helpArea.text tag bind ref <Double-Button-1> {
	set curX %x
        set curY %y
	set i1 [$gvar(iracqHelpWdg) index "@$curX,$curY"]
	set idx [$gvar(iracqHelpWdg) tag prevrange ref \
		"$i1 + 1 chars" "$i1 linestart"]
	set selItem [$gvar(iracqHelpWdg) get [lindex $idx 0] [lindex $idx 1]] 
	if {$selItem != ""} {
	    iracqHelpChooseTopic $selItem
	}
    }

    bind $w <Prior> {
	$gvar(iracqHelpWdg) yview scroll -1 pages
    }
    bind $w <Next> {
	$gvar(iracqHelpWdg) yview scroll 1 pages
    }
    bind $w <Up> {
	$gvar(iracqHelpWdg) yview scroll -1 units
    }
    bind $w <Down> {
	$gvar(iracqHelpWdg) yview scroll 1 units
    }
    bind $w <Home> {
	$gvar(iracqHelpWdg) see 1.0
    }
    bind $w <End> {
	$gvar(iracqHelpWdg) see end
    }

    $helpArea.text tag bind choose <Double-Button-1> {
	set curX %x
        set curY %y
	set li [$gvar(iracqHelpWdg) index "@$curX,$curY linestart"]
	set selItem [$gvar(iracqHelpWdg) get $li "$li lineend"]
	if {$selItem != ""} {
	    iracqHelpChooseTopic $selItem
	}
    }

    # pack help text widgets
    #$helpArea create window 0 0 -window $helpArea.text
    pack $helpArea.scrolly -side right -fill y -expand 0
    pack $helpArea.text -side left -fill both -expand 1 

    set gvar(iracqHelpWdg) $helpArea.text
    set gvar(iracqHelpCv) $helpArea
    
    # show index after creation of widget
    iracqHelpIndex
}


#
# ___oOo___
