#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifVolt.tcl,v 1.113 2005/09/13 07:27:26 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  18/06/99  created
# jstegmei  21/21/99  fixed bug: allow spaces in voltage names (NOCCS only)
#
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


proc iracqEnableDacCtrl {} {
    global gvar
    global cv

    $cv.iracqDacEnable enable    
}

# restore original value
proc iracqVoltRestore {} {
    global gvar cv
    global iracqVset
    
    if {$gvar(iracqSetVoltValueStore) != ""} {
	if {[iracqCmdVoltChan $gvar(iracqSetVoltChan) \
		$gvar(iracqSetVoltValueStore) \
		0]} {
	    set gvar(iracqSetVoltValue) $gvar(iracqSetVoltValueStore)
	    set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
	    $cv.iracqVoltTuner setValue $gvar(iracqVoltT)
	    iracqUpdateTelValue
	}
    }
}

# update selected telemetry
proc iracqUpdateTelValue {} {
    global gvar

    if {$gvar(iracqSrvState) != "ONLINE"} {
	return 1
    }

    if {$gvar(iracqSetVoltChan) == ""} {
	return 1
    }
    
    if {$gvar(iracqSetTelChan1) != ""} {
	if {![iracqFunction "telchan $gvar(iracqSetTelChan1)" 0 tmpStr]} {
	    return 0
	}
    set gvar(iracqSetTelVal1) [iracqScanString [iracqSgets tmpStr]]
    }

    if {$gvar(iracqSetTelChan2) != ""} {
	if {![iracqFunction "telchan $gvar(iracqSetTelChan2)" 0 tmpStr]} {
	    return 0
	}
	set gvar(iracqSetTelVal2) [iracqScanString [iracqSgets tmpStr]]
    }
    return 1
}

proc iracqTrimName {value} {
    upvar $value string

    set i1 [string first "|" $string]
    if {$i1 == -1} {
	return
    }

    set name [string range $string 0 [expr {$i1 - 1}]]
    set name [string trim $name]
    regsub -all " " $name "_" name

    set string "$name [string range $string [expr {$i1 + 1}] end]"
}

#get next dc value from setup string
proc iracqScanDc {config idx} {
    upvar $config string
    global iracqVset

    # scan low value
    set tmpStr [iracqScanString [iracqSgets string]]
    iracqTrimName tmpStr
    scan $tmpStr "%s %f \[%f, %f\] %d %d" \
	    iracqVset(dcName_$idx)\
	    iracqVset(dcVal_$idx)\
	    iracqVset(dcR1_$idx)\
	    iracqVset(dcR2_$idx)\
            iracqVset(dcDac_$idx)\
            iracqVset(dcTel_$idx)
        
}

# get next clock value from setup string
proc iracqScanClk {config idx} {
    upvar $config string
    global iracqVset

    # scan low value
    set tmpStr [iracqScanString [iracqSgets string]]
    iracqTrimName tmpStr
    scan $tmpStr "%s %f \[%f, %f\] %d %d" \
	    iracqVset(ckLoName_$idx)\
	    iracqVset(ckLoVal_$idx)\
	    iracqVset(ckLoR1_$idx)\
	    iracqVset(ckLoR2_$idx)\
            iracqVset(ckLoDac_$idx)\
            iracqVset(ckLoTel_$idx)
    
    # scan high value
    set tmpStr [iracqScanString [iracqSgets string]]
    iracqTrimName tmpStr
    scan $tmpStr "%s %f \[%f, %f\] %d %d" \
	    iracqVset(ckHiName_$idx)\
	    iracqVset(ckHiVal_$idx)\
	    iracqVset(ckHiR1_$idx)\
	    iracqVset(ckHiR2_$idx)\
            iracqVset(ckHiDac_$idx)\
            iracqVset(ckHiTel_$idx)
}

# select clock low
proc iracqSelClkLo {i} {
    global iracqVset
    global gvar
    global cv

    set gvar(iracqVoltSelName) $iracqVset(ckLoName_$i)
    set gvar(iracqSetVoltId) "ckLoVal_$i"
    set gvar(iracqSetVoltR1) $iracqVset(ckLoR1_$i)
    set gvar(iracqSetVoltR2) $iracqVset(ckLoR2_$i)
    set gvar(iracqSetVoltChan) $iracqVset(ckLoDac_$i)
    set gvar(iracqSetTelChan1) $iracqVset(ckLoTel_$i)
    set gvar(iracqSetTelChan2) ""
    set gvar(iracqSetTelVal2) ""
    set value $iracqVset(ckLoVal_$i)
    iracqRound value 3
    set gvar(iracqSetVoltValue) $value
    set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
    $cv.iracqVoltTuner setValue $gvar(iracqVoltT)
    iracqUpdateTelValue
    set gvar(iracqSetVoltValueStore) $gvar(iracqSetVoltValue)
}

# select clock high
proc iracqSelClkHi {i} {
    global iracqVset
    global gvar
    global cv

    set gvar(iracqVoltSelName) $iracqVset(ckHiName_$i)
    set gvar(iracqSetVoltId) "ckHiVal_$i"
    set gvar(iracqSetVoltR1) $iracqVset(ckHiR1_$i)
    set gvar(iracqSetVoltR2) $iracqVset(ckHiR2_$i)
    set gvar(iracqSetVoltChan) $iracqVset(ckHiDac_$i)
    set gvar(iracqSetTelChan1) $iracqVset(ckHiTel_$i)
    set gvar(iracqSetTelChan2) ""
    set gvar(iracqSetTelVal2) ""
    set value $iracqVset(ckHiVal_$i)
    iracqRound value 3
    set gvar(iracqSetVoltValue) $value
    set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
    $cv.iracqVoltTuner setValue $gvar(iracqVoltT)
    iracqUpdateTelValue
    set gvar(iracqSetVoltValueStore) $gvar(iracqSetVoltValue)
}

# select dc-voltage
proc iracqSelDc {i} {
    global iracqVset
    global gvar
    global cv

    set gvar(iracqVoltSelName) $iracqVset(dcName_$i)
    set gvar(iracqSetVoltId) "dcVal_$i"
    set gvar(iracqSetVoltR1) $iracqVset(dcR1_$i)
    set gvar(iracqSetVoltR2) $iracqVset(dcR2_$i)
    set gvar(iracqSetVoltChan) $iracqVset(dcDac_$i)
    set gvar(iracqSetTelChan1) $iracqVset(dcTel_$i)
    set gvar(iracqSetTelChan2) [expr {$iracqVset(dcTel_$i) + 8}]
    set value $iracqVset(dcVal_$i)
    iracqRound value 3
    set gvar(iracqSetVoltValue) $value
    set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
    $cv.iracqVoltTuner setValue $gvar(iracqVoltT)
    iracqUpdateTelValue
    set gvar(iracqSetVoltValueStore) $gvar(iracqSetVoltValue)
}

# scan voltage setup string
proc iracqScanVsetup {setupStr} {
    global iracqVset
    global gvar
    global cv
    global i

    # skip 
    iracqSgets setupStr
    iracqSgets setupStr
    iracqSgets setupStr

    $gvar(iracqVoltSelOption) deleteEntry 0 last  

    for {set i 0} {$i<16} {incr i 1} {
	iracqScanClk setupStr $i
	if {$gvar(iracqSelClkV) == "CLK"} {
	    if {$iracqVset(ckLoDac_$i) != -1} {
		$gvar(iracqVoltSelOption) addItem $iracqVset(ckLoName_$i)\
			"iracqSelClkLo $i"
	    }
	    if {$iracqVset(ckHiDac_$i) != -1} {
		$gvar(iracqVoltSelOption) addItem $iracqVset(ckHiName_$i)\
			"iracqSelClkHi $i"
	    }
	}
    }

    iracqSgets setupStr
    iracqSgets setupStr
    iracqSgets setupStr

    for {set i 0} {$i<16} {incr i 1} {
	iracqScanDc setupStr $i
	if {$gvar(iracqSelClkV) == "DC"} {
	    if {$iracqVset(dcDac_$i) != -1} {
		$gvar(iracqVoltSelOption) addItem $iracqVset(dcName_$i)\
			"iracqSelDc $i"
	    }
	}
    }

    if {$gvar(iracqSetVoltId) != ""} {
	iracqUpdateVoltId $gvar(iracqSetVoltId)
	set gvar(iracqSetVoltValue) $iracqVset($gvar(iracqSetVoltId))
	set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
	$cv.iracqVoltTuner setValue $gvar(iracqVoltT)
	iracqUpdateTelValue
	set gvar(iracqSetVoltValueStore) $gvar(iracqSetVoltValue)
    }

    set gvar(iracqVsetupDone) 1
}

# update channel id
proc iracqUpdateVoltId {id} {
    global iracqVset
    global gvar

    for {set i 0} {$i<16} {incr i 1} {
	if {$id == "ckLoVal_$i"} {
	    set gvar(iracqVoltSelName) $iracqVset(ckLoName_$i)
	    set gvar(iracqSetVoltValue) $iracqVset(ckLoVal_$i)
	    set gvar(iracqSetVoltChan) $iracqVset(ckLoDac_$i)
	    set gvar(iracqSetTelChan1) $iracqVset(ckLoTel_$i)
	    set gvar(iracqSetTelChan2) ""
	    set gvar(iracqSetTelVal2) ""
	    set gvar(iracqSetVoltR1) $iracqVset(ckLoR1_$i)
	    set gvar(iracqSetVoltR2) $iracqVset(ckLoR2_$i)
	    return
	}
	if {$id == "ckHiVal_$i"} {
	    set gvar(iracqVoltSelName) $iracqVset(ckHiName_$i)
	    set gvar(iracqSetVoltValue) $iracqVset(ckHiVal_$i)
	    set gvar(iracqSetVoltChan) $iracqVset(ckHiDac_$i)
	    set gvar(iracqSetTelChan1) $iracqVset(ckHiTel_$i)
	    set gvar(iracqSetTelChan2) ""
	    set gvar(iracqSetTelVal2) ""
	    set gvar(iracqSetVoltR1) $iracqVset(ckHiR1_$i)
	    set gvar(iracqSetVoltR2) $iracqVset(ckHiR2_$i)
	    return
	}
    }
 
    for {set i 0} {$i<16} {incr i 1} {
	if {$id == "dcVal_$i"} {
	    set gvar(iracqVoltSelName) $iracqVset(dcName_$i)
	    set gvar(iracqSetVoltValue) $iracqVset(dcVal_$i)
	    set gvar(iracqSetVoltChan) $iracqVset(dcDac_$i)
	    set gvar(iracqSetTelChan1) $iracqVset(dcTel_$i)
	    set gvar(iracqSetTelChan2) [expr {$iracqVset(dcTel_$i) + 8}]
	    set gvar(iracqSetVoltR1) $iracqVset(dcR1_$i)
	    set gvar(iracqSetVoltR2) $iracqVset(dcR2_$i)
	    return
	}
    }

    return 
}

# update channel id
proc iracqUpdateVoltChan {channel} {
    global iracqVset
    global gvar

    if {$gvar(iracqVsetupDone) == 0} {
	set gvar(iracqSetVoltId) ""
	set gvar(iracqSetTelChan1) ""
	set gvar(iracqSetTelChan2) ""
	set gvar(iracqSetTelVal1) ""
	set gvar(iracqSetTelVal2) ""
	set gvar(iracqSetVoltValue) ""
	set gvar(iracqSetVoltR1) -10.0
	set gvar(iracqSetVoltR2) 10.0
	return
    }

    for {set i 0} {$i<16} {incr i 1} {
	if {$channel == $iracqVset(ckLoDac_$i)} {
	    set gvar(iracqVoltSelName) $iracqVset(ckLoName_$i)
	    set gvar(iracqSetVoltId) "ckLoVal_$i"
	    set gvar(iracqSetVoltValue) $iracqVset(ckLoVal_$i)
	    set gvar(iracqSetTelChan1) $iracqVset(ckLoTel_$i)
	    set gvar(iracqSetTelChan2) ""
	    set gvar(iracqSetTelVal2) ""
	    set gvar(iracqSetVoltR1) $iracqVset(ckLoR1_$i)
	    set gvar(iracqSetVoltR2) $iracqVset(ckLoR2_$i)
	    return
	}
	if {$channel == $iracqVset(ckHiDac_$i)} {
	    set gvar(iracqVoltSelName) $iracqVset(ckHiName_$i)
	    set gvar(iracqSetVoltId) "ckHiVal_$i"
	    set gvar(iracqSetVoltValue) $iracqVset(ckHiVal_$i)
	    set gvar(iracqSetTelChan1) $iracqVset(ckHiTel_$i)
	    set gvar(iracqSetTelChan2) ""
	    set gvar(iracqSetTelVal2) ""
	    set gvar(iracqSetVoltR1) $iracqVset(ckHiR1_$i)
	    set gvar(iracqSetVoltR2) $iracqVset(ckHiR2_$i)
	    return
	}
    }
 
    for {set i 0} {$i<16} {incr i 1} {
	if {$channel == $iracqVset(dcDac_$i)} {
	    set gvar(iracqVoltSelName) $iracqVset(dcName_$i)
	    set gvar(iracqSetVoltId) "dcVal_$i"
	    set gvar(iracqSetVoltValue) $iracqVset(dcVal_$i)
	    set gvar(iracqSetTelChan1) $iracqVset(dcTel_$i)
	    set gvar(iracqSetTelChan2) [expr {$iracqVset(dcTel_$i) + 8}]
	    set gvar(iracqSetVoltR1) $iracqVset(dcR1_$i)
	    set gvar(iracqSetVoltR2) $iracqVset(dcR2_$i)
	    return
	}
    }

    set gvar(iracqSetVoltId) ""
    set gvar(iracqSetTelChan1) ""
    set gvar(iracqSetTelChan2) ""
    set gvar(iracqSetTelVal1) ""
    set gvar(iracqSetTelVal2) ""
    set gvar(iracqSetVoltValue) ""
    return 
}

# convert slider values to voltage
proc iracqSlide2Volt {slide} {
    global gvar

    set diff [expr {$gvar(iracqSetVoltR2) - $gvar(iracqSetVoltR1)}]
    set volt [expr {(($diff/4096.0) * $slide) + $gvar(iracqSetVoltR1)}]
    return $volt
}

# convert voltage to slider value
proc iracqVolt2Slide {volt} {
    global gvar

    set diff [expr {$gvar(iracqSetVoltR2) - $gvar(iracqSetVoltR1)}]
    if {$diff != 0} {
	set slide [expr {($volt-$gvar(iracqSetVoltR1)) * (4096.0/$diff)}]
    } else {
	set slide 2048.0
    }
    return $slide
}

# enable voltage tuner
proc iracqVoltTunerEnable {} {
    global gvar
    global cv

    trace variable gvar(iracqVoltT) [string tolower w] {iracqVoltTunerSet}

    bind $cv.iracqVoltTuner.uifScale  <ButtonRelease> {
	after 500 iracqUpdateTelValue
    }
}

# disable voltage tuner
proc iracqVoltTunerDisable {} {
    global gvar
    global cv

    trace vdelete gvar(iracqVoltT) [string tolower w] {iracqVoltTunerSet}
    bind $cv.iracqVoltTuner.uifScale  <ButtonRelease> {}
}

# set voltage tuner value
proc iracqVoltTunerSet {name1 name2 op} {
    global gvar 
    global iracqVset

    if {$gvar(iracqCCS)} {
	seq_evtSingleDisable $gvar(iracqClkVoltEvtId)
	seq_evtSingleDisable $gvar(iracqDcVoltEvtId)
    }

    iracqVoltTunerDisable

    set value [iracqSlide2Volt $gvar(iracqVoltT)]

    iracqRound value 3

    set gvar(iracqSetVoltValue) $value

    iracqCmdVoltChan $gvar(iracqSetVoltChan) $gvar(iracqSetVoltValue) 0
 
    if {$gvar(iracqSetVoltId) != ""} {
	set iracqVset($gvar(iracqSetVoltId)) $gvar(iracqSetVoltValue)
    }

    iracqVoltTunerEnable

    if {$gvar(iracqCCS)} {
	seq_evtSingleEnable $gvar(iracqClkVoltEvtId)
	seq_evtSingleEnable $gvar(iracqDcVoltEvtId)
    }

}

proc iracqSetClkVList {value} {
    global gvar

    if {!$gvar(iracqCCS)} {
	if {[iracqFunction "vsetup" 0 tmpStr]} {
	    iracqScanVsetup $tmpStr
	}
    } else {
	iracqEvtDcVolt
	iracqEvtClkVolt
    }

    set gvar(iracqVoltSelName) ""
}

proc iracqEnableVoltageSetting {} {
    global gvar cv

    set gvar(iracqVsetEnabled) 1
    $cv.iracqVoltTuner enable
    $cv.iracqSetVoltValue enable
    $cv.iracqVoltRestore enable

    if {$gvar(iracqCfgEnabled)} {
	$cv.iracqSysCldc enable
	$cv.iracqSysVoltage enable
	$cv.iracqBrowseVolt enable
    }
}

proc iracqDisableVoltageSetting {} {
    global gvar cv

    set gvar(iracqVsetEnabled) 0
    $cv.iracqVoltTuner disable
    $cv.iracqSetVoltValue disable
    $cv.iracqVoltRestore disable

    $cv.iracqSysCldc disable
    $cv.iracqSysVoltage disable
    $cv.iracqBrowseVolt disable
}

proc iracqVoltageSetting {} {
    global gvar cv

    if {$gvar(iracqVsetEnabled)} {
	iracqEnableVoltageSetting 
    } else {
	iracqDisableVoltageSetting 
    }
}

proc iracqUpdateCldcList {} {
    global gvar cv

    set cldcList [split [string trim $gvar(iracqCldcList)] " "]

    $cv.iracqSysCldcSel deleteEntry 0 last  

    foreach ele $cldcList {
	if {$ele != "-1"} {
	    $cv.iracqSysCldcSel addItem "$ele"\
		    "iracqCmdSetCldc $ele"
	}
    }

}


#
# ___oOo___
