#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: irlParseLib.tcl,v 1.104 2005/09/12 09:21:01 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  09/02/99  created
# jstegmei  06/06/01  test eof condition on stdin
#

#************************************************************************
#   NAME
#   ASSIGN, USE, EXEC, LOOP, END, SYNC, PROGEND - irace readout preprocessor
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   irace readout preprocessor
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#



proc irlReadParam {} {
    global irlVar gvar

    set str "IVLD"
    while {$str != "!" && $str != "#"} {
	set str [gets stdin]
	if {[eof stdin]} {
	    exit 0
	}
	if {$str == "#"} {
	    exit 0
	}
	if {$str != "!"} {
	    if {[scan $str "%d %f" num value] == 2} {
		set gvar(T_$num) $value
	    }
	}
    }
    set str "IVLD"
    while {$str != "!" && $str != "#"} {
	set str [gets stdin]
	if {[eof stdin]} {
	    exit 0
	}
	if {$str == "#"} {
	    exit 0
	}
	if {$str != "!"} {
	    if {[scan $str "%s %f" name value] == 2} {
		set irlVar($name) $value
	    }
	}
    }
}

proc irlInit {} {
    global gvar irlInitFlag irlDebug

    set irlInitFlag 1
    fconfigure stdout -translation lf
    set gvar(irlOut) ""
    set gvar(irlCnt) 0
    set gvar(irlMapFirst) 1
    set gvar(irlFrameList) ""
    set gvar(irlVarList) ""
  
    if {![info exists irlDebug]} {
	set irlDebug 0
    }
}

proc USE {args} {
    global argv0 irlVar irlInitFlag irlDebug gvar

    if {![info exists irlInitFlag]} {
	irlInit
    }

    if {!$irlDebug} {
	if {!$gvar(irlMapFirst)} {
	    puts $gvar(irlMap)
	}
	puts "!"
	flush stdout
	
	foreach i [lsort $args] {
	    puts $i
	}
	puts "!"
	flush stdout

	irlReadParam

	set success 1
	set errorString ""
	foreach i [lsort $args] {
	    if {[catch {set tmp $irlVar($i)}]} {
		set erms "error parsing $argv0 - variable $i is not defined"
		set errorString "$errorString$erms\n"
		set success 0
	    }
	}

	if {!$success} {
	    puts "ERROR:"
	    puts "$errorString\!"
	    flush stdout
	    exit 1
	}
    } else {
	foreach i [lsort $args] {
	    lappend gvar(irlVarList) $i
	}
    }
}

proc ASSIGN {frame fileName} {
    upvar $frame frameLoc
    global gvar irlInitFlag irlDebug

    if {![info exists irlInitFlag]} {
	irlInit
    }

    if {$gvar(irlMapFirst)} {
	set gvar(irlMapFirst) 0
	set gvar(irlMap) "$gvar(irlCnt):$fileName"
    } else {
	set gvar(irlMap) "$gvar(irlMap)\n$gvar(irlCnt):$fileName"
    }
    set frameLoc $gvar(irlCnt)

    if {$irlDebug} {
	lappend gvar(irlFrameList) "$frame $gvar(irlCnt)"
    }

    incr gvar(irlCnt) 1    
}

proc EXEC {frame {rep 1}} {
    global gvar irlInitFlag

    if {![info exists irlInitFlag]} {
	USE
    }

    set tmp [expr {round($rep)}] 
    set gvar(irlOut) "$gvar(irlOut) $frame $tmp"
}

proc LOOP {rep} {
    global gvar irlInitFlag

    if {![info exists irlInitFlag]} {
	USE
    }

    if {$rep == -1} {
	set gvar(irlOut) "$gvar(irlOut) -101 $rep"
    } else {
	set tmp [expr {round($rep)}]
	set gvar(irlOut) "$gvar(irlOut) -101 $tmp"
    }
}

proc END {} {
    global gvar irlInitFlag

    if {![info exists irlInitFlag]} {
	USE
    }

    set gvar(irlOut) "$gvar(irlOut) -102"
}

proc SYNC {} {
    global gvar irlInitFlag

    if {![info exists irlInitFlag]} {
	USE
    }

    set gvar(irlOut) "$gvar(irlOut) -104"
}

proc PROGEND {} {
    global gvar irlInitFlag irlDebug
    global irlVar

    if {![info exists irlInitFlag]} {
	USE
    }

    if {!$irlDebug} {
	puts "SUCCESS:"
	puts "!"
	flush stdout
	foreach i [lsort [array names irlVar]] {
	    puts "$i $irlVar($i)" 
	}
	puts "!"
	flush stdout
	
	puts "$gvar(irlOut) -103"
	puts "!"
	flush stdout
	
	exit 0
    }
}

proc RETERR {args} {
    global gvar irlInitFlag

    if {![info exists irlInitFlag]} {
	USE
    }

    puts "ERROR:"
    
    foreach i $args {
	puts $i
    }
    puts "!"
    flush stdout

    exit 1
}
#
# ___oOo___
