#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: irtdRapid.tcl,v 1.22 2005/04/18 14:33:35 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
#
# irtdRapid.tcl -- rapid frame display

#************************************************************************
#   NAME
#   irtdRapid.tcl - rapid frame display
# 
#   SYNOPSIS
#   irtdRapid <path> ?options?
# 
#   DESCRIPTION
#   irtdRapid builds the popup window which is needed to control the rapid frame
#   display. It consists of an RTD image and widgets to start/stop the display, to
#   set cut levels, to set the scaling and to select the frame type to be displayed.
#   irtdRapid handles the region in the main image canvas and displays it in the
#   pan window.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual irtdRapid {}

class irtdRapid {
    inherit ::rtdbDefwin

    constructor {args} {
	eval itk_initialize $args
    }

    method init {} {
	::rtdbDefwin::init
	config -center 1

	# make the rapid popup window
	make_rapid_frame
	make_rapid_controls
	make_rapid_help

	global ::rtdb
	$rtdb(Rtd) add_updateValues [code $this cutsfrom]
	wm protocol $w_ WM_DELETE_WINDOW [code $this remove_RF]
	config -notify [code $this changed]
    }

    # create the rapid frame window

    method make_rapid_frame {} {
	itk_component add image {
	    RtdImage $w_.image \
		-canvasheight 250 \
		-canvaswidth  250 \
		-xscale $scale_ -yscale $scale_ \
		-displaymode 1 \
		-with_warp 1 \
		-graphics 1 \
		-shorthelpwin $itk_option(-shorthelpwin) \
		-shelp "" \
		-drag_scroll 0 \
		-usexshm 1 \
		-verbose 0 \
		-canvasbackground grey35 \
		-subsample 1 \
		-zoomwin $itk_option(-zoomwin) \
		-scrollbars 1
	}
	pack $itk_component(image) -fill both -expand 1

	set canvas_ [$itk_component(image) get_canvas]
	set image_  [$itk_component(image) get_image]

        # add a new view (rapid frame) to the main image
        $target_image_ view add $image_ 0 1
	set frameId_ [$image_ frameid]
    }

    method make_rapid_controls {} {
        itk_component add hull2 {
            frame $w_.hull2
        }
        pack $itk_component(hull2) -side top -fill none -expand 0 -pady 1

        itk_component add panel {
            frame $itk_component(hull2).panel -borderwidth 1 -relief groove
        }
        pack $itk_component(panel) -side top -fill x -expand 0 -pady 1

	lassign [$target_image_ cut] low high
	itk_component add low {
	    LabelEntry $itk_component(panel).low \
		    -text "Low:" \
		    -command [code $this setCutLevels] \
		    -labelwidth 3 \
		    -anchor w \
		    -value $low \
		    -relief sunken
	}

	itk_component add high {
	    LabelEntry $itk_component(panel).high \
		    -text "High:" \
		    -command [code $this setCutLevels] \
		    -labelwidth 3 \
		    -anchor w \
		    -value $high \
		    -relief sunken
	}

	# add controls to zoom in and out, flip, rotate, ...
	itk_component add trans {
	    RtdImageTrans $itk_component(panel).trans \
		    -show_trans 0 \
		    -show_Zz_buttons 1 \
		    -relief flat -borderwidth 0 \
		    -image $itk_component(image)
	} {
	    keep -state
	}

	[$itk_component(trans) component choose].mb config -text "2x"
	[$itk_component(trans) component choose] config -value "2x"
	[$itk_component(trans) component choose].mb.m config -tearoff 0
	[$itk_component(trans) component choose] config -valuewidth 1

	itk_component add data {
	    LabelMenu $itk_component(panel).data \
		    -text "Data:" \
		    -relief groove \
		    -anchor e \
		    -labelwidth 5 \
		    -valuewidth 11
	}

	$itk_component(data).mb.m config -tearoff 0


	# buttons --

        itk_component add buttons {
            frame $itk_component(hull2).buttons -borderwidth 1 -relief groove
        }
        pack $itk_component(buttons) -side top -fill x -expand 0 -pady 1

	itk_component add autocut {
	    button $itk_component(buttons).autocut \
		    -text "Auto Cut" \
		    -pady 3 \
		    -width 8 \
		    -command [code $this busy "$this auto_set_cut_levels"]	    
	}

	global ::$w_.start

	itk_component add start {
	    checkbutton $itk_component(buttons).start \
		    -text "Start" \
		    -width 6 \
		    -variable $w_.start \
		    -onvalue 1 -offvalue 0 \
		    -anchor w \
		    -pady 3 \
		    -bd 2 -relief raised \
		    -command [code "$this start_stop"]
	}
	set_start 0

	itk_component add close {
	    button $itk_component(buttons).close \
		    -text "Close" \
		    -pady 3 \
		    -width 8 \
		    -command [code $this remove_RF]	
	}

	global ::$w_.cutsfrom
	itk_component add cutsfrom {
	    checkbutton $itk_component(buttons).cutsfrom \
		    -text "CutOn" \
		    -width 6 \
		    -variable $w_.cutsfrom \
		    -onvalue 1 -offvalue 0 \
		    -anchor w \
		    -pady 3 \
		    -bd 2 -relief raised \
		    -command [code $this busy "$this cutsfrom"] 
	}

	blt::table $itk_component(panel) \
		$itk_component(low)        1,0  -anchor w -fill none -columnspan 2 \
		$itk_component(high)       2,0  -anchor w -fill none -columnspan 2 \
		$itk_component(trans)      1,2  -anchor e -fill none -columnspan 2 \
		$itk_component(data)       2,2  -anchor e -fill none

	blt::table $itk_component(buttons) \
		$itk_component(start)      1,0  \
		$itk_component(cutsfrom)   1,1  \
		$itk_component(autocut)    1,2  \
		$itk_component(close)      1,3

	set_cuts_state
    }

    # short help text

    method make_rapid_help {} {
	
	add_short_help $canvas_ $itk_option(-shelp)
	
	add_short_help $itk_component(data) \
		{Data: {bitmap b1} = Select data to be displayed in Rapid Frame: \
		DIT, Non-destructive (ND) Samples, Running average (INT) data}
	add_short_help $itk_component(start) \
		{Start/Stop: {bitmap b1} = Toggle Start / Stop Rapid Frame display}
	add_short_help $itk_component(close) \
		{Close: {bitmap b1} = Stop Rapid Frame display and close this window}
	add_short_help $itk_component(low) \
		{Image low cut value: {bitmap b1} = click to enter. Type Enter after editing value}
	add_short_help $itk_component(high) \
		{Image high cut value: {bitmap b1} = click to enter. Type Enter after editing value}
	add_short_help $itk_component(autocut) \
		{Auto cut: {bitmap b1} = Set the image cut levels using median filtering}
	add_short_help $itk_component(cutsfrom) \
		{CutOn: {bitmap b1} = When the button CutOn is off the cut levels from the \
		main window are applied to the rapid frame}
	add_short_help $itk_component(trans) \
		{Scale: {bitmap b1} = Set the image scale factor}
	add_short_help $itk_component(image).imagef.vscrollf \
		{Scrollbar: {bitmap dragb1} = Scroll image vertically}
	add_short_help $itk_component(image).imagef.hscrollf \
		{Scrollbar: {bitmap dragb1} = Scroll image horizontally}
    }

    # method which is called before RF is displayed

    method rapidPreCommand {{frameId 0}} {
	$itk_component(start) config -fg black
    }

    # method which is called after RF is displayed

    method rapidPostCommand {{frameId 0}} {
	if {! $firstRf_ || [$image_ isclear]} {
	    update
	    return
	}
        $itk_component(image) scale $scale_ $scale_
	cutsfrom

	set firstRf_ 0
	config -state normal
	setCutLevels
    }

    # method to set/get the start button

    method set_start {{ss -1}} {
	global ::$w_.start
	if {![info exists $w_.start]} {
	    return 0
	}
	if {$ss == -1} {
	    return [set $w_.start]
	}
	set $w_.start $ss
    }

    # method to set the data menu items

    method set_data {{frames 2}} {
	irtdData_menu $frames $itk_component(data) $this start_data
    }

    # indicate "waiting for data" on start button

    method waiting_msg {waiting} {
        if {[set_start] && $waiting} {
            $itk_component(start) config -fg blue
        } else {
            $itk_component(start) config -fg black
        }
    }

    # send start/stop rapid frame to camera depending on button "Start" in RF

    method start_stop {} {
	global ::irtd
	set irtd(crf) 0
	if {[set_start]} {
	    irtdSendDart "RAPID $frameId_ 1 $rapidData_"
	    $itk_component(start) config -text Stop
	    waiting_msg 1
	} else {
	    irtdSendDart "RAPID $frameId_ 0 $rapidData_"
	    $itk_component(start) config -text Start -fg black
	    waiting_msg 0
	}
	update
    }

    # mark rapid frame in pan window. Delete it when the button "Rapid Frame" is off
    method rapid_pan_mark {on} {
        set win [$pan_ component image]
        set img_ [$win get_image]
	set can_ [$win component canvas]
        if { "[$can_ coords markpan]" != "" } {
            $can_ delete markpan
        }
        if {! $on} {
            return 0
        }
	lassign [region_image_coord] ix0 iy0 ix1 iy1
	$img_ convert coords $ix0 $iy0 image x0 y0 canvas
	$img_ convert coords $ix1 $iy1 image x1 y1 canvas

        set markpanId [$can_ create line $x0 $y0 $x1 $y0 $x1 $y1 $x0 $y1 $x0 $y0 \
                -width 2 \
                -fill red \
                -stipple pat3 \
                -tags markpan]
        set mark2panId [$can_ create line $x0 $y0 $x1 $y0 $x1 $y1 $x0 $y1 $x0 $y0 \
                -width 1 \
                -fill green \
                -tags markpan]
    }

    method raise_dwin {} {
	rtdbDefwin::raise_dwin
	set firstRf_ 1
	notify_cmd move
    }

    # remove rapid frame

    method remove_RF {} {
	# save scale factors and transformations
	lassign [$image_ scale] sx sy
	if {"$sx" != ""} { set scale_ $sx }
	set_start 0
	start_stop
	rapid_pan_mark 0
	catch {eval [cget -command]}
	cancel
    }

    # request data for rapid frame display

    method start_data {frame} {
	if {! [info exists frameId_]} { return }
	waiting_msg 1
	set rapidData_ [irtdGetFrameType $frame]
	irtdSendDart "RAPID $frameId_ [set_start -1] $rapidData_"
    }

    # set cut levels (called by class irtdRtd)

    method cut {low high} {
	global ::$w_.cutsfrom
	if {![set $w_.cutsfrom]} {
	    $image_ cut $low $high
	}
	update_cut
    }

    # set the cut levels when the user types them in and hits return
    
    method setCutLevels {args} {
	busy {
	    catch {$image_ cut [$itk_component(low) get] [$itk_component(high) get]}
	    update_cut
	}
    }

    # update cut level display

    method update_cut {} {
	if {[$image_ isclear]} {
	    return
	}
	lassign [$image_ cut] low high
	set state [$itk_component(low) cget -state]
	foreach el "low high" {
	    $itk_component($el) config -state normal -value [set $el]
	    $itk_component($el) config -state $state
	}
    }

    # this method is called by the "Cut On" button

    method cutsfrom {} {
	global ::$w_.cutsfrom
	if {![set $w_.cutsfrom]} {
	    lassign [$target_image_ cut] low high
	    $image_ cut $low $high
	}
	set_cuts_state
	update_cut
    }

    # enable/disable widgets for cuts

    method set_cuts_state {} {
	global ::$w_.cutsfrom
	if {![set $w_.cutsfrom]} {
	    set state disabled
	} else {
	    set state normal
	}
	foreach widg {low high autocut} {
	    $itk_component($widg) config -state $state
	}
    }

    # set the cut levels automatically using median filtering...
    
    method auto_set_cut_levels {} {
	$image_ autocut
	update_cut
    }

    method changed {args} {
	set firstRf_ 1
	update
	if {! [rtdbIsVideoImage]} { return }
	rapid_pan_mark 1
	lassign $args op x0 y0 w h
	irtdSendDart "WIN_RF 1 $x0 $y0 $w $h"
    }
    
    # -- public vars --
    
    # target rtdimage
    itk_option define -target_image target_image Target_image {} {
	set pan_ [$itk_option(-target_image) component pan]
    }
   
    # tcl command to be evaluated when the rapid frame is closed
    itk_option define -command command Command {}

    # text of short help message to be displayed whenever
    # the mouse enters the image window (see Toplevel.tcl)
    itk_option define -shelp shelp Shelp "Rapid frame window"

    # max. window size of rapid frame in #of pixels
    itk_option define -maxframesize maxframesize Maxframesize 256

    # name of zoom window to update when mouse enters this window
    itk_option define -zoomwin zoomWin ZoomWin ""

    # -- protected vars --

    # RtdImagePan object, for setting the rectangular in the pan window
    protected variable pan_

    # internal rtdimage for rapid frame
    protected variable image_

    # canvas window containing rapid frame image, 
    # different than target_canvas for popup frames
    protected variable canvas_

    # frame id of rapid frame
    protected variable frameId_

    # flag for first rapid frame
    protected variable firstRf_ 1

    # current data for the rapid frame
    protected variable rapidData_ 2

    # data reception task stopped, flag
    protected variable dartStopped_ 0

    # scale factor
    protected variable scale_ 2
}
