#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: osrtdGraphics.tcl,v 1.20 2004/01/23 15:21:48 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
# sskole    15/11/02  osrtdRect & osrtdOvals: Added mask center
# sskole    17/11/02  osrtdRect: Small bug fix
# pbiereic  23/01/04  previous introduced bug fixed

#************************************************************************
#   NAME
#   osrtdGraphics.tcl - Draw graphics (lines, ovals, crosses, etc.)
# 
#   SYNOPSIS
#   osrtdGraphics <path> ?options?
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual osrtdGraphics {}

class ::osrtdGraphics {
    inherit util::FrameWidget

    constructor {args} {
	eval itk_initialize $args
    }

    method init {} {
	global ::rtdb

	set RtdWdg_ $rtdb(Rtd)     ; # name of the Rtd (main) object
	set canvas_ [$RtdWdg_ get_canvas]

	# CCS must be initialized
	if { ! $rtdb(CCSINIT) } {
	    return
	}
	set withCCS_ 1

	initDbEvents
	showGraphics

	$RtdWdg_ add_notify [code $this showGraphics]
    }

    # attach to db events

    method initDbEvents {} {
        if {! $withCCS_ } { return }
	foreach attr "Lines Ovals Rects Text Crosses" {
	    if {! [catch {seq_evtAttach "$dbRoot_$attr\.show$attr" W [code $this osrtd$attr]} msg]} {
		lappend dbPnts_ osrtd$attr
	    } else {
		rtdbFeedback $msg
	    }
	}
	if [catch {seq_evtAttach "$dbRoot_\Graphics.showGraphics" NE [code $this showGraphics]}] {
	    rtdbFeedback $msg
	}
    }

    # turn graphics on/off
    
    method showGraphics {args} {
	foreach graphProc $dbPnts_ {
	    if {"$graphProc" != ""} { catch $graphProc }
	}
    }

    # draw lines

    method osrtdLines {args} {
	if {! $withCCS_ } { return }
        # get on/off values
        set on [seq_dbReadSymbolic "$dbRoot_\Lines.showLines"]
	set graphOn [seq_dbReadSymbolic "$dbRoot_\Graphics.showGraphics"]

        # delete graphics
        set ourTags oslines
        $canvas_ delete $ourTags

        if { ! $on || ! $graphOn || [rtdbIsImageCleared] } { return }
	set number  [seq_dbReadSymbolic "$dbRoot_\Lines.number"]
	set lineTbl [seq_dbReadSymbolic "$dbRoot_\Lines.lineTbl"]

        for {set i 0} {$i < $number} {incr i} {
            lassign [lindex $lineTbl $i] x0 y0 x1 y1 fill stipple tags width show
	    if {! $show } { continue }
	    # convert to canvas coordinates and draw the line
	    lassign [rtdbChipToCanvas $x0 $y0 $x1 $y1] cx0 cy0 cx1 cy1
	    $canvas_ create line $cx0 $cy0 $cx1 $cy1 \
		    -tags "$ourTags $tags" -fill $fill -width $width -stipple $stipple
	}
        # add "objects" to the tag so that the line is scaled, rotated and
        # flipped with the image
        $canvas_ addtag objects withtag $ourTags
    }

    # draw rectangles

    method osrtdRects {args} {
	global ::osrtd
	if {! $withCCS_ } { return }
        # get on/off values
        set on [seq_dbReadSymbolic "$dbRoot_\Rects.showRects"]
	set graphOn [seq_dbReadSymbolic "$dbRoot_\Graphics.showGraphics"]

        # delete graphics
        set ourTags osrects
        $canvas_ delete $ourTags

        if { ! $on || ! $graphOn || [rtdbIsImageCleared] } { return }
	set number  [seq_dbReadSymbolic "$dbRoot_\Rects.number"]
	set rectTbl [seq_dbReadSymbolic "$dbRoot_\Rects.rectTbl"]

        set endRect 0
        for {set i 0} {$i < $number} {incr i} {
            lassign [lindex $rectTbl $i] x0 y0 x1 y1 outline fill stipple tags width show
	    if {! $show } { continue }
	    # Set the polarimetric mask center for telescope offsets
	    set centerRect [expr ($number/2) + ($number%2) - 1]
	    if { $i == $centerRect } {
		set endRect $y1
	    } elseif { $i == [expr $centerRect + 1] } {
		set beginRect $y0
		$osrtd(Rtd) setSlitCenter 512 [expr abs($beginRect + $endRect)/2]
	    }
	    # convert to canvas coordinates and draw the line
	    lassign [rtdbChipToCanvas $x0 $y0 $x1 $y1] cx0 cy0 cx1 cy1
	    $canvas_ create rectangle $cx0 $cy0 $cx1 $cy1 \
		    -tags "$ourTags $tags" -fill $fill -outline $outline -width $width -stipple $stipple
	}
        $canvas_ addtag objects withtag $ourTags
    }

    # draw ovals

    method osrtdOvals {args} {
	global ::osrtd
	if {! $withCCS_ } { return }
        # get on/off value
        set on [seq_dbReadSymbolic "$dbRoot_\Ovals.showOvals"]
	set graphOn [seq_dbReadSymbolic "$dbRoot_\Graphics.showGraphics"]

        # delete graphics
        set ourTags osovals
        $canvas_ delete $ourTags

        if { ! $on || ! $graphOn || [rtdbIsImageCleared] } { return }
	set number  [seq_dbReadSymbolic "$dbRoot_\Ovals.number"]
	set ovalTbl [seq_dbReadSymbolic "$dbRoot_\Ovals.ovalTbl"]

        for {set i 0} {$i < $number} {incr i} {
            lassign [lindex $ovalTbl $i] x0 y0 x1 y1 outline fill stipple tags width show
	    if {! $show } { continue }
	    # Set the coronographic mask center for telescope offsets
	    $osrtd(Rtd) setSlitCenter [expr ($x0 + $x1)/2] [expr ($y0 + $y1)/2]
	    # convert to canvas coordinates and draw the line
	    lassign [rtdbChipToCanvas $x0 $y0 $x1 $y1] cx0 cy0 cx1 cy1
	    $canvas_ create oval $cx0 $cy0 $cx1 $cy1 \
		    -tags "$ourTags $tags" -fill $fill -outline $outline -width $width -stipple $stipple
	}
        $canvas_ addtag objects withtag $ourTags
    }

    # draw text

    method osrtdText {args} {
	if {! $withCCS_ } { return }
        # get on/off value
        set on [seq_dbReadSymbolic "$dbRoot_\Text.showText"]
	set graphOn [seq_dbReadSymbolic "$dbRoot_\Graphics.showGraphics"]

        # delete graphics
        set ourTags ostext
        $canvas_ delete $ourTags

        if { ! $on || ! $graphOn || [rtdbIsImageCleared] } { return }
	set number  [seq_dbReadSymbolic "$dbRoot_\Text.number"]
	set textTbl [seq_dbReadSymbolic "$dbRoot_\Text.textTbl"]
        for {set i 0} {$i < $number} {incr i} {
            lassign [lindex $textTbl $i] x0 y0 text fill font tags show
	    if {! $show } { continue }
	    # convert to canvas coordinates and draw the line
	    lassign [rtdbChipToCanvas $x0 $y0 0 0] cx0 cy0 d d
	    $canvas_ create text $cx0 $cy0 \
		    -tags "$ourTags $tags" -fill $fill -font $font -text $text
	}
        $canvas_ addtag objects withtag $ourTags
    }

    # draw crosses

    method osrtdCrosses {args} {
	if {! $withCCS_ } { return }
        # get on/off value
        set on [seq_dbReadSymbolic "$dbRoot_\Crosses.showCrosses"]
	set graphOn [seq_dbReadSymbolic "$dbRoot_\Graphics.showGraphics"]

        # delete graphics
        set ourTags oscross
        $canvas_ delete $ourTags

        if { ! $on || ! $graphOn || [rtdbIsImageCleared] } { return }
	set number  [seq_dbReadSymbolic "$dbRoot_\Crosses.number"]
	set crossTbl [seq_dbReadSymbolic "$dbRoot_\Crosses.crossTbl"]

        for {set i 0} {$i < $number} {incr i} {
            lassign [lindex $crossTbl $i] x0 y0 sizex sizey angle fill stipple tags width show
	    if {! $show } { continue }
	    set w [expr $sizex / 2.0]
	    set h [expr $sizey / 2.0]
	    if {$angle == 0} {
		lassign [rtdbChipToCanvas [expr $x0 - $w] $y0 [expr $x0 + $w] $y0] \
			cx0 cy0 cx1 cy1
		$canvas_ create line $cx0 $cy0 $cx1 $cy1 \
			-fill $fill -stipple $stipple -width $width -tags "$ourTags $tags"
		lassign [rtdbChipToCanvas $x0 [expr $y0 - $h] $x0 [expr $y0 + $h]] \
			cx0 cy0 cx1 cy1
		$canvas_ create line $cx0 $cy0 $cx1 $cy1 \
			-fill $fill -stipple $stipple -width $width -tags "$ourTags $tags"
	    } else {
		# convert to radian
		set rad [expr $angle/57.2958]
		# deltas for X and Y axis
		set dxX [expr cos($rad) * $w]
		set dyX [expr sin($rad) * $w]
		set dxY [expr cos($rad) * $h]
		set dyY [expr sin($rad) * $h]
		lassign [rtdbChipToCanvas [expr $x0 + $dxX] [expr $y0 + $dyX] \
			[expr $x0 - $dxX] [expr $y0 - $dyX]] cx0 cy0 cx1 cy1
		$canvas_ create line $cx0 $cy0 $cx1 $cy1 \
			-fill $fill -stipple $stipple -width $width -tags "$ourTags $tags"
		lassign [rtdbChipToCanvas [expr $x0 + $dyY] [expr $y0 - $dxY] \
			[expr $x0 - $dyY] [expr $y0 + $dxY]] cx0 cy0 cx1 cy1
		$canvas_ create line $cx0 $cy0 $cx1 $cy1 \
			-fill $fill -stipple $stipple -width $width -tags "$ourTags $tags"
	    }
	}
        $canvas_ addtag objects withtag $ourTags 
    }

    # -- public vars --

    itk_option define -osrtdRoot osrtdRoot OsrtdRoot {osrtd} {
	if {"[cget -osrtdRoot]" != ""} {
	    set dbRoot_ "<alias>[cget -osrtdRoot]"
	}
    }


    # -- protected vars -- 

    # Rtd main widget
    protected variable RtdWdg_

    # canvas for main image
    protected variable canvas_

    # list of existing database points
    protected variable dbPnts_ ""

    # use CCS flag
    protected variable withCCS_ 0

    # db root point
    protected variable dbRoot_ "<alias>osrtd"
}

# ___oOo___
