#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: osrtdRtd.tcl,v 1.20 2004/01/23 15:21:48 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
# pbiereic  08/06/00  added clearToffset
# pbiereic  11/07/00  the name of the 'slit' device can be passed with the
#                     toffset command
# sskole    15/11/02  Added setSlitCenter
# sskole    18/11/02  osrtdSlit: Replaced diagonal cross with a christian cross 
# pbiereic  13/10/03  Redraw slit after image was scaled

#************************************************************************
#   NAME
#   osrtdRtd.tcl - OS specific RTD functions
# 
#   SYNOPSIS
#   osrtdRtd <path> ?options?
# 
#   DESCRIPTION
#   osrtdRtd is a widget class to perform OS specific OS functions which are
#   - get interactively coordinates for an telescope offset
#   - display WCS
#   - draw a slit
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual osrtdRtd {}

class ::osrtdRtd {
    inherit util::FrameWidget

    constructor {args} {
	eval itk_initialize $args
	# get additional options from run string
	set argvs [rtdbArgs $this]
	if {"$argvs" != ""} {
	    eval itk_initialize $argvs
	}
    }

    method init {} {
	global ::rtdb

	set RtdWdg_   $rtdb(Rtd)     ; # name of the Rtd (main) object
	set panelWdg_ [$RtdWdg_ get_panel]
	set canvas_   [$RtdWdg_ get_canvas]
	set image_    [[$RtdWdg_ get_image] get_image]

	# CCS must be initialized
	if { ! $rtdb(CCSINIT) } {
	    return
	}
	set withCCS_ 1

	# create an instance of the osrtdGraphics widget
	osrtdGraphics $w_.graphics -osrtdRoot $itk_option(-osrtdRoot)

	initDbEvents
	handleDbEvents

	$RtdWdg_ add_postCmd [code $this osrtdWcs]
 	$RtdWdg_ add_notify [code $this osrtdSlit]
	$RtdWdg_ add_scaleCmd [code $this osrtdSlit]


	# only for test
 	###$RtdWdg_ traceXY [code $this traceXY]
    }

    # handle database events
    
    method handleDbEvents {args} {
	foreach eventProc $dbPnts_ {
	    if {"$eventProc" != ""} { catch $eventProc }
	}
    }

    # attach to db events

    method initDbEvents {} {
        if {! $withCCS_ } { return }
	foreach el "osrtdWcs osrtdSlit" {
	    regsub osrtd $el "" attr
	    if {! [catch {seq_evtAttach "$dbRoot_$attr\.show$attr" W [code $this $el]} msg]} {
		lappend dbPnts_ $el
	    } else {
		rtdbFeedback $msg
	    }
	}
    }

    # send world coordinates to RTD

    method osrtdWcs {args} {
	if {! $withCCS_ || ! [rtdbIsVideoImage]} { return }

	set nx [$image_ width]
	set ny [$image_ height]
	if {! [seq_dbReadSymbolic "$dbRoot_\Wcs.showWcs"] } {
	    catch {$image_ wcsset 0 0 0 0 0 0 0 0 2000 0 "PIXEL"}
	    return
	}
	foreach el "ra dec secPix xRefPix yRefPix rotate equinox epoch proj wcsFlags" {
	    if {[catch {set $el [seq_dbReadSymbolic "$dbRoot_\Wcs.$el"]}]} { return }
	}
	if {[info exists proj] && "$proj" != ""} {
	    set wcs "$ra $dec $secPix $xRefPix $yRefPix $nx $ny $rotate $equinox $epoch $proj"
	    if {[catch {eval $image_ wcsset $wcs} msg]} {
		rtdbFeedback $msg
		return
	    }
	    if { $wcsFlags != 0 } {
		set cdelt1 [expr -$secPix / 3600.0]
		set cdelt2 [expr $secPix / 3600.0]
		if { [expr $wcsFlags & 1] } {
		    set cdelt1 [expr -$cdelt1]
		}
		if { [expr $wcsFlags & 2] } {
		    set cdelt2 [expr -$cdelt2]
		}
		if {[catch {eval $image_ wcsdeltset $cdelt1 $cdelt2 $rotate} msg]} {
		    rtdbFeedback $msg
		    return
		}
	    }
	}
    }

    # show slit

    method osrtdSlit {args} {
	config -slitcenter ""
	if {! $withCCS_} { return }
        set on [seq_dbReadSymbolic "$dbRoot_\Slit.showSlit"]

        # delete graphics
        set ourTags osslit
        $canvas_ delete $ourTags
        if { ! $on || ! [rtdbIsVideoImage] } { return }
	foreach el "xc yc w h angle color width" {
	    if {[catch {set $el [seq_dbReadSymbolic "$dbRoot_\Slit.$el"]}]} { return }
	}
	if {! [catch {set name [seq_dbReadSymbolic "$dbRoot_\Slit.name"]}]} {
	    config -deviceName $name
	}
	config -slitcenter "$xc $yc"

	# x0,y0: lower left corner
	# x1,y1: lower right corner
	# x2,y2: upper left corner
	# x3,y3: upper right corner
	# x4,y4: lower cross corner
	# x5,y5: upper cross corner
	# x6,y6: right cross corner
	# x7,y7: left cross corner
	lassign [slit_transform $xc $yc $w $h $angle] x0 y0 x1 y1 x2 y2 x3 y3
	lassign [rtdbChipToCanvas $x0 $y0 $x1 $y1] x0 y0 x1 y1
	lassign [rtdbChipToCanvas $x2 $y2 $x3 $y3] x2 y2 x3 y3
	set x4 [expr ($x0 + $x1)/2]; set y4 [expr ($y0 + $y1)/2]
	set x5 [expr ($x2 + $x3)/2]; set y5 [expr ($y2 + $y3)/2]
	set x6 [expr ($x1 + $x3)/2]; set y6 [expr ($y1 + $y3)/2]
	set x7 [expr ($x0 + $x2)/2]; set y7 [expr ($y0 + $y2)/2]
	# Draw the box with a christian cross
	set boxCoord "$x0 $y0 $x1 $y1 $x1 $y1 $x3 $y3 $x3 $y3 $x2 $y2 $x2 $y2 $x0 $y0"
	set crossCoord "$x4 $y4 $x5 $y5 $x6 $y6 $x7 $y7"
	foreach "xa ya xb yb" "$boxCoord $crossCoord" {
	    $canvas_ create line $xa $ya $xb $yb \
		    -tags $ourTags -fill $color -width $width
	}  
        $canvas_ addtag objects withtag $ourTags
    }

    # Transform slit coordinates

    method slit_transform {xnull ynull width height degrees} {
	set pi [expr atan(1.0) * 4.]
	set angle [expr $pi / 180. * $degrees]
	set w2 [expr $width / 2.]
	set h2 [expr $height / 2.]
	foreach el "- +" {
	    set y0 "$el$w2"
	    set y1 "$el$w2"
	    set x0 "-$h2"
	    set x1 "$h2"
	    foreach el2 "0 1" {
		set x [set x$el2]
		set y [set y$el2]
		set px [expr $x * cos($angle) + $y * sin($angle) + $xnull]
		lappend res $px
		set py [expr "-$x" * sin($angle) + $y * cos($angle) + $ynull]
		lappend res $py
	    }
	}
	return $res
    }

    # method to control a line for setting the chopping parameters

    method m2xy {x0 y0 x1 y1} {
	global ::rtdb
	if {[rtdbIsImageCleared]} {
	    rtdbReply {-1 "No image currently loaded. Cannot execute osrtdM2XY command"}
	    return
	}
	set win $rtdb(Rtd).m2xy
	if {! [rtdbPopup $win]} {
	    itk_component add m2xy {
		utilReUseWidget ::osrtdM2XY $win \
			-target_image [$rtdb(Rtd) get_image] \
			-title        "Chopping throw and amplitude" \
			-shorthelpwin $rtdb(Rtd)
	    }
	    update
	}
	$itk_component(m2xy) make_line $x0 $y0 $x1 $y1
    }

    # method to clear the line of the previous m2xy command

    method clearM2xy {} {
	global ::rtdb
	if {! [info exists itk_component(m2xy)]} { return }
	$itk_component(m2xy) clearAll
    }

    # method to offset the telescope interactively

    method toffset {deviceName} {
	global ::rtdb
	if {[rtdbIsImageCleared]} {
	    rtdbReply {-1 "No image currently loaded. Cannot execute toffset command"}
	    return
	}
	set win $rtdb(Rtd).toffset
	if {"$deviceName" == ""} {
	    catch {set deviceName [seq_dbReadSymbolic "$dbRoot_\Slit.name"]}
	}
	if {"$deviceName" == ""} {
	    set deviceName slit
	}
	if {! [rtdbPopup $win]} {
	    itk_component add toffset {
		utilReUseWidget ::osrtdToffset $win \
			-target_image [$rtdb(Rtd) get_image] \
			-title        "Offset Telescope" \
			-shorthelpwin $rtdb(Rtd) \
			-slitcenter [cget -slitcenter] \
			-deviceName $deviceName
	    }
	    update
	}
	config -deviceName $deviceName
	$itk_component(toffset) make_line
    }

    # method to clear the arrow of the previous toffset

    method clearToffset {} {
	global ::rtdb
	if {! [info exists itk_component(toffset)]} { return }
	$itk_component(toffset) clearToffset
    }

    # test method for the add_traceXY callback

    method traceXY {x y} {
	puts "$x $y"
    }

    # method invoked from osrtdGraphics to define the center
    # of polarimetric and coronographic masks
    
    method setSlitCenter {xc yc} {
	config -slitcenter "$xc $yc"
    }


    # -- public vars --

    # database root point for osrtd which is the name of the instance (see the
    # definition of osrtdINSTANCE in DATABASE.db). Default is 'osrtd'
    itk_option define -osrtdRoot osrtdRoot OsrtdRoot {osrtd} {
	if {"[cget -osrtdRoot]" != ""} {
	    set dbRoot_ "<alias>[cget -osrtdRoot]"
	}
    }

    # image coordinates of the slit center
    itk_option define -slitcenter slitcenter Slitcenter {""} {
	if [info exists itk_component(toffset)] {
	    $itk_component(toffset) config -slitcenter [cget -slitcenter]
	}
    }

    # name of device
    itk_option define -deviceName deviceName DeviceName {slit} {
	if [info exists itk_component(toffset)] {
	    $itk_component(toffset) config -deviceName [cget -deviceName]
	}
    }

    # -- protected vars -- 

    # Rtd main widget
    protected variable RtdWdg_
    protected variable panelWdg_

    # internal rtdimage widget for main image
    protected variable image_

    # canvas for main image
    protected variable canvas_

    # use CCS flag
    protected variable withCCS_ 0

    # list of existing database points
    protected variable dbPnts_ ""

    # db root point
    protected variable dbRoot_ "<alias>osrtd"
}


# execute commands which are sent via SCRIPT

# telescope offset (return image coordinates: from -> to)

proc toffset {{deviceName ""}} {
    global ::osrtd
    rtdbCall $osrtd(Rtd) toffset $deviceName
}

proc toffsetClear {} {
    global ::osrtd
    $osrtd(Rtd) clearToffset
    return 0
}

# line control to set M2 chopping parameters
# returns image coordinates of the line

proc m2xy {args} {
    global ::osrtd
    eval rtdbCall $osrtd(Rtd) m2xy [join [join $args]]
}

proc m2xyClear {} {
    global ::osrtd
    $osrtd(Rtd) clearM2xy
}

# create an instance of the osrtdRtd class
# Note that we are running here in a global space

set osrtd(Rtd) $rtdb(Rtd).osrtd
osrtdRtd $osrtd(Rtd) -slitcenter {}

# ___oOo___
