#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: configFunctions.tcl,v 5.34 2005/09/26 12:44:18 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# mcomin    05/07/94  Store it into a separate file .. with comments
# mcomin    06/04/95  Fixed bug in applyColor
# mcomin    25/10/95  Enable widgets which have been disabled to edit classes
# mcomin    07/11/95  Fixed bug in panClassUnload to cope with embedded 
#                     classes
# mcomin    01/12/95  New implementation of panClassUnload
# mcomin    17/01/96  Fixed bug in initialization of current font
#
# mcomin    27/02/96  Porting to TK4.0
#
#*************************************************************************
#   NAME
#
#   configFunctions - Set of routines for the configuration actions.
# 
#   SYNOPSIS
#
# 
#
#   The procedures are called when the user press :
#
#   - OK
#   - Apply
#   - Default
#   - Cancel
#   - Destroy
# 
#
#
#   DESCRIPTION
#
#
#   FILES
#    
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

#-----------------------------------------------------------------------------
# PROCEDURE : setEntry - Change the entry value : it also checks if the
#                        widget exits
#-----------------------------------------------------------------------------

proc setConfigEntry { idx what value } {
   set w [getWdgConfig $idx $what]
   setEntry $w $value
}

proc setEntry { w value } {
   if {[winfo exists $w]} {
      $w delete 0 end 
      $w insert 0 $value
   }
}

#-----------------------------------------------------------------------------
# PROCEDURE : panDeleteWidget - Delete widget clear the data structures
#-----------------------------------------------------------------------------

proc panDeleteWidget { cv name } {
   global panDbData ckActBut panWidgetHlp panDbSymbAddr panWidget par

   incr par(SAVE_FLAG)

   if { [winfo class $cv.$name] == "UifActionButton" } {   
      incr ckActBut -1              
      if { $ckActBut <= 0 } { $cv configure -height 5 }
   }

   destroy $cv.$name                   ;# Delete widget
   $cv delete [$cv find withtag $name] ;# Delete canvas item 
   
   #
   #  Delete widget data structures : uifWidgets do this during the execution
   #  of their 'destructor'.
   # 

   if {[info exists panDbSymbAddr($name)]} { unset panDbSymbAddr($name)}     
   if {[info exists panWidgetHlp($name)]}  { unset panWidgetHlp($name)}
   if {[info exists panWidget($name)]}     { unset panWidget($name)}
   if {[info exists panDbData($name)]}     { unset panDbData($name)}
}

#------------------------------------------------------------------------------
# PROCEDURE : clearConfig - Clear Entry Fields in the configuration panel
#------------------------------------------------------------------------------

proc clearConfig { idx } {
   global wdgConfig ckDbVar ckDbOnvalue ckDbOffvalue ckDbMin ckDbMax 
   global ckShow ckGlobalName

   # Clear variables associated to checkbuttons

   set ckDbVar($idx) 0 ; set ckDbOnvalue($idx) 0 ; set ckDbOffvalue($idx) 0 
   set ckDbMin($idx) 0 ; set ckDbMax($idx) 0
   set ckShow($idx)  1 ; set ckGlobalName($idx) 0

   if {[uplevel #0 info exists wdgConfig${idx}] == 0 } { return }

   foreach item [uplevel #0 array names wdgConfig${idx}] {
      set w [uplevel #0 set wdgConfig${idx}($item)] ;# Get widget path
      if {[winfo exists $w] && [winfo class $w] == "UifEntry" } { 
         $w delete 0 end 
      }
   }
}

#------------------------------------------------------------------------------
# PROCEDURE : setDefaults -  Set Default configuration
#------------------------------------------------------------------------------

proc setDefaults { idx type } {
   global  par  wdgConfig     out_fg  out_font
   global  ckHotSpot  ckDots  ckDbVar   ckDbOnvalue    ckDbOffvalue   ckFrame
   global  ckRange    ckType  ckStatus  ckOrientation  ckDataType

   #
   #  Set Common Parameters or parameters that do not trigger any change 
   #  of the status of other widgets in the configuration panel
   #
   
   set ckStatus($idx)      1
   set ckDbVar($idx)       0      
   set ckDbOnvalue($idx)   0
   set ckDbOffvalue($idx)  0  
   set ckFrame($idx)       1
   set ckType($idx)        0
   set ckRange($idx)       0
   set ckOrientation($idx) $par(HORIZONTAL) 

   set out_font       $par(OUT_FONT)
   set out_fg         black

   #
   #  Set Input Fields
   #

   if {[cequal $type UifScale] || [cequal $type UifBarChart] || 
       [cequal $type UifFrame] } {
        setConfigEntry $idx wdgLen 100
   } else {
        setConfigEntry $idx wdgLen $par(DEF_LENGTH)
   }

   switch $type {
      UifBarChart - UifScale {
         setConfigEntry $idx  wdgMin $par(RANGE_MIN);
         setConfigEntry $idx  wdgMax $par(RANGE_MAX);
      }
      UifCheckButton - UifRadioButton {
         setConfigEntry $idx  wdgOn  $par(DEF_ONVALUE);
         setConfigEntry $idx  wdgOff $par(DEF_OFFVALUE);
      }
   }

   #
   # Set  Widget Specific Options
   # 
   
   switch $type {
      bitmap     { set ckHotSpot($idx) 0;  checkHotSpot  $idx $top }      
      UifButton  { set ckDots($idx)    0;  checkDots $idx }  
      UifEntry   { set ckDataType($idx) $par(NUMBER); checkDataType $idx }
   }; # end of switch
}

#------------------------------------------------------------------------------
# PROCEDURE : newPanel - Clears the current panel and initializes a new one
#------------------------------------------------------------------------------

proc newPanel { {confirm {y}} } {
   global gvar par ckMenuBar ckShHelp fileName panMaxWdg mainGrid mbArea
   global editButton  cv bcv panDEBUG menuBar wdgConfig
   
   if {[cequal $confirm y]} {
      if {$par(SAVE_FLAG)} { append msg "This Panel HAS BEEN MODIFIED \n" }
      append msg "Are you sure you want a NEW panel ?" 
         
      if {[tk_dialog .dialog "Dialog Box" $msg question 1 Yes No]} { return }
   }

   #
   # Delete current panel and all its sub-panels
   #

   set oldDEBUG $panDEBUG
   if {[cequal $panDEBUG dbgDEV]} { set panDEBUG dbgNONE }

   if {[winfo exists $par(ROOT_PANEL)]} {

      panGetCvNames $par(ROOT_PANEL) cv bcv  ;# Get canvas names
      if {$par(CCS)} { 
         panEvtDbOperation $cv DETACH all    ;# Detach DB events 
      }

      panClassUnload ;# Unload class definition

      # Close imported panels

      foreach child  [winfo children "." ] {
         if {$child != "$par(MAIN_ROOT)" && $child != ".wdgSet"} { 
            destroy $child 
         } 
      }
   }

   panCleanData       ;# Clean all data structures
   $par(mbFile) entryconfig 5 -state normal
   $par(mbFile) entryconfig 4 -state normal

   set ckShHelp   1   ;# Make short help appear
   set ckMenuBar  1   ;# Make MenuBar appear
   set par(ROOT_PANEL) ".panel" ;# Set default value

   panGetCvNames   $par(ROOT_PANEL) cv bcv ;# Get corresponding canvas names
   panCreateFrame  $par(ROOT_PANEL)        ;# Re-draw the panel main frame
   set fileName    "[pwd]/panel.pan"       ;# Set current filename

   if {$ckMenuBar} {
      panCreateNewMenuBar $menuBar
      panSetMenuBind $mbArea
   }
 
   set par(SAVE_FLAG)  0
   set par(MYCLASS)   ""
   set panMaxWdg       1
   panDrawGrid
   
   # Re-Enable widgets which have been disabled during editing of classes
   
   $editButton     configure -state normal
   $wdgConfig(setDEFAULT_POS)   configure -state normal
   $wdgConfig(clearDEFAULT_POS) configure -state normal
 
   # Set binding to update panel geometry

   panInstallGeoHandle $par(ROOT_PANEL)
   set panDEBUG $oldDEBUG
}


#------------------------------------------------------------------------------
# PROCEDURE : panelColor - Creates a panel to select a color
#------------------------------------------------------------------------------

proc panelColor {} {
   global par panColor panSelectedColor cv
      
   set root .panColorDialog
   set panColor $panSelectedColor

   if { ! [winfo exists $root]} {
      set ff $par(COUR_R14);
      set panColor $panSelectedColor

      set colorList {
          {white c1} {white c1} {Green c1} {LimeGreen c1} {OliveDrab c1} \
          {DarkTurquoise c2} {yellow c2} {DarkOrange c2} {Red c2} {brown c2}
          {magenta c3} {BlueViolet c3} {Blue c3} {grey50 c3} {Black c3}
      }

      # Replace first color to the default background color
      
      set colorList [lreplace $colorList 0 0 "[$cv cget -background] c1"]
      
      toplevel $root
      set top [frame $root.top]
      frame $top.c1 ; frame $top.c2; frame $top.c3

      pack $top 
      pack $top.c1 $top.c2 $top.c3 -side left

      set i 0
      foreach item $colorList {
         lassign $item color c
         set f [frame $top.$c.f${i}] ; incr i
         frame $f.color -background $color -width 30 -height 15
         radiobutton $f.rb -value $color -variable panColor
         pack $f  -fill x
         pack $f.color $f.rb -side left -padx 3
      }

      set bot [frame $root.bot]
      button $bot.close  -text "Close" -command "wm iconify $root" -width 10

      pack $bot -fill x -pady 3 
      pack $bot.close -expand 1  

      wm title $root "Color Setup"
      set tmp [wm protocol $root WM_DELETE_WINDOW]
      wm protocol $root WM_DELETE_WINDOW "$tmp ; wm iconify $root"

   } else {
      wm deiconify $root
   }

   tkwait variable panColor
   set panSelectedColor $panColor
   wm iconify $root
   return $panColor
}

#------------------------------------------------------------------------------
# PROCEDURE : changeTitle - Creates a information dialog to change the current
#                           title of the panel
#------------------------------------------------------------------------------

proc changeTitle {} {
   global par
   set newtitle [promptDialog $par(SUB_PANEL) "Enter New title " $par(TITLE) ]
   if { $newtitle != "" } { 
      wm title $par(ROOT_PANEL) "$newtitle"
      set par(TITLE)            "$newtitle"
   }
}


#------------------------------------------------------------------------------
# PROCEDURE : dogrid - Draws a grid pattern in the panel
#------------------------------------------------------------------------------

proc panDrawGrid {} {
   global par gridOn bcvFlag mainCursorGrid

   switch $mainCursorGrid {
     None   { set grid 0  }
     Small  { set grid 4  ; set gridStep 8  }
     Normal { set grid 8  ; set gridStep 16 }
     Large  { set grid 16 ; set gridStep 32 }
   }

   panGetCvNames $par(ROOT_PANEL) cv bcv    

   $cv  delete grid 
   set bcvFlag [catch { $bcv delete grid }] ; # Classes do not have this

   if {$grid == 0} {
      set par(GRID) 1
      return
   } else {
      set par(GRID) $grid
   }

   if { $gridOn == 0 } { return }

   for {set t 0} {$t < 1400} {set t [expr {$gridStep + $t} ] } {
      
       $cv create  line  $t  0  $t   1400 -tags grid
       $cv create  line  0   $t 1400 $t   -tags grid
  
       if {[winfo exists $bcv] && $bcvFlag == 0 } {
          $bcv create line  $t  0  $t   1400 -tags grid          
          $bcv create line  0   $t 1400 $t   -tags grid 
       }
   }; # end for ...
}


#------------------------------------------------------------------------------
# PROCEDURE : setColorSetup - Change the color setup
#------------------------------------------------------------------------------

proc applyColor {} {
   global fileName  par

   set  oldFileName   $fileName
   set  fileName     "tmpPanel"   
 
   savePanel $fileName ; loadPanel pan "n"
   exec rm -f "$fileName"
   set fileName $oldFileName
}


#------------------------------------------------------------------------------
# PROCEDURE : panelQuit - Quit panel editor
#------------------------------------------------------------------------------

proc panelQuit {} {
   global par seq_ccsStatus

   if { $par(SAVE_FLAG) } {
      set    msg "This Panel HAS BEEN MODIFIED \n\n "
      append msg "QUIT the Panel Editor Anyway ?"
   } else {
      set msg "Do you really want to QUIT ?"
   }

   if {[tk_dialog .dialog "Dialog Box" $msg question 1 Yes No]} { return }

   panClassUnload ;# Unload class definition
   update

   foreach child  [winfo children "." ] {
      destroy $child
      update
   }

   if {[info exists seq_ccsStatus]} {seq_ccsExit}
   destroy .
   exit
}
