#
# E.S.O. - VLT project
#
# "@(#) $Id: configListBox.tcl,v 5.34 2005/09/26 12:44:23 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.COMIN   21/03/95  created 
# M.COMIN   07/04/95  Add configuration for : Y scroll, font
# M.COMIN   27/04/95  Add proc. to configure list
# M.COMIN   10/08/95  Add binding to RETURN in the 'Pos' entry.
#
# mcomin    27/02/96  Porting to TK4.0
#
#-----------------------------------------------------------------------------
# PROCEDURE : uifListBoxPanel -  Creates config. panel for UifListBox Class
#-----------------------------------------------------------------------------

proc uifListBoxPanel { idx top type } {
   global wdgConfig ckScrollx ckScrolly panSelectMode outputFonts
                            
   UifEntry $top.f13.geo -label " Geometry" -width 8\
               -labelWidth $wdgConfig(lblWidth) -labelFont $wdgConfig(lblFont)

   label    $top.f13.lbl1  -text " (Width x Height)"

   label    $top.f14.lbl2  -text " Select Mode :" -font $wdgConfig(lblFont)\
                           -anchor w -width $wdgConfig(lblWidth)

   radiobutton $top.f14.single -text "single" -value single \
                               -variable panSelectMode($idx) -anchor w

   radiobutton $top.f14.browse -text "browse" -value browse \
                               -variable panSelectMode($idx) -anchor w

   radiobutton $top.f14.multi -text multiple -value multiple \
                              -variable panSelectMode($idx) -anchor w

   radiobutton $top.f14.ext -text extended -value extended \
                            -variable panSelectMode($idx) -anchor w

   label $top.f15.lblFont -text " Widget Font :" -font $wdgConfig(lblFont) \
                          -anchor w -width $wdgConfig(lblWidth)

   createFontButton $top.f15.font  10 wdgConfig${idx}(font)\
                       [uplevel #0 set outputFonts]

   checkbutton $top.f15.xscroll -text "X Scroll" -variable ckScrollx($idx)
   checkbutton $top.f15.yscroll -text "Y Scroll" -variable ckScrolly($idx)

   #
   #  Create configuration panel
   #

   pack $top.f13.geo $top.f13.lbl1   -side left
   pack $top.f14.lbl2 $top.f14.single -side left
   pack $top.f14.browse $top.f14.multi $top.f14.ext -side left -padx 1

   pack $top.f15.lblFont $top.f15.font -side left -pady 10
   pack $top.f15.yscroll -side left -padx 20
   pack $top.f15.xscroll -side left

   #
   # Set Short Help Text
   # 

   set configHelp [getWdgConfig $idx configHelp]
   UifShortHelp::add $configHelp $top.f15.xscroll {Add horizontal scroll bar}
   UifShortHelp::add $configHelp $top.f15.yscroll {Add vertical scroll bar}

   configListValues $idx $type ;# Create sub-panel to configure a list
}


#------------------------------------------------------------------------------
# PROCEDURE : configUifListBox - Updates the data structures describing the
#                                widget configuration 
#------------------------------------------------------------------------------

proc configUifListBox { idx cv top  name } {
   global ckDbVar ckScrollx ckScrolly panSelectMode

   set configList [getWdgConfig $idx configList]
   set numbItems  [$configList getSize]

   set itemList {}

   for { set i 0 } {$i < $numbItems} { incr i } {
       lappend itemList [$configList getItem $i]
   }

   $cv.$name configure -geometry [$top.f13.geo get] -scrollx $ckScrollx($idx) \
             -scrolly  $ckScrolly($idx) -font  [getWdgConfig $idx font] \
             -selectmode $panSelectMode($idx) -itemList $itemList
}


#------------------------------------------------------------------------------
# PROCEDURE : displayUifListBox -
#
#------------------------------------------------------------------------------

proc displayUifListBox { idx cv top name } {
   global ckScrollx ckScrolly panSelectMode panSelectedFont

   setEntry $top.f13.geo    [$cv.$name cget -geometry]

   set panSelectedFont      [$cv.$name cget -font]
   setWdgConfig $idx font   [$cv.$name cget -font]

   set ckScrollx($idx)      [$cv.$name cget -scrollx]
   set ckScrolly($idx)      [$cv.$name cget -scrolly]
   set panSelectMode($idx)  [$cv.$name cget -selectmode]

   set w [getWdgConfig $idx configList]
   $w addItemList 0  [$cv.$name cget -itemList]
}

