#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: save.tcl,v 5.34 2005/09/26 12:44:17 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# M.Comin   05/07/94  Add comments to the code
# M.Comin   06/02/95  Add env variable SEQUENCER to specify which
#                     version of the sequencer to use
# M.COMIN   10/02/95  Add storing of local procs
# M.COMIN   06/03/95  Add identifier for archive
# M.COMIN   08/03/95  Add extension .pan to fileName if missing
#                     Add parameter 'fileName' to savePanel().
# M.COMIN   05/05/95  Backup copy is created when a panel is saved
# M.COMIN   20/07/95  Bug during 'save' ... problems with '_' in file path.
#                     Sequence of statements (at the end) changed
# M.COMIN   05/09/95  Add panInit to configuration file
# M.COMIN   02/10/95  Add cmd reply area
# M.COMIN   17/10/95  Save relief parameter for uifClass
# M.COMIN   25/10/95  Add parameter panel/class to properly save canvas items
# M.COMIN   21/11/95  Create backup copy for classes and fixed bug during
#                     creation of fileName
# M.COMIN   03/12/95  Variables are re-named
# M.COMIN   14/12/95  Fixed bug with Cmd. Reply Widget
# -------------------
# mcomin    08/01/96  Add re-numbering of widgets names
#                     ( was taken away by mistake )
# mcomin    24/01/96  Add check in canvas items to see if this is a widget.
#                     (this problem made unable to call uifText as 'text' )
#
#
# mcomin    27/02/96  Porting to TK4.0
#           21/05/96  panCreateFrame called before panInit
#           01/09/96  Clean event id from panDbData
#           05/09/96  Check in pubClassfor for UIF resources which should 
#                     not be stored .. just in case
#           01/07/97  Name of menuHelp element changes according to the 
#                     new fileName
#           21/11/97  Fixed bug when saving -evtScript option
#           20/04/98  Fixed bug when saving UifFrames in classes
#                     i_subClass was not set properly.
#           01/05/98  Add panUserInitData procedure
#           17/06/98  Take away '-f' from seqWish when invoked from a shell
#

#---------------------------------------------------------------------------
# PROCEDURE : save - 
#---------------------------------------------------------------------------

proc save { {what {}} } {
   global fileName par classMatch

   set classKey $par(CLASS)
   set file     [file tail $fileName] ;
   
   if {[eval regexp {$classMatch} $file all class key ] } {

      if {$key == "$classKey" && $what != "$classKey" } { 
         set msg "This is a Class --  Use 'Save Class' "
         tk_dialog .dialog "Dialog Box" $msg info 0 OK
         return 
      }

      if { [file extension $file] == ".pan" } {
         set classFile [file rootname  $file]
      } else {
         set classFile $file
      }

      saveClass  $classFile ;# Save the class

   } else {

      if { $what == "$classKey" } {
          tk_dialog .dialog "Dialog Box" \
          "Wrong fileName for a class  \n Use instead    saveClass as .." \
          info 0 OK
          return
      }
      savePanel $fileName
   }
}

#----------------------------------------------------------------------------
# PROCEDURE : saveAs - Save a panel with a name selected by the user
#----------------------------------------------------------------------------

proc saveAs { {what {} } } {
   global fileName fsbReturn par classMatch lastFilePath

   set msg1  "File already exist "
   set msg2  "Wrong fileName for a Panel  \n Use instead    saveClass as .."
   set box   "Dialog Box"
   set classKey  $par(CLASS) ;     # Suffix for classes

   if {$what == "$classKey"} {
       set tmp [panSelectFile $lastFilePath "*${classKey}.tcl" ]   
   } else {
       set tmp  [panSelectFile $lastFilePath "*.pan" ]
       set file [file rootname [file tail $tmp]]

       if {([string length $file] > 19 )} {
          set tmp "Process Name \"$file\" Too Long\n(maximum 19 characters)"
          tk_dialog .dialog "Dialog Box" $tmp info 0 OK
          return
       }
   }
   
   if { $tmp == "" } { return }
   
   set file    [file tail    $tmp]
   set dirPath [file dirname $tmp]
   set classFlag [eval regexp {$classMatch} $file all class key]

   if {$what == "$classKey"} {

      if { $classFlag == 0 } { append file "_${classKey}" } 
   
      if {[file extension $file] == ""} { append file ".tcl" }
      set fileName   ${dirPath}/${file}

      if { [file exists $fileName ] } {
        if {[tk_dialog .dialog $box $msg1 question 1 Continue Cancel]} { 
           return 
        }
      }

      saveClass    $fileName ;
      $par(mbFile) entryconfig 5 -state disabled
      $par(mbFile) entryconfig 4 -state disabled

   } else {

      if { $classFlag } {
         tk_dialog .dialog $box $msg2 info 0 OK
         return
      }

      if { [file extension $file] == "" } { append file ".pan" } 

      set oldFile   $fileName
      set fileName  ${dirPath}/${file} 
    
      if { [file exists $fileName ] } {
        if {[tk_dialog .dialog $box $msg1 question 1 Continue Cancel]} { 
           return 
        }
      }

      savePanel     $fileName $oldFile
   }
}


#-----------------------------------------------------------------------------
# PROCEDURE : savePanel - Creates the TCL script to create the panel
#-----------------------------------------------------------------------------

proc savePanel { fileName {oldFile {}} } {
   global EDITING  par  shArea filePath mainHelp env cwp
   global colordef  tk_strictMotif  id  i_type i_subClass db_env
   global ckShHelp  ckMenuBar ckActBut ckCCS  ckCmdReply panWdgReply
   global panUsrProcs panUsrLib  panWidget panWidgetHlp panDbData menu
   global namespaceList ckHeartBeat panDEBUG
   global panWinXSize panWinYSize panCvXSize panCvYSize panXPos panYPos 

   if {[file exists $fileName] } {
       if { [catch { exec cp $fileName ${fileName}~ }] } {
          set msg "Failed to create Panel BACKUP copy"
          set tmp "Dialog Box" 
          if {[tk_dialog .dialog $tmp $msg question 1 Continue Cancel]} { 
             return 
          }
       }
   }

   set par(SAVE_FLAG) 0
   set par(EDIT_WHAT) ".pan"
   $mainHelp displayText "SAVING ...." ; update idletasks

   panGetCvNames   $par(ROOT_PANEL) cv bcv ;  # Get canvas Names
   set myRoot      [getMyRoot $fileName];     # Get current root name
   set id          [open $fileName w+]
   set par(SPLEN)  16;
   set par(SPAD)   ""

   set geo    [wm geometry $par(ROOT_PANEL)]
   scan $geo "%dx%d+%d+%d" wid hei posx posy    ;# Calculate geometry

   if {$par(CCS)} { 
       panEvtDbOperation $cv DETACH all ;# Disable all events
   }

   #
   # Writes in the file the standard part
   #   
  
   if {[info exists env(PANEL_WISH)]} {
      set tmp $env(PANEL_WISH)
   } else {
      set tmp "seqWish"   
   }    
  
   puts $id "#!/bin/sh"
   puts $id "# Define which version of the sequencer to use \\"
   puts $id "  exec $tmp \"\$0\"  \$\{1+\"\$@\"\}"
   
   saveHeader $id $fileName

   puts $id "#"
   puts $id "global par gvar cv bcv actAreaHeight shArea panClassLib "
   puts $id "global geoCmdReply panClassLib panUsrProcs panUsrLib"
   puts $id "global argv0 env panWidget panWidgetHlp panDbData cwp ckHeartBeat"
   puts $id "global menu menuBar menuHelp ckMenuBar ckActBut ckShHelp ckCCS"

   puts $id "# "

   puts $id " if \{ \[catch \{seq_findFile lib/libpanPublic.tcl \} ret\] \} \{ "
   puts $id "   puts stderr \"panel EXITS : Tcl library libpanPublic.tcl not found \" "
   puts $id "   exit -1"
   puts $id "\} else \{"
   puts $id "   lappend auto_path \$ret"
   puts $id "\}"

   #################################################
   #         Set auto_path to find the libraries
   #################################################

   set panClassLib {}
    
   foreach name [panCvWidgetNames $cv all]  { 
 
      # Skip 'normal' widgets and those which are sub-classes -i.e.- classes
      # nested into other classes
      
#      puts "DBG > find libraries : $name"

      if { [getWidget $name $i_type ] != "UifClass" } { continue }

      set libList [split [$cv.$name cget -library] " "]

      foreach myLib $libList {

	  if {[cequal $myLib ""]} { continue }

	  set myLib [file rootname $myLib]

	  if { [lsearch $panClassLib $myLib ] == "-1" } {

	      # Check if the ibrary really exists
	      if {! [cequal [panCheckLibPath $myLib] "NOT_FOUND"]} {
		  lappend panClassLib $myLib 
	      } else {
		  puts "WARNING : library $myLib NOT FOUND"
	      }
	  }

      }
   }

   puts $id "\n#\n# Link panel to user libraries \n#"  

   puts $id "set panUsrLib   [list $panUsrLib]  "
   puts $id "set panClassLib [list $panClassLib]\n"
   puts $id "panSetAutoPath  \$panUsrLib \$panClassLib \n"

   if {[info exists namespaceList] && ![lempty $namespaceList]} {
      set namespaceList [lrmdups $namespaceList]
      puts $id "panAddNamespace $namespaceList\n"
   }

   #
   # Set root panel
   #

   puts $id "if {\[info exists EDITING\]} \{ "

   panSetProcList $oldFile ; panInitProcEditor

   puts $id "   set panRoot .${myRoot} "
   puts $id "   set panUsrProcs [list $panUsrProcs]"
   puts $id "   set argc 0 ; set argv {}"
   puts $id "\} else \{" 
   puts $id "   set par(MY_NAME) \[file tail \$argv0\] "
   puts $id "   set EDITING 0"
   puts $id "   set tk_strictMotif $tk_strictMotif"     
   puts $id "   setPublicVar"
   puts $id "   if \{\[info exists panImportFlag\] && \$panImportFlag != 0\} \{"
   puts $id "      set panRoot .${myRoot} "
   puts $id "   \} else \{ "
   puts $id "      set panRoot ."
   puts $id "   \}"
   puts $id "\}" 

   #
   # Store global data and init procedures
   #

##   puts "DBG > save : oldRoot = $oldRoot , panUsrProcs = $panUsrProcs"

   if {[lsearch $panUsrProcs ${myRoot}GlobalData ] != -1} {
      puts $id "\n# User's Global Data "
      puts $id "# It contains definitions of constant data\n"
      puts $id [panGetProcDef ${myRoot}GlobalData ]    
      puts $id "\n"
   }

   if {[lsearch $panUsrProcs ${myRoot}InitData] != -1} {
      puts $id "\n# User's Init Data : called just after panInit"
      puts $id "# It contains data to be defined before widgets are created\n"
      puts $id [panGetProcDef ${myRoot}InitData ]
      puts $id "\n"
   }

   if {[lsearch $panUsrProcs ${myRoot}Init] != -1} {
      puts $id "\n# User's Init Procedure : called in panMainStart"
      puts $id "# It is executed after the widgets have been created\n"
      puts $id [panGetProcDef ${myRoot}Init ]
      puts $id "\n"
   }

   puts $id "\n#"
   puts $id "#-----------------------------------------------"
   puts $id "#            Widget Definition  "
   puts $id "#-----------------------------------------------"
   puts $id "# \n"

   puts $id "set ckHeartBeat   $ckHeartBeat ;# Enable Heart-beat"
   puts $id "set ckShHelp      $ckShHelp ;# Set Short Help"
   puts $id "set ckActBut      $ckActBut ;# Set Action Button Area"
   puts $id "set ckMenuBar     $ckMenuBar ;# Setup Menu Bar"
   puts $id "set ckCCS         $ckCCS ;# Initialize CCS"
   puts $id "set ckCmdReply    $ckCmdReply ;# Display Cmd. Reply"
   puts $id "set cwp           \"$cwp\" ;# Set Current Working Point" 

   if {$ckCmdReply} {
      if {[info exists par(sizeCmdReply)] && $par(sizeCmdReply) != 2 } {
        puts $id "set par(sizeCmdReply) $par(sizeCmdReply) ;# Size of reply window"
      }
   }

   set height [lindex [$bcv configure -height] 4]
   puts $id "set actAreaHeight $height; # Set height of action area \n"

   puts $id "panReadOption $colordef" 

   if {$panXPos != "" && $panYPos != ""} {
      puts $id "set par(PANELX) $panXPos ;# Panel X position"
      puts $id "set par(PANELY) $panYPos ;# Panel Y position"
   }

   puts $id "panCreateFrame $myRoot ;# Create empty frame"
   puts $id "panGetCvNames  $myRoot cv bcv ;# Get Canvas Names"
   puts $id "panInit        \$argc \$argv $myRoot ;# Initialization" 

   # Check which are the env. names used 

   set tmpEnvList {}

   foreach name [panCvWidgetNames $cv all]  {
       set type [getWidget $name $i_type ]
       set tmpEnv ""

       if {[cequal $type UifClass]} {
          set tmpEnv [$cv.$name cget -envName]
       } else {
	   if {[info exists panDbData($name)]} {
              set tmpEnv [getDbConfig $name $db_env]
           }
       }

       if {[string index $tmpEnv 0] == "\$"} {
          lappend tmpEnvList [string range $tmpEnv 1 end]
       }
   }

   if {! [lempty $par(ENV_LIST)]} {
      set tmp [lrmdups [intersect $tmpEnvList $par(ENV_LIST)]]
      puts $id "panInitEnvNames $tmp"
   }

   puts $id "\n\n"
   #
   # Take away all definition concerning widgets of a class
   #
   saveWidgetDefinition  $id  $cv $bcv ;          # Save Widgets
   saveCanvasItem        $id  $cv $myRoot panel ; # Save canvas items 
   
   saveArray  "" panWidget     $id  "Set Widget Configuration"          
   saveArray  "" panDbData     $id  "Set Database Configuration"            
   saveArray  "" panWidgetHlp  $id  "Set Short Help for each widget"
      
   saveMenuBar $id $myRoot
     
   puts $id "\n#\n#-----------------------------------------------"
   puts $id "#            Create the Panel  "
   puts $id "#-----------------------------------------------\n#\n"

   if {$par(TITLE) == "Dummy" } { 
      set title $myRoot 
   } else {
      set title $par(TITLE) 
   }

   puts $id "wm geometry  \$panRoot ${wid}x${hei}"       
   puts $id "wm title     \$panRoot \"$title\""
   puts $id "set par(TITLE) \"$title\" \n"

   puts $id "panMainStart  $myRoot \$argc \$argv"


   puts $id "\n#--------------------    End of Main   --------------------\n"

   if { [llength $panUsrProcs] != 0} {

      puts $id "\n\n#####################################################"
      puts $id "#           Definition of Local Procedures"
      puts $id "#####################################################"

      panStoreProc $id $myRoot
   }
  
   close $id
   exec chmod ug+x "$fileName"

   if {[cequal panDEBUG "dbgDEV"] } {
       puts  "Panel SAVED with title : $par(TITLE)"
   }

   $mainHelp displayText "Panel SAVED with title : $par(TITLE)"
#   puts "Current title = $par(TITLE) \n"   
}


#-----------------------------------------------------------------------------
# PROCEDURE : saveCanvasItems - Save canvas item such as free text, etc ..
#-----------------------------------------------------------------------------

proc saveCanvasItem { id cv root what } {
   global par

   puts $id "\n$par(SPAD)#"
   puts $id "$par(SPAD)#  Canvas Items"
   puts $id "$par(SPAD)#\n"

   set counter 1

   foreach item [$cv find withtag text] {

      set tmp    [lindex [$cv itemconfigure $item -tags ] 4]

      if {[lsearch $tmp "widget"] != -1 } { continue }

      set name "${root}-item${counter}"
      set posx [lindex [$cv coords $item ] 0]; set posx [expr {int($posx)}]
      set posy [lindex [$cv coords $item ] 1]; set posy [expr {int($posy)}]
      set tmp  [lindex [$cv itemconfigure $item -text ] 4]

      # Substitute all '[' and ']' with '(' and ')'

      regsub -all {\[} $tmp {\(} tmp ; regsub {\]} $tmp {\)} text

      # Do not save empty canvas freetext items
      if {[string trim $text " "] == "" } { continue }

      set font [lindex [$cv itemconfigure $item -font ] 4]

      # This statement is used to automatically convert "helvR14"
      # with the proper name ... some old panel have it still !!

      if {$font == "helvR14"} {
         set font "-Adobe-helvetica-medium-r-normal--14*"
      }

      set Tags "text $name"

      if {$what != "panel" } {   
         append Tags " classItem $root" 
      }

      set Anchor  [lindex [$cv itemconfigure $item -anchor ] 4] 
      set color   [lindex [$cv itemconfigure $item -fill ] 4]

      set tmp "$par(SPAD)\$cv create text $posx $posy -tags \"$Tags\" -anchor $Anchor \\"
      append tmp "\n$par(SPAD)          -text \"$text\" \\"
      append tmp "\n$par(SPAD)          -fill $color -font $font \n"
      
      incr counter
      puts $id $tmp
   }
}


#-----------------------------------------------------------------------------
# PROCEDURE : saveWidgetDefinition - Saves the TK calls to define the 
#                                    widgets.
#-----------------------------------------------------------------------------

proc saveWidgetDefinition { id cv {bcv {} }  } {

   global nosave  panWidgetHlp nameMap par gvar fileName
   global panWidget i_type i_text  i_var db_var i_default
   global widget panDbData saveList panArrayList
   global nameMatch keep widgetOption
   
   #
   # Save Widget definition
   #

   set classRef 0

   if { [string match "*uifClass.tcl.pan" $fileName ] } {
      set tmp [split $fileName "/"]
      set len [llength $tmp]
       set tmp [split [lindex $tmp [expr {$len - 1}]]  "."]
      set classRef [lindex $tmp 0]
   } 
   
   set tmp ""
   if { $bcv != "" } { set tmp [panCvWidgetNames $bcv uifType] }
      
   set allWdg [lsort [ concat [panCvWidgetNames $cv all] $tmp ]]

   set counterId 1
   catch {unset panArrayList}

   foreach name $allWdg {
      
      if {[info exists panWidget($name)] == 0 } {
         set tmp "No Data for widget\n$name \nWIDGET NOT SAVED"
         tk_dialog .dialog "Dialog Box" $tmp info 0 OK 
         continue
      }

      if {[info exists widgetOption]} { unset widgetOption }
      set type  [getWidget $name $i_type]

      #
      # Build the string containing the widget definition which does not
      # contain configuration parameters related to :
      #
      # - default values
      # - definition of synonyms
      # - parameters not to be saved (listed in 'nosave')
      #

      #
      # Re-name the widgets (only those which are automatically named )
      # 

      set newName [panNewWdgName $name]

      if {[eval regexp {$nameMatch} ${newName} all file wdgName ] } {

         if {[regexp {([a-zA-Z]+)([0-9]+)} ${wdgName} all prefix idx] } {
	    if {$prefix == "wdg"} {
               set newName "${file}-${prefix}${counterId}"
               incr counterId    
            }
	 }
      }

      set wdgName $newName 
      set panArrayList($name) $newName     

#      puts "DBG > save widget ($type) widget name = $newName "

      if { $type == "UifActionButton" } {

          set w      $bcv.$name
          set class  [$w info class]
          set wdgName "\$bcv.$newName"

      } elseif { $type == "UifClass" } {

          set w       $cv.$name
          set class   [$w info class]
          set wdgName "\$cv.$newName"

          set widgetOption(-library)       [$w cget -library]
          set widgetOption(-canvasAnchor)  [$w cget -canvasAnchor]

	  # For better backwards compatibility add these options
          # only if different from defaults

	  if {[$w cget -show] == 0} {
	      set widgetOption(-show)  [$w cget -show]
	  }

	  if {[$w cget -globalName]} {
	      set widgetOption(-globalName) [$w cget -globalName]
	  }

          if {![cequal $par(EDIT_WHAT) ".pan"]} {

             # Check if cwp of current class is not absolute 
             # or <alias>
        
             set tmp [string index [$w cget -cwp] 0]

	     if {![cequal $tmp "<"] && ![cequal $tmp ":"] } {
                set widgetOption(-parentCwp) "\$itk_option(-cwp)"
             }
          }

          if {![cequal [$w cget -cwp] ""]} {
            set widgetOption(-cwp) [$w cget -cwp]
          }

          if {![cequal [$w cget -envName] ""]} {
             set widgetOption(-envName) [$w cget -envName]
          }

          set widgetOption(-relief) [$w cget -relief]   
          saveResources $class $wdgName $id 
          continue
      } else {
          set w       $cv.$name
          set class   [$w info class]
          set wdgName "\$cv.$newName"
      }

      #
      # Form list of X resources 
      # Each resource is defined by a list of four elements : name, initial
      # value,current value, configuration command.
      #
      set dbFlag 1

      if {[cequal [winfo class $w] classFrame] ||
          [cequal [winfo class $w] classContainer] } {
          set widgetOption(-helparea) "\$shArea"
      }

      foreach item [$w configure ] {
         set name [lindex $item 0];   # Get resource name

         if {[set value [$w cget $name]] == "" } {
            continue  ;# Skip empty definitions
         }

#        puts "DBG > type name value : $type $name $value"
         
	 if { $type == "PubClass"} {
            # Take away resources which could be common to UIF widgets
            # and are normally not stored

            if {[lsearch $nosave(UIF_ALL) $name] == -1} {
               set widgetOption($name) $value
            }
            continue
         }

         #---------------------------------------------
         # Take away resources for specific widgets
         #---------------------------------------------

         if {[lsearch $keep(ALL) $name] == -1}  {
	    if {[info exists keep($type)] == 0 }    { continue }
            if {[lsearch $keep($type) $name] == -1} { continue }
#            if {[lsearch $nosave(ALL) $name] != -1} { continue }
	 }
             
         # If dbMode == none , the widget is not associated to any
         # database attribute, therefore we ignore some configuration
         # parameters.

         if {$name == "-dbMode" && $value == "none" } { 
            set dbFlag 0 ; continue
	 }

         if {$dbFlag == 0 && $name == "-evtScript" } { continue }
         if {$dbFlag == 0 && $name == "-evtFilter" } { continue }


         if { $name == "-evtScript" } {
             set value [lreplace $value 1 1 $newName]               	     
	 } elseif { $name == "-variable" || $name == "-stateVar" } {

            if {[panParseArray $value array elem]} { set value $elem }
            set value "gvar([panNewVarName $value])"
         }

         # This statement is used to automatically convert "helvR14"
         # with the proper name ... some old panel have it still !!
 
         if {$name == "-font"} {
	     if {$value == "helvR14"} { 
                set value "-Adobe-helvetica-medium-r-normal--14*"
             }
         }
         set widgetOption($name) $value

      }; #end foreach ...
  
      saveResources  $class $wdgName $id      
      
   }; # end foreach ...
}


#-----------------------------------------------------------------------------
# PROCEDURE : saveResources
#-----------------------------------------------------------------------------

proc saveResources { class  wdgName id} {
    global par widgetOption

    set class [string trimleft $class "::"]
    
    set maxPad   $par(SPLEN);   
    set padBuf   "                                                        "
    set padLen   [expr {$maxPad - [string length $class]}];   
    set pad      [string range $padBuf 0 $padLen];      # Set padding string
    
    set hline    "${class} ${pad} $wdgName "
    set len      [string length $hline] 
    set line     ""
    
    # puts "$class $wdgName  $maxPad"

    set optList [lsort [array names widgetOption]]
    set lastOpt [lindex $optList end]
    
   foreach option $optList {

       set value $widgetOption($option)
       set len1 [string length $option]
       set len2 [string length $value]

       set optLen [expr {$len1 + $len2}]
       set newLen [expr {$optLen + $len + 3}]

       # puts "    $option $value [llength $value]"
       if { $newLen > 75 } {
          puts $id  "${hline}${line} \\";
          set line  ""
	   set hline "$par(SPAD)[string range $padBuf 0 [expr {$maxPad + 1}]]"   
       }

       append line  "$option "

       # Process specific resources

       if {[cequal $option -wdgConfig] || [cequal $option -envName] ||
           [cequal $option -command]   || [cequal $option -units]   ||
           [cequal $option -label] } {

          append line "\{$value\} "

       } elseif { [cequal $option -helparea]} {
          append line "$value "
       } else {

          if {[llength $value] != 1} {
              append line "\{$value\} "
          } else {
              append line "$value "
          }
       }

       set len  [expr {[string length $line] + [string length $hline]}]
   }
   
   if { ![cequal $line ""] } { puts $id  ${hline}${line} }
}

#-----------------------------------------------------------------------------
# PROCEDURE : saveArray - Save the content of a given array
# Array parArrayList gives the new name correspondence :
# - panArrayList(name) newName 
#
#-----------------------------------------------------------------------------

proc saveArray { what arrayname id comment} {
   global par fileName panArrayList i_var i_type i_subClass db_event
   upvar #0 $arrayname array

   if {[info exists panArrayList] == 0 } { return }
   if {[info exists buffer]} { unset buffer }

   puts $id "\n$par(SPAD)#"
   puts $id "$par(SPAD)# $comment"
   puts $id "$par(SPAD)#"   

   # Assign values to temporary buffer
   
   foreach name [lsort [array names panArrayList]] {

      if {[info exists array($name)] == 0 } { continue }

      set new          $panArrayList($name)
      set buffer($new) $array($name)

      if {$arrayname == "panWidget"} {
         set type [getWidget $name $i_type]         
         set var  0;

         if {$type == "UifClass" || $type == "PubClass" } {
             continue
	 }

         set var    [lindex $buffer($new) $i_var]
         set newVar [panNewVarName $var]
         set buffer($new) [lreplace $buffer($new) $i_var $i_var $newVar]

         # Flag widget as class component
         if {$par(EDIT_WHAT) != ".pan"} {
            set buffer($new) [lreplace $buffer($new) $i_subClass $i_subClass 1]
         }
      }
      
      if {$arrayname == "panDbData"} {
        set buffer($new) [lreplace $buffer($new) $db_event $db_event ""]
      }

#      puts "DBG > $name - buffer($newName) = $buffer($newName)"

   } ; # end foreach .. panArrayList

   if {[info exists buffer] == 0 } { return }

   foreach name [lsort [array names buffer]] {

      if {[set value [string trim $buffer($name) " "]] == ""} { continue }  

      if {$what == "$par(CLASS)"} {
         set tmp      [split $name "-"]   
         set len      [llength $tmp] ; incr len -1
         set nameName [lindex $tmp $len]
         set  myArray [format "%s(\$\{top\}.%s)" $arrayname $name ]
      } else {
         set  myArray [format "%s(%s)" $arrayname $name ]
      }

      if {$arrayname == "panDbData"} {
         puts $id [format "$par(SPAD)set %s  \[list %s\]" $myArray $value]
      } else {
	 puts $id [format "$par(SPAD)set %s  {%s}" $myArray $value]
      }

   }; # end foreach ... on buffer
}

#----------------------------------------------------------------------------
# PROCEDURE : getMyRoot - Returns the window root name from the fileName
#----------------------------------------------------------------------------

proc getMyRoot { file } {
   global par classMatch

   set tmp [file tail $file]

   if { [eval regexp {$classMatch} $tmp all class key ] } {
      if {$key == "$par(CLASS)" } { return $class }
   }
#   puts "DBG > getMyRoot $tmp, [file rootname $tmp]"
   return     [file rootname $tmp]
}

#----------------------------------------------------------------------------
# PROCEDURE : saveHeader - 
#----------------------------------------------------------------------------

proc saveHeader { id fileName } {

   global sccsId

   puts $id "#"
   puts $id "#*************************************************************"
   puts $id "# E.S.O. - VLT project"
   puts $id "#"
   puts $id "# \"@(#) \$Id\$\" "
   puts $id "#"
   puts $id "# [file tail $fileName ]"
   puts $id "#"
   puts $id "# This file has been produced by the panel editor"
   puts $id "#   version: $sccsId"
   puts $id "#   date   : [clock format [clock seconds]]"
   puts $id "#"
   puts $id "#      !!!!!!!!!!  DO NOT MANUALLY EDIT THIS FILE  !!!!!!!!! "
   puts $id "#"
   puts $id "#--------------------------------------------------------------"
}


#----------------------------------------------------------------------------
# PROCEDURE : saveMenuBar - 
#----------------------------------------------------------------------------

proc saveMenuBar { id newRoot } {
   global cv wdgConfig  topMenu menu ckMenuBar
      
   if { $ckMenuBar == 0 } { return }
      
   puts $id "\n#\n#-----------------------------------------------"
   puts $id "#           Menu Bar Definition  "
   puts $id "#-----------------------------------------------\n# \n"

   set w [winfo toplevel $cv]

   foreach i [lsort [array names menu]] {
      puts $id "set menu($i) \{[string trim $menu($i) " "]\}"
      set wdgName [lindex $menu($i) 2]
      lappend menuList $w.topMenu.menu.$wdgName
   }
   puts $id ""

   set idx 0
   foreach item $menuList {
      set mnLabel [$item cget -text]
      set mnUline [$item cget -underline]
      set wdgName [winfo name $item]
         
      if {[string tolower [string trim $mnLabel " "]] == "help"} {
          set side right
      } else {
          set side left
      }

      puts $id "\n# Add MenuButton <$mnLabel> \n" 
      puts $id "set mb \[addMenuButton \$menuBar $wdgName \"$mnLabel\" $mnUline $side\]"

      # Save the definition of all sub-menus
      saveMenu $id $idx [$item cget -menu]
   }

   saveMenuHelp $id $w.topMenu.menu $newRoot   ;# Save Menu Help
}

proc saveMenu { fh depth menuList } {
   global wdgConfig topMenu menu panDEBUG

   # For each menu 
   # - Save menu configuration
   # - visit the sub-menu in the same way

   if {[lempty $menuList]} { return }

   foreach tmp $menuList {
      lassign $tmp menuPath menuID lblMenu
      puts $fh "\n# Menu $lblMenu \n" ;# Separator between menus

      set subMenuList {}
      set maxItem     [$menuPath index end]

      if {$panDEBUG == "dbgDEV" } {
         puts "DBG> Process Menu ([winfo name $menuPath]) path = $menuPath"
      }

      for {set i 1} {$i <= $maxItem} {incr i } {

         if {[set type [$menuPath type $i]] == "cascade" } {
             set tmp     [$menuPath entrycget $i -menu]
             set lblMenu [$menuPath entrycget $i -label]
             lappend subMenuList [list $tmp ${depth}${i} $lblMenu ]
         }

         menuSaveConfig $fh $depth $menuID $menuPath $i $type

      } ;# End loop on menu entries
        
      saveMenu $fh [incr depth] $subMenuList

   } ; # end foreach ...        
}   



proc menuSaveConfig { fh depth id w idx type } {
   global panDEBUG

   if {$type == "separator" } {
      puts $fh "\$mb${id} add separator"

      if {$panDEBUG == "dbgDEV" } { puts "DBG> $idx $type" }
      return
   }

   set mnLabel [$w entrycget $idx -label]
   set mnUline [$w entrycget $idx -underline]
   set mnCmd   [$w entrycget $idx -command ]

   if {$panDEBUG == "dbgDEV" } { puts "DBG> $idx $type - $mnLabel" }

   if {[string index $mnCmd 0] != "\"" } { set mnCmd "\{$mnCmd\}" }

   if { $type == "command" } {

      puts $fh "addMenuCmd \$mb${id} \"$mnLabel\" $mnUline $mnCmd"

   } elseif {$type == "radiobutton" } {

      set mnVar [$w entrycget $idx -variable]
      set mnVal [$w entrycget $idx -value]
      puts $fh "addMenuRb \$mb${id} \"$mnLabel\" $mnUline $mnCmd $mnVar \"$mnVal\""

   } elseif {$type == "checkbutton" } {

      set mnVar [$w entrycget $idx -variable]
      puts $fh "addMenuCk \$mb${id} \"$mnLabel\" $mnUline $mnCmd $mnVar"

   } elseif {$type == "cascade" } {

      set wdgName [winfo name [$w entrycget $idx -menu]]
      puts $fh "set mb${depth}${idx} \[addCascadeMenu \$mb${id} $wdgName \"$mnLabel\" $mnUline\]"
   }
}


proc saveMenuHelp { fh topMenu newRoot} {
   global menuHelp

   puts $fh "\n#\n#-----------------------------------------------"
   puts $fh "#           Menu Help Definition  "
   puts $fh "#-----------------------------------------------\n# \n"

   foreach name [array names menuHelp] {

      # Skip definitions of other panels
      if {[string match ${topMenu}* $name] == 0 } { continue }

      # Check if there is any active help : if all help text are empty
      # the current definition is not saved

      set saveFlag 0

      foreach tmp $menuHelp($name) {
         if {[string trim $tmp " "] != "" } { incr saveFlag }
      }

      if {[winfo exists $name] == 0 || $saveFlag == 0} { continue }

      # Change first name according to the 'newRoot' and save
      # the menuHelp definition

      set tmp [string trimleft $name "."]
      set tmp [split $tmp "."]
      set tmp [lreplace $tmp 0 0 $newRoot]
      regsub -all { } $tmp "." tmp
      
      puts $fh "set menuHelp(.${tmp}) \{ \\"

      set tmpLine ""
      foreach tmp $menuHelp($name) {

         set text   [string trim $tmp " "  ]
         set len    [string length $text   ]
         set allLen [string length $tmpLine]

         # Consecutive help text not longer than 75 chars is saved 
         # in a single line.

         if {[expr {$len + $allLen}] < 75} {

	    append tmpLine "\{$text\} "

         } else {

	    if {$tmpLine != ""} { puts $fh "    $tmpLine \\" }
            set tmpLine "\{$text\} "
         }
      } ;# End foreach menuHelp($name)

      if {$tmpLine != ""} { puts $fh "    $tmpLine \\" }

      puts $fh "\} \n"  ;# End of the definition
   }
}
