#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: saveClass.tcl,v 5.34 2005/09/26 12:44:21 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   22/11/94  Created
# M.COMIN   06/03/95  Add identifier for archive
# M.COMIN   08/03/95  Add 'fileName' parameter to saveClass()
# M.COMIN   10/03/95  Add check fileName executing  saveClass 
# M.COMIN   27/07/95  Add reset of par(SAVE_FLAG)
# mcomin    24/10/95  Use of 'args' during construction instead of 'config'
# mcomin    30/10/95  Members of the class are marked (i_subClass)
#
# mcomin    27/02/96  Porting to TK4.0
#           22/07/96  Add check on first character of a class name
# mcomin    25/08/99  Fixed problem with libraries for mega-widgets 
#
#************************************************************************
#   NAME
#
#   saveClass -  This files contains a set of routine called to save a panel
#                as a class
# 
#   SYNOPSIS
#
#  
# 
#   DESCRIPTION
#
#
#   FILES
#    
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

#---------------------------------------------------------------------------
# PROCEDURE : saveClass - 
#---------------------------------------------------------------------------

proc saveClass { fileName } {
   global cv bcv par panDbData filePath auto_path panUsrLib namespaceList
   global id panWidget gvar panWidgetHlp saveList mainHelp
   global i_subClass i_type db_log i_clib

   if {[file exists $fileName] } {
       if { [catch { exec cp $fileName ${fileName}~ }] } {
          set msg  "Failed to create CLASS BACKUP copy"
          tk_dialog .dialog "Dialog Box" $msg info 0 OK
          return
       }
   }

   set myClass [file  rootname [file tail $fileName]]

   if {[ctype upper [string index $myClass 0]]} {
      set msg    "Failed to SAVE class : Name\n"
      append msg "must start with lower-case letter"
      tk_dialog .dialog "Dialog Box" $msg info 0 OK
      return
   }

   set id             [open $fileName w]
   set par(SPLEN)     16;
   set par(SPAD)      "    "
   set counter        0
   set par(SAVE_FLAG) 0
   set par(EDIT_WHAT) [file extension $fileName] 
     
   set myRoot [getMyRoot $fileName]  ;# Get current root name

   scan [winfo geometry $par(ROOT_PANEL)]  "%dx%d+%d+%d" tmp1 tmp2 posx posy
   scan [winfo geometry $cv]               "%dx%d+%d+%d" wid  hei  tmp1 tmp2
 
   set myBorder 2                    ;# Take border dimensions away
    set wid [expr {$wid - 2*$myBorder}]
    set hei [expr {$hei - 2*$myBorder}]
   #
   # Writes in the file the standard part
   #   
   saveHeader $id $fileName

   if {![lempty $par(ENV_LIST)] } {
      puts $id "\n# Used only in EDITING"
      puts $id "uplevel #0 lappend par(ENV_LIST) [lrmdups $par(ENV_LIST)]"
   }

   #################################################
   #         Set auto_path to find the libraries
   #################################################

   set panClassLib {}

   foreach name [panCvWidgetNames $cv all] {
     
      switch [getWidget $name $i_type ] {
          UifClass {
             set tmp   [$cv.$name cget -library]   
             set myLib [file rootname $tmp]
          }
          PubClass {
             set tmp [getWidget $name $i_clib]
             set myLib [file rootname $tmp] 
          } 
          default { set myLib "" }
      } ;# End switch ...
 
      if {! [cequal [panCheckLibPath $myLib] "NOT_FOUND"]} {
         lappend panClassLib $myLib 
      }

   } ;# End foreach ...

   set panClassLib [lrmdups $panClassLib]
   #
   # Reset db_log flag
   #
   foreach name [array names panDbData] {
      if {$name == "none" } { continue } 
      setDbConfig  $name $db_log 0
   }

   puts $id "\n\nclass $myClass \{ "
   puts $id "  inherit UifPanel \n"
   puts $id "  constructor {args} \{ \n"
   puts $id "    global ::shArea"

   puts $id "    panSetAutoPath [list $panUsrLib $panClassLib]"

   if {[info exists namespaceList] && ![lempty $namespaceList]} {
      set namespaceList [lrmdups $namespaceList]
      puts $id "    panAddNamespace $namespaceList\n"
   }

   puts $id "    parseClassName \$this \n"
   puts $id "    itk_component add top \{"
   puts $id "       canvas \$itk_interior.cv -width $wid -height $hei \\"
   puts $id "              -borderwidth $myBorder -highlightthickness 0"
   puts $id "    \} \{"
   puts $id "       keep -relief "
   puts $id "    \}\n"

   puts $id "    pack \$itk_component(top) -fill both -expand 1 "
   puts $id "    eval itk_initialize \$args\n"

   saveWidgetDefinition  $id $cv  
   saveCanvasItem        $id $cv $myRoot class ; # Save canvas items 

   puts $id "  \}; # end of constructor\n" 

   puts $id "  #-------------------------------------------"
   puts $id "  #*# Method init"
   puts $id "\n  public method init { } \{"
   puts $id "    global ::panWidget ::gvar ::panDbData ::panWidgetHlp "
   puts $id "    set top \$className "

   saveArray  $par(CLASS) panWidget    $id  "Set Widget Configuration"
   saveArray  $par(CLASS) panDbData    $id  "Set Database Configuration"
   saveArray  $par(CLASS) panWidgetHlp $id  "Set Short Help for each widget"

   puts $id "\n$par(SPAD)setInstanceData"
   puts $id "$par(SPAD)set initDone 1"
   puts $id "  \}; # end of init\n"

   puts $id "\} ; #end of the class"

   close $id

   $mainHelp displayText "Panel Class SAVED with name : $myClass"  
}
