#
# E.S.O. - VLT project
#
# "@(#) $Id: saveConfig.tcl,v 5.34 2005/09/26 12:44:21 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini ??/??/??  created
# M.Comin   16/03/95  Add configuration for 'frame'
# M.COMIN   23/03/95  Add UifOutputString
# M.COMIN   06/04/94  Add UifText
# M.COMIN   27/04/95  Add UifOptionButton
# M.COMIN   04/05/95  Add proc saveDbInterface
# mcomin    25/10/95  Events are not automatically set during editing of
#                     a class.
# mcomin    16/12/95  Modified naming for dummy variables
# ------------------
# mcomin    08/01/96  Fix bug in creating bitmap
#                     Fix bug for initial value of ckStatus
#           23/02/96  Fixed bug when configuring default values for radio 
#                     buttons. Loop is now on canvas children widgets and 
#                     not on panWidget
#

#------------------------------------------------------------------------------
# PROCEDURE : saveConfig - Updates the widgets data structures with the new 
#                          configuration specified in the configuration panel
#------------------------------------------------------------------------------

proc saveConfig {idx type cv} {
   global gvar par shArea panWidgetHlp panDbSymbAddr panWidget panDbData
   global ckReSize ckHotSpot ckStatus ckDbVar ckDbOnvalue ckDbOffvalue
   global ckDbMin ckDbMax panSelectedFont
   global i_type  i_posX  i_posY  i_var i_bitmap i_default i_reSize

   #---------------------------------------------------------
   #   Save Common Configuration parameters
   #---------------------------------------------------------

   if {[set name [getConfigInput $idx wdgName]] == ""} {
      set msg "Empty Widget Name\nOPERATION ABORTED"
      tk_dialog .dialog "Dialog Box" $msg info 0 OK
      return "FAILURE"
   }

   set oldName [getWdgConfig $idx oldName]
   set top     [getWdgConfig $idx topLevel].top
   set optList [getWdgConfig $idx optList]

   ###
   ###  Check Validity of  posX posY coordinates
   ###
   
   set posx [getConfigInput $idx wdgX]  ;  # Get widget coordinates
   set posy [getConfigInput $idx wdgY]

   if { $posx == "" || $posy == "" } {  
       set pos [$cv coords $oldName];
       set posx [lindex $pos 0]
       set posy [lindex $pos 1]
   }  

    set posx [expr {int($posx)}]
    set posy [expr {int($posy)}]

   ###
   ###  Check If the name of the widget has been changed
   ###

#   puts "DBG > saveConfig : cv = $cv name = $name"

   if {![cequal $type freeText]} {

      if { ![cequal $name $oldName] } {

         # Duplicate widget with the new name

         if {[panCopyWidget $cv $oldName $name] == "FAILURE" } {
             return "FAILURE" 
         } 
 
         panDeleteWidget  $cv $oldName        ;# Delete previous widget
         set par(X) $posx ; set par(Y) $posy  ;# Set widget position
         setWdgConfig $idx oldName $name      ;# Update current widget name
         setWdgConfig $idx oldWdgPath $cv.$name
      }
  
      saveWidget $name $i_posX $posx
      saveWidget $name $i_posY $posy
   }
      
   $cv coords $name $posx $posy;    #  Set posX posY coordinates

   ###
   ###  Set Short Help Text 
   ###
   
   set panWidgetHlp($name) [getConfigInput $idx wdgSH ]
   if { $panWidgetHlp($name) != "" } {
      $cv.$name configure -helparea $shArea -helptext $panWidgetHlp($name)
   }
       
   panSetWidgetBind  $cv $name $type

   #------------------------------------------------------------
   #  Process Other Widgets
   #------------------------------------------------------------

   ###
   ### Save Widget Length
   ###

   if {[lsearch $optList "length"] != -1 } {

      if { [set tmp [getConfigInput $idx wdgLen]] == "" } {
         set tmp $par(DEF_LENGTH) 
      }

      switch $type {
         UifScale         - 
         UifBarChart      -
         UifOptionButton  { $cv.$name configure -length $tmp }
         UifListBox       { set do nothing }
         default          { $cv.$name configure -width  $tmp }
      }
   }

   ###
   ### Save Command
   ###

   if {[lsearch $optList command] != -1 } {
      if {[cequal $type UifArrowsButton]} {
          $cv.$name configure -script [getConfigInput $idx wdgCmd]      
      } else {
          $cv.$name configure -command [getConfigInput $idx wdgCmd]         
      }
   }

   ###
   ### Save Widget Label
   ###

   if {[lsearch $optList label] != -1 && ! [cequal $type freeText]} {
       $cv.$name configure -label [getConfigInput $idx wdgLabel]
   }

   if {[lsearch $optList lblOptions] != -1 } {
      set w [getWdgConfig $idx lblParFrame]

      if {[cequal $type freeText]} {
         set panSelectedFont [getWdgConfig $idx lblFont]
         $cv itemconfigure $name -font $panSelectedFont
      } else {
         $cv.$name configure -labelFont [getWdgConfig $idx lblFont]\
                             -labelWidth [getInput $w.lblSize]
      }
   }

   ###
   ### Save State Variable Bound
   ###

   if {[lsearch $optList stateVar] != -1 } {
       set tmp [getConfigInput $idx wdgStateVar]
       $cv.$name configure -stateVar gvar($tmp) 
       set gvar($tmp) [$cv.$name queryState]

       # This is to remove the stateVar name from array panWidget
       # because it is not used any longer.

       saveWidget $name $i_var ""
   }


   ###
   ### Save Variable Bound
   ###

   if {[lsearch $optList "noVar"] == -1 } {

      set varBound [getConfigInput $idx wdgVar]
      set oldVar   [getWdgConfig $idx oldVar]

      if { [set varBound [string trim $varBound " "]] == "" } {
         set varBound "$name" ;  # Define a dummy variable
      }

      if { $oldVar != $varBound  && [info exists gvar($oldVar) ] } {
         unset gvar($oldVar)
      }      

#     puts "DBG > varBound = $varBound, oldVar = $oldVar"

      panSaveConfigVar $cv $name $type $varBound

     # puts "DBG > ckDbVar = $ckDbVar, dbFlag = $dbFlag, varBound = $varBound"
   }

   ###
   ### Save Database Interface
   ###

   ### Check if the widget is enabled to get DB events and configure widget 
   ### for Database Event

   if {[catch {$cv.$name cget -dbMode}] == 0} {
      configDbEvent $idx $cv $name
   }

   if {[lsearch $optList "noVar"] == -1 } {
       set envName [getConfigInput $idx wdgEnv]
       saveDbInterface $cv $name $type $envName $varBound $ckDbVar($idx) \
          $ckDbOnvalue($idx) $ckDbOffvalue($idx) $ckDbMin($idx) $ckDbMax($idx)
   }

   ###
   ### Save Default
   ###

   if {[lsearch $optList "default"] != -1 } {

      set newDefault [getConfigInput $idx wdgDef]

      saveWidget $name $i_default $newDefault ;# Backward compatibility
      $cv.$name configure -default $newDefault

      if {[cequal $type UifRadioButton]} {

         # Look for all radiobuttons associated to this one !!
         # Related radiobutton have the same variable name

         foreach item [winfo children $cv] {
             if {[winfo class $item] != "UifRadioButton"} { continue }

             # Compare radiobutton's variable name

             if {[$item cget -variable] == "gvar($varBound)"} {
                saveWidget [winfo name $item] $i_default $newDefault
             }
         }
     }
   };# end of Save Default 


   #---------------------------------------------------------
   #   Save Specific  Configuration parameters
   #---------------------------------------------------------

   incr par(SAVE_FLAG)

   switch $type {

      UifBitmap        { configBitmap   $idx $top $cv.$name ; return }
      UifClass         { configUifClass $idx $cv $top $name ; return }
      PubClass         { configPubClass $idx $cv $top $name ; return }
      UifText          { configUifText  $idx $cv $top $name ; return }
      freeText         { configFreeText $idx $cv $name ;      return }

      UifButton        { configUifButton       $idx $cv $type $name }
      UifArrowsButton  { configArrowsButton    $idx $cv $type $name }
      UifActionButton  { configUifButton       $idx $cv $type $name }
      UifCheckButton   { configRCButton        $idx $cv $type $name }
      UifRadioButton   { configRCButton        $idx $cv $type $name }
      UifOutput        { configUifOutput       $idx $cv $top $type $name }
      UifOutputList    { configUifOutput       $idx $cv $top $type $name }
      UifOutputLed     { configOutputLed       $idx $cv $type $name }
      UifOutputArray   { configOutputArray     $idx $cv $type $name } 
      UifOutputIcon    { configOutputIcon      $idx $cv $type $name } 
      UifOutputString  { configUifOutputStr    $idx $cv $top $name 
                         configUifOutput       $idx $cv $top $type $name }

      UifOptionButton  { configUifOptButton    $idx $cv $top $name }
      UifEntry         { configUifEntry        $idx $cv $top $name }
      UifScale         { configUifScale        $idx $cv $top $name }
      UifBarChart      { configUifBarChart     $idx $cv $top $name }
      UifListBox       { configUifListBox      $idx $cv $top $name }
    
      UifFrame         { 
                       $cv.$name configure -height [getInput $top.f10.hei]
####                   pack propagate $cv.$name  $ckReSize
                       saveWidget $name $i_reSize $ckReSize($idx)
                       return
      }
      default          { puts "$type : Not Yet Implemented " }            
   }; # end of switch 
}

#------------------------------------------------------------------------------
# PROCEDURE : saveDbInterface - 
#------------------------------------------------------------------------------

proc saveDbInterface { cv name type envName varName \
                       flagVar flagOn flagOff flagMin flagMax } {

   global panDbData panDbSymbAddr par
   global db_var   db_type  db_env db_def i_reSize i_default db_poll
   global i_type   i_dbVar  i_var   db_log

   if { $flagVar == 0 && $flagOn  == 0 && $flagOff == 0 &&\
        $flagMin == 0 && $flagMax == 0 } {

#       puts "DBG > DB not used .."
        saveWidget   $name $i_dbVar 0
        saveEvtDbConfig $cv $name $type "CLEAR"
        if {[info exists panDbData($name)]} { unset panDbData($name) }

        return ;# No Database reference is used
   }

   if {[info exists panDbData($name)] == 0} { 
     
      set panDbData($name) [list 0 local 0 0 0 0 0 0 0 1 ""];

      switch $type {
         UifOutput - UifOutputList - UifOutputString - UifBarChart - 
         UifOutputLed - UifOutputArray {
            setDbConfig  $name  $db_type  "out"
         }
         UifBitmap - UifButton - UifActionButton - UifArrowsButton {
            setDbConfig  $name  $db_type  "0"
         }
         default { setDbConfig  $name  $db_type  "in" }
      }
   }

   ###
   ### Save Environment Name
   ###

   # Check if the env. name refers to a variable
   panCheckEnvName $envName

   if { $envName == ""} {
      setDbConfig  $name  $db_env  "local"
   } else {
      setDbConfig  $name  $db_env  $envName
   } 

   ###
   ###  Set Database Variable Bound
   ###  
  
   if { $flagVar } {
#         puts "DBG > DB Variable defined .."
         
         set symbAddr [panGetWdgAttribute $name $varName]
         set panDbSymbAddr($name) $symbAddr
   
         setDbConfig  $name $db_var $varName ; # Set   DB variable
         saveWidget   $name $i_dbVar 1 
         saveEvtDbConfig $cv $name $type "SET"
         
   } else {
#        puts "DBG > DB Variable not defined ... but other DB values used"
         
         saveEvtDbConfig $cv $name $type "CLEAR"
         setDbConfig  $name $db_var   0  ;  # Reset 
         saveWidget   $name $i_dbVar  0 
   }

   setDbConfig  $name $db_log 0 ; # Reset Log Flag
#  puts "DBG > DB($name) - $panDbData($name) \n" 

}

proc saveEvtDbConfig { cv name type op } {
    global par db_poll panDEBUG

    # Check if the widget is enabled to get DB events

    if {[catch {set dbMode [$cv.$name cget -dbMode]}]} {
       return
    }

    if { $op == "SET" } {

       # Set Poll or Event Database Updating 

       panEvtDbDetach $name   ;# Detach previous event

       if {$dbMode == "dbPoll"} {
           setDbConfig  $name $db_poll 1
       } elseif { $dbMode == "dbEvent" } {

           # Avoid to attach an event during editing of a class
           # because the CWP could not be properly defined

           set wdgPath [panGetWdgPath $cv $name]

           if { $par(EDIT_WHAT) == ".pan" } {
               set tmp $panDEBUG    ;# Save current value
               set panDEBUG dbgDEV  ;# Force message on failure
               panEvtDbAttach $wdgPath $name ;  # Attach event to new variable
               set panDEBUG $tmp    ;# Restore value
           }
           setDbConfig  $name $db_poll 0
       }

    } elseif { $op == "CLEAR" } {
       panEvtDbDetach $name ;           # Detach previous event
    } else {
       panEvtDbDetach $name ;           # Detach previous event
    }
}

#------------------------------------------------------------------------------
# PROCEDURE : getValue - 
#------------------------------------------------------------------------------

proc getValue { idx name w ckFlag } {
   global par  panDbData
   global ckDbMin ckDbMax ckDbOnvalue ckDbOffvalue 
   global db_min   db_max   db_onval db_offval
   global i_dbMin  i_dbMax  i_dbOn   i_dbOff

   set dbFlag [info exists panDbData($name)]

#   puts "ckFlag      = $ckFlag,     dbFlag      =  $dbFlag"
#   puts "ckDbMin     = $ckDbMin,    ckDbMax      = $ckDbMax"
#   puts "ckDbOnvalue = $ckDbOnvalue ckDbOffvalue = $ckDbOffvalue"

   switch $ckFlag {
    
     ckDbMin      { 
                  set myFlag $ckDbMin($idx);  set default $par(RANGE_MIN)    
                  set dbIdx  $db_min;   set pos $i_dbMin 
     }
     ckDbMax      { 
                  set myFlag $ckDbMax($idx);  set default $par(RANGE_MAX)   
                  set dbIdx  $db_max;   set pos $i_dbMax 
     }
     ckDbOnvalue  { 
                  set myFlag $ckDbOnvalue($idx); set default $par(DEF_ONVALUE)
                  set dbIdx  $db_onval;  set pos $i_dbOn 
     }
     ckDbOffvalue { 
                  set myFlag $ckDbOffvalue($idx); 
                  set default $par(DEF_OFFVALUE)
                  set dbIdx   $i_dbMin; 
                  set dbIdx  $db_offval;  set pos $i_dbOff
     }    
     default      { return 0 }
   }

   
   if { [set myVal [getInput  $w]] == "" } { set onVal $default }
            
   if { $myFlag } { 
           
      setDbConfig  $name $dbIdx $myVal ;# save configuration
      saveWidget   $name $pos  1 
      set myVal    [panGetWdgAttribute $name $myVal]

   } elseif { $dbFlag } { 
                     
      setDbConfig  $name $dbIdx 0 ;# Reset field           
      saveWidget   $name $pos   0 ;# Reset flag
      
   } else {
   
      saveWidget   $name $pos   0 ;# Reset flag
   }

#   puts "DBG > getVal : myVal = $myVal "
   return $myVal    
}

#------------------------------------------------------------------------------
# PROCEDURE : panSaveConfigVar - 
#------------------------------------------------------------------------------

proc panSaveConfigVar { cv name type var } {
   global gvar i_var

#   puts "DBG > save variable $var"
   if { ![uplevel info exists  gvar($var)] } { set gvar($var) 0 }

   if {[cequal $var ""]} {
      $cv.$name configure -variable ""
   } else {
      $cv.$name configure -variable gvar($var)
   } 
   saveWidget $name $i_var $var
}

#------------------------------------------------------------------------------
# PROCEDURE : panSaveEnvName - Check if the env. name is a variable
#------------------------------------------------------------------------------

proc panCheckEnvName { envName } {
   global par

   # Check if the env. name refers to a variable
   if {[string index $envName 0] != "\$"} { return }

   set var [string range $envName 1 end] 

   lappend par(ENV_LIST) $var  ;# Add new env. name to the list

   if {[uplevel #0 info exists $var] == 0} {
         uplevel #0 set $var local      ;# Set to default value
   }
}

