#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: setPrivateVar.tcl,v 5.34 2005/09/26 12:44:21 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   07/07/94  created 
# M.COMIN   29/03/95  Add fonts in their full name
#
#
#
#************************************************************************
#   NAME
#
#   setPrivateVar- Initialize variables used inside by the panel editor
# 
#   SYNOPSIS
#
#   setLocalVar
# 
#   DESCRIPTION
#
#   The file initializes the following variables :
#
#   - par     Array with general parameters
#
#   - title   Default title
#   - host    Name of the current process ??? .. to be verified
#
#   - panCmdList   Array with the following fields :
#
#             (browse_dots)    List of panels
#             (ecbrowse_dots)  Associated list of commands to call the panels
#             (browse)         List of actions
#             (ecbrowse )      Associated list of commands for the actions     
#             (new)            List of standard templates
#
#
#   FILES
#    
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   This is also included in the panel files.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

proc setPrivateVar { } {
   global panCmdList  
   global nosave par title  grid  shArea keep panLaptopSize
     
   global ckHotSpot  ckDots    ckDbVar   ckDbOnvalue    ckDbOffvalue ckFrame
   global ckRange    ckType    ckStatus  ckOrientation  ckDataType   ckReSize 
   global ckShHelp   ckMenuBar ckActBut  ckDbMin        ckDbMax      ckCCS  
   global ckCmdReply ckXb  ckHeartBeat   ckShow         ckGlobalName
    
   global out_fg     panSelectedFont   outFormat 
   global outputFonts freeTextFonts 

   global menu 

   # Matching patterns
   global classMatch nameMatch

   set panLaptopSize 800x600
   
   #
   # Set Defaults for widget configuration
   #

   set par(RANGE_MIN)            0;  # Default values for scale range
   set par(RANGE_MAX)          100;
   set par(FONT_TEXT)          "-Adobe-helvetica-medium-r-normal--14*-iso8859-1";
   set par(DEF_LENGTH)          10;  # Default length value   
   set par(DEF_ONVALUE)          1;  # CheckButton  'on' value 
   set par(DEF_OFFVALUE)         0;  # CheckButton  'off' value   
   set par(CONFIG_PANEL) .configPanel; # Root Name for config. panel

   set par(COUR_R14)           "-Adobe-courier-medium-r-normal--14*-iso8859-1"
   set par(HELV_R14)           "-Adobe-helvetica-medium-r-normal--14*-iso8859-1"
   set par(COUR_B14)           "-Adobe-courier-bold-r-normal--14*-iso8859-1"
   set par(COUR_B12)           "-Adobe-courier-bold-r-normal--12*-iso8859-1"
   set par(HELV_B12)           "-Adobe-helvetica-bold-r-normal--12*-iso8859-1"
   set par(CONFIG_FONT_TEXT)   "-Adobe-courier-medium-r-normal--14*-iso8859-1" ;

   set par(HORIZONTAL)           h   ;# Value for 'Horizontal' config.
   set par(VERTICAL)             v   ;# Value for 'Vertical'   config.
   set par(X)                   20   ;# Current Cursor X coord
   set par(Y)                   20   ;# Current Cursor Y coord
   set par(LCV_BG)           White   ;# Initial value for canvas bg. color

   set par(EDIT_PROC_ROOT)  .editProc           ;# proc editor root window
   set par(EDIT_TEXT)  .editProc.top.edit       ;# Text editor widget
   set par(EDIT_MENU)  .editProc.top.line1.proc ;# Menu button

   set par(CLASS)        "uifClass" ;# Suffix for classes
   set par(HOME)         ".";
   set par(ROOT_PANEL)   .panel
   set par(EDIT_MODE)    create
   set par(WIDGET_INFO_FORMAT) "%-35s %-6s %-6s %-20s %-30s %-50s %-25s %s"

   #
   # Set pattern to parse names
   #
   set classMatch "(\[a-zA-Z0-9\]+)_(\[a-zA-Z\]+).(\[a-zA-Z\]+)"
   set nameMatch  "(\[a-zA-Z0-9\]+)-(\[a-zA-Z0-9\]+)"

   #
   # Set global variables to describe the current configuration
   #

   set grid          0;
   set ckCCS         0; # Use CCS 
    
   set ckHotSpot(0)   0; # Config. of 'Hot-Spot' check button
   set ckDots(0)      0; # Config. of (...) check button
   set ckDbVar(0)     0; # Config. of 'DB'/'Global' radio button for Variable
   set ckXbm(0)         0; # Denotes bitmap label 
   set ckDbOnvalue(0)   0; # Config. of 'DB' check button for ON Value
   set ckDbOffvalue(0)  0; # Config. of 'DB' check button for OFF Value
   set ckDbMin(0)       0; # Config. of 'DB' check button for min value
   set ckDbMax(0)       0; # Config. of 'DB' check button for max value
   set ckRange(0)       0; # Config. of 'Range Checking' for entry widget
   set ckType(0)        0; # Config. of 'Type Checking'  for entry widget
   set ckStatus(0)      0; # Config. of 'Status'  for pushButton
   set ckOrientation(0) 0; # Config. of 'Orientation' for scale and bar charts    
   set ckDataType(0)    N; # Config. of 'Data Type' for entry widget
   set ckFrame(0)       0; # Config. of 'Frame' for output widgets
   set ckReSize(0)      0; 
   set ckShow(0)        1;
   set ckGlobalName(0)  0;
  
   set minVal        0; # Config. of 'Range'
   set maxVal        0; # Config. of 'Range'

   set ckCmdReply    0
   set ckShHelp      0
   set ckMenuBar     0
   set ckActBut      0
   set ckHeartBeat   1
    
   #
   # Current setup for output widget
   #  

   set out_fg           Black;            # fg color 
   set panSelectedFont  "-Adobe-courier-medium-r-normal--14*-iso8859-1";   # font
   set outFormat        ""
                     
   set title "\<module\> - \<title of the panel\>"

   #
   # Set font list for Output Widgets
   #  

   set outputFonts { "-Adobe-courier-medium-r-normal--12*-iso8859-1" \
                     "-Adobe-courier-medium-r-normal--14*-iso8859-1" \
                     "-Adobe-courier-medium-r-normal--24*-iso8859-1" \
                     "-Adobe-courier-bold-r-normal--12*-iso8859-1" \
                     "-Adobe-courier-bold-r-normal--14*-iso8859-1" \
                     "-Adobe-courier-bold-r-normal--24*-iso8859-1" 
                     }
   #
   # Set font list for freeText  Widgets
   #  

   set freeTextFonts { "-Adobe-helvetica-medium-r-normal--12*-iso8859-1" \
                       "-Adobe-helvetica-medium-r-normal--14*-iso8859-1" \
                       "-Adobe-helvetica-medium-r-normal--24*-iso8859-1" \
                       "-Adobe-helvetica-bold-r-normal--12*-iso8859-1" \
                       "-Adobe-helvetica-bold-r-normal--14*-iso8859-1" \
                       "-Adobe-helvetica-bold-r-normal--24*-iso8859-1"
                     }

#
# Set list of widget config. parameters not to be saved
#
   set nosave(UIF_ALL)  { -helptext -helparea -qualityBg -canvasAnchor }

#   set nosave(ALL)  { -bd -bg -fg -borderwidth -text -vltCommand \
#                      -textvariable -onvalue -offvalue -anchor -relief\
#                      -background -wdgAnchor -pady -padx -qualityBg }

   set keep(ALL)    {-variable  -stateVar -label -canvasAnchor -evtFilter \
                     -evtScript -dbMode -default -labelFont -labelWidth }

   set keep(UifArrowsButton) {-command -decrStep -incrStep -script -stateVar}
   set keep(UifText)         {-height -width -font -wrap }
   set keep(UifListBox)      {-selectmode -geometry -itemList -font -scrolly -scrollx }
   set keep(UifFrame)        {-height -width }
   set keep(UifButton)       {-initialState -popup -width -command -bitmapFile }
   set keep(UifCheckButton)  {-anchor -command -offVal -onVal -width }
   set keep(UifRadioButton)  {-anchor -command -offVal -onVal -width }
   set keep(UifOutput)       {-units -anchor -frame  -font -width -color }
   set keep(UifOutputList)   {-units -anchor -frame  -font -width -color -listKey -listColor}
   set keep(UifOutputArray)  {-anchor -frame  -font -format -width -color \
                              -padding -labelFont -labelWidth -orient -size }
   set keep(UifOutputLed)    {-flashTime -flashing -label -labelFont -labelWidth -colorOn -colorOff }
   set keep(UifOutputIcon)   {-listValues -listPictures }
   set keep(UifOptionButton) {-itemList -itemValue -selectMode -command -length }
   set keep(UifEntry)        {-minVal -maxVal -width -units }
   set keep(UifScale)        {-minVal -maxVal -orient -length -digits -resolution}
   set keep(UifOutputString) {-listValues -listStrings -listColors -frame \
	                      -font -width -anchor}
   set keep(UifBarChart)     {-ticks -axis -minVal -maxVal -length -orient }
   set keep(UifBitmap)       {-command -hotSpot -hotSpotColor -frame }
   set keep(UifActionButton) {-initialState -width -command }

#
# Set contents of the ESO Selection Box : list of panels
#
     set panCmdList(browse_dots) {
            {Call   a panel}
            {Import a panel}
     }	

     set panCmdList(ecbrowse_dots) {     
	    {panCallPanel   <file> <parameter list>}
	    {panImportPanel <file> ?-iconify? <parameter list>}
     }
#
# Set contents of the ESO Selection Box : list of generic actions
#	
     set panCmdList(browse) { 
         {Send Command }
         {Write DB Attribute}
         {Write All Class DB Widget Values}
         {Write All Panel Widget Values}
         {Read DB Attribute}
     }

     set panCmdList(ecbrowse) {  
         {panSendCommand <env> <proc> <cmd> <flags> <timeout>  ?args?}
         {panDbWrite <symbAddress> <value>}
         {panDbWriteClass  <class_widget_path> all}    
         {panDbWriteWidget <canvas> all}
         {panDbRead  <symbAddress> <retValue>}
     }
#
# Set contents of the ESO Selection Box : list of actions for 'action Buttons'
#	
     set panCmdList(actButton) { 
         {Send Command }
         {Refresh Input Widgets}     
         {Close the panel}
         {Store to database}
         {Set Defaults}
         {Clear Panel}
     }

     set panCmdList(ecactButton) {  
         {panSendCommand <env> <proc> <cmd> <flag> <timeout> <parameter list> }
         {panUpdateCanvas $cv}
         {panClosePanel}
         {panDbWriteWidget $cv all}
         {panSetDefaults   $cv}
         {panResetPanel    $cv}
     }
#
# Set contents of the ESO Selection Box : list of standard templates
#
     set panCmdList(new) {
            {ESO Selection Box} 
            {File Selection Box} 
            {Extended Help} 
     }	

#
# ????
#	
   set help_list {  Quit {End Session}
                    Help {Show Extended Help}
                    Save {Save panel to file}
                    New {Create a new panel}
                    Execute {Perform action}
                    Defaults {Insert default values}
                  }

#    Other Widgets 

     if {![info exists env(PRINTER)]} { set env(PRINTER) {} }
}

