#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ptlibMisc.tcl,v 1.21 2003/11/18 12:40:56 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/09/95  created
# mcomin    05/10/95  ptlibSetApplCwp return the application's cwp
#                     improved version of ptlibPanelExist : it re-tries
#                     several times up to 30 sec. before aborting the test.
# mcomin    15/12/95  Use string operation ...instead of list operation
#
# mcomin    10/03/96  Porting to TK4.0 and itcl2.0.
#                     Add procedure ptlibAckDialog
#           10/11/97  Add check on process name length
#           02/05/98  Add ptlibDisplayComment
#
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
#
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


proc ptlibInit { appl } {
  global env ptlibTitleList

  eval global ::${appl}_SOURCE

  set  ptlibTitleList(dummy) dummy
  set  libPath [findVLTFile lib/libptlib.tcl]
  set  ${appl}_SOURCE 1
  send $appl source ${libPath}/ptlibSourceFile.tcl
}



# CCS Initialization

proc ptlibCcsInit { procName } {
  global env

  set name [string range [file tail $procName] 0 18]
  if { [catch {seq_ccsInit $name } ret] } {
     puts "CANNOT INIT $name IN LOCAL ENV. $env(RTAPENV): $ret\n"
     exit
  }
}

#
# Check and wait until the GUI is active on the screen.
# The routine waits for at max. 60 sec.
#
proc ptlibWaitPanel { appl {toplevel {.}} {delay {60}} } {

   while { $delay > 0 } { 
     after 1000
#     puts "DBG > delay = $delay" 

     if {[lsearch [winfo interps] $appl] != -1} {
        foreach top [split $toplevel " "] {
	    if {[send $appl winfo exists $top]} { return $top }
        }
     }  
     incr delay -1
   } 
  
   return ""
}


proc ptlibPanelExist { appl {toplevel {.}} {delay {60}} } {
   after 3000
   if { [ptlibWaitPanel $appl $toplevel $delay] != "" } {
      after 3000 ;# To compensate some unexpected delays ...
      set tmp [ptlibGetTitle $appl $toplevel] ;
      puts "GUI \"${tmp}\" ON SCREEN"
      send $appl raise $toplevel
      return
   }

   puts "EXIT TEST : GUI $appl NOT ACTIVE" ;
   exit 
}

#
# Acknowledge a dialog box
#
proc ptlibAckDialog { appl {toplevel {.}} {idx {0}} {delay {30}} } {
   global ptlibDUMP_STACK

   # Wait until dialog box on screen

   if {[set myDialog [ptlibWaitPanel $appl $toplevel $delay]] != "" }  {

       send $appl raise $myDialog

       # Process a normal tk-dialog window

       if {$myDialog == ".dialog"} {
          set wdg  $myDialog.button0
          if {[ptlibLogAction $appl $wdg]} { return }
          send $appl simulClickButton $wdg
          return
       }

       # Process the VLT error dialog
       # Quitting the error stack panels ... also the errDialog
       # disappears from screen

       if {[info exists ptlibDUMP_STACK] && $ptlibDUMP_STACK } {
          ptlibDumpErrStack $appl $myDialog
          return
       }

       set wdg $myDialog.errDialogPanel.b0.fOK.ok

       if {[ptlibLogAction $appl $wdg]} { return }
       send $appl simulClickButton $wdg
   }
}

proc ptlibDumpErrStack { appl myDialog } {

    set wdg $myDialog.errDialogPanel.b0.stack

    ptlibLogAction $appl $wdg

    send -async $appl simulClickButton $wdg
             
    set mbFile .seqErrStack.errStackPanel.menu.file 
    after 5000
    ptlibClickMenu $appl $mbFile  "Print On Screen"
    ptlibClickMenu $appl $mbFile  "Quit"
}

# Set Current CWP according to the application CWP

proc ptlibSetApplCwp { appl } {
   set cwp [ptlibGetValue $appl cwp]

   ptlibSetDbCwp $cwp 
   return $cwp
}

# Set Current CWP according to the application 'cwp' for a specific env.

proc ptlibSetEnvCwp { appl {envName {}} } {

    if {[catch {set cwp [send $appl seq_dbGetCwp $envName]}]} {
      puts "CANNOT GET CWP TO from $appl"
      exit
   }

   if {$envName != ""} {
      if {[string index $envName 0] != "@"} {
         set cwp @${envName}${cwp}
      } else {
         set cwp ${envName}${cwp}
      }
   } 

   ptlibSetDbCwp $cwp 
}


# Set Current Working Point

proc ptlibSetDbCwp { cwp } {

   if {$cwp == ""} { return }

   puts "SET GUI CWP TO \"$cwp\""

   if { [catch {seq_dbSetCwp $cwp } ret] } {
      puts "CANNOT SET CWP TO \"$cwp\""
      exit
   }

   return $cwp
}


# Get panel editor GUI canvas names

proc ptlibGetCanvasNames { appl cv bcv } {

   upvar $cv  cv1
   upvar $bcv bcv1

   set cv1   [ptlibGetValue $appl cv]
   set bcv1  [ptlibGetValue $appl bcv] 
}


proc ptlibAckPanTerm { appl args } {

  ptlibPanelExist $appl .panTerm 120
  flush stdout

  if {[lsearch $args "NO_PRINT"] == -1} {
     ptlibClickMenu  $appl .panTerm.top.menu.file  "Print On Screen"
     flush stdout
  }

  ptlibClickMenu  $appl .panTerm.top.menu.file  "Quit"
  flush stdout
}


proc ptlibSetVariable { appl var value } {
    send $appl set $var \"${value}\"
}

proc ptlibDisplayComment { appl text} {
    send $appl panDisplayComment \"$text\"
}

proc ptlibGetWdgOption { appl root name option } {

   if {[catch { set wdgPath [send $appl panGetWdgPath $root $name] } ret]} {
      puts "WIDGET $name DOES NOT EXIST IN \"$title\" "
      continue
   }

   return [send $appl $wdgPath cget $option]   
}

proc ptlibExecMethod { appl root name method args } {

   if {[catch { set wdgPath [send $appl panGetWdgPath $root $name] } ret]} {
      puts "WIDGET $name DOES NOT EXIST IN \"$title\" "
      continue
   }

   return [send $appl $wdgPath $method $args]   
}






#
# ___oOo___
