#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ptlibPanelConfig.tcl,v 1.21 2003/11/18 12:40:56 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/09/95  created
# mcomin    04/11/95  Erase tag "current" from canvas items
# mcomin    06/11/95  Modified to support classes
# mcomin    14/12/95  panGetWidgetReference replaced with panWdgReferenceList
# ------------------
# mcomin    07/06/96  List of widget is now sorted
#           05/03/97  Take away item number ... otherwise there are problems
#                     reproducing the test after deleting elements
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
#      
#      ptlibPanelConfig      <appl> <widget root> <itemsFlag>
#
#      ptlibCanvasItemConfig <appl> <canvas name> <tag list>
#
#   DESCRIPTION
#      
#      ptlibPanelConfig
#
#        Dumps the configuration of all widgets of a specific 
#        window -i.e.- a canvas.
#
#        appl         <IN>  Name of the Tcl/Tk application
#        widget root  <IN>  Full name of the parent-widget.
#        itemsFlag    <IN>  1/0  : if 1 it dumps the canvas items configuration
#
#      ptlibCanvasItemConfig
#
#        Dumps the configuration of all canvas 
#
#        appl         <IN>  Name of the Tcl/Tk application
#        canvas name  <IN>  Full name of the canvas-widget
#        tag list     <IN>  List of tags to select proper canvas items.
#                           Example : text, rectangle, etc ...
#
#
#     The procedures generates a log on the standard output of the user action.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   Canvas items belonging to classes are not dumped.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc ptlibPanelConfig { appl root args } {

    global ptlibFullConfig

    set ptlibFullConfig 1
    set itemsFlag       1

    if {[set idx [lsearch $args "NO_ITEM_CONFIG"]] != -1 } {
	set args [lreplace $args $idx $idx]
	set itemsFlag 0
    }

    set title [ptlibGetTitle $appl $root]

    ptlibGetCanvasNames $appl cv bcv

    set tmp "\nPANEL CONFIGURATION FOR \"${title}\""
    if {$root == $bcv} { append tmp " (ACTION AREA)"}
    puts $tmp

    set wdgList [send $appl panWdgReferenceList $root all all]
    
    if {[llength $wdgList] == 0} {

	puts "WINDOW \"$title\" NO WIDGETS\n"

    } else {

	# Get poll rate and wait for a whole cycle to wait 
	#for proper data updating

	set pollRate [ptlibGetValue $appl pollRate]
	after [int [expr {1.5 * $pollRate}]]

	eval ptlibWidgetConfig $appl $root $args [lsort $wdgList]
    }

    if {$itemsFlag} { ptlibCanvasItemConfig $appl $root }

    set ptlibFullConfig 0
    puts "\n"
}


#
# Dump configuration of camvas items
#

proc ptlibCanvasItemConfig { appl root { tags {all}} } {

    set itemIgnoreList [ list -qualityBg -qualityFg -offset -activefill \
			     -disabledstipple -disabledfill -activestipple -state]

    set title [ptlibGetTitle $appl $root]

    ptlibGetCanvasNames $appl cv bcv

    set tmp "\nPANEL CONFIGURATION (canvas items) FOR \"${title}\""
    if {$root == $bcv} { append tmp " (ACTION AREA)"}
    puts $tmp
    puts "SELECTED TAGS       : $tags " 
    
    if { $tags == "all"} {  
	set itemList [send $appl $root find all]
    } else {
	set itemList [send $appl $root find withtag $tags]
    }  
    
    set flag 0
    foreach item $itemList { 
	
	set myType [send $appl $root type $item ]      
	
	if {$myType == "window"} { continue }
	
	set flag 1
	set config [send $appl $root itemconfig $item]
	
        puts "\nWINDOW \"${title}\" CONFIGURATION OF ITEM TYPE $myType "

	foreach tmp $config {
	    set optName [lindex $tmp 0]
	    if {[lsearch $itemIgnoreList $optName] != "-1" } { continue }

	    if {[cequal $optName "-tags"]} {
		regsub " current" $tmp "" tmp
	    }
	    puts "OPTION $tmp"
	}      
    }
    
    if {[llength $itemList] == 0 || $flag == 0} {
	puts "WINDOW \"$title\" NO CANVAS ITEMS\n"
    }
    
    puts "\n"

}

#
# ___oOo___
