#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ptlibTypeConfig.tcl,v 1.21 2003/11/18 12:40:58 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/09/95  created
# mcomin    04/11/95  Erase tag "current" from canvas items
# mcomin    06/11/95  Modified to support classes
# mcomin    14/12/95  panGetWidgetReference replaced with panWdgReferenceList
# ------------------
# mcomin    07/06/96  List of widget is now sorted
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
#      
#      ptlibPanelConfig      <appl> <widget root> <itemsFlag>
#
#      ptlibCanvasItemConfig <appl> <canvas name> <tag list>
#
#   DESCRIPTION
#      
#      ptlibPanelConfig
#
#        Dumps the configuration of all widgets of a specific 
#        window -i.e.- a canvas.
#
#        appl         <IN>  Name of the Tcl/Tk application
#        widget root  <IN>  Full name of the parent-widget.
#        itemsFlag    <IN>  1/0  : if 1 it dumps the canvas items configuration
#
#      ptlibCanvasItemConfig
#
#        Dumps the configuration of all canvas 
#
#        appl         <IN>  Name of the Tcl/Tk application
#        canvas name  <IN>  Full name of the canvas-widget
#        tag list     <IN>  List of tags to select proper canvas items.
#                           Example : text, rectangle, etc ...
#
#
#     The procedures generates a log on the standard output of the user action.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   Canvas items belonging to classes are not dumped.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc ptlibTypeConfig { appl root args } {

   global ptlibFullConfig

   set ptlibFullConfig 1
   set configFlag      ""

   if {[set idx [lsearch $args "SHORT_CONFIG"]] != "-1"} { 
      set configFlag "SHORT_CONFIG"
      set args [lreplace $args $idx $idx]
   }

   set title [ptlibGetTitle $appl $root]

   ptlibGetCanvasNames $appl cv bcv

   if {$root == $bcv} {
      puts "ERROR : ptlibWidgetTypeConfig NOT APPLICABLE TO ACTION AREA"
      return 
   }

   puts "\nPANEL CONFIGURATION FOR \"${title}\" ( SELECTED WIDGETS : $args )"

   set wdgList [send $appl panWdgReferenceList $root all uifType]
   
   if {[llength $wdgList] == 0} {
      puts "WINDOW \"$title\" NO WIDGETS\n"
   }

   # Get poll rate and wait for a whole cycle to wait for proper data updating

   set pollRate [ptlibGetValue $appl pollRate]
   after [int [expr {1.5 * $pollRate}]]

   foreach name  [lsort $wdgList]  {

      if {[catch { set wdgPath [send $appl panGetWdgPath $root $name] } ret]} {
         puts "WIDGET $name DOES NOT EXIST IN \"$title\" "
         continue
      }

      set type [send $appl winfo class $wdgPath]; # Get Widget Type

      if {[lsearch $args $type] == -1} { continue }

      if {$configFlag == ""} {
         ptlibWidgetConfig $appl $root $name
      } else {
         ptlibWidgetConfig $appl $root $configFlag $name     
      }
   }

   set ptlibFullConfig 0
   puts "\n"
}

#
# ___oOo___
