#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbDraw.tcl,v 1.24 2005/06/19 19:36:29 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
#

#************************************************************************
#   NAME
#   rtdbDraw.tcl - IRDB specific methods of CanvasDraw.tcl
# 
#   SYNOPSIS
#   rtdbDraw <path> ?options?
# 
#   DESCRIPTION
#   This class defines the IRDB specific methods of CanvasDraw.tcl
#   It contains the following modifications:
#   - additional buttons: defaults and set
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   none
#
#   EXAMPLES
#
#   SEE ALSO
#   CanvasDraw.tcl, RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual rtdbDraw {}

class ::rtdbDraw {
    inherit util::CanvasDraw

    # create an instance of this class

    constructor {args} {
	# get the options before building
	eval itk_initialize $args
    }

    protected method init {args} {
	util::CanvasDraw::init

        if {!$itk_option(-withtoolbox)} {
            return
        }
        set checkDone_ 0

	# add button to set the default drawing modes
	itk_component add defaults {
	    button $w_.defaults -text Defaults -command "$w_ set_defaults"
	}

        # add button to set drawing modes to selected object
        itk_component add set {
            button $w_.set -text Set -command "$w_ set_selected"
        }
	set i 1
	foreach compo "set defaults delete clear close" {
	    blt::table $w_.bot \
		$itk_component($compo)       0,$i -fill x -anchor w
	    incr i
	}
	
        # add short help 
	add_more_help_texts

	set_defaults
    }    
    
    method add_more_help_texts {} {
	# add short help texts
	set item $w_.drawmodes
	add_short_help $itk_component(set) \
		{Set: {bitmap b1} = Set current drawing options (Width, Arrow...) for selected object}
	add_short_help $itk_component(defaults) \
		{Defaults: {bitmap b1} = Set the default drawing options (Width, Arrow...)}
    }

    # sets flag to finish object after 1st <ButtonRelease-1>
    method set_check_done {val} {
        set checkDone_ $val
    }
    
    # redefined method check_done

    method check_done {x y {update_method "update_object"}} {
        clip x y
        if {$x == $startx_ && $y == $starty_ && ! $checkDone_} {
            bind $canvas_ <Motion> [code eval $this $update_method $canvasX_ $canvasY_]
            bind $canvas_ <1> [code eval $this create_done $canvasX_ $canvasY_]
        } else {
            set checkDone_ 0
            create_done $x $y
        }
    }

    # set the defaults for drawing mode

    method set_defaults {} {
        set_drawing_mode anyselect
        deselect_objects

        set_linewidth 1
        set_arrowtype none
        set_arrowshape {8 10 3}
        set_fillcolor {}
        global ::$w_.fill
        set $w_.fill None
        set_outlinecolor yellow
        set_stipplepattern 0
        set_textfont "-*-courier-bold-r-*-*-*-180-*-*-*-*-*-*"
    }

    # set the drawing modes for selected object

    method set_selected {} {
        config_selected -width $itk_option(-linewidth)
        config_selected -arrow $itk_option(-arrowtype)
        config_selected -arrowshape $itk_option(-arrowshape)
        config_selected -outline $itk_option(-outlinecolor)
        config_selected -stipple $itk_option(-stipplepattern)
        config_selected -font $itk_option(-textfont)
        config_selected -fill $itk_option(-fillcolor)
    }
   
    # -- protected member variables --
    
    # Finish object after 1st <ButtonRelease-1>
    protected variable checkDone_ 0
}

# ___oOo___
