#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbPick.tcl,v 1.24 2005/06/19 19:36:29 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
# pbiereic  05/05/00  fixed: width and height for "view update"
# pbiereic  19/12/00  changed layout and some configuration of widgets
# pbiereic  27/10/03  using more methods of class RtdImagePick
#

#************************************************************************
#   NAME
#   rtdbPick.tcl - widget to select an object in an image using a centroid
#   algorithm
# 
#   SYNOPSIS
#   rtdbPick <path> ?options?
# 
#   DESCRIPTION
#   This class inherits rtd::RtdImagePick and redefines a number of methods
#   to work well for the remote PICK options as WAIT, NOW etc.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual rtdbPick {}

class ::rtdbPick {
    inherit ::rtd::RtdImagePick

    constructor {args} {
	# the constructor of RtdImagePick needs the args list
	eval RtdImagePick::constructor $args
    } {
    }

    protected method init {} {
	rtd::RtdImagePick::init
	wm protocol $w_ WM_DELETE_WINDOW [code $this close]

	# change layout and some configuration of widgets
	$itk_component(object) config -text "Level above bg"
	pack $w_.mainf -ipadx 0 -ipady 0 -padx 0 -pady 0 -fill both -expand 0
	pack $itk_component(buttons) -fill x -expand 1 -padx 0
	foreach el "close cancel pick" {
	    pack $itk_component($el) -ipadx 0 -ipady 0 -padx 0 -pady 0 -fill x
	}
	set labelf $w_.mainf.topf.labelf
	$labelf config -relief flat
	foreach wdg [winfo children $labelf] {
	    if {[winfo exists $wdg.entry]} {
		$wdg.entry config -highlightthickness 0 -borderwidth 2
		$wdg.label config -pady 0
	    }
	    pack $wdg -padx 0 -pady 0 -expand 0 -fill none
	}
	foreach wdg "$w_.mainf.topf [winfo children $w_.mainf.topf]" {
	    pack $wdg -expand 1 -fill both -pady 0
	}
	cancel

        $itk_component(zoomView) zoom
        $itk_component(zoomView) config -command "$this scale_changed"
    }

    method pick_object {{parms ""} {wait 0}} {
        if {"$parms" == ""} {
            utilRaiseWindow $w_
	}
	RtdImagePick::pick_object $parms $wait
        $target_image_ zoomview stop 2
    }
 
    public method set_result_value {} {
        set set_result_ 1
    }

    # Create the window used to display the part of the image being examined.
    # $w is the parent frame.
    # The optional "side" arg is used for packing.

    protected method make_rect {w {side left}} {
        # add a frame to show the size of the square selected by the scale
        pack [set rf [frame $w.rf \
			  -bd 0 -relief flat \
			  -width $itk_option(-maxsize) \
			  -height $itk_option(-maxsize)]] \
	    -fill none -expand 1 -anchor c

        # This component displays the section of the image that will be used for
        # the centroid algorithm and statistics.
        itk_component add zoomView {
            rtdbZoomView $rf.zoomView \
		-target_image $itk_option(-target_image) \
		-verbose $itk_option(-verbose) \
		-factor $itk_option(-factor) \
		-width $itk_option(-maxsize)  \
		-height $itk_option(-maxsize) \
		-propagate 0 \
		-usexshm 1 \
		-usexsync 1 \
		-borderwidth 2 \
		-relief groove \
		-shelp {PickImage: displays section of image being examined}
        }
	set zoomImage [$itk_component(zoomView) component image]
        set image_ [$zoomImage get_image]
        set canvas_ [$zoomImage get_canvas]

        catch {destroy [$itk_component(zoomView) component check]}
        global ::$itk_component(zoomView).dozoom
        set $itk_component(zoomView).dozoom 1

        pack [$itk_component(zoomView) component larger] -expand 1
        pack [$itk_component(zoomView) component smaller] -expand 1
        pack $itk_component(zoomView) -anchor c -fill none -expand 0
        $itk_component(zoomView) inc_zoom 0
    }
    
    # set the values of the labels from the list (results of "pick_object" call).
    # If list is empty the labels are cleared.
    # If the list is not empty, mark the ra,dec spot in the image.

    protected method set_values {list {with_rmt 1}} {
        lassign $list x y ra dec equinox fwhmX fwhmY angle object background

        # create a dot which is used to mark the center pixel
	$target_canvas_ delete $pickc_
	if {"$x" != "" && "$y" != ""} {
            $target_image_ convert coords $x $y image xc yc canvas
            $target_canvas_ create oval $xc $yc $xc $yc \
                -tags "$pickc_ objects" 
	}

	# display x,y in chip coords
	if {"$x" != "" && "$y" != ""} {
	    $target_image_ convert coords $x $y image xc yc chip
	} else {
	    set xc $x
	    set yc $y
	}

	$itk_component(x)       config -value [format_val $xc]
        $itk_component(y)       config -value [format_val $yc]
        $itk_component(ra)      config -value $ra
        $itk_component(dec)     config -value $dec
        $itk_component(equinox) config -value $equinox

        $image_ convert dist 0 $itk_option(-maxsize) canvas 0 nsize image
        $itk_component(nsize)      config -value [format_val $nsize]
        $itk_component(object)     config -value [format_val $object]
        $itk_component(background) config -value [format_val $background]

        if {"$fwhmX" == "" || $fwhmX > 0 && $fwhmY > 0} {
            $itk_component(fwhm)   config -value "[format_val $fwhmX] : [format_val $fwhmY]"
            [$itk_component(fwhm)  component entry] config -foreground black
            $itk_component(angle)  config -value [format_val $angle]
        } else {
            $itk_component(fwhm)   config -value "Can't do"
            [$itk_component(fwhm)  component entry] config -foreground red
            $itk_component(angle)  config -value ""
        }
        if {"$x" != "" && "$y" != ""} {
            set imageX_ $x
            set imageY_ $y
        }
        if {"$x" != "" && "$y" != "" && $fwhmX < 50 && $fwhmY < 50} {
            mark_spot $x $y $image_ $canvas_ $angle $fwhmX $fwhmY
            mark_spot $x $y $target_image_ $target_canvas_ $angle $fwhmX $fwhmY 1

            if {$x < 0 || $y < 0 || $x > [$image_ width] || $y > [$image_ height]} {
		set_result -1
            } else {
                foreach el "ra dec equinox" {
                    if {"[set $el]" == ""} {set $el "{}"}
                }
                set_result \
		    "$x $y $ra $dec $equinox $object $background $fwhmX $fwhmY $angle $nsize"
            }
        }
    }

    # close this window
    
    public method close {} {
        cancel
	set_result -1
        after idle "wm withdraw $w_"
    }
    
    # cancel the current pick operation

    method cancel {{with_pick 1}} {
	rtd::RtdImagePick::cancel $with_pick
	if {$with_pick} {
	    set list_ ""
	    $itk_component(zoomView) leave_image $image_
	}
    }

    # set result if requested

    method set_result {args} {
        if {$set_result_} {
            set set_result_ 0
            rtdbReply $args
        }
    }
}

# ___oOo___
