#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbPubCoords.tcl,v 1.24 2005/06/19 19:36:30 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created

#************************************************************************
#   NAME
#   rtdbPubCoords.tcl - public library for the RTD base module
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#        rtdbCanvasToImage {x0 y0 x1 y1 {image}}
#              Return a tcl list with converted canvas coordinates to image coordinates
#
#        rtdbCanvasToChip {x0 y0 x1 y1 {image}}
#              Return a tcl list with converted canvas coordinates to chip coordinates
#
#        rtdbImageToCanvas {x0 y0 x1 y1 {image}}
#              Return a tcl list with converted image coordinates to canvas coordinates
#
#        rtdbImageToChip {x0 y0 x1 y1 {image}}
#              Return a tcl list with converted image coordinates to canvas coordinates
#
#        rtdbChipToCanvas {x0 y0 x1 y1 {image}}
#              Return a tcl list with converted chip coordinates to canvas coordinates
#
#        rtdbChipToImage {x0 y0 x1 y1 {image}}
#              Return a tcl list with converted chip coordinates to image coordinates
#
#        "image" is the name of the Tk image object. Default is the main image.
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation, rtdimage(3)
#
#   BUGS     
#
#------------------------------------------------------------------------

# generate coordinate-convertion convenience routines on-the-fly

proc rtdbConv {x0 y0 x1 y1 from to targetImage} {
    if {"$targetImage" == ""} {
	set targetImage [[rtdbRtdCmd get_image] get_image]
    }
    if {[catch {$targetImage convert coords $x0 $y0 $from rx0 ry0 $to}]} {
	rtdbFeedback "Illegal coordinates: $x0 $y0"
	return "0 0 0 0"
    }
    if {[catch {$targetImage convert coords $x1 $y1 $from rx1 ry1 $to}]} {
	rtdbFeedback "Illegal coordinates: $x1 $y1"
	return "0 0 0 0"
    }
    return "$rx0 $ry0 $rx1 $ry1"
}

set rtdbCnv "Canvas Image Chip Screen Wcs Deg"
set rtdbN [llength $rtdbCnv]
for {set i 0} {$i < $rtdbN} {incr i} {
    set name1 [lindex $rtdbCnv $i]
    for {set j 0} {$j < $rtdbN} {incr j} {
	set name2 [lindex $rtdbCnv $j]
	proc rtdb$name1\To$name2 {x0 y0 x1 y1 {targetImage ""}} [::format { \
		rtdbConv $x0 $y0 $x1 $y1 %s %s $targetImage} \
		[string tolower $name1] [string tolower $name2]]
    }
}

proc rtdbConvDist {x y from to targetImage} {
    if {"$targetImage" == ""} {
	set targetImage [[rtdbRtdCmd get_image] get_image]
    }
    if {[catch {$targetImage convert dist $x $y $from rx ry $to}]} {
	rtdbFeedback "Illegal coordinates: $x $y"
	return "0 0"
    }
    return "$rx $ry"
}

for {set i 0} {$i < $rtdbN} {incr i} {
    set name1 [lindex $rtdbCnv $i]
    for {set j 0} {$j < $rtdbN} {incr j} {
	set name2 [lindex $rtdbCnv $j]
	proc rtdbDist$name1\To$name2 {x y {targetImage ""}} [::format { \
		rtdbConvDist $x $y %s %s $targetImage} \
		[string tolower $name1] [string tolower $name2]]
    }
}

unset rtdbCnv
unset rtdbN

# dummy to for auto loading this library

proc rtdbConvGen {} {}

# ___oOo___
