#
# E.S.O. - VLT project
#
# "@(#) $Id: menuBar.tcl,v 3.49 2005/09/07 11:45:54 vltsccm Exp $" 
#

proc uifCheckNameSpace { var } {

    # If variable exist or if its an empty string .. do nothing
    if {[info exist $var] || [cequal $var ""]} { return }
   
    # Check if variable belongs to a namespace
    set nspaceName [namespace qualifiers $var]
    if {![cequal $nspaceName ""]} {
	set varName [namespace tail $var]
	# Create variable varName within the specified namespace.
	namespace eval $nspaceName set $varName 0
    }
}


proc makeMenuFrame { top } {
    frame  $top.menu  -border 1 -relief raised 
    pack   $top       -side top -padx 1 -pady 0 -fill x
    pack   $top.menu  -side top -expand yes -padx 2 -pady 2 -fill x
    return $top.menu
}

#----------------------------------------------------------------------------
#  Add a 'cascade' menu
#----------------------------------------------------------------------------

proc addCascadeMenu { w wdgName label { line {0}} } {
   return [insertCascadeMenu end $w $wdgName $label $line]
}


proc insertCascadeMenu { pos w wdgName label { line {0}} } {
    if {[cequal $wdgName ""]} {
       regsub -all {\.} [string trim $label " ."] {_}  wdgName ;# Remove '.'
    }

    set wdgPath  $w.[string tolower $wdgName]

    # Check if menu already exists
    if {[winfo exists $wdgPath]} { return "" }

    $w insert $pos cascade -label $label -underline $line -menu $wdgPath
    return [menu $wdgPath]
}

#----------------------------------------------------------------------------
#  Add a Menu-Button
#----------------------------------------------------------------------------

proc addMenuButton { w wdgName label { line {0}} { side "left"} } {

    if {[cequal $wdgName ""]} {
       regsub -all {\.} [string trim $label " ."] {_}  wdgName ;# Remove '.'
    }
 
    set wdgPath  $w.[string tolower $wdgName]

    # Check if menu already exists
    if {[winfo exists $wdgPath]} { return "" } 
     
    menubutton $wdgPath -menu $wdgPath.menu -text $label -underline $line
    pack $wdgPath -padx 2 -side $side     
                                                                         
    return [menu $wdgPath.menu]
}



#----------------------------------------------------------------------------
#  Add 'Command' Item  to a menu
#----------------------------------------------------------------------------

proc addMenuCmd { menu label line command } {
   return [insertMenuCmd end $menu $label $line $command]
}

proc insertMenuCmd { pos menu label line command } {
   $menu insert $pos command -label $label -underline $line -command $command
}



#----------------------------------------------------------------------------
#  Add 'checkButton' Item  to a menu
#----------------------------------------------------------------------------

proc addMenuCk { menu label line command { var {}}  } {
   return [insertMenuCk end $menu $label $line $command $var] 
}


proc insertMenuCk { pos menu label line command { var {}}  } {

    uifCheckNameSpace $var

    $menu insert $pos checkbutton -label $label -underline $line \
	-command $command -variable $var
}


#----------------------------------------------------------------------------
#  Add 'radioButton' Item  to a menu
#----------------------------------------------------------------------------

proc addMenuRb { menu label line command {var {a}} {value  {0}}  } {
   return [insertMenuRb end $menu $label $line $command $var $value] 
}


proc insertMenuRb { pos menu label line command {var {a}} {value {0}}  } {
    uifCheckNameSpace $var

    $menu insert $pos radiobutton -label $label -value $value \
         -command $command -variable $var -underline $line
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#                  HERE FOR BACKWARD COMPATIBILITY
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

proc addSubMenu { subMenu label { line {0}} } {

    set tmp [string trim $label " ."]
    regsub -all {\.} $tmp {}  tmp;      # Remove '.'
    regsub -all { }  $tmp {_} tmp; 
   
    set name    [string tolower $tmp]        
    set myMenu  "$subMenu.$name"
    set param   "add cascade -label \"$label\"  "
  
    if { $line != 0 } { incr line -1; append param "-underline $line" }  

    eval $subMenu  $param -menu $myMenu
                                                                         
    return [menu $myMenu]
}


proc addMenu { menuBar label { line {0}} {side "left"} } {
    set tmp [string trim $label " ."]
    regsub -all {\.} $tmp {}  tmp;      # Remove '.'
    regsub -all { }  $tmp {_} tmp; 

    set name    [string tolower $tmp]           
    set myMenu  "$menuBar.$name.menu"
    set param   "-text \"$label\" " 
  
    if { $line != 0 } { incr line -1; append param "-underline $line" }

    eval menubutton $menuBar.$name -menu $myMenu $param
                           
    pack $menuBar.$name -side $side -fill x -padx 2        
                                                                         
    return [menu $myMenu]
}

