#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifStripChartPlotter.tcl,v 3.49 2005/09/07 11:46:00 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# cerpello  22/10/96  created
#

#************************************************************************
#   NAME
#     UifStripChartPlotter - Dynamic Plotter with control buttons
#   SYNOPSIS
#     UifStripChartPlotter <widget-name>
#   DESCRIPTION
#     UifStripChartPlotter is a widget that can be configured to display a 
#     dynamic plotting that can be controlled with some buttons.
#
#     The widget is formed by:
#
#     - A 'blt' graph widget : used to display the dynamic plotting.
#     - Four buttons: the first used to start plotting, the second used to 
#       stop plotting and clear graph, the third used to pause plotting and 
#       the last used to continue plotting after a pause.
# 
#   COMPONENTS
#
#   METHODS
#     Inherit all methods by UifStripChart class.
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS 
#     Inherit all options by UifStripChart class and supports :
#
#     configFlag <normal/disabled>  Enable the user to configure 
#                                   the widget directly from the panel
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#     UifPlot UifStripChart
#   BUGS     
#
#------------------------------------------------------------------------
#
class UifStripChartPlotter {
   inherit UifStripChart
   constructor {args} {}

   itk_option define -configFlag configFlag ConfigFlag "disabled"

   public  method setAction { action }   
   private method buttonState { a b c d e}
}

#---------------------------------------------------------------------------
# Constructor
#---------------------------------------------------------------------------

body UifStripChartPlotter::constructor {args} {

   set font "-Adobe-helvetica-bold-r-normal--14*"
   
   itk_component add fbuttons {
      frame $itk_interior.fbuttons
   }
           
   itk_component add start {
      button $itk_component(fbuttons).start -text "Start" -width 6\
             -command "$this setAction start" -font $font
   } {
     keep -cursor
   } 
            
   itk_component add stop {
      button $itk_component(fbuttons).stop -text "Stop" -width 6\
             -command "$this setAction stop" -font $font -state disabled
   } {
     keep -cursor
   } 
   
   itk_component add pause {
      button $itk_component(fbuttons).pause -text "Pause" -width 6 \
             -command "$this setAction pause" -font $font -state disabled
   } {
     keep -cursor
   } 

   itk_component add continue {
      button $itk_component(fbuttons).continue -text "Continue" -width 9 \
             -command "$this setAction continue" -font $font -state disabled
   } {
     keep -cursor
   }
      
   itk_component add config {
      button $itk_component(fbuttons).config -text "Configure..." -width 11\
             -command "$this setAction configure" -font $font -state normal
   } {
     keep -cursor
   }
   
   pack $itk_component(fbuttons) -side top -fill x
   
   pack $itk_component(start) $itk_component(stop) \
        $itk_component(pause) $itk_component(continue) $itk_component(config)\
        -side left -expand 1 -pady 3
       
   set initialized 0         ;# trick to wait 'til the real initialization
   eval itk_initialize $args 
   set initialized 1

   configure -graphWidth 300  

   if {![lempty $args]} { eval configure $args } ;# User's configuration
}

#---------------------------------------------------------------------------
#  OPTION : -enableConfig
#--------------------------------------------------------------------------
configbody UifStripChartPlotter::configFlag {
   if {$initialized} {
      $itk_component(config) configure -state $itk_option(-configFlag)
   }
}

#-----------------------------------------------------------------
# METHOD : setAction
#-----------------------------------------------------------------

body UifStripChartPlotter::setAction { action } {

   switch $action {
      "start"  { buttonState disabled normal   normal   disabled normal; plot }
      "stop"   { buttonState normal   disabled disabled disabled normal; stopPlot}
      "pause"  { buttonState disabled normal   disabled normal   normal; pausePlot}
      "continue" { buttonState disabled normal   normal   disabled normal; continuePlot} 
      "configure" { 
          stripChartConfig; \
          switch $plotState  {
             "IDLE"  {buttonState normal disabled disabled disabled normal}
             "PLOT"  {buttonState disabled normal   normal   disabled normal}
             "PAUSE" {buttonState disabled normal   disabled normal   normal}
             "STOP"  {buttonState normal   disabled disabled disabled normal}
          }
      } ;# end of 'configure' switch .."
   }
}
 
 
#-----------------------------------------------------------------
# METHOD : buttonState
#-----------------------------------------------------------------

body UifStripChartPlotter::buttonState { startFlag stopFlag pauseFlag \
					 continueFlag configureFlag} {

   $itk_component(start)    configure -state $startFlag         
   $itk_component(stop)     configure -state $stopFlag
   $itk_component(pause)    configure -state $pauseFlag   
   $itk_component(continue) configure -state $continueFlag

   # If the widget is set not to be configured by the user
   # the 'configure' button is always disabled

   if {$itk_option(-configFlag) == "disabled"} {
      $itk_component(config) configure -state disabled
   } else {  
      $itk_component(config) configure -state $configureFlag
   }     
}

#
# ___oOo___
