#***********************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: wpLoadAccount.tcl,v 1.23 1998/03/09 08:55:24 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    12/04/95  created
# gfilippi  01/06/95  get year and week from the loaded file
# mcomin    24/06/06  Modifiy call to short help area
#
#************************************************************************
#   NAME
#
#     wpLoadAccount - Loads a file from the local database
#
#   SYNOPSIS
#
#     wpLoadAccount
#
#   DESCRIPTION
#
#     The fileName has the format : $HOME/.wp-<user>[_<week>]
#
#     This function is used to load both the current time-sheet
#     $HOME/.wp-<user> at the start-up or one of the previously "SUBMIted"
#     by prompting the user for the name of the file.
#
#   FILES
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#------------------------------------------------------------------------
#

proc wpLoadAccount { {file {} } } {

   global wpLastSelection wpLocalAccDir wpAccountDir wpActRecord  wpListWidget wpLocalFile
   global cv gvar wp_name shArea wpUser env

   #
   # according to the input parameters and to the current environment
   # prepare the file to be opened

   if {$file == ""} {

      #
      # select a file name from the local database
      #
      set filePath [panSelectFile $wpLocalAccDir "WP${wpUser}*"]
      if { [ string length $filePath] == 0 } { update idletasks ; return }

   } else {

      #
      # get the current time-sheet or the default one
      #
      if {([file exists $file] == 0) || ([file size $file] == 0)} {
	 set tmp "Nothing to load from the current database. Using defaults"
         $shArea displayText $tmp
         set filePath "${wpAccountDir}/../WPdefault"
      } else {

         #
         # The file exists and is not empty. Let's use it.
         set filePath $file
      }
   }

   #
   # is the selected file a real file and also accessible?
   #
   
   if { [file type $filePath] != "file" } {
      set tmp "$filePath not an ASCII file."
      $shArea displayText $tmp
      return
   }   
   
   if { [catch { open $filePath "r" } fid] } {
      set tmp "Cannot open file : $filePath"
      $shArea displayText $tmp
      return
   }

   #
   # Clear the current data and the screen

   wpClear ;                           # Clear current screen 
   $wpListWidget deleteItem 0 end ;    # Clear List Box

   #
   # load the selected file
   #
   set wpMax 0
   if { [info exists wpActRecord] } { unset wpActRecord }
   set wpActRecord(0) {}

   #
   # get year and week from the first line
   #
   gets $fid line
   if { [regexp {(Week: )([0-9]+)/([0-9]+)} $line dummy1 dummy2 year week ] } {     
      set gvar(year)  $year
      set gvar(week)  $week
   } else {
      set gvar(year)  "????"
      set gvar(week)  "??"
   }
   
   # skip the following 3 lines (header)
   gets $fid line
   gets $fid line
   gets $fid line

   while {[gets $fid line] && [eof $fid] == 0} {
      set wpActRecord($wpMax) $line
      incr wpMax
   }

   #-------------------------------------------
   #          show the data to the user
   #--------------------------------------------

   # Initialize activity list widget

   for { set i 0 } {$i < $wpMax} {incr i} {
       $wpListWidget addItem end [lindex $wpActRecord($i) $wp_name]
   }   

   # Initialize table of entry fields

   for {set i 0} {$i < $wpMax} {incr i} {
       wpSetRowValues $i
   }

   # Configure current selected entry widgets

   $wpListWidget  setCursor 1 ;
   if { $wpLastSelection == "" } { set wpLastSelection 1 }

   wpSetSelection ; # Set Default selection
   wpTotal ;        # Calculate total values
}

#------------------------------------------------------------------------
# PROCEDURE : showData  - Debugging procedure
#------------------------------------------------------------------------
proc showData { } {

   global wpActRecord  wpListWidget wp_name

   set max [$wpListWidget getSize]

   puts "----- Show Data -----"

   set max    [$wpListWidget getSize]
   set maxLen 0

   foreach member [array names wpActRecord] {
      set len [string length [getRecField $member $wp_name]]
      if { $len > $maxLen } { set maxLen $len } 
   }

   set myFormat "DBG > Rec(%3d) = %-${maxLen}s   %4d %4d %4d %4d %4d %4d %4d   %6d"

   for {set i 0} {$i < $max} {incr i} {
       set tmp $wpActRecord($i)
       set line [format $myFormat $i [lindex $tmp 0] \
                                  [lindex $tmp 1] [lindex $tmp 2] \
                                  [lindex $tmp 3] [lindex $tmp 4] \
                                  [lindex $tmp 5] [lindex $tmp 6] \
                                  [lindex $tmp 7] [lindex $tmp 8] ]
       puts $line
   }
}

# ___oOo___
