.\" This Man Page has been automatically produced. DO NOT EDIT THIS FILE!!!!!!
.\" Input file:  seqError.c
.\" Formatter :  @(#) $Id: docDoManPages.c,v 1.37 2005/09/30 13:12:01 vltsccm Exp $
.\" -----------------------------------------------
.TH seq_errResetStack n "VLTSW-JAN2006 27/10/05-16:47"
.SH NAME 
seq_errResetStack, seq_errAdd, seq_errCloseStack, seq_errPrint,
seq_errDisplay, seq_errGetStackSize, seq_errGetFromStack -
         interface to the CCS error logging

.SH SYNOPSIS 
.nf 
.LP 
\fBseq_errResetStack ?-internal|-reply?\fR

\fBseq_errAdd ?-internal|-reply? <error>\fR \fI<sourceId>\fR

\fBseq_errCloseStack ?-internal|-reply?\fR

\fBseq_errPrint ?-internal|-reply?\fR

\fBseq_errDisplay ?-internal|-reply? ?-suspend?\fR

\fBseq_errGetStackSize ?-internal|-reply?\fR

\fBseq_errGetFromStack ?-internal|-reply?\fR \fI<frameNr>\fR

.SH DESCRIPTION 
.nf 
.LP 
These commands will handle the logging of errors for the Sequencer (see
further below for a description of the \fI-internal\fR and \fI-reply\fR options):

\fBseq_errResetStack\fR: resets the error stack; to be used after the
  successful recovery from an error, or at initialization time, before
  calling \fIseq_errAdd\fR for the first time (see also \fICAUTIONS\fR section).

\fBseq_errAdd\fR adds the string <error> to the error stack, pointing to
  <sourceId> (a string) as the originator of the error.  The moduleId
  (default: "seq") with which it will be logged can be modified via the
  global Tcl variable \fIseq_moduleId\fR.  Of course this would then require that
  a proper error definition file exist for this new moduleId.  In
  particular, error number 1 is reserved for this command; it needs to have
  been defined with the format-string ".256s".

\fBseq_errCloseStack\fR logs the last added error and closes the stack; used
  if no error recovery is possible.  Remark that after this the error
  stack is reset automatically.

\fBseq_errPrint\fR will return a list with all the elements of the last CCS
  error, which was  either produced internally or received as a reply.
  The order of the list is as follows (see also a description of the CCS
  error structure for more details):
      o environment name
      o stackId structure (hostId and localNumber) as a list
      o stack sequence number
      o location identifier
      o module identifier
      o error number (must be > 0)
      o severity (W for Warning, S for Serious, F for Fatal)
      o error message (between braces if it contains spaces)
      o timestamp of the error

\fBseq_errDisplay\fR will pop up a panel showing the stack of the last CCS
  error, which was either produced internally or received as a reply.  If
  the \fI-suspend\fR option is specified, the Sequencer script will wait for
  this error panel to be closed before continuing.

\fBseq_errGetStackSize\fR returns the size of the stack of the last CCS error;
  0 means it is empty

\fBseq_errGetFromStack\fR will return the contents of the frame <frameNr> in
  the error stack as a list, formatted similar as seq_errPrint

The Sequencer uses 3 different error stacks, which can be selected via the
presence or absence of the options \fI-internal\fR and \fI-reply\fR. Also the logging
of these stacks can be influenced, via the setting of the Tcl-variable
\fIseq_errLogging\fR. The use of this variable, and the selection plus purpose
of the different stacks is explained in the next few paragraphs.

Whenever the Sequencer gets a CCS-error it cannot recover from, the
internal error stack will be submitted to an errCloseStack call
(\fIseq_errLogging\fR set to \fIauto\fR).  This means that the Sequencer script does
not need to include any instructions to close and log any error stack.
In this case, the call to errCloseStack() will actually be preceded by an
errAdd() call, identifying the Sequencer as the top-level.

If the Sequencer receives via seq_msgRecvReply an error reply, this reply
error stack will also be logged automatically (if the default setting of
the variables \fIseq_errLogging\fR and \fIseq_errReplyMerging\fR is used, see below),
however without the Sequencer appending its own message. If you want to add
your own error messages to this reply stack, you can set \fIseq_errLogging\fR
to \fI"manual"\fR and then give the proper seq_errAdd / seq_errCloseStack
commands using the option \fI-reply\fR. Alternatively, one can use the
\fIseq_errReplyMerging\fR variable to have this stack automatically merged into
one of the two other stacks (see below).

In both cases, a copy of this error stack is kept in the \fI"display stack"\fR,
so that it still can be displayed with e.g. the seq_errGet* commands even
after errCloseStack() was called automatically (which resets the stack,
rendering its previous contents unaccessible).

For seq_errResetStack, seq_errAdd and seq_errCloseStack the error stack
referred to is completely independent from the Sequencer's internal error
stack (used for returned CCS errors, as described above), unless the
\fI-internal\fR option is specified.  This option is intended for the cases
where one does not want automatic logging, by setting the global
Tcl-variable \fIseq_errLogging\fR to \fImanual\fR - see the description of
\fIseq_errLogging\fR below. Remark however that issuing a seq_errCloseStack
command with \fIseq_errLogging\fR set to \fImanual\fR behaves the same as for \fIauto\fR,
i.e. the error stack will be logged, and it will be copied into the
display stack" for further display purposes.

The error stack referred to by seq_errPrint, seq_errDisplay,
seq_errGetStackSize and seq_errGetFromStack is by default the \fI"display
stack"\fR, i.e. a copy of either the last error message received by
seq_msgRecvReply, the last internal error logged automatically or the last
error stack logged explictly via \fIseq_errCloseStack\fR.  Again, the \fI-internal\fR
and \fI-reply\fR options are only useful when the global Tcl-variable
\fIseq_errLogging\fR is set to \fImanual\fR. The \fI-reply\fR option is to refer to the last
error which was returned as a reply to a command.  This is useful in cases
where due to further script execution the "display stack" may get
overwritten by internal errors.

If both the \fI-internal\fR and \fI-reply\fR options are given, the one given last
prevails.

.SH ENVIRONMENT 
.nf 
.LP 
The environment variable \fBNO_ERR_DISPLAY\fR affects the behaviour of
seq_errDisplay, in that it will send its output to stdout instead of
popping up a new window if set.  See also errDisplay(3).

The global Tcl variable \fBseq_errLogging\fR reflects how the logging of errors
occuring during the execution of seq_* commands are dealt with; \fI"auto"\fR
(default) leads to automatic logging of all unrecoverable errors (by
issuing an errCloseStack()), \fI"off"\fR will not log any error (by issuing an
errResetStack), while \fI"manual"\fR will not issue any err*Stack command.In the
latter case it is assumed that the script will take care of the error stack
via "seq_err* -internal|-reply" commands. In all cases, the corresponding
error stack will be copied over to the "display stack".

The global Tcl variable \fBseq_errReplyMerging\fR determines how the merging of
error-replies will take place: \fI"internal"\fR will append the reply-stack to
the internal error stack, \fI"local"\fR will make the reply-stack append to the
default stack of seq_errAdd etc, while \fI"off"\fR (default) leaves the error
reply stack alone. In the former 2 cases, the error reply stack will be
reset after merging, without any logging. In the latter case, it will get
logged automatically if seq_errLogging is set to \fI"auto"\fR, in which case it
will also be copied over to the "display stack".

The global Tcl variable \fBseq_moduleId\fR has its effect on how errors are
logged with seq_errAdd - see above.

.SH RETURN VALUES 
.nf 
.LP 
OK return if no errors occurred.  The result string is empty.
ERROR return if one of the following conditions is met (with corresponding
error message strings here printed in boldface):

  - \fBwrong # args : should be "seq_errResetStack ?-internal|-reply?"\fR
    \fBwrong # args : should be "seq_errAdd <error> ?-internal|-reply?
    <sourceId>"\fR
    \fBwrong # args : should be "seq_errCloseStack ?-internal|-reply?"\fR
    \fBwrong # args : should be "seq_errGetStackSize ?-internal|-reply?"\fR
    \fBwrong # args : should be "seq_errGetFromStack ?-internal|-reply?
                    <frameNr>"\fR
    \fBwrong # args : should be "seq_errPrint ?-internal|-reply?"\fR
    \fBwrong # args : should be "seq_errDisplay ?-internal|-reply?
                    ?-suspend?"\fR
    the corresponding Sequencer command was given with the wrong number
    of arguments;

  - \fBsourceId "<sourceId>" too long\fR if the string length of <sourceId>
    exceeds the limits imposed by the ccsERROR structure;

  - error message left by Tcl_GetInt, when trying to interpret <frameNr>
    as an integer;

  - \fBcould not do an errAdd - check the log\fR if during the processing of
    the seq_errAdd command the call to errAdd(3) failed.

  - \fBcould not get error-frame <frameNr> from stack\fR if seq_errGetFromStack
    returns an error (e.g. <frameNr> exceeds the current stack size)

  - \fBwrong option: <option> should be "-internal", "-reply" or "-suspend"\fR
    if any of these words were misspelled for the seq_errDisplay command

.SH CAUTIONS 
.nf 
.LP 
Under NOCCS, only seq_errResetStack, seq_errAdd and seq_errCloseStack
are avaliable.

Under CCS-lite, the current setting of RTAPENV is taken whenever the first
CCS-function gets called. If this first CCS-function is not a ccsInit(),
this value of RTAPENV will anyway be used later when ccsInit() get called,
independent of RTAPENV's value at that later time. This means that e.g. in
the \fIwtctest\fR environment, issuing first a \fIseq_errResetStack\fR command
followed by a \fIset env(RTAPENV) xyz\fR and a \fIseq_ccsInit\fR command, will lead
to a ccsInit() in the \fIwtctest\fR environment instead of the \fIxyz\fR environment.
This is not inherent to seq, but rather to CCS-lite. In fact, to enable an
effective modification of the environment variable RTAPENV within a
Sequencer script (e.g. based on runstring options), the Sequencer's internal
initialization does not call any CCS-function under CCS-lite, thereby
shifting the responsibility to do things in the proper order to the
application.

seq_errGetStackSize and seq_errGetFromStack manipulate either the reply or
the internal error stack.  The stack set aside for seq_errAdd,
seq_errCloseStack and seq_errResetStack is a different one, and hence
cannot be seen by the seq_errGet* commands.

The arguments of seq_errAdd will all be logged as errors with a severity
of "warning".

seq_errDisplay will by default use the information of the last CCS error,
which is stored in a global C-variable for use by all seq-commands.  So if
the seq_errDisplay command is not called immediately after detecting such
an error, or if the -suspend option is not used, the displayed values may
not correspond to what is expected.
Subsequent calls to seq_errDisplay, without destroying windows in between,
will simply update the information displayed in these error stack windows.
This may not be the desired effect for panels with classes that can try
simultaneously to display errors via the seq_errDisplay command.

.SH EXAMPLES 
.nf 
.LP 
(For clarity's sake, the example is for an interactive session with
input typed by the user coming after the "seqWish>" prompt, and all
replies on a new line)

 .....
seqWish>seq_errResetStack
seqWish>seq_errAdd "Who the hell deleted this file??" "my procedure"
seqWish>seq_errCloseStack
 ...(This error can now be seen with the log-monitor)...
 .....
seqWish>seqSendCommand wte67 otherSeq SCRIPT {puts hello}
seqWIsh>seq_msgRecvReply cmd0 en lr -all
seqERR_NO_CMDS_ALLOWED : Command "SCRIPT" rejected - no commands allowed
seqWish>seq_errGetStackSize
1
seqWish>seq_errGetFromStack 1
wte67 {0 4869} 1 seqMsgDispatch.c seq 122 W {seqERR_NO_CMDS_ALLOWED :
Command "SCRIPT" rejected - no commands allowed} {97-02-14
15:35:53.324656}
seqWish>.....

.SH SEE ALSO 
.nf 
.LP 
"Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
errResetStack(3), errAdd(3), errCloseStack(3), errGetStackSize(3),
errGetFromStack(3), errDisplay


